//-----------------------------------------------------------------------------
//  (c) 2006 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "ChunkTest.h"
#include "GenApi/GenApi.h"
#include "GenApi/ChunkAdapterGEV.h"
#include "GenApi/ChunkAdapterDcam.h"
#include "GenApi/ChunkAdapterGeneric.h"
#include "StopWatch.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

#ifndef ARRAYSIZE
#  define ARRAYSIZE(x) (sizeof(x) / sizeof (x)[0])
#endif

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ChunkTestSuite, "GenApiTest");

class CCallbackTarget
{
public:
    CCallbackTarget() :
        m_CallbackFired(false)
    {}

    void OnUpdate(INode* /*pNode*/)
    {
        m_CallbackFired = true;
    }

    bool HasCallbackFired()
    {
        bool temp = m_CallbackFired;
        m_CallbackFired = false;
        return temp;
    }

    bool m_CallbackFired;
};

unsigned long CRC16(const unsigned char *pData, unsigned long nbyLength )
{
    unsigned long i, j, c, bit;
    unsigned long crc = 0;
    for (i=0; i<nbyLength; i++) {
        c = (unsigned long)*pData++;
        for (j=0x80; j; j>>=1) {
            bit = crc & 0x8000;
            crc <<= 1;
            if (c & j) bit^= 0x8000;
            if (bit) crc^= 0x1021;
        }
    }
    return (unsigned long) (crc & 0xffff);
};


void ChunkTestSuite::TestChunkAdapterDcam()
{
    // added ChunkBaseAddress and ChunkLenght
    if(GenApiSchemaVersion == v1_0)
        return;

    // create a camera description file
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <IntReg Name=\"Value1\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value2\" >                                \n"
    "        <Address>0x04</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"ChunkBaseAddress1\" >                                \n"
    "        <Address>0x7fffffffffffffff</Address>                                \n"
    "        <Length>8</Length>                                    \n"
    "        <AccessMode>RO</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Signed</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"ChunkLength1\" >                                \n"
    "        <Address>0x7ffffffffffffff0</Address>                                \n"
    "        <Length>8</Length>                                    \n"
    "        <AccessMode>RO</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Signed</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value3\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value4\" >                                \n"
    "        <Address>-4</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"ChunkBaseAddress2\" >                                \n"
    "        <Address>0x7fffffffffffffff</Address>                                \n"
    "        <Length>8</Length>                                    \n"
    "        <AccessMode>RO</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Signed</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"ChunkLength2\" >                                \n"
    "        <Address>0x7ffffffffffffff0</Address>                                \n"
    "        <Length>8</Length>                                    \n"
    "        <AccessMode>RO</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Signed</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <Port Name=\"MyPort1\" >                                \n"
    "        <ChunkID>E22B3A4791154560BD468608AA90AE30</ChunkID>    \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    "    <Port Name=\"MyPort2\" >                                \n"
    "        <ChunkID>8135B3ECE1A240548B5A51322FF5DC89</ChunkID>    \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    + CDFFooter;

    // create and connect a camera object
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Create and connect the event adapter
    CChunkAdapterDcam ChunkAdapter( Camera._Ptr );
    // understood by msc and gcc 4++
#   pragma pack(push, 1)

    // dummy ChunkData
    typedef struct Buffer_t
    {
         uint32_t Value1 ;
         uint32_t Value2 ;
         DCAM_CHUNK_TRAILER Trailer1 ;
         uint32_t Value3 ;
         uint8_t Data1[504] ;
         uint32_t Value4 ;
         DCAM_CHUNK_TRAILER Trailer2 ;
    } Buffer_t;

    typedef GUID ChunkID_t;

#   pragma pack(pop)

    Buffer_t Buffer;

    memset(&Buffer, 0, sizeof(Buffer_t));
    Buffer.Value1 = 12;
    Buffer.Value2 = 34;

    // {E22B3A47-9115-4560-BD46-8608AA90AE30}
    SET_GUID(Buffer.Trailer1.ChunkID,
        0x473a2be2, 0x1591, 0x6045, 0xbd, 0x46, 0x86, 0x8, 0xaa, 0x90, 0xae, 0x30);
    Buffer.Trailer1.ChunkLength = 2 * sizeof(uint32_t) + sizeof(DCAM_CHUNK_TRAILER);
    Buffer.Trailer1.InverseChunkLength = ~Buffer.Trailer1.ChunkLength;

    Buffer.Value3 = 56;
    Buffer.Value4 = 78;
    // {8135B3EC-E1A2-4054-8B5A-51322FF5DC89}
    SET_GUID(Buffer.Trailer2.ChunkID ,
        0xecb33581, 0xa2e1, 0x5440, 0x8b, 0x5a, 0x51, 0x32, 0x2f, 0xf5, 0xdc, 0x89);
    Buffer.Trailer2.ChunkLength = sizeof(uint32_t) + 508 + sizeof(DCAM_CHUNK_TRAILER);
    Buffer.Trailer2.InverseChunkLength = ~Buffer.Trailer2.ChunkLength;

    CIntegerPtr ptrValue1 = Camera._GetNode("Value1");
    CPPUNIT_ASSERT( ptrValue1.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue1) );

    CIntegerPtr ptrValue2 = Camera._GetNode("Value2");
    CPPUNIT_ASSERT( ptrValue2.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue2) );

    CIntegerPtr ptrValue3 = Camera._GetNode("Value3");
    CPPUNIT_ASSERT( ptrValue3.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );

    CIntegerPtr ptrValue4 = Camera._GetNode("Value4");
    CPPUNIT_ASSERT( ptrValue4.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue4) );

    CIntegerPtr ptrChunkBaseAddress1 = Camera._GetNode("ChunkBaseAddress1");
    CPPUNIT_ASSERT( ptrChunkBaseAddress1.IsValid() );
    CIntegerPtr ptrChunkLength1 = Camera._GetNode("ChunkLength1");
    CPPUNIT_ASSERT( ptrChunkLength1.IsValid() );
    CIntegerPtr ptrChunkBaseAddress2 = Camera._GetNode("ChunkBaseAddress2");
    CPPUNIT_ASSERT( ptrChunkBaseAddress2.IsValid() );
    CIntegerPtr ptrChunkLength2 = Camera._GetNode("ChunkLength2");
    CPPUNIT_ASSERT( ptrChunkLength2.IsValid() );


    CCallbackTarget CallbackTarget;
    CNodePtr ptrValue3Node( ptrValue3 );
    GenApi::Register( ptrValue3Node, CallbackTarget, &CCallbackTarget::OnUpdate );

    /*--------------------------------------*/

   CPPUNIT_ASSERT( !ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, -1 ) );
   CPPUNIT_ASSERT( !ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, sizeof(DCAM_CHUNK_TRAILER) ) );
    CPPUNIT_ASSERT( ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, sizeof(Buffer_t) ) );

    /*--------------------------------------*/

    AttachStatistics_t AttachStatistics;

    // Attach the buffer and check the callback and values
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics );
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumAttachedChunks);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunkPorts);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunks);
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)78, ptrValue4->GetValue() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)&Buffer.Value1 , ptrChunkBaseAddress1->GetValue() );
    // Beware, the trailer is not(!) part of the chunk's lenght
    CPPUNIT_ASSERT_EQUAL( (int64_t)(sizeof Buffer.Value1 + sizeof Buffer.Value2), ptrChunkLength1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( Buffer.Value1, *(uint32_t*)ptrChunkBaseAddress1->GetValue() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)&Buffer.Value3, ptrChunkBaseAddress2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)(sizeof Buffer.Value3 + sizeof Buffer.Data1 + sizeof Buffer.Value4), ptrChunkLength2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( Buffer.Value3, *(uint32_t*)ptrChunkBaseAddress2->GetValue() );

    CRegisterPtr ptrRegValue4 = (CRegisterPtr) ptrValue4;
    CPPUNIT_ASSERT( ptrRegValue4.IsValid() );
    int64_t Value4Address = ptrChunkBaseAddress2->GetValue() + ptrRegValue4->GetAddress();
    CPPUNIT_ASSERT_EQUAL( Buffer.Value4, *(uint32_t*)Value4Address );

    /*--------------------------------------*/

    // create a buffer with CRC
    uint8_t pBufferWithCRC[sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM)];

    memcpy(pBufferWithCRC, &Buffer, sizeof(Buffer_t) );

    *(uint32_t*)(pBufferWithCRC + sizeof(Buffer_t)) = CRC16(pBufferWithCRC, sizeof(Buffer_t));

   //Check the layout of the buffer
   CPPUNIT_ASSERT( ChunkAdapter.CheckBufferLayout( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM)) );

    // Attach the buffer and check the callback and values
     CPPUNIT_ASSERT( !ChunkAdapter.HasCRC( pBufferWithCRC, sizeof(DCAM_CHECKSUM) - 1 ) );  // buffer too small
     CPPUNIT_ASSERT( !ChunkAdapter.HasCRC( pBufferWithCRC, sizeof(Buffer_t ) ));  // buffer has no checksum
    CPPUNIT_ASSERT( ChunkAdapter.HasCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ) );
    CPPUNIT_ASSERT( ChunkAdapter.CheckCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ) );
    ChunkAdapter.AttachBuffer( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM));
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)78, ptrValue4->GetValue() );


    // regression test for trac #909  (chunk parser reports invalid layout if CRC equals length of last chunk )
    *(uint32_t*)(pBufferWithCRC + sizeof(Buffer_t)) = Buffer.Trailer2.ChunkLength;
   CPPUNIT_ASSERT( ChunkAdapter.CheckBufferLayout( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM)) );

    // ruin the CRC
    *(uint32_t*)(pBufferWithCRC + sizeof(Buffer_t)) = 0x12345678;
    CPPUNIT_ASSERT( !ChunkAdapter.CheckCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ) );

    /*--------------------------------------*/

    // Copy the buffer and change the values
    Buffer_t Buffer2;
    memcpy(&Buffer2, &Buffer, sizeof(Buffer_t));
    Buffer2.Value1 = 98;
    Buffer2.Value2 = 76;
    Buffer2.Value3 = 54;
    Buffer2.Value4 = 32;

    // update the buffer (same layout)
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.UpdateBuffer( NULL ), GenICam::RuntimeException );
    ChunkAdapter.UpdateBuffer( (uint8_t*)&Buffer2 );
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)98, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)76, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)54, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)32, ptrValue4->GetValue() );

   /*--------------------------------------*/

   //Modify chunk ports through pointers to the IntRegs and check
   //for corresponding buffer modifications.
   ptrValue1->SetValue(20);
   ptrValue2->SetValue(40);
   ptrValue3->SetValue(60);
   ptrValue4->SetValue(70);

   CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
   CPPUNIT_ASSERT(20 == Buffer2.Value1);
   CPPUNIT_ASSERT(40 == Buffer2.Value2);
   CPPUNIT_ASSERT(60 == Buffer2.Value3);
   CPPUNIT_ASSERT(70 == Buffer2.Value4);

    /*--------------------------------------*/

    // Change bue buffer's layout by changing the 2nd ChunkID
    // {D11006E8-A9E9-4819-99A0-8FFF3468A7B5}
    SET_GUID(Buffer.Trailer2.ChunkID,
        0xd11006e8, 0xa9e9, 0x4819, 0x99, 0xa0, 0x8f, 0xff, 0x34, 0x68, 0xa7, 0xb5);

    // Attach the buffer (new layout, so don't use UpdateBuffer)
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics );
    CPPUNIT_ASSERT_EQUAL(1, AttachStatistics.NumAttachedChunks);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunkPorts);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunks);
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );

   /*--------------------------------------*/

   //Detach the buffer and check for callback and values
   ChunkAdapter.DetachBuffer();

   //These writes must fail since the ports are not mapped to any memory address by the chunk adapter.
   CPPUNIT_ASSERT_THROW_EX( ptrValue1->SetValue(36), GenICam::GenericException );
   CPPUNIT_ASSERT_THROW_EX( ptrValue2->SetValue(72), GenICam::GenericException );

   //The buffer that was previously attached to the chunk adapter must not be modified.
   CPPUNIT_ASSERT( 12 == Buffer.Value1 );
   CPPUNIT_ASSERT( 34 == Buffer.Value2 );

   {
     // The buffer Layout is incorrect
     uint8_t Buffer[sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM)];
     memset(Buffer, 10, sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM));
     CPPUNIT_ASSERT(ChunkAdapter.CheckBufferLayout((uint8_t *)&Buffer, sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM)) == false);
   }
   Buffer_t MyBuffer;
   memcpy(&MyBuffer, &Buffer, sizeof (Buffer_t));
   MyBuffer.Trailer1.InverseChunkLength = 0;
   MyBuffer.Trailer2.InverseChunkLength = 0;
   CPPUNIT_ASSERT(ChunkAdapter.CheckBufferLayout((uint8_t *)&MyBuffer, sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM)) == false);


    // try to attach invalid buffers
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( 0, sizeof(Buffer_t), &AttachStatistics ), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, 0, &AttachStatistics ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = 0;
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = 65000;
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = 2 * sizeof(uint32_t) + sizeof(DCAM_CHUNK_TRAILER);

    // and do other weird things
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.CheckCRC( NULL, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ), GenICam::RuntimeException );
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.CheckCRC( pBufferWithCRC, 0 ), GenICam::RuntimeException );
    ((Buffer_t*)pBufferWithCRC)->Trailer2.ChunkLength = 1000;
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.CheckCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ), GenICam::RuntimeException );

}

void ChunkTestSuite::TestChunkAdapterDcam_legacy_v1_0()
{
    // create a camera description file
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <IntReg Name=\"Value1\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value2\" >                                \n"
    "        <Address>0x04</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value3\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value4\" >                                \n"
    "        <Address>-4</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <Port Name=\"MyPort1\" >                                \n"
    "        <ChunkID>E22B3A4791154560BD468608AA90AE30</ChunkID>    \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    "    <Port Name=\"MyPort2\" >                                \n"
    "        <ChunkID>8135B3ECE1A240548B5A51322FF5DC89</ChunkID>    \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    + CDFFooter;

    // create and connect a camera object
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Create and connect the event adapter
    CChunkAdapterDcam ChunkAdapter( Camera._Ptr );

#   pragma pack(push, 1)

    // dummy ChunkData
    typedef struct Buffer_t
    {
         uint32_t Value1 ;
         uint32_t Value2 ;
         DCAM_CHUNK_TRAILER Trailer1 ;
         uint32_t Value3 ;
         uint8_t Data1[504] ;
         uint32_t Value4 ;
         DCAM_CHUNK_TRAILER Trailer2 ;
    } Buffer_t;

    typedef GUID ChunkID_t;

#   pragma pack(pop)

    Buffer_t Buffer;

    memset(&Buffer, 0, sizeof(Buffer_t));
    Buffer.Value1 = 12;
    Buffer.Value2 = 34;

    // {E22B3A47-9115-4560-BD46-8608AA90AE30}
    SET_GUID(Buffer.Trailer1.ChunkID,
        0x473a2be2, 0x1591, 0x6045, 0xbd, 0x46, 0x86, 0x8, 0xaa, 0x90, 0xae, 0x30);
    Buffer.Trailer1.ChunkLength = 2 * sizeof(uint32_t) + sizeof(DCAM_CHUNK_TRAILER);
    Buffer.Trailer1.InverseChunkLength = ~Buffer.Trailer1.ChunkLength;

    Buffer.Value3 = 56;
    Buffer.Value4 = 78;
    // {8135B3EC-E1A2-4054-8B5A-51322FF5DC89}
    SET_GUID(Buffer.Trailer2.ChunkID ,
        0xecb33581, 0xa2e1, 0x5440, 0x8b, 0x5a, 0x51, 0x32, 0x2f, 0xf5, 0xdc, 0x89);
    Buffer.Trailer2.ChunkLength = sizeof(uint32_t) + 508 + sizeof(DCAM_CHUNK_TRAILER);
    Buffer.Trailer2.InverseChunkLength = ~Buffer.Trailer2.ChunkLength;

    CIntegerPtr ptrValue1 = Camera._GetNode("Value1");
    CPPUNIT_ASSERT( ptrValue1.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue1) );

    CIntegerPtr ptrValue2 = Camera._GetNode("Value2");
    CPPUNIT_ASSERT( ptrValue2.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue2) );

    CIntegerPtr ptrValue3 = Camera._GetNode("Value3");
    CPPUNIT_ASSERT( ptrValue3.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );

    CIntegerPtr ptrValue4 = Camera._GetNode("Value4");
    CPPUNIT_ASSERT( ptrValue4.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue4) );

    CCallbackTarget CallbackTarget;
    CNodePtr ptrValue3Node( ptrValue3 );
    GenApi::Register( ptrValue3Node, CallbackTarget, &CCallbackTarget::OnUpdate );

    /*--------------------------------------*/

   CPPUNIT_ASSERT( !ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, -1 ) );
   CPPUNIT_ASSERT( !ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, sizeof(DCAM_CHUNK_TRAILER) ) );
    CPPUNIT_ASSERT( ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, sizeof(Buffer_t) ) );

    /*--------------------------------------*/

    AttachStatistics_t AttachStatistics;

    // Attach the buffer and check the callback and values
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics );
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumAttachedChunks);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunkPorts);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunks);
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)78, ptrValue4->GetValue() );

    /*--------------------------------------*/

    // create a buffer with CRC
    uint8_t pBufferWithCRC[sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM)];

    memcpy(pBufferWithCRC, &Buffer, sizeof(Buffer_t) );

    *(uint32_t*)(pBufferWithCRC + sizeof(Buffer_t)) = CRC16(pBufferWithCRC, sizeof(Buffer_t));

   //Check the layout of the buffer
   CPPUNIT_ASSERT( ChunkAdapter.CheckBufferLayout( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM)) );

    // Attach the buffer and check the callback and values
     CPPUNIT_ASSERT( !ChunkAdapter.HasCRC( pBufferWithCRC, sizeof(DCAM_CHECKSUM) - 1 ) );  // buffer too small
     CPPUNIT_ASSERT( !ChunkAdapter.HasCRC( pBufferWithCRC, sizeof(Buffer_t ) ));  // buffer has no checksum
    CPPUNIT_ASSERT( ChunkAdapter.HasCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ) );
    CPPUNIT_ASSERT( ChunkAdapter.CheckCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ) );
    ChunkAdapter.AttachBuffer( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM));
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)78, ptrValue4->GetValue() );

    // ruin the CRC
    *(uint32_t*)(pBufferWithCRC + sizeof(Buffer_t)) = 0x12345678;
    CPPUNIT_ASSERT( !ChunkAdapter.CheckCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ) );

    /*--------------------------------------*/

    // Copy the buffer and change the values
    Buffer_t Buffer2;
    memcpy(&Buffer2, &Buffer, sizeof(Buffer_t));
    Buffer2.Value1 = 98;
    Buffer2.Value2 = 76;
    Buffer2.Value3 = 54;
    Buffer2.Value4 = 32;

    // update the buffer (same layout)
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.UpdateBuffer( NULL ), GenICam::RuntimeException );
    ChunkAdapter.UpdateBuffer( (uint8_t*)&Buffer2 );
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)98, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)76, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)54, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)32, ptrValue4->GetValue() );

   /*--------------------------------------*/

   //Modify chunk ports through pointers to the IntRegs and check
   //for corresponding buffer modifications.
   ptrValue1->SetValue(20);
   ptrValue2->SetValue(40);
   ptrValue3->SetValue(60);
   ptrValue4->SetValue(70);

   CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
   CPPUNIT_ASSERT(20 == Buffer2.Value1);
   CPPUNIT_ASSERT(40 == Buffer2.Value2);
   CPPUNIT_ASSERT(60 == Buffer2.Value3);
   CPPUNIT_ASSERT(70 == Buffer2.Value4);

    /*--------------------------------------*/

    // Change bue buffer's layout by changing the 2nd ChunkID
    // {D11006E8-A9E9-4819-99A0-8FFF3468A7B5}
    SET_GUID(Buffer.Trailer2.ChunkID,
        0xd11006e8, 0xa9e9, 0x4819, 0x99, 0xa0, 0x8f, 0xff, 0x34, 0x68, 0xa7, 0xb5);

    // Attach the buffer (new layout, so don't use UpdateBuffer)
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics );
    CPPUNIT_ASSERT_EQUAL(1, AttachStatistics.NumAttachedChunks);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunkPorts);
    CPPUNIT_ASSERT_EQUAL(2, AttachStatistics.NumChunks);
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );

   /*--------------------------------------*/

   //Detach the buffer and check for callback and values
   ChunkAdapter.DetachBuffer();

   //These writes must fail since the ports are not mapped to any memory address by the chunk adapter.
   CPPUNIT_ASSERT_THROW_EX( ptrValue1->SetValue(36), GenICam::GenericException );
   CPPUNIT_ASSERT_THROW_EX( ptrValue2->SetValue(72), GenICam::GenericException );

   //The buffer that was previously attached to the chunk adapter must not be modified.
   CPPUNIT_ASSERT( 12 == Buffer.Value1 );
   CPPUNIT_ASSERT( 34 == Buffer.Value2 );

   {
     // The buffer Layout is incorrect
     uint8_t Buffer[sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM)];
     memset(Buffer, 10, sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM));
     CPPUNIT_ASSERT(ChunkAdapter.CheckBufferLayout((uint8_t *)&Buffer, sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM)) == false);
   }
   Buffer_t MyBuffer;
   memcpy(&MyBuffer, &Buffer, sizeof (Buffer_t));
   MyBuffer.Trailer1.InverseChunkLength = 0;
   MyBuffer.Trailer2.InverseChunkLength = 0;
   CPPUNIT_ASSERT(ChunkAdapter.CheckBufferLayout((uint8_t *)&MyBuffer, sizeof(DCAM_CHUNK_TRAILER) + sizeof(DCAM_CHECKSUM)) == false);


    // try to attach invalid buffers
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( 0, sizeof(Buffer_t), &AttachStatistics ), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, 0, &AttachStatistics ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = 0;
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = 65000;
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStatistics ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = 2 * sizeof(uint32_t) + sizeof(DCAM_CHUNK_TRAILER);

    // and do other weird things
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.CheckCRC( NULL, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ), GenICam::RuntimeException );
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.CheckCRC( pBufferWithCRC, 0 ), GenICam::RuntimeException );
    ((Buffer_t*)pBufferWithCRC)->Trailer2.ChunkLength = 1000;
    CPPUNIT_ASSERT_THROW_EX( ChunkAdapter.CheckCRC( pBufferWithCRC, sizeof(Buffer_t) + sizeof(DCAM_CHECKSUM) ), GenICam::RuntimeException );

}

void ChunkTestSuite::TestChunkAdapterGEV()
{
    // create a camera description file
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <IntReg Name=\"Value1\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                \n"
    "    </IntReg>                                                   \n"
    "                                                                  \n"
    "    <IntReg Name=\"Value2\" >                                \n"
    "        <Address>0x04</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value3\" >                                \n"
    "        <Address>0x00</Address>                              \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value4\" >                                \n"
    "        <Address>-4</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <Port Name=\"MyPort1\" >                                \n"
    "        <ChunkID>12344321</ChunkID>                            \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    "    <Port Name=\"MyPort2\" >                                \n"
    "        <ChunkID>56788765</ChunkID>                            \n"
    "    </Port>                                                    \n"
   "    <Port Name=\"BogusPort\" >                                \n"
    "        <ChunkID>ff</ChunkID>                            \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    + CDFFooter;

    // create and connect a camera object
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Create and connect the chunk adapter
    CChunkAdapterGEV ChunkAdapter( Camera._Ptr );
    ChunkAdapter.AttachNodeMap( Camera._Ptr ); // attach the same nodemap again, should not break anything
    CChunkAdapterGEV BlankChunkAdapter(NULL);

#   pragma pack(push, 1)

    // dummy ChunkData
    typedef struct Buffer_t
    {
         uint32_t Value1 ;
         uint32_t Value2 ;
         GVCP_CHUNK_TRAILER Trailer1 ;
         uint32_t Value3 ;
         uint8_t Data1[504] ;
         uint32_t Value4 ;
         GVCP_CHUNK_TRAILER Trailer2 ;
    } Buffer_t;

    typedef struct ChunkID_t
    {
         uint8_t Byte1 ;
         uint8_t Byte2 ;
         uint8_t Byte3 ;
         uint8_t Byte4 ;
    } ChunkID_t;

#   pragma pack(pop)

   AttachStatistics_t AttachStats;
    Buffer_t Buffer;

    memset(&Buffer, 0, sizeof(Buffer_t));
    Buffer.Value1 = 12;
    Buffer.Value2 = 34;
    // 0x12344321
    ChunkID_t ChunkID1 = { 0x12, 0x34, 0x43, 0x21 };
    Buffer.Trailer1.ChunkID = *(uint32_t*)&ChunkID1;
    Buffer.Trailer1.ChunkLength = htonl(2 * sizeof(uint32_t));

    Buffer.Value3 = 56;
    Buffer.Value4 = 78;
    // 0x56788765
    ChunkID_t ChunkID2 = { 0x56, 0x78, 0x87, 0x65 };
    Buffer.Trailer2.ChunkID = *(uint32_t*)&ChunkID2;
    Buffer.Trailer2.ChunkLength = htonl(sizeof(uint32_t) + 508);

    CIntegerPtr ptrValue1 = Camera._GetNode("Value1");
    CPPUNIT_ASSERT( ptrValue1.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue1) );

    CIntegerPtr ptrValue2 = Camera._GetNode("Value2");
    CPPUNIT_ASSERT( ptrValue2.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue2) );

    CIntegerPtr ptrValue3 = Camera._GetNode("Value3");
    CPPUNIT_ASSERT( ptrValue3.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );

    CIntegerPtr ptrValue4 = Camera._GetNode("Value4");
    CPPUNIT_ASSERT( ptrValue4.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue4) );

    CCallbackTarget CallbackTarget;
    CNodePtr ptrValue3Node( ptrValue3 );
    GenApi::Register( ptrValue3Node, CallbackTarget, &CCallbackTarget::OnUpdate );

   //Check the layout of the buffer
   CPPUNIT_ASSERT( ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, sizeof(Buffer_t)) );

   //Check the return value of CheckBufferLayout if the buffer is not as large as a standard GVCP chunk trailer
   CPPUNIT_ASSERT( !ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, -1) );

    /*--------------------------------------*/

    // Attach the buffer and check the callback and values
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t) );
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT( IsReadable(ptrValue4) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)78, ptrValue4->GetValue() );

    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : Value1 <=> Port1");
    LOG_PERFORMANCE("        Value2 <=> Port1");
    LOG_PERFORMANCE("        Value3 <=> Port2");
    LOG_PERFORMANCE("                   Port3");

    TEST_BEGIN(1000)
        ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t) );
        StopWatch.PauseBegin();
        ChunkAdapter.DetachBuffer();
        StopWatch.PauseEnd();
    TEST_END( attach buffer - firing 1 callback )

    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t) );
    TEST_BEGIN(1000)
        ChunkAdapter.UpdateBuffer( (uint8_t*)&Buffer );
    TEST_END( update buffer - firing 1 callback )

    /*--------------------------------------*/

    // Copy the buffer and change the values
    Buffer_t Buffer2;
    memcpy(&Buffer2, &Buffer, sizeof(Buffer_t));
    Buffer2.Value1 = 98;
    Buffer2.Value2 = 76;
    Buffer2.Value3 = 54;
    Buffer2.Value4 = 32;

    // update the buffer (same layout)
    ChunkAdapter.UpdateBuffer( (uint8_t*)&Buffer2 );
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( IsReadable(ptrValue3) );
    CPPUNIT_ASSERT( IsReadable(ptrValue4) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)98, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)76, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)54, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)32, ptrValue4->GetValue() );

   /*--------------------------------------*/

   //Modify chunk ports through pointers to the IntRegs and check
   //for corresponding buffer modifications.
   ptrValue1->SetValue(20);
   ptrValue2->SetValue(40);
   ptrValue3->SetValue(60);
   ptrValue4->SetValue(80);

   CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
   CPPUNIT_ASSERT(20 == Buffer2.Value1);
   CPPUNIT_ASSERT(40 == Buffer2.Value2);
   CPPUNIT_ASSERT(60 == Buffer2.Value3);
   CPPUNIT_ASSERT(80 == Buffer2.Value4);

    // Modify the data through Read/Write port methods
    CPortPtr ptrMyPort2 = Camera._GetNode ("MyPort2");
    CPPUNIT_ASSERT (ptrMyPort2.IsValid());

    uint32_t Val = 11;
    ptrMyPort2->Write (&Val, -4, 4);
    CPPUNIT_ASSERT(11 == Buffer2.Value4);
    Val = 0;
    ptrMyPort2->Read (&Val, -4, 4);
    CPPUNIT_ASSERT(11 == Val);

    /*--------------------------------------*/

    // Change bue buffer's layout by changing the 2nd ChunkID
    // 0xabcddcba
    ChunkID_t ChunkID2a = { 0xab, 0xcd, 0xdc, 0xba };
    Buffer.Trailer2.ChunkID = *(uint32_t*)&ChunkID2a;

    // Attach the buffer (new layout, so don't use UpdateBuffer)
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStats );
    CPPUNIT_ASSERT( CallbackTarget.HasCallbackFired() );
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT( IsReadable(ptrValue2) );
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );
    CPPUNIT_ASSERT( !IsReadable(ptrValue4) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
   CPPUNIT_ASSERT( 3 == AttachStats.NumChunkPorts );
   CPPUNIT_ASSERT( 2 == AttachStats.NumChunks );
   CPPUNIT_ASSERT( 1 == AttachStats.NumAttachedChunks );

   /*--------------------------------------*/

   //Detach the buffer and check for callback and values
   ChunkAdapter.DetachBuffer();

   //These writes must fail since the ports are not mapped to any memory address by the chunk adapter.
   CPPUNIT_ASSERT_THROW_EX( ptrValue1->SetValue(36), GenICam::GenericException );
   CPPUNIT_ASSERT_THROW_EX( ptrValue2->SetValue(72), GenICam::GenericException );

   //The buffer that was previously attached to the chunk adapter must not be modified.
   CPPUNIT_ASSERT( 12 == Buffer.Value1 );
   CPPUNIT_ASSERT( 34 == Buffer.Value2 );


    // try to attach invalid buffers
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( 0, sizeof(Buffer_t), &AttachStats ), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, 0, &AttachStats ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = htonl(1);
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStats ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = htonl(1000);
    CPPUNIT_ASSERT_THROW_EX (ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t), &AttachStats ), GenICam::RuntimeException);
    Buffer.Trailer1.ChunkLength = htonl(2 * sizeof(uint32_t));

}

void ChunkTestSuite::TestChunkAdapterDcamCached()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    // create a camera description file
    gcstring CameraDescriptionFile =

    CDFHeader +
    "    <IntReg Name=\"Value1\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value2\" >                                \n"
    "        <Address>0x04</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"ChunkBaseAddress1\" >                                \n"
    "        <Address>0x7fffffffffffffff</Address>                                \n"
    "        <Length>8</Length>                                    \n"
    "        <AccessMode>RO</AccessMode>                        \n"
    "        <pPort>MyPort1</pPort>                                \n"
    "        <Sign>Signed</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value3\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"Value4\" >                                \n"
    "        <Address>-4</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <IntReg Name=\"ChunkBaseAddress2\" >                                \n"
    "        <Address>0x7fffffffffffffff</Address>                                \n"
    "        <Length>8</Length>                                    \n"
    "        <AccessMode>RO</AccessMode>                        \n"
    "        <pPort>MyPort2</pPort>                                \n"
    "        <Sign>Signed</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                    \n"
    "    </IntReg>                                                \n"
    "                                                            \n"
    "    <Port Name=\"MyPort1\" >                                \n"
    "        <ChunkID>E22B3A4791154560BD468608AA90AE30</ChunkID>    \n"
    "        <CacheChunkData>Yes</CacheChunkData>    \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    "    <Port Name=\"MyPort2\" >                                \n"
    "        <ChunkID>8135B3ECE1A240548B5A51322FF5DC89</ChunkID>    \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    + CDFFooter;

    // create and connect a camera object
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Create and connect the chunk adapter
    CChunkAdapterDcam ChunkAdapter( Camera._Ptr );

#   pragma pack(push, 1)

    // dummy ChunkData
    typedef struct Buffer_t
    {
         uint32_t Value1 ;
         uint32_t Value2 ;
         DCAM_CHUNK_TRAILER Trailer1 ;
         uint32_t Value3 ;
         uint8_t Data1[504] ;
         uint32_t Value4 ;
         DCAM_CHUNK_TRAILER Trailer2 ;
    } Buffer_t;

    typedef GUID ChunkID_t;

#   pragma pack(pop)

    Buffer_t Buffer;

    memset(&Buffer, 0, sizeof(Buffer_t));
    Buffer.Value1 = 12;
    Buffer.Value2 = 34;

    // {E22B3A47-9115-4560-BD46-8608AA90AE30}
    SET_GUID(Buffer.Trailer1.ChunkID,
        0x473a2be2, 0x1591, 0x6045, 0xbd, 0x46, 0x86, 0x8, 0xaa, 0x90, 0xae, 0x30);
    Buffer.Trailer1.ChunkLength = 2 * sizeof(uint32_t) + sizeof(DCAM_CHUNK_TRAILER);
    Buffer.Trailer1.InverseChunkLength = ~Buffer.Trailer1.ChunkLength;

    Buffer.Value3 = 56;
    Buffer.Value4 = 78;
    // {8135B3EC-E1A2-4054-8B5A-51322FF5DC89}
    SET_GUID(Buffer.Trailer2.ChunkID ,
        0xecb33581, 0xa2e1, 0x5440, 0x8b, 0x5a, 0x51, 0x32, 0x2f, 0xf5, 0xdc, 0x89);
    Buffer.Trailer2.ChunkLength = sizeof(uint32_t) + 508 + sizeof(DCAM_CHUNK_TRAILER);
    Buffer.Trailer2.InverseChunkLength = ~Buffer.Trailer2.ChunkLength;

    CIntegerPtr ptrValue1 = Camera._GetNode("Value1");
    CPPUNIT_ASSERT( ptrValue1.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue1) );

    CIntegerPtr ptrValue2 = Camera._GetNode("Value2");
    CPPUNIT_ASSERT( ptrValue2.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue2) );

    CIntegerPtr ptrValue3 = Camera._GetNode("Value3");
    CPPUNIT_ASSERT( ptrValue3.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );

    CIntegerPtr ptrValue4 = Camera._GetNode("Value4");
    CPPUNIT_ASSERT( ptrValue4.IsValid() );
    CPPUNIT_ASSERT( !IsReadable(ptrValue4) );

    CIntegerPtr ptrChunkBaseAddress1 = Camera._GetNode("ChunkBaseAddress1");
    CPPUNIT_ASSERT( ptrChunkBaseAddress1.IsValid() );

    CIntegerPtr ptrChunkBaseAddress2 = Camera._GetNode("ChunkBaseAddress2");
    CPPUNIT_ASSERT( ptrChunkBaseAddress2.IsValid() );

    /*--------------------------------------*/

    // Attach the buffer and check the callback and values
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)78, ptrValue4->GetValue() );

    // check again after update buffer
    ChunkAdapter.UpdateBuffer((uint8_t*)&Buffer);
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValue2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValue3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)78, ptrValue4->GetValue() );

    // check direct buffer access
    CRegisterPtr ptrRegValue4 = (CRegisterPtr) ptrValue4;
    CPPUNIT_ASSERT( ptrRegValue4.IsValid() );
    int64_t Value4Address = ptrChunkBaseAddress2->GetValue() +  ptrRegValue4->GetAddress();
    CPPUNIT_ASSERT_EQUAL( Buffer.Value4, *(uint32_t*)Value4Address );

    /*--------------------------------------*/

    // Detach the buffer
    ChunkAdapter.DetachBuffer();

    // Chunkport 1 is cached...
    CPPUNIT_ASSERT( IsReadable(ptrValue1) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValue1->GetValue() );
    ptrValue1->SetValue(42);
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrValue1->GetValue() );
    ptrValue1->SetValue(12);
    // if the chunk comes from cache the base address is NULL
    CPPUNIT_ASSERT_EQUAL( (int64_t)0, ptrChunkBaseAddress1->GetValue() );

    // ...but Chunk port 2 is not
    CPPUNIT_ASSERT( !IsReadable(ptrValue3) );

    // now clear the caches resulting in port 1 becoming unaccessible too
    ChunkAdapter.ClearCaches();
    CPPUNIT_ASSERT( !IsReadable(ptrValue1) );

}


void ChunkTestSuite::TestInvalidPort()
{

    if(GenApiSchemaVersion == v1_0)
        return;


    // create a camera description file
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                \n"
  "    <Port Name=\"MyPort\">        \n"
    "        <ChunkID>12</ChunkID>                        \n"
    "        <CacheChunkData>Yes</CacheChunkData>    \n"
    "    </Port>                                                    \n"
  "    <Port Name=\"MyPortNoID\">        \n"
    "    </Port>                                                    \n"
    "                                                                \n"
    + CDFFooter;

    // create and connect a camera object
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CPortPtr ptrPort = Camera._GetNode("MyPort");
    CPortPtr ptrPortNoID = Camera._GetNode("MyPortNoID");

    CChunkPort *pChunkPort = new CChunkPort (NULL);

    // test actions on invalid port
    delete pChunkPort;
    pChunkPort = new CChunkPort (NULL);
    CPPUNIT_ASSERT_NO_THROW (pChunkPort->InvalidateNode());
    uint8_t Buffer[4];
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Write (Buffer, 0, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Write (Buffer, -4, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Write (Buffer, 4, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Read (Buffer, 0, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Read (Buffer, -4, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Read (Buffer, 4, 4), GenICam::RuntimeException);

    // attach the port
    CPPUNIT_ASSERT_EQUAL (false, pChunkPort->AttachPort (ptrPortNoID));
    CPPUNIT_ASSERT_EQUAL (true, pChunkPort->AttachPort (ptrPort));
    CPPUNIT_ASSERT_EQUAL (true, pChunkPort->AttachPort (ptrPort)); // re-attach the same one

    // attach the chunk
    uint8_t Chunk[4];
    pChunkPort->AttachChunk (Chunk, 0, 1, true); // happy path
    pChunkPort->AttachChunk (Chunk, 0, 4, true);
    pChunkPort->AttachChunk (Chunk, 0, 4, true); // happy path


    // now the port is valid, exercise some special conditions
    CPPUNIT_ASSERT_NO_THROW (pChunkPort->Write (Buffer, 0, 4));
    CPPUNIT_ASSERT_NO_THROW (pChunkPort->Write (Buffer, -4, 4)); // access with address counted from right
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Write (Buffer, -8, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Write (Buffer, 4, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_NO_THROW (pChunkPort->Read (Buffer, 0, 4));
    CPPUNIT_ASSERT_NO_THROW (pChunkPort->Read (Buffer, -4, 4)); // access with address counted from right
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Read (Buffer, -8, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Read (Buffer, 4, 4), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Read (Buffer, CHUNK_BASE_ADDRESS_REGISTER, CHUNK_BASE_ADDRESS_REGISTER_LEN-1), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Read (Buffer, CHUNK_LENGTH_REGISTER, CHUNK_LENGTH_REGISTER_LEN-1), GenICam::RuntimeException);
    // out of bounds access
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Write (Buffer, 0, -4), RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (pChunkPort->Write (Buffer, 1, INT64_MAX), RuntimeException);

    // test principal interface
    CPPUNIT_ASSERT_EQUAL( intfIPort, pChunkPort->GetPrincipalInterfaceType() );

}

void ChunkTestSuite::TestChunkIDs()
{
    // create a camera description file
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                            \n"
    "    <IntReg Name=\"ValueLongID\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPortLongID</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                \n"
    "    </IntReg>                                                   \n"
    "                                                            \n"
    "    <IntReg Name=\"ValueShortID\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPortShortID</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                \n"
    "    </IntReg>                                                   \n"
    "                                                            \n"
    "    <IntReg Name=\"ValueZeroID\" >                                \n"
    "        <Address>0x00</Address>                                \n"
    "        <Length>4</Length>                                    \n"
    "        <AccessMode>RW</AccessMode>                        \n"
    "        <pPort>MyPortZeroID</pPort>                                \n"
    "        <Sign>Unsigned</Sign>                                \n"
    "        <Endianess>LittleEndian</Endianess>                \n"
    "    </IntReg>                                                   \n"
    "                                                            \n"
    "    <Port Name=\"MyPortLongID\" >                                \n"
    "        <ChunkID>12345678</ChunkID>                            \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    "    <Port Name=\"MyPortShortID\" >                                \n"
    "        <ChunkID>ab</ChunkID>                            \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    "    <Port Name=\"MyPortZeroID\" >                                \n"
    "        <ChunkID>00</ChunkID>                            \n"
    "    </Port>                                                    \n"
    "    <Port Name=\"MyPorxtLongID\" >                                \n"
    "        <ChunkID>1234567812345678</ChunkID>                            \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    "    <Port Name=\"MyPorytLongID\" >                                \n"
    "        <ChunkID>123456781234567800</ChunkID>                            \n"
    "    </Port>                                                    \n"
    "                                                            \n"
    + CDFFooter;

    // create and connect a camera object
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Create and connect the chunk adapter
    CChunkAdapterGEV ChunkAdapter( Camera._Ptr );

#   pragma pack(push, 1)

    // dummy ChunkData
    typedef struct Buffer_t
    {
         uint32_t ValueLongID ;
         GVCP_CHUNK_TRAILER TrailerLongID ;
         uint32_t ValueShortID ;
         GVCP_CHUNK_TRAILER TrailerShortID ;
         uint32_t ValueZeroID ;
         GVCP_CHUNK_TRAILER TrailerZeroID ;
    } Buffer_t;

    typedef struct ChunkID_t
    {
         uint8_t Byte1 ;
         uint8_t Byte2 ;
         uint8_t Byte3 ;
         uint8_t Byte4 ;
    } ChunkID_t;

#   pragma pack(pop)

    Buffer_t Buffer;

    memset(&Buffer, 0, sizeof(Buffer_t));
    Buffer.ValueLongID = 12;
    ChunkID_t ChunkLongID = { 0x12, 0x34, 0x56, 0x78 };
    Buffer.TrailerLongID.ChunkID = *(uint32_t*)&ChunkLongID;
    Buffer.TrailerLongID.ChunkLength = htonl(sizeof(uint32_t));
    Buffer.ValueShortID = 34;
    ChunkID_t ChunkShortID = { 0x00, 0x00, 0x00, 0xab };
    Buffer.TrailerShortID.ChunkID = *(uint32_t*)&ChunkShortID;
    Buffer.TrailerShortID.ChunkLength = htonl(sizeof(uint32_t));
    Buffer.ValueZeroID = 56;
    ChunkID_t ChunkZeroID = { 0x00, 0x00, 0x00, 0x00 };
    Buffer.TrailerZeroID.ChunkID = *(uint32_t*)&ChunkZeroID;
    Buffer.TrailerZeroID.ChunkLength = htonl(sizeof(uint32_t));

    //Check the layout of the buffer and attach it
    CPPUNIT_ASSERT( ChunkAdapter.CheckBufferLayout( (uint8_t*)&Buffer, sizeof(Buffer_t)) );
    ChunkAdapter.AttachBuffer( (uint8_t*)&Buffer, sizeof(Buffer_t) );

    // Verify the values
    CIntegerPtr ptrValueLongID = Camera._GetNode("ValueLongID");
    CPPUNIT_ASSERT( ptrValueLongID.IsValid() );
    CPPUNIT_ASSERT( IsReadable(ptrValueLongID) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)12, ptrValueLongID->GetValue() );
    CIntegerPtr ptrValueShortID = Camera._GetNode("ValueShortID");
    CPPUNIT_ASSERT( ptrValueShortID.IsValid() );
    CPPUNIT_ASSERT( IsReadable(ptrValueShortID) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)34, ptrValueShortID->GetValue() );
    CIntegerPtr ptrValueZeroID = Camera._GetNode("ValueZeroID");
    CPPUNIT_ASSERT( ptrValueZeroID.IsValid() );
    CPPUNIT_ASSERT( IsReadable(ptrValueZeroID) );
    CPPUNIT_ASSERT_EQUAL( (int64_t)56, ptrValueZeroID->GetValue() );

    CChunkAdapterGeneric ChunkAdapterGen( Camera._Ptr );
    {// repeat the same tests with the generic adapter and SingleChunkData_t
        SingleChunkData_t ChkData[3];
        ChkData[0].ChunkID = 0x12345678;
        ChkData[0].ChunkOffset = 0;
        ChkData[0].ChunkLength = 4;
        ChkData[1].ChunkID = 0xab;
        ChkData[1].ChunkOffset = 12;
        ChkData[1].ChunkLength = 4;
        ChkData[2].ChunkID = 0x00;
        ChkData[2].ChunkOffset = 24;
        ChkData[2].ChunkLength = 4;
        Buffer.ValueLongID = 212;
        Buffer.ValueShortID = 434;
        Buffer.ValueZeroID = 656;
        ChunkAdapterGen.AttachBuffer( (uint8_t*)&Buffer, ChkData, ARRAYSIZE( ChkData ) );

        // Verify the values
        CIntegerPtr ptrGenLongID = Camera._GetNode("ValueLongID");
        CPPUNIT_ASSERT( ptrGenLongID.IsValid() );
        CPPUNIT_ASSERT( IsReadable(ptrGenLongID) );
        CPPUNIT_ASSERT_EQUAL( (int64_t)212, ptrGenLongID->GetValue() );
        CIntegerPtr ptrGenShortID = Camera._GetNode("ValueShortID");
        CPPUNIT_ASSERT( ptrGenShortID.IsValid() );
        CPPUNIT_ASSERT( IsReadable(ptrGenShortID) );
        CPPUNIT_ASSERT_EQUAL( (int64_t)434, ptrGenShortID->GetValue() );
        CIntegerPtr ptrGenZeroID = Camera._GetNode("ValueZeroID");
        CPPUNIT_ASSERT( ptrGenZeroID.IsValid() );
        CPPUNIT_ASSERT( IsReadable(ptrGenZeroID) );
        CPPUNIT_ASSERT_EQUAL( (int64_t)656, ptrGenZeroID->GetValue() );
    }

    {// repeat the same tests with the generic adapter and SingleChunkDataStr_t
        SingleChunkDataStr_t ChkDataStr[3];
        ChkDataStr[0].ChunkID = "12345678";
        ChkDataStr[0].ChunkOffset = 0;
        ChkDataStr[0].ChunkLength = 4;
        ChkDataStr[1].ChunkID = "ab";
        ChkDataStr[1].ChunkOffset = 12;
        ChkDataStr[1].ChunkLength = 4;
        ChkDataStr[2].ChunkID = "00";
        ChkDataStr[2].ChunkOffset = 24;
        ChkDataStr[2].ChunkLength = 4;
        Buffer.ValueLongID = 21;
        Buffer.ValueShortID = 43;
        Buffer.ValueZeroID = 65;
        ChunkAdapterGen.AttachBuffer( (uint8_t*)&Buffer, ChkDataStr, ARRAYSIZE( ChkDataStr ) );

        // Verify the values
        CIntegerPtr ptrGenLongID = Camera._GetNode("ValueLongID");
        CPPUNIT_ASSERT( ptrGenLongID.IsValid() );
        CPPUNIT_ASSERT( IsReadable(ptrGenLongID) );
        CPPUNIT_ASSERT_EQUAL( (int64_t)21, ptrGenLongID->GetValue() );
        CIntegerPtr ptrGenShortID = Camera._GetNode("ValueShortID");
        CPPUNIT_ASSERT( ptrGenShortID.IsValid() );
        CPPUNIT_ASSERT( IsReadable(ptrGenShortID) );
        CPPUNIT_ASSERT_EQUAL( (int64_t)43, ptrGenShortID->GetValue() );
        CIntegerPtr ptrGenZeroID = Camera._GetNode("ValueZeroID");
        CPPUNIT_ASSERT( ptrGenZeroID.IsValid() );
        CPPUNIT_ASSERT( IsReadable(ptrGenZeroID) );
        CPPUNIT_ASSERT_EQUAL( (int64_t)65, ptrGenZeroID->GetValue() );
    }
}
