//-----------------------------------------------------------------------------
//  (c) 2006 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Hartmut Nebelung
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of Command node
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "CommandTestSuite.h"
#include "TestPort.h"
#include "CallbackHelper.h"
#include "GenApi/GenApi.h"
#include "GenApi/StructPort.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

#undef min

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CommandTestSuite, "GenApiTest");
//----------------------------------------------------------------------------
// void CommandTestSuite::TestCommand01()
//----------------------------------------------------------------------------
/*!
* Test the basic functionality of a command node.
*/
void CommandTestSuite::TestCommand01()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                 \n"
    "<Category Name=\"Root\">                         \n"
    "   <pFeature>command</pFeature>                  \n"
    "</Category>                                      \n"
    "                                                 \n"
    "<Command Name=\"command\">                       \n"
    "   <pValue>myIntReg</pValue>                     \n"
    "   <CommandValue>0x4321</CommandValue>           \n"
    "</Command>                                       \n"
    "                                                 \n"
    "<IntReg Name=\"myIntReg\">                       \n"
    "    <Address>0x00ff</Address>                    \n"
    "    <Length>4</Length>                           \n"
    "    <AccessMode>RW</AccessMode>                  \n"
    "    <pPort>MyPort</pPort>                        \n"
    "    <Sign>Unsigned</Sign>                        \n"
    "    <Endianess>BigEndian</Endianess>             \n"
    "</IntReg>                                        \n"
    "                                                 \n"
    "<Port Name=\"MyPort\"/>                          \n"
    "                                                 \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x00ff, 42UL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

  //
  // Test the ICommand interface
  //

  // retrieve the node and convert it to ICommand
  CNodePtr ptrNode( Camera._GetNode( "command" ) );
  CCommandPtr ptrCmd( ptrNode );
  CPPUNIT_ASSERT( (bool) ptrNode );
  CPPUNIT_ASSERT( (bool) ptrCmd );
  CPPUNIT_ASSERT_EQUAL (gcstring("ICommand"), GetInterfaceName(ptrCmd));

  // execute function
  ptrCmd->Execute();
  // is executing?
  CPPUNIT_ASSERT_EQUAL( false, ptrCmd->IsDone() );
  // manipulate the command register in order to simulate a self-resetting toggle
  int32_t tmp;
  Port.Read( &tmp, 0x00ff, 4 );
  CPPUNIT_ASSERT_EQUAL( 0x21430000, tmp );
  tmp = 0;
  Port.Write( &tmp, 0x00ff, 4 );
  CPPUNIT_ASSERT_EQUAL( true, ptrCmd->IsDone() );

  // once again with verification
  ptrCmd->Execute();
  CPPUNIT_ASSERT_EQUAL( false, ptrCmd->IsDone(true) );
  Port.Write( &tmp, 0x00ff, 4 );
  CPPUNIT_ASSERT_EQUAL( true, ptrCmd->IsDone(true) );

  // happy path (what to test on execution without verification?)
  ptrCmd->Execute(false);

  // execute function
  ptrCmd->operator()();
  // is executing?
  CPPUNIT_ASSERT_EQUAL( false, ptrCmd->IsDone() );
  // manipulate the command register in order to simulate a self-resetting toggle
  Port.Read( &tmp, 0x00ff, 4 );
  CPPUNIT_ASSERT_EQUAL( 0x21430000, tmp );
  tmp = 0;
  Port.Write( &tmp, 0x00ff, 4 );
  CPPUNIT_ASSERT_EQUAL( true, ptrCmd->IsDone() );

  // TestAccessMode
   CPPUNIT_ASSERT_EQUAL( RW, ptrCmd->GetAccessMode() );

  //
  // Test the IValue interface
  //
  CValuePtr ptrValue( ptrCmd );
  gcstring strExec( "0" );
  // function is done => value is "0"
  CPPUNIT_ASSERT_EQUAL( strExec, ptrValue->ToString() );
  // execute using a string
  strExec = "1";
  ptrValue->FromString( strExec );
  // function is executing => value is "1"
  CPPUNIT_ASSERT_EQUAL( strExec, ptrValue->ToString() );
  CPPUNIT_ASSERT( NULL != ptrValue->GetNode() );

  // try to use an illegal string
  CPPUNIT_ASSERT_THROW_EX( ptrValue->FromString( "foo" ), GenICam::InvalidArgumentException );
  CPPUNIT_ASSERT_NO_THROW( ptrValue->FromString( "true" ) );

  {
    CCommandRef refCommand;
    // unitialized internal pointer => throws exception, except GetNode
    CPPUNIT_ASSERT_THROW_EX( refCommand.Execute(), AccessException );
    CPPUNIT_ASSERT_THROW_EX( refCommand.FromString( "true" ), AccessException );
    CPPUNIT_ASSERT_EQUAL( NI, refCommand.GetAccessMode() );
    CPPUNIT_ASSERT_THROW_EX( refCommand.GetNode(), AccessException );
    CPPUNIT_ASSERT_THROW_EX( refCommand.IsDone(), AccessException );
    CPPUNIT_ASSERT_THROW_EX( refCommand.ToString(), AccessException );
    // Pleora FG: try operator() - should throw
    CPPUNIT_ASSERT_THROW_EX( refCommand.operator()(), AccessException );
    // initialize pointer and try to excecute
    CPPUNIT_ASSERT_NO_THROW( refCommand.SetReference( ptrValue ) );
    CPPUNIT_ASSERT_NO_THROW( refCommand.Execute() );
    strExec = "1";
    CPPUNIT_ASSERT_NO_THROW( refCommand.FromString( strExec ) );
    CPPUNIT_ASSERT( NULL != refCommand.GetNode() );
    // try to use an illegal string
    CPPUNIT_ASSERT_THROW_EX( refCommand.FromString( "foo" ), GenICam::InvalidArgumentException );
    // try with correct on
    CPPUNIT_ASSERT_NO_THROW( refCommand.FromString( "true" ) );
    CPPUNIT_ASSERT_EQUAL( false, refCommand.IsDone() );
    CPPUNIT_ASSERT_EQUAL( strExec, refCommand.ToString() );
    // Pleora FG - execute function
    CPPUNIT_ASSERT_NO_THROW( refCommand.operator()() );
    // Pleora FG - is executing?
    CPPUNIT_ASSERT_EQUAL( false, refCommand.IsDone() );

  }


}



void CommandTestSuite::TestCommand02()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                \n"
    "<Command Name=\"command\">                      \n"
    "    <Value>0</Value>                            \n"
    "    <CommandValue>1</CommandValue>              \n"
    "</Command>                                      \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

      //
      // Test the ICommand interface
      //

      // retrieve the node and convert it to ICommand
      CNodePtr ptrNode( Camera._GetNode( "command" ) );
      CCommandPtr ptrCmd( ptrNode );
      CPPUNIT_ASSERT( (bool) ptrNode );
      CPPUNIT_ASSERT( (bool) ptrCmd );

      //
      // Test the IValue interface
      //
      // Execute function
      CValuePtr ptrValue( ptrCmd );
      gcstring strExec( "0" );
      // Function not executing => CommandValue is 0
      CPPUNIT_ASSERT_EQUAL( strExec, ptrValue->ToString() );

      // Execute function
      // 1st Method
      strExec = "0";
      CPPUNIT_ASSERT_THROW_EX( ptrValue->FromString( strExec ), GenICam::InvalidArgumentException );

      // 2nd Method
      strExec = "1";
      ptrValue->FromString( strExec );  // equivalent to "ptrCmd->Execute();"

      // Function not executing (a floating command is immediatelx ready )
      strExec = "0";
      CPPUNIT_ASSERT_EQUAL( strExec, ptrValue->ToString() );

      // is executing?
      CPPUNIT_ASSERT_EQUAL( true, ptrCmd->IsDone() );
}

void CommandTestSuite::TestCommand03()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                    \n"
    "<Command Name=\"Command\">                          \n"
    "    <pValue>Value</pValue>                          \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValue\">                     \n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"Value\">                            \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CCommandPtr ptrCommand( Camera._GetNode( "Command" ));
    CPPUNIT_ASSERT( (bool)ptrCommand );

    CIntegerPtr ptrValue( Camera._GetNode( "Value" ));
    CPPUNIT_ASSERT( (bool)ptrValue );

    CIntegerPtr ptrCommandValue( Camera._GetNode( "CommandValue" ));
    CPPUNIT_ASSERT( (bool)ptrCommandValue );

    CallbackObject CallbackTarget;
    Register(ptrCommand->GetNode(), CallbackTarget, &CallbackObject::Callback );

    // nothing happened so no callback expected
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0, CallbackTarget.Count() );

    // a first callback happens after execute
    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget.Count() );

    // no callback on IsDone because the value has not changed yet
    CPPUNIT_ASSERT_EQUAL( false, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrValue->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget.Count() );

    // now the value has changed so we expect a callback
    ptrValue->SetValue(0);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallbackTarget.Count() );

    // On IsDone the Command node notives that the vale has changed and fires another time
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)3, CallbackTarget.Count() );

}

void CommandTestSuite::TestCommand04()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                    \n"
    "<Command Name=\"Command\">                          \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>       \n"
    "    <pValue>Value</pValue>                          \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValue\">                     \n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"Value\">                            \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CCommandPtr ptrCommand( Camera._GetNode( "Command" ));
    CPPUNIT_ASSERT( (bool)ptrCommand );

    CIntegerPtr ptrValue( Camera._GetNode( "Value" ));
    CPPUNIT_ASSERT( (bool)ptrValue );

    CIntegerPtr ptrCommandValue( Camera._GetNode( "CommandValue" ));
    CPPUNIT_ASSERT( (bool)ptrCommandValue );

    CallbackObject CallbackTarget;
    Register(ptrCommand->GetNode(), CallbackTarget, &CallbackObject::Callback );

    // due to the execute a callback is fired
    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget.Count() );

    // since the command node is WO IsDone does not trigger a callback
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget.Count() );

}

void CommandTestSuite::TestCommand05()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                    \n"
    "<Command Name=\"Command\">                          \n"
    "    <pValue>Value</pValue>                          \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "    <PollingTime>1000</PollingTime>                 \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValue\">                     \n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"Value\">                            \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Command Name=\"CommandWO\">                          \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                          \n"
    "    <pValue>Value</pValue>                          \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "    <PollingTime>1000</PollingTime>                 \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Command Name=\"Command_ValueWO\">                          \n"
    "    <pValue>ValueWO</pValue>                          \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "    <PollingTime>1000</PollingTime>                 \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"ValueWO\">                            \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                          \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CCommandPtr ptrCommand( Camera._GetNode( "Command" ));
    CPPUNIT_ASSERT( (bool)ptrCommand );

    CIntegerPtr ptrValue( Camera._GetNode( "Value" ));
    CPPUNIT_ASSERT( (bool)ptrValue );

    CIntegerPtr ptrCommandValue( Camera._GetNode( "CommandValue" ));
    CPPUNIT_ASSERT( (bool)ptrCommandValue );

    CallbackObject CallbackTarget;
    Register(ptrCommand->GetNode(), CallbackTarget, &CallbackObject::Callback );

    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget.Count() );
    CPPUNIT_ASSERT_EQUAL( false, ptrCommand->IsDone() );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget.Count() );
    CPPUNIT_ASSERT_EQUAL( false, ptrCommand->IsDone() );

    Camera._Poll(500);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallbackTarget.Count() );
    CPPUNIT_ASSERT_EQUAL( false, ptrCommand->IsDone() );

    // now the self-clearing flag is clearing
    ptrValue->SetValue(0);
    // this is due to the fact that Command is dependent on value
    CPPUNIT_ASSERT_EQUAL( (uint32_t)3, CallbackTarget.Count() );

    Camera._Poll(1000);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)4, CallbackTarget.Count() );
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone() );

    // similar tests with WO node (done immediatelly, no polling)
    CCommandPtr ptrCommandWO( Camera._GetNode( "CommandWO" ));
    CPPUNIT_ASSERT( (bool)ptrCommandWO );

    CallbackObject CallbackTargetWO;
    Register(ptrCommandWO->GetNode(), CallbackTargetWO, &CallbackObject::Callback );

    ptrCommandWO->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTargetWO.Count() );
    CPPUNIT_ASSERT_EQUAL( true, ptrCommandWO->IsDone() );

    Camera._Poll(1000);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTargetWO.Count() );
    CPPUNIT_ASSERT_EQUAL( true, ptrCommandWO->IsDone() );

    // and again with command having a WO value
    CCommandPtr ptrCommand_ValueWO( Camera._GetNode( "Command_ValueWO" ));
    CPPUNIT_ASSERT( (bool)ptrCommand_ValueWO );

    CallbackObject CallbackTarget_ValueWO;
    Register(ptrCommand_ValueWO->GetNode(), CallbackTarget_ValueWO, &CallbackObject::Callback );

    ptrCommand_ValueWO->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget_ValueWO.Count() );
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand_ValueWO->IsDone() );

    Camera._Poll(1000);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget_ValueWO.Count() );
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand_ValueWO->IsDone() );
}


void CommandTestSuite::TestCommand06()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                    \n"
    "<Command Name=\"Command\">                          \n"
    "    <pIsLocked>LockIt</pIsLocked>                   \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>       \n"
    "    <pValue>Value</pValue>                          \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValue\">                     \n"
    "    <pIsImplemented>CommandValueIsImpl</pIsImplemented>\n"
    "    <pIsAvailable>CommandValueIsAvail</pIsAvailable>\n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"Value\">                            \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"LockIt\">                           \n"
    "    <Value>1</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValueIsAvail\">              \n"
    "    <Value>1</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValueIsImpl\">              \n"
    "    <Value>1</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CCommandPtr ptrCommand( Camera._GetNode( "Command" ));
    CPPUNIT_ASSERT( ptrCommand.IsValid() );

    CPPUNIT_ASSERT_EQUAL( NA, ptrCommand->GetAccessMode() );

    CPPUNIT_ASSERT_THROW_EX( ptrCommand->FromString("42"), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( ptrCommand->Execute(), GenICam::AccessException );

    // disable the command value
    CIntegerPtr ptrCmdValueAvailability( Camera._GetNode( "CommandValueIsAvail" ) );
    CPPUNIT_ASSERT( ptrCmdValueAvailability.IsValid() );
    CIntegerPtr ptrLockCommand( Camera._GetNode( "LockIt" ) );
    ptrLockCommand->SetValue( 0 );
    ptrCmdValueAvailability->SetValue( 0 );
    CPPUNIT_ASSERT_EQUAL( NA, ptrCommand->GetAccessMode() );
    CPPUNIT_ASSERT_THROW_EX( ptrCommand->FromString("42"), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( ptrCommand->Execute(), GenICam::AccessException );

    CIntegerPtr ptrCmdValueImpl( Camera._GetNode( "CommandValueIsImpl" ) );
    CPPUNIT_ASSERT( ptrCmdValueImpl.IsValid() );
    *ptrCmdValueImpl = 0;
    CPPUNIT_ASSERT_EQUAL( NI, ptrCommand->GetAccessMode() );
}

void CommandTestSuite::TestCommand07()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +
        "                                                    \n"
        "<Command Name=\"CommandRO\">                          \n"
        "    <ImposedAccessMode>RO</ImposedAccessMode>                          \n"
        "    <pValue>Value</pValue>                          \n"
        "    <CommandValue>1</CommandValue>     \n"
        "</Command>                                          \n"
        "                                                    \n"
        "<Command Name=\"Command_ValueWO\">                          \n"
        "    <pValue>Value</pValue>                          \n"
        "    <pCommandValue>CommandValueWO</pCommandValue>     \n"
        "</Command>                                          \n"
        "                                                    \n"
        "<Integer Name=\"CommandValueWO\">                     \n"
        "    <ImposedAccessMode>WO</ImposedAccessMode>                               \n"
        "    <Value>42</Value>                               \n"
        "</Integer>                                          \n"
        "                                                    \n"
        "<Integer Name=\"Value\">                            \n"
        "    <Value>0</Value>                                \n"
        "</Integer>                                          \n"
        "                                                    \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CCommandPtr ptrCommandRO( Camera._GetNode( "CommandRO" ));
    CPPUNIT_ASSERT( (bool)ptrCommandRO );
    CCommandPtr ptrCommand_ValueWO( Camera._GetNode( "Command_ValueWO" ));
    CPPUNIT_ASSERT( (bool)ptrCommand_ValueWO );

    CPPUNIT_ASSERT_THROW_EX (ptrCommandRO->Execute(), GenICam::AccessException);
    CPPUNIT_ASSERT_THROW_EX (ptrCommand_ValueWO->Execute(), GenICam::AccessException);
}

// Local CallbackObject for testing Mantis Bug #250
    class CallbackObjectMantis250
    {
        public:
            CallbackObjectMantis250() :
            m_Count(0)
            {}

            void Reset()
            {
                m_Count = 0;
            }

            uint32_t Count() const
            {
                return m_Count;
            }


            void Callback( GenApi::INode* /*pNode*/)
            {
                ++m_Count;
            }
         private:
            uint32_t m_Count;
    };

void CommandTestSuite::TestCommandMantis250()
{


    // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +

    "<Integer Name=\"BinningVertical\">                  \n"
    "  <pValue>VerticalBinningReg</pValue>              \n"
    "  <Representation>Linear</Representation>           \n"
    "</Integer>                                          \n"

    "<IntReg Name=\"VerticalBinningReg\">               \n"
    "  <Address>0x4</Address>                            \n"
    "  <Length>4</Length>                                \n"
    "  <AccessMode>RW</AccessMode>                       \n"
    "  <pPort>MyPort</pPort>                             \n"
    "  <pInvalidator>UserSetLoadReg</pInvalidator>      \n"
    "  <Sign>Unsigned</Sign>                             \n"
    "  <Endianess>BigEndian</Endianess>                  \n"
    "</IntReg>                                           \n"
    "                                                    \n"
    "<Command Name=\"Command\">                          \n"
    "    <pValue>UserSetLoadReg</pValue>                \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValue\">                     \n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<IntReg Name=\"UserSetLoadReg\">                   \n"
    "  <Address>0x8</Address>                            \n"
    "  <Length>4</Length>                                \n"
    "  <AccessMode>RW</AccessMode>                       \n"
    "  <pPort>MyPort</pPort>                             \n"
    "  <Cachable>NoCache</Cachable>                      \n"
    "  <Sign>Unsigned</Sign>                             \n"
    "  <Endianess>BigEndian</Endianess>                  \n"
    "</IntReg>                                           \n"
    "                                                    \n"
    "<Port Name=\"MyPort\"/>                             \n"
    "                                                    \n"
    + CDFFooter;
    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescription);

    CTestPort Port;
    Port.CreateEntry(0x0004, 2UL, RW);  // VerticalBinningReg
    Port.CreateEntry(0x0008, 0UL, RW);  // UserSetLoadReg

    // connect the node map to the port
    Camera._Connect(&Port, "MyPort");

    CCommandPtr ptrCommand( Camera._GetNode( "Command" ));
    CPPUNIT_ASSERT( ptrCommand.IsValid() );

    CIntegerPtr ptrBinning( Camera._GetNode( "BinningVertical" ));
    CPPUNIT_ASSERT( ptrBinning.IsValid() );

    CallbackObjectMantis250 CallbackBinning;
    Register(ptrBinning->GetNode(), CallbackBinning, &CallbackObjectMantis250::Callback );

    CallbackObjectMantis250 CallbackCommand;
    Register(ptrCommand->GetNode(), CallbackCommand, &CallbackObjectMantis250::Callback );

    // this writes 42 to the UserSetLoadReg
    // because VerticalBinningReg has a pInvalidates pointer to UserSetLoadReg it is invalidated
    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackCommand.Count() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackBinning.Count() );

    // since UserSetLoadReg has not change no callback fires on IsDone
    CPPUNIT_ASSERT_EQUAL( false, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackBinning.Count() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackCommand.Count() );

    // now we reset UserSetLoadReg
    unsigned long value = 0;
    EAccessMode AccessMode(NI);
    Port.UpdateEntry(0x0008, 4, &value, AccessMode);

    // on the next IsDone the callback fires
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallbackCommand.Count() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallbackBinning.Count() );

}

    // Local CallbackObject for testing Mantis Bug #257
    class CallbackObjectMantis257
    {
        public:
            CallbackObjectMantis257() :
            m_Count(0),
            m_NodeError (false)
            {}

            void Reset()
            {
                m_Count = 0;
            }

            uint32_t Count() const
            {
                return m_Count;
            }

            bool NodeError() const
            {
                return m_NodeError;
            }

            void Callback( GenApi::INode* pNode)
            {
                ++m_Count;

                if (m_Count >3)
                    return;

                GenApi::ICommand* pCommand = dynamic_cast<GenApi::ICommand*>(pNode);

                if (pCommand)
                    pCommand->IsDone(); // Call ICommand->IsDone() to check of fireing recursive callbacks
                else
                    m_NodeError = true;

            }
         private:
            uint32_t m_Count;
            bool     m_NodeError;
    };

void CommandTestSuite::TestCommandMantis257()
{


    // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                    \n"
    "<Command Name=\"Command\">                            \n"
    "    <pValue>Value</pValue>                            \n"
    "    <pCommandValue>CommandValue</pCommandValue>        \n"
    "</Command>                                            \n"
    "                                                    \n"
    "<Integer Name=\"CommandValue\">                    \n"
    "    <Value>42</Value>                                \n"
    "</Integer>                                            \n"
    "                                                    \n"
    "<Integer Name=\"Value\">                            \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                            \n"
    "                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CCommandPtr ptrCommand( Camera._GetNode( "Command" ));
    CPPUNIT_ASSERT( ptrCommand.IsValid() );

    CIntegerPtr ptrValue( Camera._GetNode( "Value" ));
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    CIntegerPtr ptrCommandValue( Camera._GetNode( "CommandValue" ));
    CPPUNIT_ASSERT( ptrCommandValue.IsValid() );

    CallbackObjectMantis257 CallbackTarget;
    Register(ptrCommand->GetNode(), CallbackTarget, &CallbackObjectMantis257::Callback );

    // due to the execute a callback is fired
    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackTarget.Count() );
    CPPUNIT_ASSERT_EQUAL( false, CallbackTarget.NodeError());

    // due to changing the value another callback fires
    // BEWARE: inside the callback function (see CallbackObjectMantis257 class)
    // there is a call to IsDone. Since the cale has changed to
    // zero the command is now done and another callback fires
    ptrValue->SetValue( 0 );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)3, CallbackTarget.Count() );

    // since we have already called IsDone once no additional callback is fired.
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)3, CallbackTarget.Count() );


}

// modification of TestCommandMantis250, where the pInvalidator points directly to the command node
// the depending features should get updated twice per command as well
void CommandTestSuite::TestCommandMantis250_sch11()
{
    // <pInvalidator> pointing to a non-register node
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "<Integer Name=\"BinningVertical\">                  \n"
    "  <pValue>pVerticalBinningReg</pValue>              \n"
    "  <Representation>Linear</Representation>           \n"
    "</Integer>                                          \n"
    "<IntReg Name=\"pVerticalBinningReg\">               \n"
    "  <Address>0x4</Address>                            \n"
    "  <Length>4</Length>                                \n"
    "  <AccessMode>RW</AccessMode>                       \n"
    "  <pPort>MyPort</pPort>                             \n"
    "  <pInvalidator>Command</pInvalidator>      \n"
    "  <Sign>Unsigned</Sign>                             \n"
    "  <Endianess>BigEndian</Endianess>                  \n"
    "</IntReg>                                           \n"
    "                                                    \n"
    "<Command Name=\"Command\">                          \n"
    "    <pValue>pUserSetLoadReg</pValue>                \n"
    "    <pCommandValue>CommandValue</pCommandValue>     \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValue\">                     \n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<IntReg Name=\"pUserSetLoadReg\">                   \n"
    "  <Address>0x8</Address>                            \n"
    "  <Length>4</Length>                                \n"
    "  <AccessMode>RW</AccessMode>                       \n"
    "  <pPort>MyPort</pPort>                             \n"
    "  <Cachable>NoCache</Cachable>                      \n"
    "  <Sign>Unsigned</Sign>                             \n"
    "  <Endianess>BigEndian</Endianess>                  \n"
    "</IntReg>                                           \n"
    "                                                    \n"
    "<Port Name=\"MyPort\"/>                             \n"
    "                                                    \n"
    + CDFFooter;
    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescription);

    CTestPort Port;
    Port.CreateEntry(0x0004, 2UL, RW);
    Port.CreateEntry(0x0008, 0UL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

    CCommandPtr ptrCommand( Camera._GetNode( "Command" ));
    CPPUNIT_ASSERT( ptrCommand.IsValid() );

    CIntegerPtr ptrBinning( Camera._GetNode( "BinningVertical" ));
    CPPUNIT_ASSERT( ptrBinning.IsValid() );

    CallbackObjectMantis250 CallbackBinning;
    Register(ptrBinning->GetNode(), CallbackBinning, &CallbackObjectMantis250::Callback );

    CallbackObjectMantis250 CallbackCommand;
    Register(ptrCommand->GetNode(), CallbackCommand, &CallbackObjectMantis250::Callback );

    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackBinning.Count() );

    CPPUNIT_ASSERT_EQUAL( false, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( false, ptrCommand->IsDone(false) );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackBinning.Count() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackCommand.Count() );

    unsigned long value = 0;
    EAccessMode AccessMode(NI);
    Port.UpdateEntry(0x0008, 4, &value, AccessMode);

    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone(false) );
    CPPUNIT_ASSERT_EQUAL( true, ptrCommand->IsDone() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallbackCommand.Count() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)2, CallbackBinning.Count() );

}

void CommandTestSuite::TestCornerCases()
{
  // create and initilize node map
    gcstring CameraDescription =
    CDFHeader +
    "                                                    \n"
    "<Command Name=\"CommandValWO\">                          \n"
    "    <pValue>Value</pValue>                          \n"
    "    <pCommandValue>CommandValueWO</pCommandValue>     \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValueWO\">                     \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                               \n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"Value\">                            \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Command Name=\"CommandValNI\">                          \n"
    "    <pValue>Value</pValue>                          \n"
    "    <pCommandValue>CommandValueNI</pCommandValue>     \n"
    "</Command>                                          \n"
    "                                                    \n"
    "<Integer Name=\"CommandValueNI\">                     \n"
    "    <pIsImplemented>Zero</pIsImplemented>                               \n"
    "    <Value>42</Value>                               \n"
    "</Integer>                                          \n"
    "                                                    \n"
    "<Integer Name=\"Zero\">                            \n"
    "    <Value>0</Value>                                \n"
    "</Integer>                                          \n"
    "                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // test command which does not have readable command value
    CCommandPtr ptrCommandValWO( Camera._GetNode( "CommandValWO" ));
    CPPUNIT_ASSERT( (bool)ptrCommandValWO );
    CPPUNIT_ASSERT_THROW_EX (ptrCommandValWO->Execute(), AccessException);
    //??CPPUNIT_ASSERT_THROW_EX (ptrCommandValWO->IsDone(), AccessException);
    CPPUNIT_ASSERT_EQUAL (NA, ptrCommandValWO->GetAccessMode());

    // test command which has command value NI
    CCommandPtr ptrCommandValNI( Camera._GetNode( "CommandValNI" ));
    CPPUNIT_ASSERT( (bool)ptrCommandValNI );
    CPPUNIT_ASSERT_THROW_EX (ptrCommandValNI->Execute(), AccessException);
    CPPUNIT_ASSERT_EQUAL (NI, ptrCommandValNI->GetAccessMode());
    CPPUNIT_ASSERT_THROW_EX (ptrCommandValNI->IsDone(), AccessException);

}

#   pragma pack(push, 1)
typedef struct _CommandTestSuiteTestRegister_t
{                               // addr:len
    uint32_t ValueCommand ;     // 0:4
} TestRegister_t;               // 4
#   pragma pack(pop)

void CommandTestSuite::TestCommandCaching()
{

    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +
        "                                                 \n"
        "<Command Name=\"Command\">                       \n"
        "   <pValue>Value</pValue>                        \n"
        "   <pCommandValue>CommandValue</pCommandValue>   \n"
        "</Command>                                       \n"
        "                                                 \n"
        "<Integer Name=\"Value\">                         \n"
        "   <Value>0</Value>                              \n"
        "</Integer>                                       \n"
        "                                                 \n"
        "<IntReg Name=\"CommandValue\">                   \n"
        "    <Address>0x0000</Address>                    \n"
        "    <Length>4</Length>                           \n"
        "    <AccessMode>RW</AccessMode>                  \n"
        "    <pPort>Device</pPort>                        \n"
        "    <pInvalidator>Value</pInvalidator>           \n"
        "    <Sign>Unsigned</Sign>                        \n"
        "    <Endianess>LittleEndian</Endianess>          \n"
        "</IntReg>                                        \n"
        "                                                 \n"
        "<Port Name=\"Device\"/>                          \n"
        "                                                 \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CTestPortStruct<TestRegister_t> Port;
    Camera._Connect(&Port, "Device");

    CCommandPtr ptrCommand = Camera._GetNode("Command");
    CPPUNIT_ASSERT( ptrCommand.IsValid() );

    CIntegerPtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    Port.ValueCommand = 1;
    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (int64_t)1LL, ptrValue->GetValue() );

    Port.ValueCommand = 2;
    ptrCommand->Execute();
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, ptrValue->GetValue() );
}

#   pragma pack(push, 1)
    typedef struct _CommandTestSuiteTestRegister768_t
    {                                // addr:len
        uint32_t ValueCommand1 ;     // 0:4
        uint32_t ValueCommand2 ;     // 4:4
        uint32_t ValueCommand3 ;     // 8:4
    }  TestRegister768_t;            // b
#   pragma pack(pop)

void CommandTestSuite::TestTicket768()
{


     // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +
        "                                                 \n"
        "<Command Name=\"MyCommand1\">                    \n"
        "   <pValue>ValueReg1</pValue>                    \n"
        "   <CommandValue>1</CommandValue>                \n"
        "</Command>                                       \n"
        "                                                 \n"
        "<IntReg Name=\"ValueReg1\">                      \n"
        "    <Address>0x0000</Address>                    \n"
        "    <Length>4</Length>                           \n"
        "    <AccessMode>RW</AccessMode>                  \n"
        "    <pPort>Device</pPort>                        \n"
        "    <Cachable>NoCache</Cachable>                 \n"
        "    <Sign>Unsigned</Sign>                        \n"
        "    <Endianess>LittleEndian</Endianess>          \n"
        "</IntReg>                                        \n"
        "                                                 \n"
        "<Command Name=\"MyCommand2\">                    \n"
        "   <pValue>ValueReg2</pValue>                    \n"
        "   <CommandValue>1</CommandValue>                \n"
        "</Command>                                       \n"
        "                                                 \n"
        "<IntReg Name=\"ValueReg2\">                      \n"
        "    <Address>0x0004</Address>                    \n"
        "    <Length>4</Length>                           \n"
        "    <AccessMode>WO</AccessMode>                  \n"
        "    <pPort>Device</pPort>                        \n"
        "    <Cachable>NoCache</Cachable>                 \n"
        "    <Sign>Unsigned</Sign>                        \n"
        "    <Endianess>LittleEndian</Endianess>          \n"
        "</IntReg>                                        \n"
        "                                                 \n"
        "<Command Name=\"MyCommand3\">                    \n"
        "   <ImposedAccessMode>WO</ImposedAccessMode>     \n"
        "   <pValue>ValueReg3</pValue>                    \n"
        "   <CommandValue>1</CommandValue>                \n"
        "</Command>                                       \n"
        "                                                 \n"
        "<IntReg Name=\"ValueReg3\">                      \n"
        "    <Address>0x0008</Address>                    \n"
        "    <Length>4</Length>                           \n"
        "    <AccessMode>RW</AccessMode>                  \n"
        "    <pPort>Device</pPort>                        \n"
        "    <Cachable>NoCache</Cachable>                 \n"
        "    <Sign>Unsigned</Sign>                        \n"
        "    <Endianess>LittleEndian</Endianess>          \n"
        "</IntReg>                                        \n"
        "                                                 \n"
        "<Port Name=\"Device\"/>                          \n"
        "                                                 \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CTestPortStruct<TestRegister768_t> Port;
    Camera._Connect(&Port, "Device");

    {
        cout << "1) RW Value\n";

        NODE_POINTER( Command, MyCommand1 );
        Port.ValueCommand1 = 0;
        Port.ResetStatistics();
        cout << "Reset   : NumReads= " << Port.GetNumReads() << "\n";
        
        cout << "Execute!\n";
        ptrMyCommand1->Execute();
        cout << "NumReads= " << Port.GetNumReads() << "\n";
        CPPUNIT_ASSERT_EQUAL( (int64_t)0, Port.GetNumReads() );

        cout << "IsDone?\n";
        CPPUNIT_ASSERT_EQUAL( false, ptrMyCommand1->IsDone() );
        cout << "NumReads= " << Port.GetNumReads() << "\n";
        CPPUNIT_ASSERT_EQUAL( (int64_t)1, Port.GetNumReads() );
        
        cout << "Done!\n";
        Port.ValueCommand1 = 0;
        
        cout << "IsDone?\n";
        CPPUNIT_ASSERT_EQUAL( true, ptrMyCommand1->IsDone() );
        cout << "NumReads= " << Port.GetNumReads() << "\n";
        CPPUNIT_ASSERT_EQUAL( (int64_t)2, Port.GetNumReads() );
    }

    {
        cout << "2) WO Value\n";

        NODE_POINTER( Command, MyCommand2 );
        Port.ValueCommand2 = 0;
        Port.ResetStatistics();
        cout << "Reset   : NumReads= " << Port.GetNumReads() << "\n";
        
        cout << "Execute!\n";
        ptrMyCommand2->Execute();
        cout << "NumReads= " << Port.GetNumReads() << "\n";
        CPPUNIT_ASSERT_EQUAL( (int64_t)0, Port.GetNumReads() );

        cout << "IsDone?\n";
        CPPUNIT_ASSERT_EQUAL( true, ptrMyCommand2->IsDone() );
        cout << "NumReads= " << Port.GetNumReads() << "\n";
        CPPUNIT_ASSERT_EQUAL( (int64_t)0, Port.GetNumReads() );
        
    }

    {
        cout << "3) Imposed WO on the Command\n";

        NODE_POINTER( Command, MyCommand3 );
        Port.ValueCommand3 = 0;
        Port.ResetStatistics();
        cout << "Reset   : NumReads= " << Port.GetNumReads() << "\n";
        
        cout << "Execute!\n";
        ptrMyCommand3->Execute();
        cout << "NumReads= " << Port.GetNumReads() << "\n";
        CPPUNIT_ASSERT_EQUAL( (int64_t)0, Port.GetNumReads() );

        cout << "IsDone?\n";
        CPPUNIT_ASSERT_EQUAL( true, ptrMyCommand3->IsDone() );
        cout << "NumReads= " << Port.GetNumReads() << "\n";
        CPPUNIT_ASSERT_EQUAL( (int64_t)0, Port.GetNumReads() );
        
    }
}
