//-----------------------------------------------------------------------------
//  (c) 2006 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Hartmut Nebelung
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
Test the dependency implementation of all nodes
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "DependencyTest.h"
#include "TestPort.h"
#ifdef _MSC_VER
#   include <xutility>
#endif
#include <algorithm>
#include <list>
#include <fstream>
#include "GenApi/GenApi.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

#include "GenApi/impl/PointerImpl.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

#undef min

bool operator == (const NodeList_t &lhs, const NodeList_t &rhs)
{
    if (lhs.size() != rhs.size())
        return false;

    for (NodeList_t::const_iterator it=lhs.begin(); it!=lhs.end(); ++it)
    {
        NodeList_t::const_iterator init=rhs.begin();
        while (init != rhs.end() && *init != *it)
        {
            ++init;
        }

        NodeList_t::const_iterator fit = std::find( rhs.begin(), rhs.end(), *it );
        CPPUNIT_ASSERT( fit == init );
        if (init == rhs.end())
            return false;
    }

    return true;

}

static void FillNodeList( const CNodeMapRef& Map, const char *Names[], NodeList_t& Nodes )
{
    const char **ppCur = Names;
    while (*ppCur != NULL)
    {
        Nodes.push_back( Map._GetNode( *ppCur ) );

        ++ppCur;
    }

}

static bool TestDependency( const CNodeMapRef& NodeMap, const char Name[], const char *Dependents[] )
{
    NodeList_t expected;
    FillNodeList( NodeMap, Dependents, expected );

    NodeList_t actual;
    INodePrivate* pNode = dynamic_cast<INodePrivate*>( NodeMap._GetNode( Name ) );
    pNode->GetAllDependingNodes( actual );

    return expected == actual;
}

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( DependencyTestSuite, "GenApiTest");

//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency01()
//----------------------------------------------------------------------------
/*!
* Test the basic functionality of a command node.
*/
void DependencyTestSuite::TestDependency01()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +
        "                   \n"
        "<Category Name=\"Root\"> \n"
        " <pFeature>command</pFeature> \n"
        "</Category> \n"
        "                   \n"
        "<Command Name=\"command\"> \n"
        " <pValue>myIntReg</pValue> \n"
        " <CommandValue>0x4321</CommandValue> \n"
        "</Command> \n"
        "                                        \n"
        "<IntReg Name=\"myIntReg\">                \n"
        "    <Address>0x00ff</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        " <Cachable>NoCache</Cachable> \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"
        "                                        \n"
        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x00ff, 42UL, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "MyPort");

    NodeList_t lstIntReg;
    lstIntReg.push_back( Camera._GetNode( "command" ) );
    lstIntReg.push_back( Camera._GetNode( "Root" ) );
    NodeList_t lstActual;
    INodePrivate* pNode = dynamic_cast<INodePrivate*>( Camera._GetNode( "myIntReg" ) );
    pNode->GetAllDependingNodes( lstActual );
    CPPUNIT_ASSERT( lstIntReg ==lstActual );

}



//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency02()
//----------------------------------------------------------------------------
/*!
* Test the integer register
*/
void DependencyTestSuite::TestDependency02()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<IntReg Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "    <pAddress>F</pAddress>            \n"
        //        "    <pLength>G</pLength>                    \n"
        "   <Length>2</Length> \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "   <pInvalidator>H</pInvalidator> \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"G\">                \n"
        "    <Address>0x8</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }

    // node 'F'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }

    // node 'G'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "G", names ) );
    }

    // node 'H'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "G", "H", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}



//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency03()
//----------------------------------------------------------------------------
/*!
* Test the float register
*/
void DependencyTestSuite::TestDependency03()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<FloatReg Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "    <pAddress>F</pAddress>            \n"
        //        "    <pLength>G</pLength>                    \n"
        "   <Length>4</Length> \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "   <pInvalidator>H</pInvalidator> \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</FloatReg>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"G\">                \n"
        "    <Address>0x8</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }

    // node 'F'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }

    // node 'G'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "G", names ) );
    }

    // node 'H'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "G", "H", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}


//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency04()
//----------------------------------------------------------------------------
/*!
* Test the string register
*/
void DependencyTestSuite::TestDependency04()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<StringReg Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "    <pAddress>F</pAddress>            \n"
        "    <pLength>G</pLength>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "   <pInvalidator>H</pInvalidator> \n"
        "</StringReg>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"G\">                \n"
        "    <Address>0x8</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }

    // node 'F'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }

    // node 'G'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "G", names ) );
    }

    // node 'H'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "G", "H", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}




//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency05()
//----------------------------------------------------------------------------
/*!
* Test the masked integer register
*/
void DependencyTestSuite::TestDependency05()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<MaskedIntReg Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "    <pAddress>F</pAddress>            \n"
        //        "    <pLength>G</pLength>                    \n"
        "   <Length>2</Length> \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "   <pInvalidator>H</pInvalidator> \n"
        "   <Bit>12</Bit> \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "    <pSelected>I</pSelected>            \n"
        "    <pSelected>J</pSelected>            \n"
        "</MaskedIntReg>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"G\">                \n"
        "    <Address>0x8</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<MaskedIntReg Name=\"I\">                \n"
        "    <Address>0xC</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "   <Bit>1</Bit>                        \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</MaskedIntReg>                                \n"

        "<MaskedIntReg Name=\"J\">                \n"
        "    <Address>0xC</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "   <Bit>2</Bit>                        \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</MaskedIntReg>                                \n"


        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", "I", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", "I", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", "I", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", "I", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }

    // node 'F'
    {
        const char *names[] = { "A", "B", "I", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }

    // node 'G'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "G", names ) );
    }

    // node 'H'
    {
        const char *names[] = { "A", "B", "I", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }

    // node 'I'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "I", names ) );
    }

    // node 'J'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "J", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}



//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency06()
//----------------------------------------------------------------------------
/*!
* Test the smart feature adress
*/
void DependencyTestSuite::TestDependency06()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<IntReg Name=\"B\">                \n"
        "    <pAddress>C</pAddress>            \n"
        "   <Length>2</Length> \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<SmartFeature Name=\"C\">                \n"
        "   <FeatureID>F2D0D149-16F1-4c34-9E1D-85B04CBCB7D6</FeatureID> \n"
        "    <Address>0x0</Address>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "</SmartFeature>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }


    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}

//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency07()
//----------------------------------------------------------------------------
/*!
* Test the ConfRom entry
*/
void DependencyTestSuite::TestDependency07()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\" NameSpace=\"Custom\"> \n"
        " <pFeature>B</pFeature> \n"
        " <pFeature>C</pFeature> \n"
        "</Category> \n"

        "<IntKey Name=\"B\" NameSpace=\"Custom\"> \n"
        "  <p1212Parser>E</p1212Parser>\n"
        "  <Key>12</Key> \n"
        "</IntKey> \n"

        "<TextDesc Name=\"C\" NameSpace=\"Custom\"> \n"
        "  <p1212Parser>E</p1212Parser> \n"
        "  <Key>0x81</Key>\n"
        "</TextDesc> \n"

        "<IntSwissKnife Name=\"D\" NameSpace=\"Custom\" > \n"
        "    <Formula>0x400</Formula> \n"
        "</IntSwissKnife> \n"

        "<ConfRom Name=\"E\" NameSpace=\"Custom\"> \n"
        "  <Unit>0x12</Unit> \n"
        "  <pAddress>D</pAddress> \n"
        "  <Length>256</Length>\n"
        "  <pPort>MyPort</pPort>\n"
        "</ConfRom> \n"


        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", "C", "E", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }
    // node 'E'
    {
        const char *names[] = { "A", "B", "C",  NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "E", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}


//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency08()
//----------------------------------------------------------------------------
/*!
* Test the advanced feature lock
*/
void DependencyTestSuite::TestDependency08()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<IntReg Name=\"B\">                \n"
        "   <pIsAvailable>C</pIsAvailable>  \n"
        "    <pAddress>C</pAddress>            \n"
        "   <Length>2</Length>              \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<AdvFeatureLock Name=\"C\">                \n"
        "   <pIsImplemented>I</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "    <pAddress>F</pAddress>            \n"
        "   <Length>2</Length> \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "   <pInvalidator>H</pInvalidator> \n"
        "   <FeatureID>305533</FeatureID> \n"
        "</AdvFeatureLock>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"G\">                \n"
        "    <Address>0x8</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"I\">                \n"
        "    <Address>0xC</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", "C", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", "C", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }

    // node 'F'
    {
        const char *names[] = { "A", "B", "C", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }

    // node 'G'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "G", names ) );
    }

    // node 'H'
    {
        const char *names[] = { "A", "B", "C", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }

    // node 'I'
    {
        const char *names[] = { "A", "B", "C", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "I", names ) );
    }


    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "G", "H", "I", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}

//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency09()
//----------------------------------------------------------------------------
/*!
* Test the swiss knife
*/
void DependencyTestSuite::TestDependency09()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<IntSwissKnife Name=\"B\">                \n"
        "    <pVariable Name=\"A\">C</pVariable>        \n"
        "    <pVariable Name=\"B\">D</pVariable>        \n"
        "   <Formula>A*B</Formula>               \n"
        "</IntSwissKnife>                        \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "   <Length>2</Length> \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "   <Length>2</Length> \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}


//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency10()
//----------------------------------------------------------------------------
/*!
* Test the integer node
*/
void DependencyTestSuite::TestDependency10()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<Integer Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "   <pValue>F</pValue> \n"
        "   <pMin>G</pMin> \n"
        "   <pMax>H</pMax> \n"
        "   <pInc>I</pInc> \n"
        "    <pSelected>J</pSelected> \n"
        "</Integer>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"G\">                \n"
        "    <Address>0x8</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"I\">                \n"
        "    <Address>0xC</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"J\">                \n"
        "    <Address>0xE</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", "J", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", "J",NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", "J",NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", "J",NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }
    // node 'F'
    {
        const char *names[] = { "A", "B", "J",NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }
    // node 'G'
    {
        const char *names[] = { "A", "B", "J",NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "G", names ) );
    }
    // node 'H'
    {
        const char *names[] = { "A", "B", "J",NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }
    // node 'I'
    {
        const char *names[] = { "A", "B", "J",NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "I", names ) );
    }
    // node 'J'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "J", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "G", "H", "I", "J" , NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}




//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency11()
//----------------------------------------------------------------------------
/*!
* Test the float node
*/
void DependencyTestSuite::TestDependency11()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<Float Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "   <pValue>F</pValue> \n"
        "   <pMin>G</pMin> \n"
        "   <pMax>H</pMax> \n"
        "</Float>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<FloatReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</FloatReg>                                \n"

        "<FloatReg Name=\"G\">                \n"
        "    <Address>0x8</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</FloatReg>                                \n"

        "<FloatReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</FloatReg>                                \n"

        "<FloatReg Name=\"I\">                \n"
        "    <Address>0xC</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</FloatReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
//        const char *names[] = { "A", "J", NULL };
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }
    // node 'F'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }
    // node 'G'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "G", names ) );
    }
    // node 'H'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }
    // node 'I'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "I", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "G", "H", "I", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}






//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency12()
//----------------------------------------------------------------------------
/*!
* Test the boolean node
*/
void DependencyTestSuite::TestDependency12()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<Boolean Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "   <pValue>F</pValue> \n"
        "</Boolean>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }
    // node 'F'
    {
        const char *names[] = { "A", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}

//----------------------------------------------------------------------------
// void DependencyTestSuite::TestDependency13()
//----------------------------------------------------------------------------
/*!
* Test the enum node
*/
void DependencyTestSuite::TestDependency13()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Category Name=\"A\"> \n"
        " <pFeature>B</pFeature> \n"
        "</Category> \n"

        "<Enumeration Name=\"B\">                \n"
        "   <pIsImplemented>C</pIsImplemented>\n"
        "   <pIsAvailable>D</pIsAvailable>\n"
        "   <pIsLocked>E</pIsLocked>\n"
        "   <EnumEntry Name=\"G\"> \n"
        "     <pIsImplemented>H</pIsImplemented>\n"
        "     <pIsAvailable>I</pIsAvailable>\n"
        "     <pIsLocked>J</pIsLocked>\n"
        "     <Value>1</Value> \n"
        "   </EnumEntry> \n"
        "   <pValue>F</pValue> \n"
        "    <pSelected>K</pSelected> \n"
        "</Enumeration>                                \n"

        "<IntReg Name=\"C\">                \n"
        "    <Address>0x0</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"D\">                \n"
        "    <Address>0x2</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"E\">                \n"
        "    <Address>0x4</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"F\">                \n"
        "    <Address>0x6</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"


        "<IntReg Name=\"H\">                \n"
        "    <Address>0xA</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"I\">                \n"
        "    <Address>0xC</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"J\">                \n"
        "    <Address>0xE</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<IntReg Name=\"K\">                \n"
        "    <Address>0x10</Address>            \n"
        "    <Length>2</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>MyPort</pPort>                \n"
        "    <Sign>Unsigned</Sign>                \n"
        "    <Endianess>BigEndian</Endianess>    \n"
        "</IntReg>                                \n"

        "<Port Name=\"MyPort\"/>                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", "K", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }

    // node 'C'
    {
        const char *names[] = { "A", "B", "K", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "C", names ) );
    }

    // node 'D'
    {
        const char *names[] = { "A", "B", "K", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "D", names ) );
    }

    // node 'E'
    {
        const char *names[] = { "A", "B", "K", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "E", names ) );
    }
    // node 'F'
    {
        const char *names[] = { "A", "B", "K", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "F", names ) );
    }
    // node 'G'
    {
        const char *names[] = { "A", "K", "B", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "EnumEntry_B_G", names ) );
    }
    // node 'H'
    {
        const char *names[] = { "A", "K", "B", "EnumEntry_B_G", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "H", names ) );
    }
    // node 'I'
    {
        const char *names[] = { "A", "K", "B", "EnumEntry_B_G", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "I", names ) );
    }
    // node 'J'
    {
        const char *names[] = { "A", "K", "B", "EnumEntry_B_G", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "J", names ) );
    }


    // node 'K'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "K", names ) );
    }

    // node 'MyPort'
    {
        const char *names[] = { "A", "B", "C", "D", "E", "F", "EnumEntry_B_G", "H", "I", "J", "K" , NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "MyPort", names ) );
    }
}

void DependencyTestSuite::TestDependency14()
{
    // invalidators can now exist between any nodes
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Node Name=\"A\">                      \n"
        "   <pInvalidator>B</pInvalidator>      \n"
        "</Node>                                \n"
        "                                       \n"
        "<Node Name=\"B\">                      \n"
        "</Node>                                \n"
        "                                       \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    // node 'A'
    {
        const char *names[] = { NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "A", names ) );
    }

    // node 'B'
    {
        const char *names[] = { "A", NULL };
        CPPUNIT_ASSERT( TestDependency( Camera, "B", names ) );
    }
}

//! this is used to sort node lists 
bool CompareNodeNames(INode* pA, INode* pB) 
{ 
    return (pA->GetName() < pB->GetName()); 
};


void DependencyTestSuite::DependencyDump()
{

    //! Gets a list of files or directories matching a given FileTemplate
    gcstring_vector FileNames;
    gcstring FileFolder( XMLPATH "Test/Performance/" );
    ReplaceEnvironmentVariables(FileFolder);

    GetFiles( FileFolder + "*.xml", FileNames );

    for( gcstring_vector::iterator itFileName=FileNames.begin(); itFileName!=FileNames.end(); itFileName++ )
    {
        gcstring InputFileName(FileFolder + *itFileName);
        gcstring DependencyFileName(InputFileName + gcstring(".dependencies.txt"));
        gcstring TerminalsFileName(InputFileName + gcstring(".terminals.txt"));
        gcstring StatisticsFileName(InputFileName + gcstring(".statistics.txt"));

        std::ofstream DependencyFile(DependencyFileName, std::ios_base::out);
        std::ofstream TerminalsFile(TerminalsFileName, std::ios_base::out);
        std::ofstream StatisticsFile(StatisticsFileName, std::ios_base::out);

        CNodeMapRef Camera;
        Camera._LoadXMLFromFile(InputFileName);

        NodeList_t AllNodes;
        Camera._GetNodes(AllNodes);
        std::sort( AllNodes.begin(), AllNodes.end(), CompareNodeNames );

        int NumDependents = 0;
        int NumFeatures = 0;
        int NumTopLevels = 0;
        int NumDependencyTopLevels = 0;
        int NumTerminals = 0;
        NodeList_t::iterator itNode;
        for( itNode=AllNodes.begin(); itNode!=AllNodes.end(); itNode++ )
        {
            CNodePrivatePtr ptrNodePrivate(*itNode);
            if(ptrNodePrivate->IsFeature())
            {
                NumFeatures++;
                StatisticsFile << "#F";
            }
            else
                StatisticsFile << "  ";

            if(ptrNodePrivate->IsTopLevelNode())
            {
                NumTopLevels++;
                StatisticsFile << "#W";
            }
            else
                StatisticsFile << "  ";

            if(ptrNodePrivate->IsDependencyTopLevelNode())
            {
                NumDependencyTopLevels++;
                StatisticsFile << "#D";
            }
            else
                StatisticsFile << "  ";

            if(ptrNodePrivate->IsTerminalNode())
            {
                NumTerminals++;
                StatisticsFile << "#T";
            }
            else
                StatisticsFile << "  ";

            StatisticsFile << "  " << (*itNode)->GetName() << endl;
            DependencyFile << (*itNode)->GetName() << endl;
            TerminalsFile << (*itNode)->GetName() << endl;

            //dump dependents
            {
                gcstring ValueStr;
                gcstring AttribStr;
                (*itNode)->GetProperty("pDependent", ValueStr, AttribStr);

                gcstring_vector _Dependents;
                Tokenize( ValueStr, _Dependents, "\t");
                std::list<gcstring> Dependents;

                gcstring_vector::iterator _itDependent;
                for( _itDependent=_Dependents.begin(); _itDependent!=_Dependents.end(); _itDependent++ )
                    Dependents.push_back(*_itDependent);

                Dependents.sort();
                std::list<gcstring>::iterator itDependent;
                for( itDependent=Dependents.begin(); itDependent!=Dependents.end(); itDependent++ )
                {
                    DependencyFile << "    " << (*itDependent) << endl;
                    NumDependents++;
                }
            }

            //dump terminals
            {
                gcstring ValueStr;
                gcstring AttribStr;
                (*itNode)->GetProperty("pTerminal", ValueStr, AttribStr);

                gcstring_vector _Terminals;
                Tokenize( ValueStr, _Terminals, "\t");
                std::list<gcstring> Terminals;

                gcstring_vector::iterator _itTerminals;
                for( _itTerminals=_Terminals.begin(); _itTerminals!=_Terminals.end(); _itTerminals++ )
                    Terminals.push_back(*_itTerminals);

                Terminals.sort();
                std::list<gcstring>::iterator itTerminals;
                for( itTerminals=Terminals.begin(); itTerminals!=Terminals.end(); itTerminals++ )
                {
                    TerminalsFile << "    " << (*itTerminals) << endl;
                }
            }
        }

        StatisticsFile << "Nodes = " << AllNodes.size() << endl;
        StatisticsFile << "Dependents = " << NumDependents << endl;
        StatisticsFile << "#F - Features = " << NumFeatures << endl;
        StatisticsFile << "#W - Write TopLevels = " << NumTopLevels << endl;
        StatisticsFile << "#D - Dependency TopLevels = " << NumDependencyTopLevels << endl;
        StatisticsFile << "#T - Terminals = " << NumTerminals << endl;
        
        DependencyFile.close();
        TerminalsFile.close();
        StatisticsFile.close();
    }
}
