//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks, Ivonne Puhlmann
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "EnumerationTest.h"
#include "CallbackTestTarget.h"
#include "TestPort.h"
#include "GenApi/StructPort.h"
#include "StopWatch.h"
#include "GenApi/GenApi.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

#include <limits>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( EnumerationTestSuite, "GenApiTest");

void EnumerationTestSuite::TestValueAccess()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<Enumeration Name=\"Enum\">            \n"
        "    <EnumEntry Name=\"EnumValue1\">     \n"
        "        <Value>10</Value>                \n"
        "   </EnumEntry>                        \n"
        "    <EnumEntry Name=\"EnumValue2\">     \n"
        "        <Value>20</Value>                \n"
        "   </EnumEntry>                        \n"
        "    <pValue>Value</pValue>                \n"
        "</Enumeration>                            \n"
        "                                        \n"
        "<Integer Name=\"Value\">                \n"
        "    <Value>10</Value>                    \n"
        "</Integer>                                \n"
        "                                        \n"
        "<Enumeration Name=\"Enum2\">            \n"
        "    <EnumEntry Name=\"EnumValue1\">     \n"
        "        <Value>10</Value>                \n"
        "   </EnumEntry>                        \n"
        "    <EnumEntry Name=\"EnumValue2\">     \n"
        "        <Value>20</Value>                \n"
        "   </EnumEntry>                        \n"
        "    <EnumEntry Name=\"EnumValue3\">     \n"
        "        <pIsAvailable>False</pIsAvailable>                \n"
        "        <Value>30</Value>                \n"
        "   </EnumEntry>                        \n"
        "    <pValue>Value2</pValue>                \n"
        "</Enumeration>                            \n"
        "                                        \n"
        "<IntReg Name=\"Value2\">        \n"
        "    <Address>0</Address>            \n"
        "    <Length>4</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Signed</Sign>                    \n"
        "    <Endianess>LittleEndian</Endianess>    \n"
        "</IntReg>                                \n"
        "                                        \n"
        "<Integer Name=\"False\">                \n"
        "    <Value>0</Value>                    \n"
        "</Integer>                                \n"
        "                                        \n"
        "<Port Name=\"Port\"/>                    \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CPPUNIT_ASSERT( ptrEnum.IsValid() );
    CPPUNIT_ASSERT_EQUAL (gcstring("IEnumeration"), GetInterfaceName(ptrEnum));

    CIntegerPtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    // happy path
    ptrEnum->FromString("EnumValue1");
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue1"), ptrEnum->ToString() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)10, ptrValue->GetValue() );

    ptrEnum->FromString("EnumValue2");
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue2"), ptrEnum->ToString() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)20, ptrValue->GetValue() );

    CEnumEntryPtr ptrEnumEntry = ptrEnum->GetCurrentEntry();
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue2"), ptrEnumEntry->GetSymbolic() );

    // Ahaem - this should not change over time :-)
    ptrEnumEntry->GetNode()->InvalidateNode();
    CPPUNIT_ASSERT_EQUAL( RO, ptrEnumEntry->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrEnumEntry->GetAccessMode() );


    // value too small
    CPPUNIT_ASSERT_THROW_EX(
        ptrEnum->FromString("BlaBla"),
        GenICam::InvalidArgumentException
        );

    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : Enumeration <=> Integer");

    ptrValue->GetNode()->InvalidateNode();
    TEST_BEGIN(1000)
        ptrEnum->FromString("EnumValue1");
    StopWatch.PauseBegin();
    ptrValue->GetNode()->InvalidateNode();
    StopWatch.PauseEnd();
    TEST_END( Enumeration::FromString )

    // now some tests with the other enum, connected to a real port
    CTestPort Port;
    Port.CreateEntry(0x000, (int32_t)10, RW);
    Camera._Connect(&Port, "Port");
    CEnumerationPtr ptrEnum2 = Camera._GetNode("Enum2");
    CPPUNIT_ASSERT_EQUAL ((int64_t)10, ptrEnum2->GetIntValue());
    // modify the underlying value, not visible through the cache
    int32_t Value = 20;
    Port.Write (&Value, 0x00, 4);
    CPPUNIT_ASSERT_EQUAL ((int64_t)10, ptrEnum2->GetIntValue());
    CPPUNIT_ASSERT_EQUAL (gcstring("EnumValue1"), ptrEnum2->ToString());
    // but visible when bypassing the cache
    CPPUNIT_ASSERT_EQUAL ((int64_t)20, ptrEnum2->GetIntValue(false, true));
    CPPUNIT_ASSERT_EQUAL (gcstring("EnumValue2"), ptrEnum2->ToString(false, true));
    // now change it to point to the NA entry
    Value = 30;
    Port.Write (&Value, 0x00, 4);
    // and test without/with verification
    CPPUNIT_ASSERT_EQUAL ((int64_t)30, ptrEnum2->GetIntValue(false, true));
    CPPUNIT_ASSERT_EQUAL (gcstring("EnumValue3"), ptrEnum2->ToString(false, true));
    CPPUNIT_ASSERT_THROW (ptrEnum2->GetIntValue(true, true), GenericException);
    CPPUNIT_ASSERT_THROW (ptrEnum2->ToString(true, true), GenericException);

}

void EnumerationTestSuite::TestValueAccessStatic()
{
    {
        using namespace Test_TestCamera_v1_0;

        CTestCamera_v1_0 Camera;
        Camera._LoadDLL();

        // happy path
        Camera.Enum = Enum_EnumValue1;
        CPPUNIT_ASSERT_EQUAL( Enum_EnumValue1, Camera.Enum.GetValue() );

        Camera.Enum = Enum_EnumValue2;
        CPPUNIT_ASSERT_EQUAL( Enum_EnumValue2, Camera.Enum.GetValue() );

        LOG_PERFORMANCE("-------------------------------------------------");
        LOG_PERFORMANCE("Setup : Enumeration <=> Integer");

        Camera.EnumValue.GetNode()->InvalidateNode();
        TEST_BEGIN(1000)
            Camera.Enum = Enum_EnumValue1;
        StopWatch.PauseBegin();
        Camera.EnumValue.GetNode()->InvalidateNode();
        StopWatch.PauseEnd();
        TEST_END( EnumerationT::SetValue )
    }
    {
        using namespace Test_TestCamera_v1_1;

        CTestCamera_v1_1 Camera;
        Camera._LoadDLL();

        // happy path
        Camera.Enum = Enum_EnumValue1;
        CPPUNIT_ASSERT_EQUAL( Enum_EnumValue1, Camera.Enum.GetValue() );

        Camera.Enum = Enum_EnumValue2;
        CPPUNIT_ASSERT_EQUAL( Enum_EnumValue2, Camera.Enum.GetValue() );

        LOG_PERFORMANCE("-------------------------------------------------");
        LOG_PERFORMANCE("Setup : Enumeration <=> Integer");

        Camera.EnumValue.GetNode()->InvalidateNode();
        TEST_BEGIN(1000)
            Camera.Enum = Enum_EnumValue1;
        StopWatch.PauseBegin();
        Camera.EnumValue.GetNode()->InvalidateNode();
        StopWatch.PauseEnd();
        TEST_END( EnumerationT::SetValue )
    }
}

void EnumerationTestSuite::TestRegValueAccessRW()
{


}
void EnumerationTestSuite::TestEnumEntry()
{
    // create and initilize node map
    // (one of the Enumerations is a terminal node, the other not)
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                        \n"
        "<Enumeration Name=\"Value\">                            \n"
        "    <EnumEntry Name=\"MyEnumEntry0\">                    \n"
        "        <Value>0</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"MyEnumEntry1\">                    \n"
        "        <Value>1</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"MyEnumEntry2\">                    \n"
        "        <Value>2</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"MyEnumEntry3\">                    \n"
        "        <pIsAvailable>EntryAvailable</pIsAvailable>        \n"
        "        <Value>3</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"MyEnumEntry4\">                    \n"
        "        <pIsImplemented>EntryImplemented</pIsImplemented>\n"
        "        <Value>4</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <pValue>Value2</pValue>                                \n"
        "</Enumeration>                                            \n"
        "                                                        \n"
        "<Integer Name=\"EntryAvailable\">                        \n"
        "    <Value>0</Value>                                    \n"
        "</Integer>                                                \n"
        "                                                        \n"
        "<Integer Name=\"EntryImplemented\">                    \n"
        "    <Value>0</Value>                                    \n"
        "</Integer>                                                \n"
        "                                                        \n"
        "<Integer Name=\"Value2\">                                \n"
        "    <Value>10</Value>                                    \n"
        "</Integer>                                                \n"
        "                                                        \n"
        "<Enumeration Name=\"OtherValue\">                        \n"
        "    <EnumEntry Name=\"MyOtherEnumEntry0\">                \n"
        "        <Value>0</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"MyOtherEnumEntry1\">                \n"
        "        <Value>1</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <Value>0</Value>                                    \n"
        "</Enumeration>                                            \n"
        "                                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    INode* featureNode = Camera._GetNode("Value");
    CValuePtr ptrFeature(featureNode);
    if(ptrFeature)
    {
        IEnumeration* pEnumeration =  dynamic_cast<IEnumeration*>(featureNode);
        if (pEnumeration != NULL)
        {
            gcstring_vector symbolics;
            pEnumeration->GetSymbolics(symbolics);
            int i=0;
            CPPUNIT_ASSERT_EQUAL( (int)symbolics.size(), 3 );
            CPPUNIT_ASSERT_EQUAL( gcstring("MyEnumEntry0"), symbolics[i++] );
            CPPUNIT_ASSERT_EQUAL( gcstring("MyEnumEntry1"), symbolics[i++] );
            CPPUNIT_ASSERT_EQUAL( gcstring("MyEnumEntry2"), symbolics[i++] );
        }

    }
    CEnumerationPtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    ptrValue->FromString("MyEnumEntry1");
    gcstring gstr( "MyEnumEntry1" );
    CPPUNIT_ASSERT_EQUAL(gstr, ptrValue->ToString() );

    ptrValue->FromString("MyEnumEntry2");
    gstr = "MyEnumEntry2";
    CPPUNIT_ASSERT_EQUAL(gstr, ptrValue->ToString() );

    ptrValue->FromString("MyEnumEntry0");
    gstr = "MyEnumEntry0";
    CPPUNIT_ASSERT_EQUAL(gstr, ptrValue->ToString() );

    IEnumEntry *MyEntry;
    MyEntry = ptrValue->GetEntryByName("MyEnumEntry1");
    //    CPPUNIT_ASSERT_EQUAL( (gcstring)"1", MyEntry->ToString(MyEntry->GetValue()) ); // *JS* removed warning
    CPPUNIT_ASSERT_EQUAL( (gcstring)"1", MyEntry->ToString(MyEntry->GetValue() != 0) );
    //    gcstring str = MyEntry->ToString( MyEntry->GetValue() ); // *JS* removed warning
    gcstring str = MyEntry->ToString( MyEntry->GetValue() != 0);
    // entries are not writeable
    CPPUNIT_ASSERT_THROW( MyEntry->FromString(str), GenICam::GenericException);

    gstr = MyEntry->GetSymbolic();
    CPPUNIT_ASSERT_EQUAL((gcstring)"MyEnumEntry1" ,gstr);

    CEnumEntryRef refEnumEntry;
    CPPUNIT_ASSERT_NO_THROW(refEnumEntry.SetReference( ptrValue ) );
    CPPUNIT_ASSERT_THROW_EX( refEnumEntry.GetSymbolic(),AccessException );
    CPPUNIT_ASSERT_THROW_EX( refEnumEntry.GetValue(),AccessException );
    CPPUNIT_ASSERT_NO_THROW( refEnumEntry.SetReference( MyEntry ) );
    CPPUNIT_ASSERT_EQUAL( (gcstring)"MyEnumEntry1", refEnumEntry.GetSymbolic());
    CPPUNIT_ASSERT_EQUAL( (int64_t)1,refEnumEntry.GetValue());
    CPPUNIT_ASSERT_EQUAL (RO, refEnumEntry.GetAccessMode());

    // now do some additional tests with the terminal node
    CEnumerationPtr ptrEnum = Camera._GetNode( "OtherValue" );
    ptrEnum->SetIntValue( 1 );
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrEnum->GetIntValue());
    CPPUNIT_ASSERT_NO_THROW ( ptrEnum->GetIntValue( true ) );

    CPPUNIT_ASSERT_THROW_EX ( ptrEnum->SetIntValue( 100 ), GenICam::InvalidArgumentException );
    // note that the Verify=false does not disable the checking against valid enum values...
    CPPUNIT_ASSERT_THROW_EX ( ptrEnum->SetIntValue( 100, false ), GenICam::InvalidArgumentException );

    // what if value points to a NI entry?
    CIntegerPtr ptrIntVal = Camera._GetNode ("Value2");
    CPPUNIT_ASSERT ( ptrIntVal.IsValid() );
    ptrIntVal->SetValue (3); // this entry is NA
    CPPUNIT_ASSERT_THROW_EX (ptrValue->ToString(true), AccessException);
    CPPUNIT_ASSERT_THROW_EX (ptrValue->FromString("MyEnumEntry4"), AccessException);
}
void EnumerationTestSuite::TestEnumFalseEntry()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "<Enumeration Name=\"NoValue\">                            \n"
        "    <EnumEntry Name=\"MyEnumEntry1\">                    \n"
        "    <Value>3</Value>                                    \n"
        "    </EnumEntry>                                        \n"
        "    <pValue>Value2</pValue>                                \n"
        "</Enumeration>                                            \n"
        "                                                        \n"
        "<Integer Name=\"Value2\">                                \n"
        "    <Value>10</Value>                                    \n"
        "</Integer>                                                \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    gcstring_vector symbolics;
    CEnumerationPtr FalsePtr = Camera._GetNode("NoValue");
    CPPUNIT_ASSERT_EQUAL( RW,FalsePtr->GetAccessMode() );
    FalsePtr->GetSymbolics(symbolics);
    CPPUNIT_ASSERT_THROW_EX( FalsePtr->ToString(), InvalidArgumentException );
    FalsePtr->SetIntValue(3);
    CPPUNIT_ASSERT_EQUAL( gcstring("MyEnumEntry1"), FalsePtr->ToString() );

}
void EnumerationTestSuite::TestEnumRef()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "<Enumeration Name=\"PixelFormat\">        \n"
        "  <EnumEntry Name=\"Mono8\">              \n"
        "     <Value>0</Value>                     \n"
        "  </EnumEntry>                            \n"
        "  <EnumEntry Name=\"Mono16\">             \n"
        "     <Value>1</Value>                     \n"
        "  </EnumEntry>                            \n"
        "  <EnumEntry Name=\"RGB24\">              \n"
        "     <Value>2</Value>                     \n"
        "  </EnumEntry>                            \n"
        "  <pValue>Value</pValue>                  \n"
        "</Enumeration>                            \n"
        "                                          \n"
        "<Integer Name=\"Value\">                  \n"
        "    <Value>10</Value>                     \n"
        "</Integer>                                \n"
        "                                          \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CTestPort Port;
    Port.CreateEntry(0x0104, (uint32_t)1024, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    CEnumerationPtr ptrValue = Camera._GetNode("PixelFormat");
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    // not valid pointer for testing false cases
    CEnumerationTRef<int> refFalse;

    CEnumerationTRef<int> refEnumT;
    CPPUNIT_ASSERT_NO_THROW( refEnumT.SetReference( ptrValue ) );
    CPPUNIT_ASSERT_NO_THROW( refEnumT.SetEnumReference(0, "PixelFormat" ) );

    CPPUNIT_ASSERT_THROW_EX( refFalse.IsValueCacheValid(),AccessException );
    CPPUNIT_ASSERT( !refEnumT.IsValueCacheValid() );

    ptrValue = refEnumT.GetNode();
    CPPUNIT_ASSERT_THROW_EX( refFalse.GetNode(),AccessException );

    CEnumerationTRef<EAccessMode> refEnumAcc;
    EAccessMode AMode = RW;
    CPPUNIT_ASSERT_NO_THROW( refEnumAcc.GetAccessMode());
    CPPUNIT_ASSERT_NO_THROW( refEnumAcc.SetReference( ptrValue ) );
    CPPUNIT_ASSERT_EQUAL( AMode, refEnumAcc.GetAccessMode());

    gcstring_vector symbolics;
    CPPUNIT_ASSERT_THROW_EX( refFalse.GetSymbolics(symbolics),AccessException );
    refEnumT.GetSymbolics(symbolics);
    int i(0);
    CPPUNIT_ASSERT_EQUAL( gcstring("Mono8"), symbolics[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("Mono16"), symbolics[i++] );
    CPPUNIT_ASSERT_EQUAL( gcstring("RGB24"), symbolics[i++] );
    CPPUNIT_ASSERT_EQUAL( size_t(i), symbolics.size() );

    NodeList_t entries;
    i=0;
    gcstring str1 = "EnumEntry_PixelFormat_Mono8",
        str2 = "EnumEntry_PixelFormat_Mono16",
        str3 = "EnumEntry_PixelFormat_RGB24";
    CPPUNIT_ASSERT_THROW_EX( refFalse.GetEntries(entries),AccessException );
    refEnumT.GetEntries(entries);
    CPPUNIT_ASSERT_EQUAL( str1, entries[i++]->GetName() );
    CPPUNIT_ASSERT_EQUAL( str2, entries[i++]->GetName() );
    CPPUNIT_ASSERT_EQUAL( str3, entries[i++]->GetName() );
    CPPUNIT_ASSERT_EQUAL( size_t(i), entries.size() );

    /** AH: no longer allowed since Mantis #9 is fixed.
    ptrValue->SetIntValue(1234);  // AH: now checked in test EnumerationTestSuite::TestInvalidValues()
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1234LL,refEnumT.GetIntValue() );
    */

    CPPUNIT_ASSERT_THROW_EX( refFalse.SetIntValue(5432),AccessException );

    /** AH: no longer allowed since Mantis #9 is fixed
    refEnumT.SetIntValue(5432); // AH: now checked in test EnumerationTestSuite::TestInvalidValues()
    CPPUNIT_ASSERT_EQUAL( (int64_t) 5432LL,(int64_t)refEnumT.GetIntValue() );
    */

    CPPUNIT_ASSERT_THROW( refFalse.GetEntry((int64_t)0), GenericException );
    CPPUNIT_ASSERT_THROW( refFalse.GetCurrentEntry(), GenericException );
    CPPUNIT_ASSERT_THROW( refFalse.GetEntry((int)0), GenericException ); // falls into the EnumT version of GetEntry (EnumT=int, above)

    IEnumEntry *MyEntry;
    CPPUNIT_ASSERT_THROW_EX( refFalse.GetEntryByName("RGB24"),AccessException );
    MyEntry = refEnumT.GetEntryByName("RGB24");
    CPPUNIT_ASSERT_EQUAL((gcstring)"2", MyEntry->ToString());

    //    CPPUNIT_ASSERT_EQUAL( (int64_t) 10LL, refEnumT.GetIntValue(MyEntry->GetValue())); // *JS* removed warning
    CPPUNIT_ASSERT_THROW_EX( refEnumT.GetIntValue(MyEntry->GetValue() != 0), OutOfRangeException);
    //    CPPUNIT_ASSERT_THROW_EX( refFalse.GetIntValue(MyEntry->GetValue()),AccessException ); // *JS* removed warning
    CPPUNIT_ASSERT_THROW_EX( refFalse.GetIntValue(MyEntry->GetValue() != 0),AccessException );

    CPPUNIT_ASSERT_THROW_EX( refEnumT.ToString(0), InvalidArgumentException );
    refEnumT.SetIntValue(1);
    CPPUNIT_ASSERT_EQUAL((gcstring)"Mono16",refEnumT.ToString(0) );
    CPPUNIT_ASSERT_EQUAL ((gcstring)"Mono16",refEnumT.GetCurrentEntry()->GetSymbolic());
    CPPUNIT_ASSERT_EQUAL ((gcstring)"Mono8",refEnumT.GetEntry((int64_t)0)->GetSymbolic());
    CPPUNIT_ASSERT_EQUAL ((IEnumEntry*)NULL,refEnumT.GetEntry((int64_t)234));
    CPPUNIT_ASSERT_THROW_EX( refFalse.ToString(0),AccessException );

    refEnumT.FromString("Mono8"); // ???
    CPPUNIT_ASSERT_THROW_EX( refFalse.FromString("Mono8"),AccessException );

    CPPUNIT_ASSERT_THROW_EX( refEnumT.GetValue(),AccessException );

    refEnumT.SetNumEnums(3);
#   undef max
    CPPUNIT_ASSERT_THROW_EX( refEnumT.SetValue(int(MyEntry->GetValue() & numeric_limits<unsigned>::max() )),AccessException );
    CPPUNIT_ASSERT_THROW_EX( refFalse.SetValue(int(MyEntry->GetValue() & numeric_limits<unsigned>::max() )),AccessException );


    // With the fix of Mantis 376, the following assertion would fail.
    //    CPPUNIT_ASSERT_EQUAL( 0, refEnumT.GetValue() );

    // Since no enum references are set, GetValue() must throw an exception
    CPPUNIT_ASSERT_THROW_EX( refEnumT.GetValue(), AccessException ); // checks if Mantis 376 works

    CPPUNIT_ASSERT_THROW_EX( refFalse.GetValue(),AccessException );

    refEnumT.SetEnumReference(0, "Mono8");
    refEnumT.SetEnumReference(1, "Mono16");
    refEnumT.SetEnumReference(2, "RGB24");

    // this one does nothing (the call is ignored), it is here just to complete the coverage
    CPPUNIT_ASSERT_NO_THROW( refFalse.SetEnumReference(0, "FooBar") );

    refEnumT.SetValue(1);
    CPPUNIT_ASSERT_EQUAL( 1, refEnumT.GetValue() );
    CPPUNIT_ASSERT_EQUAL (true, refEnumT.IsValueCacheValid());

    refEnumT = "Mono16";
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, refEnumT.GetIntValue() );
    CPPUNIT_ASSERT_THROW_EX( refFalse = "Mono16", AccessException );

    refEnumT = gcstring("RGB24");
    CPPUNIT_ASSERT_EQUAL( (int64_t)2, refEnumT.GetIntValue() );
    CPPUNIT_ASSERT_THROW_EX( refFalse = gcstring("Mono16"), AccessException );

    CPPUNIT_ASSERT_EQUAL( gcstring("RGB24"), *refEnumT );
    CPPUNIT_ASSERT_THROW_EX( *refFalse, AccessException );

    CPPUNIT_ASSERT_EQUAL( 2, refEnumT() );
    CPPUNIT_ASSERT_THROW_EX( refFalse(),AccessException );

    INode* featureNode = Camera._GetNode("PixelFormat");
    gcstring Value, Attribute;


}



void EnumerationTestSuite::TestInvalidValues()
{
    {
        using namespace Test_TestCamera_v1_0;

        CTestCamera_v1_0 Camera;
        Camera._LoadDLL();

        CPPUNIT_ASSERT_NO_THROW(Camera.Enum.SetValue(Enum_EnumValue1));

        // invalid string value
        CPPUNIT_ASSERT_THROW_EX(Camera.Enum.FromString("foo"), GenICam::InvalidArgumentException );

        // invalid integer value
        CPPUNIT_ASSERT_THROW_EX(Camera.Enum.SetIntValue(42), GenICam::InvalidArgumentException );

        // try to write an value that isn't available
        CEnumEntryPtr ptrEntry = Camera._GetNode("EnumEntry_Enum_EnumValue1");
        CPPUNIT_ASSERT( ptrEntry.IsValid() );
        CPPUNIT_ASSERT_EQUAL (gcstring("IEnumEntry"), GetInterfaceName(ptrEntry));

        ptrEntry->GetNode()->ImposeAccessMode(NA);

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.FromString(ptrEntry->GetSymbolic()), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetValue(Enum_EnumValue1), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetIntValue( ptrEntry->GetValue() ), GenICam::AccessException );


        // try to write an value to an enumeration node that isn't available
        Camera.Enum.GetNode()->ImposeAccessMode(NA);

        ptrEntry = Camera._GetNode("EnumEntry_Enum_EnumValue2");
        CPPUNIT_ASSERT( ptrEntry.IsValid() );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.FromString(ptrEntry->GetSymbolic()), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetValue(Enum_EnumValue1), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetIntValue( ptrEntry->GetValue() ), GenICam::AccessException );
    }
    {
        using namespace Test_TestCamera_v1_1;

        CTestCamera_v1_1 Camera;
        Camera._LoadDLL();

        CPPUNIT_ASSERT_NO_THROW(Camera.Enum.SetValue(Enum_EnumValue1));

        // invalid string value
        CPPUNIT_ASSERT_THROW_EX(Camera.Enum.FromString("foo"), GenICam::InvalidArgumentException );

        // invalid integer value
        CPPUNIT_ASSERT_THROW_EX(Camera.Enum.SetIntValue(42), GenICam::InvalidArgumentException );

        // try to write an value that isn't available
        CEnumEntryPtr ptrEntry = Camera._GetNode("EnumEntry_Enum_EnumValue1");
        CPPUNIT_ASSERT( ptrEntry.IsValid() );
        CPPUNIT_ASSERT_EQUAL (gcstring("IEnumEntry"), GetInterfaceName(ptrEntry));

        ptrEntry->GetNode()->ImposeAccessMode(NA);

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.FromString(ptrEntry->GetSymbolic()), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetValue(Enum_EnumValue1), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetIntValue( ptrEntry->GetValue() ), GenICam::AccessException );


        // try to write an value to an enumeration node that isn't available
        Camera.Enum.GetNode()->ImposeAccessMode(NA);

        ptrEntry = Camera._GetNode("EnumEntry_Enum_EnumValue2");
        CPPUNIT_ASSERT( ptrEntry.IsValid() );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.FromString(ptrEntry->GetSymbolic()), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetValue(Enum_EnumValue1), GenICam::AccessException );

        CPPUNIT_ASSERT_THROW_EX( Camera.Enum.SetIntValue( ptrEntry->GetValue() ), GenICam::AccessException );
    }
}


void EnumerationTestSuite::TestDisplayName()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "<Enumeration Name=\"Enumeration\">                            \n"
        "    <EnumEntry Name=\"EnumEntry0\">                    \n"
        "        <Value>0</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"EnumEntry1\">                    \n"
        "        <Value>1</Value>                                \n"
        "        <Symbolic>Symbolic1</Symbolic>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"EnumEntry2\">                    \n"
        "        <DisplayName>DisplayName2</DisplayName>                                \n"
        "        <Value>2</Value>                                \n"
        "    </EnumEntry>                                        \n"
        "    <EnumEntry Name=\"EnumEntry3\">                    \n"
        "        <DisplayName>DisplayName3</DisplayName>                                \n"
        "        <Value>3</Value>                                \n"
        "        <Symbolic>Symbolic3</Symbolic>                                \n"
        "    </EnumEntry>                                        \n"
        "    <Value>0</Value>                                \n"
        "</Enumeration>                                            \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CEnumerationPtr ptrEnumeration = Camera._GetNode("Enumeration");
    //IEnumeration* pEnumeration =  dynamic_cast<IEnumeration*>(ValueNode);

    CPPUNIT_ASSERT_EQUAL (gcstring("EnumEntry0"), (dynamic_cast<INode*>(ptrEnumeration->GetEntryByName("EnumEntry0")))->GetDisplayName() );
    CPPUNIT_ASSERT_EQUAL ((IEnumEntry*)NULL, ptrEnumeration->GetEntryByName("EnumEntry1") );
    CPPUNIT_ASSERT_EQUAL (gcstring("Symbolic1"), (dynamic_cast<INode*>(ptrEnumeration->GetEntryByName("Symbolic1")))->GetDisplayName() );
    CPPUNIT_ASSERT_EQUAL (gcstring("DisplayName2"), (dynamic_cast<INode*>(ptrEnumeration->GetEntryByName("EnumEntry2")))->GetDisplayName() );
    CPPUNIT_ASSERT_EQUAL ((IEnumEntry*)NULL, ptrEnumeration->GetEntryByName("EnumEntry3") );
    CPPUNIT_ASSERT_EQUAL (gcstring("DisplayName3"), (dynamic_cast<INode*>(ptrEnumeration->GetEntryByName("Symbolic3")))->GetDisplayName() );

}

void EnumerationTestSuite::TestNumericValue()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                         \n"
        "<Integer Name=\"IntFromEnum\">           \n"
        "    <pValue>Enum</pValue>                \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Enumeration Name=\"Enum\">              \n"
        "    <EnumEntry Name=\"EnumValue1\">      \n"
        "        <Value>10</Value>                \n"
        "        <NumericValue>1.5</NumericValue> \n"
        "   </EnumEntry>                          \n"
        "    <EnumEntry Name=\"EnumValue2\">      \n"
        "        <Value>20</Value>                \n"
        "   </EnumEntry>                          \n"
        "    <EnumEntry Name=\"EnumValue3\">      \n"
        "        <Value>30</Value>                \n"
        "        <NumericValue>1.7</NumericValue> \n"
        "   </EnumEntry>                          \n"
        "    <Value>10</Value>               \n"
        "</Enumeration>                           \n"
        "                                         \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CEnumEntryPtr ptrEnumEntry1 = ptrEnum->GetEntryByName("EnumValue1");
    CEnumEntryPtr ptrEnumEntry2 = ptrEnum->GetEntryByName("EnumValue2");
    CIntegerPtr ptrIntFromEnum = Camera._GetNode("IntFromEnum");

    CPPUNIT_ASSERT_DOUBLES_EQUAL( 1.5, ptrEnumEntry1->GetNumericValue(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 20.0, ptrEnumEntry2->GetNumericValue(), std::numeric_limits<double>::epsilon() );

    // the same through reference
    CEnumEntryRef refEnumEntry1, refEnumEntry2;
    CPPUNIT_ASSERT_THROW_EX (refEnumEntry1.GetNumericValue(), GenICam::AccessException);
    CPPUNIT_ASSERT_NO_THROW(refEnumEntry1.SetReference( ptrEnumEntry1 ) );
    CPPUNIT_ASSERT_NO_THROW(refEnumEntry2.SetReference( ptrEnumEntry2 ) );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 1.5, refEnumEntry1.GetNumericValue(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 20.0, refEnumEntry2.GetNumericValue(), std::numeric_limits<double>::epsilon() );

    // try setting an enum through an integer
    *ptrIntFromEnum = 1;
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue1"), ptrEnum->ToString());
    *ptrIntFromEnum = 2;
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue3"), ptrEnum->ToString());
    *ptrIntFromEnum = 20;
    CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue2"), ptrEnum->ToString());

}


void TestAutoGain_Callback(INode *pNode)
{
    CValuePtr ptrValue(pNode);
    ptrValue->ToString();
}


/*============ type definitions for the register space defined below ========== */
// have to be defined at file scope for VC71 (hn)
    typedef enum _EGainAuto
    {
        Off = (uint8_t)1,
        Once = (uint8_t)2,
        Continuous = (uint8_t)3
    } EGainAuto;
    #pragma pack(push, 1)
    typedef struct _AutoGainReg_t
    {                                               // addr:len
        uint32_t Gain;           // 0:4
        EGainAuto GainAutoReg;   // 4:1
    }  GainAutoFeature_t;         // 5
    #pragma pack(pop)

void EnumerationTestSuite::TestAutoGain()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    /*============ Setup the register space ========== */
    // type definitions above

    CTestPortStruct<GainAutoFeature_t> GainAutoFeature;
    GainAutoFeature.Gain = 42;
    GainAutoFeature.GainAutoReg = Off;

    /*============ Setup the corresponding XML file ========== */

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +

        "<IntReg Name=\"Gain\">                     \n"
        "    <pBlockPolling>GainPollingBlock</pBlockPolling>\n"
        "    <Address>0</Address>                   \n"
        "    <Length>4</Length>                     \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <PollingTime>1000</PollingTime>          \n"
        "    <Sign>Unsigned</Sign>                  \n"
        "    <Endianess>LittleEndian</Endianess>    \n"
        "</IntReg>                                  \n"
        "                                           \n"
        "<IntReg Name=\"GainAutoReg\">              \n"
        "    <Address>4</Address>                   \n"
        "    <Length>1</Length>                     \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Unsigned</Sign>                  \n"
        "    <Endianess>LittleEndian</Endianess>    \n"
        "</IntReg>                                  \n"
        "                                           \n"
        "<Enumeration Name=\"GainAuto\">            \n"
        "   <EnumEntry Name=\"Off\">                \n"
        "        <Value>1</Value>                   \n"
        "   </EnumEntry>                            \n"
        "   <EnumEntry Name=\"Once\">               \n"
        "        <Value>2</Value>                   \n"
        "        <IsSelfClearing>Yes</IsSelfClearing>   \n"
        "   </EnumEntry>                            \n"
        "   <EnumEntry Name=\"Continuous\">         \n"
        "        <Value>3</Value>                   \n"
        "   </EnumEntry>                            \n"
        "   <pValue>GainAutoReg</pValue>            \n"
        "   <PollingTime>50</PollingTime>           \n"
        "</Enumeration>                             \n"
        "                                           \n"
        "<IntSwissKnife Name=\"GainPollingBlock\">                  \n"
        "    <pVariable Name=\"GainAuto\">GainAuto</pVariable>  \n"
        "    <Formula> GainAuto=1 ? 1 : 0 </Formula>         \n"
        "</IntSwissKnife>                                       \n"
        "                                           \n"
        "<Port Name=\"Port\"/>                      \n"

        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    Camera._Connect(&GainAutoFeature, "Port");

    /*============ Get pointers and do basic checks ========== */

    CIntegerPtr ptrGain = Camera._GetNode("Gain");
    CEnumerationPtr ptrGainAuto = Camera._GetNode("GainAuto");

    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, ptrGain->GetValue() );
    *ptrGain = 4711;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 4711LL, ptrGain->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (uint32_t)4711, GainAutoFeature.Gain );

    CPPUNIT_ASSERT_EQUAL( gcstring("Off"), ptrGainAuto->ToString() );
    *ptrGainAuto = "Continuous";
    CPPUNIT_ASSERT_EQUAL( gcstring("Continuous"), ptrGainAuto->ToString() );
    CPPUNIT_ASSERT_EQUAL( Continuous, GainAutoFeature.GainAutoReg );

    CEnumEntryPtr ptrGainAutoOff = ptrGainAuto->GetEntryByName("Off");
    CEnumEntryPtr ptrGainAutoOnce = ptrGainAuto->GetEntryByName("Once");
    CEnumEntryRef refGainAutoOff;
    refGainAutoOff.SetReference (ptrGainAutoOff);
    CEnumEntryRef refGainAutoOnce;
    CPPUNIT_ASSERT_THROW_EX (refGainAutoOnce.IsSelfClearing(), AccessException);
    refGainAutoOnce.SetReference (ptrGainAutoOnce);

    CPPUNIT_ASSERT_EQUAL (false, ptrGainAutoOff->IsSelfClearing());
    CPPUNIT_ASSERT_EQUAL (true, ptrGainAutoOnce->IsSelfClearing());
    CPPUNIT_ASSERT_EQUAL (false, refGainAutoOff.IsSelfClearing());
    CPPUNIT_ASSERT_EQUAL (true, refGainAutoOnce.IsSelfClearing());


    /*============ Test basic auto-reset ========== */

    CCallbackTestTarget GainAutoCallback(ptrGainAuto);

    // this callback will read the value
    Register( ptrGainAuto->GetNode(), &TestAutoGain_Callback );

    // start one cycle of auto-gain
    *ptrGainAuto = "Once";
    CPPUNIT_ASSERT_EQUAL( Once, GainAutoFeature.GainAutoReg );
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );

    // poll while waiting for the auto-gain cycle to complete
    Camera._Poll(100);
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );

    Camera._Poll(100);
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );

    Camera._Poll(25);
    CPPUNIT_ASSERT( !GainAutoCallback.HasFiredOnce() );
    Camera._Poll(25);
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );

    // the camera signals we're ready
    GainAutoFeature.GainAutoReg = Off;

    Camera._Poll(100);
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Off"), ptrGainAuto->ToString() );

    Camera._Poll(100);
    CPPUNIT_ASSERT( GainAutoCallback.HasNotFired() );


    /*============ Test different use cases of auto-reset ========== */

    *ptrGainAuto = "Once";
    GainAutoCallback.Reset();
    GainAutoFeature.GainAutoReg = Off;

    // "cleared" by setting another value
    *ptrGainAuto = "Continuous";
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );

    //////////////

    *ptrGainAuto = "Once";
    GainAutoCallback.Reset();
    GainAutoFeature.GainAutoReg = Off;

    // "cleared" by setting another value
    ptrGainAuto->SetIntValue(Continuous);
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );

    //////////////

    *ptrGainAuto = "Once";
    GainAutoCallback.Reset();
    GainAutoFeature.GainAutoReg = Off;

    // "cleared" by getting the value
    ptrGainAuto->GetIntValue();
    // but no callback, since it is a read
    CPPUNIT_ASSERT( GainAutoCallback.HasNotFired() );
    // no callback since it is cleared already
    Camera._Poll(100);
    CPPUNIT_ASSERT( GainAutoCallback.HasNotFired() );

    //////////////

    *ptrGainAuto = "Once";
    GainAutoCallback.Reset();
    GainAutoFeature.GainAutoReg = Off;

    // "cleared" by getting the value
    ptrGainAuto->ToString();
    // but no callback, since it is a read
    CPPUNIT_ASSERT( GainAutoCallback.HasNotFired() );
    // no callback since it is cleared already
    Camera._Poll(100);
    CPPUNIT_ASSERT( GainAutoCallback.HasNotFired() );

    /*============ Test continuous ========== */
    *ptrGainAuto = "Continuous";

    GainAutoFeature.Gain = 48;
    CCallbackTestTarget GainCallback(ptrGain);
    Camera._Poll(100);
    CPPUNIT_ASSERT( GainCallback.HasNotFired());
    Camera._Poll(1000);
    CPPUNIT_ASSERT( GainCallback.HasFiredOnce() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 48LL, ptrGain->GetValue() );


    /*============ Test off ========== */

    *ptrGainAuto = "Off";
    GainCallback.Reset();

    // fill the cache and then change the register value
    CPPUNIT_ASSERT_EQUAL( (int64_t) 48LL, ptrGain->GetValue() );
    GainAutoFeature.Gain = 49;

    // now polling does not happen and the cache is not cleared
    Camera._Poll(2000);
    CPPUNIT_ASSERT( GainCallback.HasNotFired() );
    // because the cache is enabled the old value is given
    CPPUNIT_ASSERT_EQUAL( (int64_t) 48LL, ptrGain->GetValue() );


    /*============ Test caching, Continuous ========== */

    *ptrGainAuto = "Continuous";  // this clears the Gain cache
    GainAutoFeature.Gain = 111;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 111LL, ptrGain->GetValue() );
    GainAutoFeature.Gain = 222;
    // Since there is no polling and Continuous is not self clearing anyway the cached value is
    // returned
    CPPUNIT_ASSERT_EQUAL( (int64_t) 111LL, ptrGain->GetValue() );


    /*============ Test caching, Off ========== */
    *ptrGainAuto = "Off";  // this clears the Gain cache
    // this fills the cache
    CPPUNIT_ASSERT_EQUAL( (int64_t) 222LL, ptrGain->GetValue() );
    GainAutoFeature.Gain = 333;
    // value comes from cache
    CPPUNIT_ASSERT_EQUAL( (int64_t) 222LL, ptrGain->GetValue() );

    /*============ Test caching, Once ========== */
    *ptrGainAuto = "Once"; // this clears the cache
    GainAutoFeature.Gain = 444;
    // this fills the cache
    CPPUNIT_ASSERT_EQUAL( (int64_t) 444LL, ptrGain->GetValue() );
    GainAutoFeature.Gain = 555;
    GainAutoFeature.GainAutoReg = Off;
    // since Once is self clearing the value is read and Gain's cache is cleard
    CPPUNIT_ASSERT_EQUAL( gcstring("Off"), ptrGainAuto->ToString() );
    // this fills the cache
    CPPUNIT_ASSERT_EQUAL( (int64_t) 555LL, ptrGain->GetValue() );

    // once again without direct read of GainAuto
    *ptrGainAuto = "Once"; // this clears the Gain cache
    GainAutoFeature.Gain = 666;
    // this fills the cache
    CPPUNIT_ASSERT_EQUAL( (int64_t) 666LL, ptrGain->GetValue() );
    GainAutoFeature.Gain = 777;
    // since GainAuto is not read this goes unnotices
    GainAutoFeature.GainAutoReg = Off;
    // value comes form cache
    CPPUNIT_ASSERT_EQUAL( (int64_t) 666LL, ptrGain->GetValue() );
    GainAutoFeature.Gain = 888;
    // value comes form cache
    CPPUNIT_ASSERT_EQUAL( (int64_t) 666LL, ptrGain->GetValue() ); // from cache

    /*============ Test auto-reset, read before poll ========== */

    // start one cycle of auto-gain
    *ptrGain = 123;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 123LL, ptrGain->GetValue() );
    GainAutoCallback.Reset();
    GainCallback.Reset();
    *ptrGainAuto = "Once";
    CPPUNIT_ASSERT_EQUAL( Once, GainAutoFeature.GainAutoReg );
    CPPUNIT_ASSERT( GainAutoCallback.HasFiredOnce() );
    CPPUNIT_ASSERT( GainCallback.HasFiredOnce() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 123LL, ptrGain->GetValue() );

    // the camera signals we're ready, get to know that through reading (before any poll)
    GainAutoFeature.Gain = 234;
    GainAutoFeature.GainAutoReg = Off;
    CPPUNIT_ASSERT_EQUAL( gcstring("Off"), ptrGainAuto->ToString() );

    // polling does not fire any more
    Camera._Poll(1000);
    CPPUNIT_ASSERT( GainAutoCallback.HasNotFired() );
    CPPUNIT_ASSERT( GainCallback.HasNotFired() );

    // the callbacks do not come, but Gain value is still invalidated
    CPPUNIT_ASSERT_EQUAL( (int64_t) 234LL, ptrGain->GetValue() );

}

void EnumerationTestSuite::TestGetEntry()
{

    {
        // create and initilize node map
        gcstring CameraDescriptionFile =
            CDFHeader +
            "                                        \n"
            "<Enumeration Name=\"Enum\">            \n"
            "    <EnumEntry Name=\"EnumValue1\">     \n"
            "        <Value>10</Value>                \n"
            "   </EnumEntry>                        \n"
            "    <EnumEntry Name=\"EnumValue2\">     \n"
            "        <Value>20</Value>                \n"
            "   </EnumEntry>                        \n"
            "    <pValue>Value</pValue>                \n"
            "</Enumeration>                            \n"
            "                                        \n"
            "<Integer Name=\"Value\">                \n"
            "    <Value>10</Value>                    \n"
            "</Integer>                                \n"
            "                                        \n"
            + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
        CPPUNIT_ASSERT( ptrEnum.IsValid() );

        CEnumEntryPtr ptrEnumEntry = ptrEnum->GetEntry(20);
        CPPUNIT_ASSERT_EQUAL( (int64_t) 20LL, ptrEnumEntry->GetValue() );
    }
    {
        using namespace Test_TestCamera_v1_0;
        CTestCamera_v1_0 Camera;
        Camera._LoadDLL();

        Camera.Enum = Enum_EnumValue2;
        CPPUNIT_ASSERT_EQUAL( gcstring("EnumValue1"), Camera.Enum.GetEntry(Enum_EnumValue1)->GetSymbolic() );

        ////CPPUNIT_ASSERT_NO_THROW( Camera.Enum.GetEntry((Test_TestCamera_v1_0::EnumEnums)117));
        ////CPPUNIT_ASSERT_EQUAL( (IEnumEntry*)NULL, Camera.Enum.GetEntry((Test_TestCamera_v1_0::EnumEnums)117));
    }

}

void EnumerationTestSuite::TestAccessMode()
{

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                       \n"
        "<Enumeration Name=\"Enum\">                            \n"
        "    <EnumEntry Name=\"EnumEntry1\">                    \n"
        "        <pIsImplemented>Toggle_I</pIsImplemented>      \n"
        "        <pIsAvailable>Toggle_A</pIsAvailable>          \n"
        "        <Value>10</Value>                              \n"
        "    </EnumEntry>                                       \n"
        "    <EnumEntry Name=\"EnumEntry2\">                    \n"
        "        <pIsImplemented>Toggle_I</pIsImplemented>      \n"
        "        <pIsAvailable>Toggle_A</pIsAvailable>          \n"
        "        <Value>20</Value>                              \n"
        "    </EnumEntry>                                       \n"
        "    <Value>10</Value>                                  \n"
        "</Enumeration>                                         \n"
        "                                                       \n"
        "<Integer Name=\"Toggle_A\">                            \n"
        "   <Value>1</Value>                                    \n"
        "</Integer>                                             \n"
        "                                                       \n"
        "<Integer Name=\"Toggle_I\">                            \n"
        "   <Value>1</Value>                                    \n"
        "</Integer>                                             \n"
        "                                                       \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Fetch node pointers
    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CPPUNIT_ASSERT(ptrEnum.IsValid());
    CEnumEntryPtr ptrEnumEntry1 = Camera._GetNode("EnumEntry_Enum_EnumEntry1");
    CPPUNIT_ASSERT(ptrEnumEntry1.IsValid());
    CEnumEntryPtr ptrEnumEntry2 = Camera._GetNode("EnumEntry_Enum_EnumEntry2");
    CPPUNIT_ASSERT(ptrEnumEntry2.IsValid());
    CIntegerPtr ptrToggleAvailabilty = Camera._GetNode( "Toggle_A" );
    CPPUNIT_ASSERT( ptrToggleAvailabilty.IsValid() );
    CIntegerPtr ptrToggleImplementationAvailabilty = Camera._GetNode( "Toggle_I" );
    CPPUNIT_ASSERT( ptrToggleImplementationAvailabilty.IsValid() );

    // What happens if both EnumEntries become not available?
    CPPUNIT_ASSERT_EQUAL( RW, ptrEnum->GetAccessMode() );
    ptrToggleAvailabilty->SetValue(0);
    CPPUNIT_ASSERT_EQUAL( NA, ptrEnum->GetAccessMode() );
    // Repeat the same test to see if correct access mode was cached
    CPPUNIT_ASSERT_EQUAL( NA, ptrEnum->GetAccessMode() );
    CPPUNIT_ASSERT_THROW( ptrEnum->GetIntValue(), GenICam::GenericException );
    ptrToggleAvailabilty->SetValue(1);
    CPPUNIT_ASSERT_EQUAL( RW, ptrEnum->GetAccessMode() );

    // What happens if both EnumEntries become not implemented?
    CPPUNIT_ASSERT_EQUAL( RW, ptrEnum->GetAccessMode() );
    ptrToggleImplementationAvailabilty->SetValue(0);
    CPPUNIT_ASSERT_EQUAL( NI, ptrEnum->GetAccessMode() );
    ptrToggleImplementationAvailabilty->SetValue(1);
    CPPUNIT_ASSERT_EQUAL( RW, ptrEnum->GetAccessMode() );

    // Same questions but triggered using ImposeAccessMode
    CPPUNIT_ASSERT_EQUAL( RW, ptrEnum->GetAccessMode() );
    ptrEnum->GetNode()->ImposeAccessMode(RO);
    CPPUNIT_ASSERT_EQUAL( RO, ptrEnum->GetAccessMode() );
    ptrEnumEntry1->GetNode()->ImposeAccessMode(NA);
    CPPUNIT_ASSERT_EQUAL( RO, ptrEnum->GetAccessMode() );
    ptrEnumEntry2->GetNode()->ImposeAccessMode(NA);
    CPPUNIT_ASSERT_EQUAL( NA, ptrEnum->GetAccessMode() );
    ptrEnumEntry1->GetNode()->ImposeAccessMode(NI);
    CPPUNIT_ASSERT_EQUAL( NA, ptrEnum->GetAccessMode() );
    ptrEnumEntry2->GetNode()->ImposeAccessMode(NI);
    CPPUNIT_ASSERT_EQUAL( NI, ptrEnum->GetAccessMode() );

}

#pragma pack(push, 1)
typedef struct _EnumEntryImplemented_t
{                                                        // addr:len
    uint32_t EnumEntry1Implemented;   // 0:4
    uint32_t EnumEntry2Implemented;   // 4:4
}  EnumEntryImplemented_t;             // 8
#pragma pack(pop)


void EnumerationTestSuite::TestAccessMode_issue_711()
{

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                                                \n"
        "<Enumeration Name=\"Enum\">                                     \n"
        "    <pIsImplemented>EnumImplemented</pIsImplemented>            \n"
        "    <pIsAvailable>EnumAvailable</pIsAvailable>                  \n"
        "    <EnumEntry Name=\"EnumValue1\">                             \n"
        "        <pIsImplemented>EnumEntry1Implemented</pIsImplemented>  \n"
        "        <Value>10</Value>                                       \n"
        "   </EnumEntry>                                                 \n"
        "    <EnumEntry Name=\"EnumValue2\">                             \n"
        "        <pIsImplemented>EnumEntry2Implemented</pIsImplemented>  \n"
        "        <Value>20</Value>                                       \n"
        "   </EnumEntry>                                                 \n"
        "    <pValue>Value</pValue>                                      \n"
        "</Enumeration>                                                  \n"
        "                                                                \n"
        "<Integer Name=\"Value\">                                        \n"
        "    <Value>10</Value>                                           \n"
        "</Integer>                                                      \n"
        "                                                                \n"
        "<Integer Name=\"EnumImplemented\">                              \n"
        "    <Value>1</Value>                                            \n"
        "</Integer>                                                      \n"
        "                                                                \n"
        "<Integer Name=\"EnumAvailable\">                                \n"
        "    <Value>1</Value>                                            \n"
        "</Integer>                                                      \n"
        "                                                                \n"
        "<IntReg Name=\"EnumEntry1Implemented\">                         \n"
        "    <Address>0</Address>                                        \n"
        "    <Length>4</Length>                                          \n"
        "    <AccessMode>RW</AccessMode>                                 \n"
        "    <pPort>Port</pPort>                                         \n"
        "    <Sign>Unsigned</Sign>                                       \n"
        "    <Endianess>LittleEndian</Endianess>                         \n"
        "</IntReg>                                                       \n"
        "                                                                \n"
        "<IntReg Name=\"EnumEntry2Implemented\">                         \n"
        "    <Address>4</Address>                                        \n"
        "    <Length>4</Length>                                          \n"
        "    <AccessMode>RW</AccessMode>                                 \n"
        "    <pPort>Port</pPort>                                         \n"
        "    <Sign>Unsigned</Sign>                                       \n"
        "    <Endianess>LittleEndian</Endianess>                         \n"
        "</IntReg>                                                       \n"
        "                                                                \n"
        "<Port Name=\"Port\"/>                                           \n"
        "                                                                \n"
        + CDFFooter;

    CTestPortStruct<EnumEntryImplemented_t> EnumEntryImplemented;
    EnumEntryImplemented.EnumEntry1Implemented = 0;
    EnumEntryImplemented.EnumEntry2Implemented = 0;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    Camera._Connect(&EnumEntryImplemented, "Port");

    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CPPUNIT_ASSERT( ptrEnum.IsValid() );

    CIntegerPtr ptrEnumImplemented = Camera._GetNode("EnumImplemented");
    CPPUNIT_ASSERT( ptrEnumImplemented.IsValid() );

    CIntegerPtr ptrEnumAvailable = Camera._GetNode("EnumAvailable");
    CPPUNIT_ASSERT( ptrEnumAvailable.IsValid() );

    CNodePtr ptrPort = Camera._GetNode("Port");
    CPPUNIT_ASSERT( ptrPort.IsValid() );

    // Test IsImplemented for EnumImplemented

    *ptrEnumImplemented = 1;
    *ptrEnumAvailable = 1;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsImplemented(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, EnumEntryImplemented.GetNumReads() );

    *ptrEnumImplemented = 0;
    *ptrEnumAvailable = 1;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsImplemented(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, EnumEntryImplemented.GetNumReads() );

    // Test IsImplemented for EnumAvailable

    *ptrEnumImplemented = 1;
    *ptrEnumAvailable = 1;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsImplemented(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, EnumEntryImplemented.GetNumReads() );

    *ptrEnumImplemented = 1;
    *ptrEnumAvailable = 0;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsImplemented(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, EnumEntryImplemented.GetNumReads() );

    // Test IsAvailable for EnumImplemented

    *ptrEnumImplemented = 1;
    *ptrEnumAvailable = 1;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsAvailable(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, EnumEntryImplemented.GetNumReads() );

    *ptrEnumImplemented = 0;
    *ptrEnumAvailable = 1;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsAvailable(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, EnumEntryImplemented.GetNumReads() );

    // Test IsAvailable for EnumAvailable

    *ptrEnumImplemented = 1;
    *ptrEnumAvailable = 1;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsAvailable(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, EnumEntryImplemented.GetNumReads() );

    *ptrEnumImplemented = 1;
    *ptrEnumAvailable = 0;
    ptrPort->InvalidateNode();
    EnumEntryImplemented.ResetStatistics();
    CPPUNIT_ASSERT( !IsAvailable(ptrEnum) );
    cout << "EnumImplemented = " << ptrEnumImplemented->GetValue() << endl;
    cout << "EnumAvailable = " << ptrEnumAvailable->GetValue() << endl;
    cout << "Number of reads in EnumEntry1/2Implemented= " << EnumEntryImplemented.GetNumReads() << endl;
    cout << endl;
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, EnumEntryImplemented.GetNumReads() );

}

void EnumerationTestSuite::TestTicket778()
{

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                           \n"
        "<Enumeration Name=\"EnumA\">               \n"
        "    <EnumEntry Name=\"EnumValue1\">        \n"
        "        <pIsAvailable>AvailableA</pIsAvailable>\n"
        "        <Value>10</Value>                  \n"
        "    </EnumEntry>                           \n"
        "    <EnumEntry Name=\"EnumValue2\">        \n"
        "        <Value>20</Value>                  \n"
        "    </EnumEntry>                           \n"
        "    <Value>10</Value>                      \n"
        "</Enumeration>                             \n"
        "                                           \n"
        "<Enumeration Name=\"EnumB\">               \n"
        "    <EnumEntry Name=\"EnumValue1\">        \n"
        "        <pIsAvailable>AvailableB</pIsAvailable>\n"
        "        <Value>10</Value>                  \n"
        "    </EnumEntry>                           \n"
        "    <EnumEntry Name=\"EnumValue2\">        \n"
        "        <Value>20</Value>                  \n"
        "    </EnumEntry>                           \n"
        "    <Value>10</Value>                      \n"
        "</Enumeration>                             \n"
        "                                           \n"
        "<IntReg Name=\"AvailableA\">               \n"
        "    <Address>0</Address>                   \n"
        "    <Length>4</Length>                     \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Cachable>NoCache</Cachable>           \n"
        "    <Sign>Signed</Sign>                    \n"
        "    <Endianess>LittleEndian</Endianess>    \n"
        "</IntReg>                                  \n"
        "                                           \n"
        "<IntReg Name=\"AvailableB\">               \n"
        "    <Address>0</Address>                   \n"
        "    <Length>4</Length>                     \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Cachable>WriteThrough</Cachable>      \n"
        "    <Sign>Signed</Sign>                    \n"
        "    <Endianess>LittleEndian</Endianess>    \n"
        "</IntReg>                                  \n"
        "                                           \n"
        "<Port Name=\"Port\"/>                      \n"
        "                                           \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    NODE_POINTER( Enumeration, EnumA );
    NODE_POINTER( EnumEntry, EnumEntry_EnumA_EnumValue1 );
    NODE_POINTER( EnumEntry, EnumEntry_EnumA_EnumValue2 );
    NODE_POINTER( Integer, AvailableA );

    NODE_POINTER( Enumeration, EnumB );
    NODE_POINTER( EnumEntry, EnumEntry_EnumB_EnumValue1 );
    NODE_POINTER( EnumEntry, EnumEntry_EnumB_EnumValue2 );
    NODE_POINTER( Integer, AvailableB );

    CPPUNIT_ASSERT_EQUAL( NoCache, ptrAvailableA->GetNode()->GetCachingMode() );
    CPPUNIT_ASSERT_EQUAL( WriteThrough, ptrAvailableB->GetNode()->GetCachingMode() );

    CPPUNIT_ASSERT_EQUAL(  No, ptrEnumEntry_EnumA_EnumValue1->GetNode()->IsAccessModeCacheable());
    CPPUNIT_ASSERT_EQUAL( Yes, ptrEnumEntry_EnumB_EnumValue1->GetNode()->IsAccessModeCacheable() );

    CPPUNIT_ASSERT_EQUAL( Yes, ptrEnumEntry_EnumA_EnumValue2->GetNode()->IsAccessModeCacheable() );
    CPPUNIT_ASSERT_EQUAL( Yes, ptrEnumEntry_EnumB_EnumValue2->GetNode()->IsAccessModeCacheable() );

    CPPUNIT_ASSERT_EQUAL(  No, ptrEnumA->GetNode()->IsAccessModeCacheable() );
    CPPUNIT_ASSERT_EQUAL( Yes, ptrEnumB->GetNode()->IsAccessModeCacheable() );

}

