//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/


#ifndef __ENUMERATIONTEST_H
#define __ENUMERATIONTEST_H

#include "GenApiTest.h"

/**
\brief Tests Enumeration
*/
class EnumerationTestSuite : public CPPUNIT_NS::TestCaseEx
{
    CPPUNIT_TEST_SUITE( EnumerationTestSuite );
        CPPUNITEX_TEST( TestValueAccess );
        CPPUNITEX_TEST( TestValueAccessStatic );
        CPPUNITEX_TEST( TestRegValueAccessRW );
        CPPUNITEX_TEST( TestEnumEntry );
        CPPUNITEX_TEST( TestEnumFalseEntry );
        CPPUNITEX_TEST( TestEnumRef );
        CPPUNITEX_TEST( TestInvalidValues );
        CPPUNITEX_TEST( TestDisplayName );
        CPPUNITEX_TEST( TestNumericValue );
        CPPUNITEX_TEST( TestAutoGain );
        CPPUNITEX_TEST( TestGetEntry );
        CPPUNITEX_TEST( TestAccessMode );
        CPPUNITEX_TEST( TestAccessMode_issue_711 );
        CPPUNITEX_TEST( TestTicket778 );
    CPPUNIT_TEST_SUITE_END();

public:
    //! Tests the basic get and set methods
    void TestValueAccess();

    //! Tests the basic  static get and set methods
    void TestValueAccessStatic();

    //! Tests the AccessMode
    void TestRegValueAccessRW();

    //! Tests the EnumEntries
    void TestEnumEntry();

    //! Tests the false cases of EnumEntries
    void TestEnumFalseEntry();

    //! Tests the Emum References
    void TestEnumRef();

    //! Tests if invalid values are rejected properly
    void TestInvalidValues();

    //! Tests enum entry display name
    void TestDisplayName();

    //! Tests the <NumericValue> entry
    void TestNumericValue();

    //! Tests selfclearing flags and switching cache of
    void TestAutoGain();

    //! test the GetEntry(int64_t) method
    void TestGetEntry();

    //! test the special AccessMode treatment
    void TestAccessMode();

    //! make sure that for not implemented Enumerations the EnumEntries ar not asked for their access mode
    /*! For details see #711 */
    void TestAccessMode_issue_711();

    //! Makes sure the cache mode of Enumeration takes into account the cache mode of the enumentries
    void TestTicket778();
};

#define __ENUMERATIONTEST_H
#endif //
