//-----------------------------------------------------------------------------
//  (c) 2008 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of CFloat
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "ErrorTest.h"
#include "GenApi/GenApi.h"

using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ErrorTestSuite, "GenApiTest");

void ErrorTestSuite::TestBasics()
{
    // added <pError> element
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescription =
    CDFHeader +
    "                                         \n"
    "<Integer Name=\"Value\">                 \n"
    "    <pError>Error</pError>               \n"
    "    <Value>42</Value>                    \n"
    "</Integer>                               \n"
    "                                         \n"
    "<Enumeration Name=\"Error\">             \n"
    "    <EnumEntry Name=\"NoError\">         \n"
    "        <Value>0</Value>                 \n"
    "   </EnumEntry>                          \n"
    "    <EnumEntry Name=\"Ahrrgs\">          \n"
    "        <ToolTip>A really awfull error has happend</ToolTip>\n"
    "        <Value>-1</Value>                \n"
    "   </EnumEntry>                          \n"
    "    <Value>0</Value>                     \n"
    "</Enumeration>                           \n"
    "                                         \n"
    "<Enumeration Name=\"EnumValue\">             \n"
    "    <pError>Error</pError>               \n"
    "    <EnumEntry Name=\"Value0\">         \n"
    "        <Value>0</Value>                 \n"
    "   </EnumEntry>                          \n"
    "    <EnumEntry Name=\"Value1\">          \n"
    "        <Value>1</Value>                \n"
    "   </EnumEntry>                          \n"
    "    <Value>0</Value>                     \n"
    "</Enumeration>                           \n"
    "                                         \n"

    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CIntegerPtr ptrValue = Camera._GetNode( "Value" );
    CPPUNIT_ASSERT( (bool) ptrValue );

    CEnumerationPtr ptrError = Camera._GetNode( "Error" );
    CPPUNIT_ASSERT( (bool) ptrError );

    CEnumerationPtr ptrEnumValue = Camera._GetNode( "EnumValue" );
    CPPUNIT_ASSERT( (bool) ptrEnumValue );

    // No error
    *ptrValue = 17;

    // error
    *ptrError = "Ahrrgs";
    CPPUNIT_ASSERT_THROW_EX( ptrValue->SetValue(17), RuntimeException );
    CPPUNIT_ASSERT_THROW_EX( ptrEnumValue->SetIntValue(1), RuntimeException );

    // error suppressed
    ptrValue->SetValue(17, false);
    ptrEnumValue->SetIntValue(1, false);
}
