//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "ExceptionTest.h"
#include "GenApi/GenApi.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( ExceptionTestSuite, "GenApiTest");




void ExceptionTestSuite::TestException()
{
    const char pcDesc[] = "ErrorDEscription";
    const char pcSourceFileName[] = ".\\MyFile.cpp";
    const char pcExceptionType[] = "ExceptionType";
    const char pcNodeName[] = "NodeName";
    const char pcEntryPoint[] = "EntryPoint";
    unsigned int nSourceLine = 9;

    {
    // using strcmp for string comparison as we don't know whether gcstring::operator== is correct
    GenericException MyGenericException(pcDesc, pcSourceFileName, nSourceLine);
        
    CPPUNIT_ASSERT(strcmp(MyGenericException.GetDescription(), pcDesc) == 0);
    CPPUNIT_ASSERT(strcmp(MyGenericException.GetSourceFileName(), pcSourceFileName) == 0);
    CPPUNIT_ASSERT_EQUAL(MyGenericException.GetSourceLine(), nSourceLine);
    }


    // test c'tors of derived exception
    {
    RuntimeException MyRuntimeException(pcDesc, pcSourceFileName, nSourceLine);

    CPPUNIT_ASSERT(strcmp(MyRuntimeException.GetDescription(), pcDesc) == 0);
    CPPUNIT_ASSERT(strcmp(MyRuntimeException.GetSourceFileName(), pcSourceFileName) == 0);
    CPPUNIT_ASSERT_EQUAL(MyRuntimeException.GetSourceLine(), nSourceLine);
    }

    {
    RuntimeException MyRuntimeException(pcDesc, pcSourceFileName, nSourceLine, pcExceptionType);

    CPPUNIT_ASSERT(strcmp(MyRuntimeException.GetDescription(), pcDesc) == 0);
    CPPUNIT_ASSERT(strcmp(MyRuntimeException.GetSourceFileName(), pcSourceFileName) == 0);
    CPPUNIT_ASSERT_EQUAL(MyRuntimeException.GetSourceLine(), nSourceLine);
    // there is no getter for exceptiontype ???
    // but we still want to test c'tor
    }


    {
    RuntimeException MyRuntimeException(pcDesc, pcSourceFileName, nSourceLine, pcEntryPoint, pcNodeName, pcExceptionType);

    CPPUNIT_ASSERT(strcmp(MyRuntimeException.GetDescription(), pcDesc) == 0);
    CPPUNIT_ASSERT(strcmp(MyRuntimeException.GetSourceFileName(), pcSourceFileName) == 0);
    CPPUNIT_ASSERT_EQUAL(MyRuntimeException.GetSourceLine(), nSourceLine);
    // there is no getter for exceptiontype, nodename and entrypoint???
    // but we still want to test c'tor
    }


    // test exception reporting using stringstream
    stringstream sstr;
    sstr << "x" << "y";
    gcstring ExceptionDesc;
    try
    {
        throw RUNTIME_EXCEPTION( sstr ) ;
    }
    catch(GenericException &e)
    {
        ExceptionDesc = e.GetDescription();
    }
    CPPUNIT_ASSERT_EQUAL(ExceptionDesc, (gcstring)"xy");

    // test exception reporting using Report()
    try
    {
        std::string str("ab");
        throw GenICam::ExceptionReporter<GenICam::RuntimeException>(__FILE__, __LINE__).Report (str);
    }
    catch(GenericException &e)
    {
        ExceptionDesc = e.GetDescription();
    }
    CPPUNIT_ASSERT_EQUAL((gcstring)"ab", ExceptionDesc);
}

void ExceptionTestSuite::TestEntryPoint()
{
    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<Integer Name=\"Value\">                \n"
        "    <pValue>DeepValue</pValue>          \n"
        "    <Min>0</Min>                        \n"
        "    <Max>100</Max>                      \n"
        "</Integer>                              \n"
        "                                        \n"
        "<Integer Name=\"DeepValue\">            \n"
        "    <Value>0</Value>                    \n"
        "    <Min>0</Min>                        \n"
        "    <Max>10</Max>                       \n"
        "</Integer>                              \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    NODE_POINTER( Integer, Value );

    // 
    CPPUNIT_ASSERT_THROW_EX( ptrValue->SetValue( 110 ), GenICam::OutOfRangeException );
    CPPUNIT_ASSERT_THROW_EX( ptrValue->SetValue( 11 ), GenICam::OutOfRangeException );

}






