//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "TestPort.h"
#include "FilestreamTest.h"
#include "GenApi/GenApi.h"
#include "GenApi/Filestream.h"
#include <sstream>
#include <limits.h>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( FilestreamTestSuite, "GenApiTest");

template <int BufferSize, int DefaultValue>
class CRawTestPort : public CTestPort{
public:
    CRawTestPort()
  {
        memset(m_Buffer,DefaultValue,BufferSize);
    };
    //! Reads a chunk of bytes from the port
    virtual void Read(void *pBuffer, int64_t Address, int64_t Length)
  {
        if ( Address>=0 && Address<BufferSize )
    {
      // *JS* removed warning
      size_t cnt = static_cast<size_t>(min(BufferSize - Address, Length));
      memcpy(pBuffer, m_Buffer + Address, cnt);
        }
    else
    {
            CTestPort::Read(pBuffer, Address, Length);
        }
    }
    virtual void Write(const void *pBuffer, int64_t Address, int64_t Length)
  {
        if ( Address>=0 && Address<BufferSize )
    {
      // *JS* removed warning
      size_t cnt = static_cast<size_t>(min(BufferSize - Address, Length));
            memcpy(m_Buffer + Address, pBuffer, cnt);
        }
    else
    {
            CTestPort::Write(pBuffer, Address, Length);
        }
    }

    char* GetBuffer(){return reinterpret_cast<char*>(&m_Buffer);};
private:
    char m_Buffer[BufferSize];
};


void FilestreamTestSuite::TestFileStreamNotExist()
{
    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        ""
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // test via attach
    CPPUNIT_ASSERT_EQUAL(false,GenICam::FileProtocolAdapter().attach(Camera._Ptr));

    ODevFileStream usersetWrite;
    usersetWrite.open(Camera._Ptr, "UserSet1");

    CPPUNIT_ASSERT(usersetWrite.fail());

    ODevFileStream usersetWrite1(Camera._Ptr, "UserSet1");

    CPPUNIT_ASSERT(usersetWrite1.fail());

    usersetWrite.close();

    IDevFileStream usersetRead;
    usersetRead.open(Camera._Ptr, "UserSet1");

    CPPUNIT_ASSERT(usersetRead.fail());

    usersetRead.close();

    IDevFileStream usersetRead1(Camera._Ptr, "UserSet1");

    CPPUNIT_ASSERT(usersetRead1.fail());

    usersetRead1.close();


}

void FilestreamTestSuite::TestFileStreamLUTAccessBufferSmaller()
{
    static const size_t LUT_SIZE = 256;

    // create and initilize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "<Category Name=\"Root\" NameSpace=\"Standard\">\n"
        "       <pFeature>GenericFileAccess</pFeature>\n"
        "   </Category>\n"
        "   <Category Name=\"GenericFileAccess\">\n"
        "       <pFeature>FileSelector</pFeature>\n"
        "       <pFeature>FileOperationSelector</pFeature>\n"
        "       <pFeature>FileOperationExecute</pFeature>\n"
        "       <pFeature>FileOpenMode</pFeature>\n"
        "       <pFeature>FileAccessOffset</pFeature>\n"
        "       <pFeature>FileAccessLength</pFeature>\n"
        "       <pFeature>FileAccessBuffer</pFeature>\n"
        "       <pFeature>FileOperationStatus</pFeature>\n"
        "       <pFeature>FileOperationResult</pFeature>\n"
        "   </Category>\n"
        "   <Enumeration Name=\"FileSelector\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"UserSetDefault\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet1\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet2\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet3\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>3</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"LUTRed\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>4</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Enumeration Name=\"FileOperationSelector\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"Open\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Close\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Read\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Write\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>3</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Enumeration Name=\"FileOpenMode\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"Read\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Write\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"ReadWrite\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Command Name=\"FileOperationExecute\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <CommandValue>1</CommandValue>\n"
        "   </Command>\n"
        "   <Integer Name=\"FileAccessOffset\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <Min>0</Min>\n"
        "       <Max>511</Max>\n"
        "   </Integer>\n"
        "   <Integer Name=\"FileAccessLength\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <Min>0</Min>\n"
        "       <Max>16</Max>\n"
        "   </Integer>\n"
        "   <Register Name=\"FileAccessBuffer\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Address>0x0</Address>\n"
        "       <pAddress>FileAccessOffset</pAddress>\n"
        "       <Length>16</Length>\n"
        "       <AccessMode>RW</AccessMode>\n"
        "       <pPort>MyPort</pPort>\n"
        "       <Cachable>NoCache</Cachable>\n"
        "   </Register>\n"
        "   <Integer Name=\"FileOperationResult\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <pValue>FileAccessLength</pValue>\n"
        "       <Min>0</Min>\n"
        "   </Integer>\n"
        "   <Enumeration Name=\"FileOperationStatus\" NameSpace=\"Standard\">\n"
        "       <Visibility>Invisible</Visibility>\n"
        "       <EnumEntry Name=\"Success\">\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Failed\">\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Port Name=\"MyPort\"/>        \n"
        + CDFFooter;


    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    // lutBuffer contains the default lut ramp
    uint16_t lutBuffer[LUT_SIZE];
    typedef CRawTestPort<4096,0xaa> port_t;

    for ( uint16_t i = 0; i < LUT_SIZE; i ++){
        lutBuffer[static_cast<uint16_t>(i)]  = i;
    }

    auto_ptr<port_t> pPort(new port_t);
    uint16_t * pPortBuffer = reinterpret_cast<uint16_t *>(pPort->GetBuffer());

    for ( size_t i = 0 ; i < LUT_SIZE; i++){
        pPortBuffer[i] = static_cast<uint16_t>(i & USHRT_MAX);
    }

    // connect the node map to the port
    Camera._Connect(pPort.get(), "MyPort");

    //
    // Test full read
    //
    {
    IDevFileStream lutRead;
    CPPUNIT_ASSERT_EQUAL (false, lutRead.is_open());

    // test not existing file
    CPPUNIT_ASSERT_THROW_EX(lutRead.open(Camera._Ptr, "file_does_not_exist"),GenICam::InvalidArgumentException);


    lutRead.open(Camera._Ptr, "LUTRed");
    // open is successful
    CPPUNIT_ASSERT_EQUAL (true, lutRead.is_open());
    CPPUNIT_ASSERT(!lutRead.fail());

    uint16_t testBuffer[LUT_SIZE];
    lutRead.read(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer));

    CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,LUT_SIZE));

    lutRead.close();
    }

    //
    // Test read less
    //
    {
        static const size_t CHECK_DELTA( 13 );

        IDevFileStream lutRead;
        lutRead.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutRead.fail());

        uint16_t testBuffer[LUT_SIZE];
        memset(testBuffer,0x42,LUT_SIZE*sizeof(uint16_t));

        lutRead.read(reinterpret_cast<char*>(&testBuffer), sizeof(testBuffer) - CHECK_DELTA);
        CPPUNIT_ASSERT(!lutRead.fail());
        size_t result(lutRead.gcount());

        CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,sizeof(testBuffer) - CHECK_DELTA));

        // check that exactly LUTSIZE-CHECK_DELTA Bytes are read
        for ( size_t i = LUT_SIZE*sizeof(uint16_t) - CHECK_DELTA; i < LUT_SIZE*sizeof(uint16_t); i++){
            CPPUNIT_ASSERT_EQUAL(static_cast<const char>(0x42), *(reinterpret_cast<const char*>(testBuffer) + i));
        }
        CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,LUT_SIZE - CHECK_DELTA));
        CPPUNIT_ASSERT_EQUAL(LUT_SIZE*sizeof(uint16_t) - CHECK_DELTA,static_cast<const size_t>(result));

        lutRead.close();
    }

    //
    // Test read more
    //
    {
        static const size_t CHECK_DELTA( 13 );

        IDevFileStream lutRead;
        lutRead.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutRead.fail());

        uint16_t testBuffer[LUT_SIZE];
        memset(testBuffer,0x42,LUT_SIZE*sizeof(uint16_t));

        lutRead.read(reinterpret_cast<char*>(&testBuffer), sizeof(testBuffer) + CHECK_DELTA);
        CPPUNIT_ASSERT(lutRead.fail());
        size_t result(lutRead.gcount());

        CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,LUT_SIZE));
        CPPUNIT_ASSERT_EQUAL(LUT_SIZE*sizeof(uint16_t),static_cast<const size_t>(result));

        lutRead.close();
    }


    //
    // Test write full
    //
    {
    ODevFileStream lutWrite;
    CPPUNIT_ASSERT_EQUAL (false, lutWrite.is_open());

    // test not existing file
    CPPUNIT_ASSERT_THROW_EX(lutWrite.open(Camera._Ptr, "file_does_not_exist"),GenICam::InvalidArgumentException);

    lutWrite.open(Camera._Ptr, "LUTRed");
    // open is successful
    CPPUNIT_ASSERT_EQUAL (true, lutWrite.is_open());
    CPPUNIT_ASSERT(!lutWrite.fail());
        uint16_t testBuffer[LUT_SIZE];
    // init the testbuffer with new LUT data
    for ( size_t i = 0; i < LUT_SIZE; i ++){
        testBuffer[i]  = 0x5555;
    }
    lutWrite.write(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer));
    lutWrite.close();

    CPPUNIT_ASSERT(0 == memcmp(pPort->GetBuffer(),testBuffer,LUT_SIZE*sizeof(uint16_t)));
    }

    //
    // Test write less
    //
    {
        static const size_t CHECK_DELTA( 13 );
        ODevFileStream lutWrite;
        lutWrite.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutWrite.fail());
        uint16_t testBuffer[LUT_SIZE];
        // init the testbuffer with new LUT data
        for ( size_t i = 0; i < LUT_SIZE; i ++){
            testBuffer[i]  = 0x5555;
}

        lutWrite.write(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer)-CHECK_DELTA);
        CPPUNIT_ASSERT(!lutWrite.fail());

        lutWrite.close();
        CPPUNIT_ASSERT(!lutWrite.fail());

        // check
        CPPUNIT_ASSERT(0 == memcmp(pPort->GetBuffer(),testBuffer,LUT_SIZE*sizeof(uint16_t)-CHECK_DELTA));
    }

    //
    // Test write more but less than buffer size
    //
    {
        static const size_t CHECK_DELTA( 13 );
        ODevFileStream lutWrite;
        lutWrite.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutWrite.fail());
        uint16_t testBuffer[LUT_SIZE];
        // init the testbuffer with new LUT data
        for ( size_t i = 0; i < LUT_SIZE; i ++){
            testBuffer[i]  = 0x5555;
        }
        lutWrite.write(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer) + CHECK_DELTA);
        CPPUNIT_ASSERT(!lutWrite.fail());

        lutWrite.close();
        CPPUNIT_ASSERT(lutWrite.fail());

        // check
        CPPUNIT_ASSERT(0 == memcmp(pPort->GetBuffer(),testBuffer,LUT_SIZE*sizeof(uint16_t)));
    }

}

void FilestreamTestSuite::TestFileStreamLUTAccessBufferEqual()
{
    static const size_t LUT_SIZE = 256;

    // create and initilize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "<Category Name=\"Root\" NameSpace=\"Standard\">\n"
        "       <pFeature>GenericFileAccess</pFeature>\n"
        "   </Category>\n"
        "   <Category Name=\"GenericFileAccess\">\n"
        "       <pFeature>FileSelector</pFeature>\n"
        "       <pFeature>FileOperationSelector</pFeature>\n"
        "       <pFeature>FileOperationExecute</pFeature>\n"
        "       <pFeature>FileOpenMode</pFeature>\n"
        "       <pFeature>FileAccessOffset</pFeature>\n"
        "       <pFeature>FileAccessLength</pFeature>\n"
        "       <pFeature>FileAccessBuffer</pFeature>\n"
        "       <pFeature>FileOperationStatus</pFeature>\n"
        "       <pFeature>FileOperationResult</pFeature>\n"
        "   </Category>\n"
        "   <Enumeration Name=\"FileSelector\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"UserSetDefault\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet1\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet2\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet3\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>3</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"LUTRed\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>4</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Enumeration Name=\"FileOperationSelector\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"Open\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Close\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Read\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Write\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>3</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Enumeration Name=\"FileOpenMode\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"Read\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Write\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"ReadWrite\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Command Name=\"FileOperationExecute\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <CommandValue>1</CommandValue>\n"
        "   </Command>\n"
        "   <Integer Name=\"FileAccessOffset\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <Min>0</Min>\n"
        "       <Max>0</Max>\n"
        "   </Integer>\n"
        "   <Integer Name=\"FileAccessLength\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <Min>0</Min>\n"
        "       <Max>512</Max>\n"
        "   </Integer>\n"
        "   <Register Name=\"FileAccessBuffer\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Address>0x0</Address>\n"
        "       <pAddress>FileAccessOffset</pAddress>\n"
        "       <Length>512</Length>\n"
        "       <AccessMode>RW</AccessMode>\n"
        "       <pPort>MyPort</pPort>\n"
        "       <Cachable>NoCache</Cachable>\n"
        "   </Register>\n"
        "   <Integer Name=\"FileOperationResult\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <pValue>FileAccessLength</pValue>\n"
        "       <Min>0</Min>\n"
        "   </Integer>\n"
        "   <Enumeration Name=\"FileOperationStatus\" NameSpace=\"Standard\">\n"
        "       <Visibility>Invisible</Visibility>\n"
        "       <EnumEntry Name=\"Success\">\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Failed\">\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Port Name=\"MyPort\"/>        \n"
        + CDFFooter;


    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    
    // lutBuffer contains the default lut ramp
    uint16_t lutBuffer[LUT_SIZE];
    typedef CRawTestPort<4096,0xaa> port_t;

    for ( uint16_t i = 0; i < LUT_SIZE; i ++){
        lutBuffer[static_cast<uint16_t>(i)]  = i;
    }

    auto_ptr<port_t> pPort(new port_t);
    uint16_t * pPortBuffer = reinterpret_cast<uint16_t *>(pPort->GetBuffer());

    for ( size_t i = 0 ; i < LUT_SIZE; i++){
        pPortBuffer[i] = static_cast<uint16_t>(i & USHRT_MAX);
    }

    // connect the node map to the port
    Camera._Connect(pPort.get(), "MyPort");

    //
    // Test full read
    //
    {
        IDevFileStream lutRead;
        lutRead.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutRead.fail());

        uint16_t testBuffer[LUT_SIZE];
        lutRead.read(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer));

        CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,LUT_SIZE));

        lutRead.close();
    }

    //
    // Test read less
    //
    {
        static const size_t CHECK_DELTA( 13 );

        IDevFileStream lutRead;
        lutRead.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutRead.fail());

        uint16_t testBuffer[LUT_SIZE];
        memset(testBuffer,0x42,LUT_SIZE*sizeof(uint16_t));

        lutRead.read(reinterpret_cast<char*>(&testBuffer), sizeof(testBuffer) - CHECK_DELTA);
        CPPUNIT_ASSERT(!lutRead.fail());
        size_t result(lutRead.gcount());

        CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,sizeof(testBuffer) - CHECK_DELTA));

        // check that exactly LUTSIZE-CHECK_DELTA Bytes are read
        for ( size_t i = LUT_SIZE*sizeof(uint16_t) - CHECK_DELTA; i < LUT_SIZE*sizeof(uint16_t); i++){
            CPPUNIT_ASSERT_EQUAL(static_cast<const char>(0x42), *(reinterpret_cast<const char*>(testBuffer) + i));
        }
        CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,LUT_SIZE - CHECK_DELTA));
        CPPUNIT_ASSERT_EQUAL(LUT_SIZE*sizeof(uint16_t) - CHECK_DELTA,static_cast<const size_t>(result));

        lutRead.close();
    }

    //
    // Test read more
    //
    {
        static const size_t CHECK_DELTA( 13 );

        IDevFileStream lutRead;
        lutRead.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutRead.fail());

        uint16_t testBuffer[LUT_SIZE];
        memset(testBuffer,0x42,LUT_SIZE*sizeof(uint16_t));

        lutRead.read(reinterpret_cast<char*>(&testBuffer), sizeof(testBuffer) + CHECK_DELTA);
        CPPUNIT_ASSERT(lutRead.fail());
        size_t result(lutRead.gcount());

        CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,LUT_SIZE));
        CPPUNIT_ASSERT_EQUAL(LUT_SIZE*sizeof(uint16_t),static_cast<const size_t>(result));

        lutRead.close();
    }


    //
    // Test write full
    //
    {
    ODevFileStream lutWrite;
    lutWrite.open(Camera._Ptr, "LUTRed");
    // open is successful
    CPPUNIT_ASSERT(!lutWrite.fail());
        uint16_t testBuffer[LUT_SIZE];
    // init the testbuffer with new LUT data
    for ( size_t i = 0; i < LUT_SIZE; i ++){
        testBuffer[i]  = 0x5555;
    }
    lutWrite.write(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer));
    lutWrite.close();

    CPPUNIT_ASSERT(0 == memcmp(pPort->GetBuffer(),testBuffer,LUT_SIZE*sizeof(uint16_t)));
    }

    //
    // Test write less
    //
    {
        static const size_t CHECK_DELTA( 13 );
        ODevFileStream lutWrite;
        lutWrite.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutWrite.fail());
        uint16_t testBuffer[LUT_SIZE];
        // init the testbuffer with new LUT data
        for ( size_t i = 0; i < LUT_SIZE; i ++){
            testBuffer[i]  = 0x5555;
        }

        lutWrite.write(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer)-CHECK_DELTA);
        CPPUNIT_ASSERT(!lutWrite.fail());

        lutWrite.close();
        CPPUNIT_ASSERT(!lutWrite.fail());

        // check
        CPPUNIT_ASSERT(0 == memcmp(pPort->GetBuffer(),testBuffer,LUT_SIZE*sizeof(uint16_t)-CHECK_DELTA));
    }

    //
    // Test write more but less than buffer size
    //
    {
        static const size_t CHECK_DELTA( 13 );
        ODevFileStream lutWrite;
        lutWrite.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutWrite.fail());
        uint16_t testBuffer[LUT_SIZE];
        // init the testbuffer with new LUT data
        for ( size_t i = 0; i < LUT_SIZE; i ++){
            testBuffer[i]  = 0x5555;
        }
        lutWrite.write(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer) + CHECK_DELTA);
        CPPUNIT_ASSERT(!lutWrite.fail());

        lutWrite.close();
        CPPUNIT_ASSERT(lutWrite.fail());

        // check
        CPPUNIT_ASSERT(0 == memcmp(pPort->GetBuffer(),testBuffer,LUT_SIZE*sizeof(uint16_t)));
    }

    //
    // Excercise some corner cases
    //
    {
    IDevFileStream lutRead;
    lutRead.open(Camera._Ptr, "LUTRed");
    // open is successful
    CPPUNIT_ASSERT(!lutRead.fail());

    uint16_t testBuffer[LUT_SIZE];
#if _MSC_VER   ////FAILS//// with gcc
    CPPUNIT_ASSERT_EQUAL (EOF, lutRead.rdbuf()->sungetc());
#endif

    lutRead.read(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer));

    // reads the same data back that was written in previous block
    CPPUNIT_ASSERT(0 == memcmp(pPort->GetBuffer(),testBuffer,sizeof(testBuffer)));

    lutRead.close();

    }
    {
        ODevFileStream lutWrite;
        lutWrite.open(Camera._Ptr, "LUTRed");
        // open is successful
        CPPUNIT_ASSERT(!lutWrite.fail());

        CPPUNIT_ASSERT_EQUAL (EOF, lutWrite.rdbuf()->sungetc());

        lutWrite.close();
    }


}

void FilestreamTestSuite::TestTicket799()
{
    static const size_t LUT_SIZE = 256;

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "<Category Name=\"Root\" NameSpace=\"Standard\">\n"
        "       <pFeature>GenericFileAccess</pFeature>\n"
        "   </Category>\n"
        "   <Category Name=\"GenericFileAccess\">\n"
        "       <pFeature>FileSelector</pFeature>\n"
        "       <pFeature>FileOperationSelector</pFeature>\n"
        "       <pFeature>FileOperationExecute</pFeature>\n"
        "       <pFeature>FileOpenMode</pFeature>\n"
        "       <pFeature>FileAccessOffset</pFeature>\n"
        "       <pFeature>FileAccessLength</pFeature>\n"
        "       <pFeature>FileAccessBuffer</pFeature>\n"
        "       <pFeature>FileOperationStatus</pFeature>\n"
        "       <pFeature>FileOperationResult</pFeature>\n"
        "   </Category>\n"
        "   <Enumeration Name=\"FileSelector\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"UserSetDefault\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet1\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet2\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"UserSet3\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>3</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"LUTRed\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>4</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Enumeration Name=\"FileOperationSelector\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"Open\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Close\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Read\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Write\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>3</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Enumeration Name=\"FileOpenMode\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <EnumEntry Name=\"Read\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Write\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"ReadWrite\">\n"
        "           <Visibility>Guru</Visibility>\n"
        "           <Value>2</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Command Name=\"FileOperationExecute\" NameSpace=\"Standard\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <CommandValue>1</CommandValue>\n"
        "   </Command>\n"
        "   <Integer Name=\"FileAccessOffset\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <Min>0</Min>\n"
        "       <Max>0</Max>\n"
        "   </Integer>\n"
        "   <Integer Name=\"FileAccessLength\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Value>0</Value>\n"
        "       <Min>0</Min>\n"
        "       <Max>512</Max>\n"
        "   </Integer>\n"
        "   <Register Name=\"FileAccessBuffer\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <Address>0x0</Address>\n"
        "       <pAddress>FileAccessOffset</pAddress>\n"
        "       <Length>512</Length>\n"
        "       <AccessMode>RW</AccessMode>\n"
        "       <pPort>MyPort</pPort>\n"
        "       <Cachable>NoCache</Cachable>\n"
        "   </Register>\n"
        "   <Integer Name=\"FileOperationResult\">\n"
        "       <Visibility>Guru</Visibility>\n"
        "       <pValue>FileAccessLength</pValue>\n"
        "       <Min>0</Min>\n"
        "   </Integer>\n"
        "   <Enumeration Name=\"FileOperationStatus\" NameSpace=\"Standard\">\n"
        "       <Visibility>Invisible</Visibility>\n"
        "       <EnumEntry Name=\"Success\">\n"
        "           <Value>0</Value>\n"
        "       </EnumEntry>\n"
        "       <EnumEntry Name=\"Failed\">\n"
        "           <Value>1</Value>\n"
        "       </EnumEntry>\n"
        "       <Value>0</Value>\n"
        "   </Enumeration>\n"
        "   <Port Name=\"MyPort\"/>        \n"
        + CDFFooter;


    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // lutBuffer contains the default lut ramp
    uint16_t lutBuffer[LUT_SIZE];
    typedef CRawTestPort<4096,0xaa> port_t;

    for ( uint16_t i = 0; i < LUT_SIZE; i ++){
        lutBuffer[static_cast<uint16_t>(i)]  = i;
    }

    auto_ptr<port_t> pPort(new port_t);
    uint16_t * pPortBuffer = reinterpret_cast<uint16_t *>(pPort->GetBuffer());

    for ( size_t i = 0 ; i < LUT_SIZE; i++){
        pPortBuffer[i] = static_cast<uint16_t>(i & USHRT_MAX);
    }

    // connect the node map to the port
    Camera._Connect(pPort.get(), "MyPort");

    IDevFileStream lutRead;
    {               
        gcstring FileName( "LUTRed" );
        lutRead.open(Camera._Ptr, FileName);
    }
    // Oops the pointer to the FileName is invalid now
    // this should not matter but it does but in the original impleentation there is a member variable storing
    // just the cost char*...

    // open is successful
    CPPUNIT_ASSERT(!lutRead.fail());

    uint16_t testBuffer[LUT_SIZE];
    lutRead.read(reinterpret_cast<char*>(&testBuffer),sizeof(testBuffer));

    CPPUNIT_ASSERT(0 == memcmp(lutBuffer,testBuffer,LUT_SIZE));

    lutRead.close();

}



