//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/


#ifndef __GENAPITEST_H
#define __GENAPITEST_H

#include "GenApi/GenApi.h"
#include "GenApi/PortImpl.h"
#include <cppunitex/HelperMacrosEx.h>

//! Indicates the GenApi schema to be tested
typedef enum _EGenApiSchemaVersion
{
    v1_0 = 1,
    v1_1 = 2,
    _Undefined = -1,
} EGenApiSchemaVersion;

//! helper class converting EGenApiSchemaVersion from and to string
class EGenApiSchemaVersionClass
{
public:
    //! Converts a string to enum value
    static bool FromString(const std::string &ValueStr, EGenApiSchemaVersion *pValue)
    {
        if (!pValue) return false;
        if( ValueStr == "v1_0" )
            *pValue = v1_0;
        else if( ValueStr == "v1_1" )
            *pValue = v1_1;
        else
            return false;
        return true;
    }
    //! Converts a string to an int32_t property
    static void ToString(std::string &ValueStr, EGenApiSchemaVersion *pValue)
    {

        assert( ! pValue );
        if( *pValue == v1_0 )
            ValueStr = "v1_0";
        else if( *pValue == v1_1 )
            ValueStr = "v1_1";
        else
            ValueStr = "_Undefined";
    }

    //! Converts a string to an int32_t property
    static std::string ToString(EGenApiSchemaVersion Value)
    {
        std::string Result;
        ToString(Result, &Value);
        return Result;
    }
};

extern GenICam::gcstring CDFHeader;
extern GenICam::gcstring CDFFooter;
extern EGenApiSchemaVersion GenApiSchemaVersion;

#pragma warning( disable : 4127 ) //conditional expression is constant
#pragma warning( once : 4995 ) // 'DeprecatedNode': name was marked as #pragma deprecated (this is by intention for test)
#   define XMLPATH "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/"
#if 0
#ifdef _MSC_VER
#elif defined (HAVE_CONFIG_H)
#   define XMLPATH "$(" GENICAM_ROOT_VERSION ")/share/genicam/xml/"
#else
#   error Unknown/unsupported platform
#endif
#endif

#define NODE_POINTER( TYPE, NAME ) \
	C ## TYPE ## Ptr ptr ## NAME = Camera._GetNode( #NAME ); \
	CPPUNIT_ASSERT( ptr ## NAME.IsValid() );


// ---------------------------------------------------------------------------
// the test callback with a counter
// ---------------------------------------------------------------------------
struct CallbackUtility
{
    static void Reset() { m_Count=0; }
    static void Callback( GenApi::INode* ) { ++m_Count; }
    static uint32_t Count() { return m_Count; }
private:
    static unsigned m_Count;
};

#endif // __GENAPITEST_H
