//-----------------------------------------------------------------------------
//  (c) 2008 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "IndexedFloatTest.h"
#include "GenApi/GenApi.h"
#include <limits>

#undef max
#undef min

using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( IndexedFloatTestSuite, "GenApiTest");

void IndexedFloatTestSuite::TestMultiplexerMode()
{
    // added <pValueIndexed> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Float Name=\"Multiplexer\">                       \n"
    "    <pIndex>Selector</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">FloatOn</pValueIndexed>     \n"
    "    <pValueIndexed Index=\"-1\">FloatOff</pValueIndexed>   \n"
    "    <pValueDefault>FloatDefault</pValueDefault>    \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatOn\">                           \n"
    "    <Value>100</Value>                             \n"
    "    <DisplayNotation>Scientific</DisplayNotation>       \n"
    "    <DisplayPrecision>3</DisplayPrecision>                   \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatOff\">                          \n"
    "    <Value>-100</Value>                            \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatDefault\">                      \n"
    "    <Value>123</Value>                             \n"
    "    <DisplayNotation>Fixed</DisplayNotation>       \n"
    "    <DisplayPrecision>2</DisplayPrecision>                   \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Enumeration Name=\"Selector\">                    \n"
    "    <EnumEntry Name=\"On\">                        \n"
    "        <Value>1</Value>                           \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Off\">                       \n"
    "        <Value>-1</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Unknown\">                   \n"
    "        <Value>99</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <Value>-1</Value>                              \n"
    "</Enumeration>                                     \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrMultiplexer = Camera._GetNode("Multiplexer");
    CEnumerationPtr ptrSelector = Camera._GetNode("Selector");

    *ptrSelector = "On";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 100.0, *ptrMultiplexer(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_EQUAL( fnScientific, ptrMultiplexer->GetDisplayNotation() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)3LL, ptrMultiplexer->GetDisplayPrecision() );
    *ptrSelector = "Off";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( -100.0, *ptrMultiplexer(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_EQUAL( fnAutomatic, ptrMultiplexer->GetDisplayNotation() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)6LL, ptrMultiplexer->GetDisplayPrecision() );
    *ptrSelector = "Unknown";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 123.0, *ptrMultiplexer(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_EQUAL( fnFixed, ptrMultiplexer->GetDisplayNotation() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)2LL, ptrMultiplexer->GetDisplayPrecision() );
}

void IndexedFloatTestSuite::TestIndexedAccess()
{
    // added polymorphic access
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Float Name=\"IndexFromInt\">                    \n"
    "    <pIndex>Int</pIndex>                           \n"
    "    <ValueIndexed Index=\"0\">0815</ValueIndexed>  \n"
    "    <ValueIndexed Index=\"1\">42</ValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Float>                                         \n"
    "                                                   \n"
    "<Float Name=\"IndexFromEnum\">                   \n"
    "    <pIndex>Enum</pIndex>                          \n"
    "    <ValueIndexed Index=\"0\">0815</ValueIndexed>  \n"
    "    <ValueIndexed Index=\"1\">42</ValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Float>                                         \n"
    "                                                   \n"
    "<Float Name=\"IndexFromBool\">                   \n"
    "    <pIndex>Bool</pIndex>                          \n"
    "    <ValueIndexed Index=\"0\">0815</ValueIndexed>  \n"
    "    <ValueIndexed Index=\"1\">42</ValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Float>                                         \n"
    "                                                   \n"
    "<Integer Name=\"Int\">                             \n"
    "    <Value>0</Value>                               \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Enumeration Name=\"Enum\">                        \n"
    "    <EnumEntry Name=\"On\">                        \n"
    "        <Value>1</Value>                           \n"
    "   </EnumEntry>                                    \n"
    "    <EnumEntry Name=\"Off\">                       \n"
    "        <Value>0</Value>                           \n"
    "   </EnumEntry>                                    \n"
    "    <Value>0</Value>                               \n"
    "</Enumeration>                                     \n"
    "                                                   \n"
    "<Boolean Name=\"Bool\">                            \n"
    "    <Value>1</Value>                               \n"
    "    <OnValue>1</OnValue>                           \n"
    "    <OffValue>0</OffValue>                         \n"
    "</Boolean>                                         \n"
    "                                                   \n"
    "<Float Name=\"IndexedFloatDetailed\">                    \n"
    "    <ImposedAccessMode>RO</ImposedAccessMode>                \n"
    "    <pIndex>Int</pIndex>                           \n"
    "    <pValueIndexed Index=\"0\">Float0</pValueIndexed>  \n"
    "    <pValueIndexed Index=\"1\">Float1</pValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "    <Unit>ParentUnit</Unit>                    \n"
    "    <Representation>Linear</Representation>    \n"
    "</Float>                                         \n"
    "                                                   \n"
    "<Float Name=\"IndexedFloatNoDetails\">                    \n"
    "    <pIndex>Int</pIndex>                           \n"
    "    <pValueIndexed Index=\"0\">Float0</pValueIndexed>  \n"
    "    <pValueIndexed Index=\"1\">Float1</pValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Float>                                         \n"
    "                                                   \n"
    "<Float Name=\"Float0\">                             \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                \n"
    "    <Value>0</Value>                               \n"
    "    <Unit>IndexedUnit</Unit>                    \n"
    "    <Representation>Linear</Representation>    \n"
    "</Float>                                         \n"
    "                                                   \n"
    "<Float Name=\"Float1\">                             \n"
    "    <Value>1</Value>                               \n"
    "</Float>                                         \n"
    "                                                   \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);
    CIntegerPtr ptrInt = Camera._GetNode("Int");
    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CBooleanPtr ptrBool = Camera._GetNode("Bool");

    CFloatPtr ptrIndexFromInt = Camera._GetNode("IndexFromInt");
    CFloatPtr ptrIndexFromEnum = Camera._GetNode("IndexFromEnum");
    CFloatPtr ptrIndexFromBool = Camera._GetNode("IndexFromBool");

    gcstring ValueStr;
    gcstring AttributeStr;
    ptrIndexFromInt->GetNode()->GetProperty("ValueDefault", ValueStr, AttributeStr);
    cout << "ValueDefault = " << ValueStr << "\n";
    ptrIndexFromInt->GetNode()->GetProperty("ValueIndexed", ValueStr, AttributeStr);
    cout << "ValueIndexed = " << ValueStr << " : " << AttributeStr << "\n";

    *ptrInt = 0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 815.0, *ptrIndexFromInt(), std::numeric_limits<double>::epsilon() );
    *ptrInt = 1;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 42.0, *ptrIndexFromInt(), std::numeric_limits<double>::epsilon() );
    *ptrInt = 2;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 13.0, *ptrIndexFromInt(), std::numeric_limits<double>::epsilon() );
    // change and re-read
    *ptrInt = 0;
    ptrIndexFromInt->SetValue (518.0);
    *ptrInt = 1;
    ptrIndexFromInt->SetValue (24.0);
    *ptrInt = 2;
    ptrIndexFromInt->SetValue (31.0);
    *ptrInt = 0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 518.0, *ptrIndexFromInt(), std::numeric_limits<double>::epsilon()  );
    *ptrInt = 1;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 24.0, *ptrIndexFromInt(), std::numeric_limits<double>::epsilon()  );
    *ptrInt = 2;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 31.0, *ptrIndexFromInt(), std::numeric_limits<double>::epsilon()  );

    *ptrEnum = "Off";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 815.0, *ptrIndexFromEnum(), std::numeric_limits<double>::epsilon()  );
    *ptrEnum = "On";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 42.0, *ptrIndexFromEnum(), std::numeric_limits<double>::epsilon()  );

    *ptrBool = false;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 815.0, *ptrIndexFromBool(), std::numeric_limits<double>::epsilon()  );
    *ptrBool = true;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 42.0, *ptrIndexFromBool(), std::numeric_limits<double>::epsilon()  );

    *ptrEnum = "Off";
    *ptrIndexFromEnum = 999.0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 999.0, *ptrIndexFromEnum(), std::numeric_limits<double>::epsilon()  );
    *ptrEnum = "On";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 42.0, *ptrIndexFromEnum(), std::numeric_limits<double>::epsilon()  );
    *ptrEnum = "Off";
    *ptrIndexFromEnum = 888.0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 888.0, *ptrIndexFromEnum(), std::numeric_limits<double>::epsilon()  );
    *ptrIndexFromEnum = 815.0;

    // now test propagation of unit/representation/accessmode
    CFloatPtr ptrFloatDetails = Camera._GetNode("IndexedFloatDetailed");
    CFloatPtr ptrFloatNoDetails = Camera._GetNode("IndexedFloatNoDetails");
    *ptrInt = 0;
    CPPUNIT_ASSERT_EQUAL( gcstring("ParentUnit"), ptrFloatDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrFloatDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( NA, ptrFloatDetails->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( gcstring("IndexedUnit"), ptrFloatNoDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrFloatNoDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( WO, ptrFloatNoDetails->GetAccessMode() );
    *ptrInt = 1;
    CPPUNIT_ASSERT_EQUAL( gcstring("ParentUnit"), ptrFloatDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrFloatDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrFloatDetails->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( gcstring(""), ptrFloatNoDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrFloatNoDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RW, ptrFloatNoDetails->GetAccessMode() );
    *ptrInt = 2;
    CPPUNIT_ASSERT_EQUAL( gcstring("ParentUnit"), ptrFloatDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrFloatDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrFloatDetails->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( gcstring(""), ptrFloatNoDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrFloatNoDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RW, ptrFloatNoDetails->GetAccessMode() );
}

void IndexedFloatTestSuite::TestPolymorphicalFloatAccess()
{
    // added polymorphic added
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Float Name=\"FloatFromInt\">           \n"
    "    <pValue>Int</pValue>                \n"
    "</Float>                              \n"
    "                                        \n"
// TODO
//    "<Float Name=\"FloatFromEnum\">          \n"
//    "    <pValue>Enum</pValue>               \n"
//    "</Float>                              \n"
    "                                        \n"
    "<Float Name=\"FloatFromFloat\">          \n"
    "    <pValue>Float</pValue>               \n"
    "</Float>                              \n"
    "                                        \n"
    "<Integer Name=\"Int\">                  \n"
    "    <Value>0</Value>                    \n"
    "    <Min>-10</Min>                      \n"
    "    <Max>10</Max>                       \n"
    "    <Inc>2</Inc>                        \n"
    "    <Representation>Linear</Representation>\n"
    "</Integer>                              \n"
    "                                        \n"
    "<Enumeration Name=\"Enum\">             \n"
    "    <EnumEntry Name=\"On\">             \n"
    "        <Value>1</Value>                \n"
    "   </EnumEntry>                         \n"
    "    <EnumEntry Name=\"Off\">            \n"
    "        <Value>0</Value>                \n"
    "   </EnumEntry>                         \n"
    "    <Value>0</Value>                    \n"
    "</Enumeration>                          \n"
    "                                        \n"
    "<Float Name=\"Float\">                 \n"
    "    <Value>1.0</Value>                    \n"
    "    <Min>-10.0</Min>                      \n"
    "    <Max>10.0</Max>                       \n"
    "    <Representation>Linear</Representation>\n"
    "</Float>                              \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrFloat = Camera._GetNode("Float");
    CIntegerPtr ptrInt = Camera._GetNode("Int");
    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CFloatPtr ptrFloatFromInt = Camera._GetNode("FloatFromInt");
    //CFloatPtr ptrFloatFromEnum = Camera._GetNode("FloatFromEnum");
    CFloatPtr ptrFloatFromFloat = Camera._GetNode("FloatFromFloat");

    CPPUNIT_ASSERT_DOUBLES_EQUAL( -10.0, ptrFloat->GetMin(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 10.0, ptrFloat->GetMax(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrFloat->GetRepresentation() );
    *ptrFloat = 2.0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 2.0, *ptrFloat(), std::numeric_limits<double>::epsilon() );
    *ptrFloat = 0.0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 0.0, *ptrFloat(), std::numeric_limits<double>::epsilon() );

    CPPUNIT_ASSERT_DOUBLES_EQUAL( -10.0, ptrFloatFromFloat->GetMin(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 10.0, ptrFloatFromFloat->GetMax(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrFloatFromFloat->GetRepresentation() );
    *ptrFloat = 4;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 4.0, *ptrFloatFromFloat(), std::numeric_limits<double>::epsilon() );
    *ptrFloat = 0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 0.0, *ptrFloatFromFloat(), std::numeric_limits<double>::epsilon() );

// TODO
#if 0
    CPPUNIT_ASSERT_DOUBLES_EQUAL( -std::numeric_limits<double>::max(), ptrFloatFromEnum->GetMin(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( std::numeric_limits<double>::max(), ptrFloatFromEnum->GetMax(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrFloatFromEnum->GetRepresentation() );
    *ptrEnum = "On";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 1.0, *ptrFloatFromEnum() );
    *ptrEnum = "Off";
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 0.0, *ptrFloatFromEnum() );
#endif

    CPPUNIT_ASSERT_DOUBLES_EQUAL( -10.0, ptrFloatFromInt->GetMin(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 10.0, ptrFloatFromInt->GetMax(), std::numeric_limits<double>::epsilon() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrFloatFromInt->GetRepresentation() );
    *ptrInt = 4;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 4.0, *ptrFloatFromInt(), std::numeric_limits<double>::epsilon() );
    *ptrInt = 0;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( 0.0, *ptrFloatFromInt(), std::numeric_limits<double>::epsilon() );
}


void IndexedFloatTestSuite::TestAccessMode()
{
    // added <pValueIndexed> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Float Name=\"Multiplexer\">                       \n"
    "    <pIndex>Selector</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">FloatNI</pValueIndexed>     \n"
    "    <pValueIndexed Index=\"2\">FloatNA</pValueIndexed>   \n"
    "    <pValueIndexed Index=\"3\">FloatWO</pValueIndexed>   \n"
    "    <pValueIndexed Index=\"4\">FloatRO</pValueIndexed>   \n"
    "    <pValueDefault>FloatRW</pValueDefault>    \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatNI\">                           \n"
    "    <pIsImplemented>Zero</pIsImplemented>                             \n"
    "    <Value>100</Value>                             \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatNA\">                           \n"
    "    <pIsAvailable>Zero</pIsAvailable>                             \n"
    "    <Value>100</Value>                             \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatWO\">                           \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                             \n"
    "    <Value>100</Value>                             \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatRO\">                           \n"
    "    <ImposedAccessMode>RO</ImposedAccessMode>                             \n"
    "    <Value>100</Value>                             \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatRW\">                           \n"
    "    <Value>100</Value>                             \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Integer Name=\"Selector\">                      \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"Zero\">                      \n"
    "    <Value>0</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Float Name=\"MultiplexerNI\">                       \n"
    "    <pIndex>SelectorNI</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">FloatRW</pValueIndexed>     \n"
    "    <pValueDefault>FloatRW</pValueDefault>    \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Integer Name=\"SelectorNI\">                      \n"
    "    <pIsImplemented>Zero</pIsImplemented>                             \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Float Name=\"MultiplexerNA\">                       \n"
    "    <pIndex>SelectorNA</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">FloatRW</pValueIndexed>     \n"
    "    <pValueDefault>FloatRW</pValueDefault>    \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Integer Name=\"SelectorNA\">                      \n"
    "    <pIsAvailable>Zero</pIsAvailable>                             \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Float Name=\"MultiplexerNAWO\">                       \n"
    "    <pIndex>SelectorWO</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">FloatRW</pValueIndexed>     \n"
    "    <pValueDefault>FloatRW</pValueDefault>    \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Integer Name=\"SelectorWO\">                      \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                             \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrMultiplexer = Camera._GetNode("Multiplexer");
    CIntegerPtr ptrSelector = Camera._GetNode("Selector");

    *ptrSelector = 1;
    CPPUNIT_ASSERT_EQUAL( NI, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 2;
    CPPUNIT_ASSERT_EQUAL( NA, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 3;
    CPPUNIT_ASSERT_EQUAL( WO, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 4;
    CPPUNIT_ASSERT_EQUAL( RO, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 222;
    CPPUNIT_ASSERT_EQUAL( RW, ptrMultiplexer->GetAccessMode() );

    CFloatPtr ptrMultiplexerNI = Camera._GetNode("MultiplexerNI");
    CPPUNIT_ASSERT_EQUAL( NI, ptrMultiplexerNI->GetAccessMode() );

    CFloatPtr ptrMultiplexerNA = Camera._GetNode("MultiplexerNA");
    CPPUNIT_ASSERT_EQUAL( NA, ptrMultiplexerNA->GetAccessMode() );

    CFloatPtr ptrMultiplexerNAWO = Camera._GetNode("MultiplexerNAWO");
    CPPUNIT_ASSERT_EQUAL( NA, ptrMultiplexerNAWO->GetAccessMode() );
}

void IndexedFloatTestSuite::TestTicket803()
{
    // added <pValueIndexed> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Float Name=\"Multiplexer\">                       \n"
    "    <pIndex>Selector</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">FloatOn</pValueIndexed>     \n"
    "    <pValueIndexed Index=\"-1\">FloatOff</pValueIndexed>   \n"
    "    <pValueDefault>FloatDefault</pValueDefault>    \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatOn\">                           \n"
    "    <Value>100</Value>                             \n"
    "    <DisplayNotation>Scientific</DisplayNotation>       \n"
    "    <DisplayPrecision>3</DisplayPrecision>                   \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatOff\">                          \n"
    "    <Value>-100</Value>                            \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Float Name=\"FloatDefault\">                      \n"
    "    <Value>123</Value>                             \n"
    "    <DisplayNotation>Fixed</DisplayNotation>       \n"
    "    <DisplayPrecision>2</DisplayPrecision>                   \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Enumeration Name=\"Selector\">                    \n"
    "    <EnumEntry Name=\"On\">                        \n"
    "        <Value>1</Value>                           \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Off\">                       \n"
    "        <Value>-1</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Unknown\">                   \n"
    "        <Value>99</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <Value>-1</Value>                              \n"
    "</Enumeration>                                     \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);

    NODE_POINTER( Node, Multiplexer );

    NodeList_t Terminals;
    ptrMultiplexer->GetChildren( Terminals, ctAllTerminalNodes );

    cout << "Terminals:\n";
    NodeList_t::iterator itTerminal;
    for( itTerminal=Terminals.begin(); itTerminal!=Terminals.end(); itTerminal++ )
    {
        cout << "- " << (*itTerminal)->GetName().c_str() << "\n";
    }

    CPPUNIT_ASSERT_EQUAL( (size_t)3, Terminals.size() );

}
