//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "IndexedIntegerTest.h"
#include "GenApi/GenApi.h"
#include "TestPort.h"
#include <algorithm>

using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( IndexedIntegerTestSuite, "GenApiTest");

void IndexedIntegerTestSuite::TestMultiplexerMode()
{
    // added <pValueIndexed> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Integer Name=\"Multiplexer\">                     \n"
    "    <pIndex>Selector</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">IntOn</pValueIndexed>     \n"
    "    <pValueIndexed Index=\"-1\">IntOff</pValueIndexed>   \n"
    "    <pValueDefault>IntDefault</pValueDefault>            \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IntOn\">                           \n"
    "    <Value>100</Value>                             \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IntOff\">                          \n"
    "    <Value>-100</Value>                            \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IntDefault\">                      \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Enumeration Name=\"Selector\">                    \n"
    "    <EnumEntry Name=\"On\">                        \n"
    "        <Value>1</Value>                           \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Off\">                       \n"
    "        <Value>-1</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Unknown\">                   \n"
    "        <Value>99</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <Value>-1</Value>                              \n"
    "</Enumeration>                                     \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrMultiplexer = Camera._GetNode("Multiplexer");
    CEnumerationPtr ptrSelector = Camera._GetNode("Selector");

    *ptrSelector = "On";
    CPPUNIT_ASSERT_EQUAL( (int64_t) 100LL, *ptrMultiplexer() );
    *ptrSelector = "Off";
    CPPUNIT_ASSERT_EQUAL( (int64_t) -100LL, *ptrMultiplexer() );
    *ptrSelector = "Unknown";
    CPPUNIT_ASSERT_EQUAL( (int64_t) 123LL, *ptrMultiplexer() );
}

void IndexedIntegerTestSuite::TestIndexedAccess()
{
    // added polymorphic accedd
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Integer Name=\"IndexFromInt\">                    \n"
    "    <pIndex>Int</pIndex>                           \n"
    "    <ValueIndexed Index=\"0\">0815</ValueIndexed>  \n"
    "    <ValueIndexed Index=\"1\">42</ValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IndexFromEnum\">                   \n"
    "    <pIndex>Enum</pIndex>                          \n"
    "    <ValueIndexed Index=\"0\">0815</ValueIndexed>  \n"
    "    <ValueIndexed Index=\"1\">42</ValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IndexFromBool\">                   \n"
    "    <pIndex>Bool</pIndex>                          \n"
    "    <ValueIndexed Index=\"0\">0815</ValueIndexed>  \n"
    "    <ValueIndexed Index=\"1\">42</ValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IndexFromFloat\">                  \n"
    "    <pIndex>Float</pIndex>                         \n"
    "    <ValueIndexed Index=\"0\">0815</ValueIndexed>  \n"
    "    <ValueIndexed Index=\"1\">42</ValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"Int\">                             \n"
    "    <Value>0</Value>                               \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Enumeration Name=\"Enum\">                        \n"
    "    <EnumEntry Name=\"On\">                        \n"
    "        <Value>1</Value>                           \n"
    "   </EnumEntry>                                    \n"
    "    <EnumEntry Name=\"Off\">                       \n"
    "        <Value>0</Value>                           \n"
    "   </EnumEntry>                                    \n"
    "    <Value>0</Value>                               \n"
    "</Enumeration>                                     \n"
    "                                                   \n"
    "<Boolean Name=\"Bool\">                            \n"
    "    <Value>1</Value>                               \n"
    "    <OnValue>1</OnValue>                           \n"
    "    <OffValue>0</OffValue>                         \n"
    "</Boolean>                                         \n"
    "                                                   \n"
    "<Float Name=\"Float\">                             \n"
    "    <Value>0</Value>                               \n"
    "</Float>                                           \n"
    "                                                   \n"
    "<Integer Name=\"IndexedIntDetailed\">                    \n"
    "    <ImposedAccessMode>RO</ImposedAccessMode>                \n"
    "    <pIndex>Int</pIndex>                           \n"
    "    <pValueIndexed Index=\"0\">Int0</pValueIndexed>  \n"
    "    <pValueIndexed Index=\"1\">Int1</pValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "    <Unit>ParentUnit</Unit>                    \n"
    "    <Representation>HexNumber</Representation>    \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IndexedIntNoDetails\">                    \n"
    "    <pIndex>Int</pIndex>                           \n"
    "    <pValueIndexed Index=\"0\">Int0</pValueIndexed>  \n"
    "    <pValueIndexed Index=\"1\">Int1</pValueIndexed>    \n"
    "    <ValueDefault>13</ValueDefault>                \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"Int0\">                             \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                \n"
    "    <Value>0</Value>                               \n"
    "    <Unit>IndexedUnit</Unit>                    \n"
    "    <Representation>Linear</Representation>    \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"Int1\">                             \n"
    "    <Value>1</Value>                               \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrInt = Camera._GetNode("Int");
    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CBooleanPtr ptrBool = Camera._GetNode("Bool");
    CFloatPtr ptrFloat = Camera._GetNode("Float");

    CIntegerPtr ptrIndexFromInt = Camera._GetNode("IndexFromInt");
    CIntegerPtr ptrIndexFromEnum = Camera._GetNode("IndexFromEnum");
    CIntegerPtr ptrIndexFromBool = Camera._GetNode("IndexFromBool");
    CIntegerPtr ptrIndexFromFloat = Camera._GetNode("IndexFromFloat");

    gcstring ValueStr;
    gcstring AttributeStr;
    ptrIndexFromInt->GetNode()->GetProperty("ValueDefault", ValueStr, AttributeStr);
    cout << "ValueDefault = " << ValueStr << "\n";
    ptrIndexFromInt->GetNode()->GetProperty("ValueIndexed", ValueStr, AttributeStr);
    cout << "ValueIndexed = " << ValueStr << " : " << AttributeStr << "\n";

    *ptrInt = 0;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 815LL, *ptrIndexFromInt() );
    *ptrInt = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrIndexFromInt() );
    *ptrInt = 2;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 13LL, *ptrIndexFromInt() );
    // change and re-read
    *ptrInt = 0;
    ptrIndexFromInt->SetValue (518LL);
    *ptrInt = 1;
    ptrIndexFromInt->SetValue (24LL);
    *ptrInt = 2;
    ptrIndexFromInt->SetValue (31LL);
    *ptrInt = 0;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 518LL, *ptrIndexFromInt() );
    *ptrInt = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 24LL, *ptrIndexFromInt() );
    *ptrInt = 2;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 31LL, *ptrIndexFromInt() );

    *ptrEnum = "Off";
    CPPUNIT_ASSERT_EQUAL( (int64_t) 815LL, *ptrIndexFromEnum() );
    *ptrEnum = "On";
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrIndexFromEnum() );

    *ptrBool = false;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 815LL, *ptrIndexFromBool() );
    *ptrBool = true;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrIndexFromBool() );

    *ptrFloat = 0.0;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 815LL, *ptrIndexFromFloat() );
    *ptrFloat = 1.2;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrIndexFromFloat() );

    *ptrEnum = "Off";
    *ptrIndexFromEnum = 999;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 999LL, *ptrIndexFromEnum() );
    *ptrEnum = "On";
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrIndexFromEnum() );
    *ptrEnum = "Off";
    *ptrIndexFromEnum = 888;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 888LL, *ptrIndexFromEnum() );
    *ptrIndexFromEnum = 815;

    // now test propagation of unit/representation/accessmode
    CIntegerPtr ptrIntDetails = Camera._GetNode("IndexedIntDetailed");
    CIntegerPtr ptrIntNoDetails = Camera._GetNode("IndexedIntNoDetails");
    *ptrInt = 0;
    CPPUNIT_ASSERT_EQUAL( gcstring("ParentUnit"), ptrIntDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( HexNumber, ptrIntDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( NA, ptrIntDetails->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( gcstring("IndexedUnit"), ptrIntNoDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( Linear, ptrIntNoDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( WO, ptrIntNoDetails->GetAccessMode() );
    *ptrInt = 1;
    CPPUNIT_ASSERT_EQUAL( gcstring("ParentUnit"), ptrIntDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( HexNumber, ptrIntDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrIntDetails->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( gcstring(""), ptrIntNoDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrIntNoDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RW, ptrIntNoDetails->GetAccessMode() );
    *ptrInt = 2;
    CPPUNIT_ASSERT_EQUAL( gcstring("ParentUnit"), ptrIntDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( HexNumber, ptrIntDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrIntDetails->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( gcstring(""), ptrIntNoDetails->GetUnit() );
    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrIntNoDetails->GetRepresentation() );
    CPPUNIT_ASSERT_EQUAL( RW, ptrIntNoDetails->GetAccessMode() );
}

void IndexedIntegerTestSuite::TestPolymorphicalIntegerAccess()
{
    // added polymorphic accedd
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<Integer Name=\"IntFromInt\">           \n"
    "    <pValue>Int</pValue>                \n"
    "</Integer>                              \n"
    "                                        \n"
    "<Integer Name=\"IntFromEnum\">          \n"
    "    <pValue>Enum</pValue>               \n"
    "</Integer>                              \n"
    "                                        \n"
    "<Integer Name=\"IntFromBool\">          \n"
    "    <pValue>Bool</pValue>               \n"
    "</Integer>                              \n"
    "                                        \n"
    "<Integer Name=\"Int\">                  \n"
    "    <Value>0</Value>                    \n"
    "    <Min>-10</Min>                      \n"
    "    <Max>10</Max>                       \n"
    "    <Inc>2</Inc>                        \n"
    "    <Representation>HexNumber</Representation>\n"
    "</Integer>                              \n"
    "                                        \n"
    "<Enumeration Name=\"Enum\">             \n"
    "    <EnumEntry Name=\"On\">             \n"
    "        <Value>1</Value>                \n"
    "   </EnumEntry>                         \n"
    "    <EnumEntry Name=\"Off\">            \n"
    "        <Value>0</Value>                \n"
    "   </EnumEntry>                         \n"
    "    <Value>0</Value>                    \n"
    "</Enumeration>                          \n"
    "                                        \n"
    "<Boolean Name=\"Bool\">                 \n"
    "    <Value>1</Value>                    \n"
    "    <OnValue>1</OnValue>                \n"
    "    <OffValue>0</OffValue>              \n"
    "</Boolean>                              \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrInt = Camera._GetNode("Int");
    CIntegerPtr ptrIntFromInt = Camera._GetNode("IntFromInt");
    CEnumerationPtr ptrEnum = Camera._GetNode("Enum");
    CIntegerPtr ptrIntFromEnum = Camera._GetNode("IntFromEnum");
    CBooleanPtr ptrBool = Camera._GetNode("Bool");
    CIntegerPtr ptrIntFromBool = Camera._GetNode("IntFromBool");

    CPPUNIT_ASSERT_EQUAL( (int64_t) -10LL, ptrInt->GetMin() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 10LL, ptrInt->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 2LL, ptrInt->GetInc() );
    CPPUNIT_ASSERT_EQUAL( HexNumber, ptrInt->GetRepresentation() );
    *ptrInt = 2;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 2LL, *ptrInt() );
    *ptrInt = 0;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, *ptrInt() );

    CPPUNIT_ASSERT_EQUAL( (int64_t) -10LL, ptrIntFromInt->GetMin() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 10LL, ptrIntFromInt->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 2LL, ptrIntFromInt->GetInc() );
    CPPUNIT_ASSERT_EQUAL( HexNumber, ptrIntFromInt->GetRepresentation() );
    *ptrInt = 4;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 4LL, *ptrIntFromInt() );
    *ptrInt = 0;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, *ptrIntFromInt() );

    CPPUNIT_ASSERT_EQUAL( INT64_MIN, ptrIntFromEnum->GetMin() );
    CPPUNIT_ASSERT_EQUAL( INT64_MAX, ptrIntFromEnum->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, ptrIntFromEnum->GetInc() );
    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrIntFromEnum->GetRepresentation() );
    *ptrEnum = "On";
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, *ptrIntFromEnum() );
    *ptrEnum = "Off";
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, *ptrIntFromEnum() );

    CPPUNIT_ASSERT_EQUAL( INT64_MIN, ptrIntFromBool->GetMin() );
    CPPUNIT_ASSERT_EQUAL( INT64_MAX, ptrIntFromBool->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, ptrIntFromBool->GetInc() );
    CPPUNIT_ASSERT_EQUAL( PureNumber, ptrIntFromBool->GetRepresentation() );
    *ptrBool = true;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, *ptrIntFromBool() );
    *ptrBool = false;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, *ptrIntFromBool() );

}

void IndexedIntegerTestSuite::TestAccessMode()
{
    // added <pValueIndexed> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initialize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Integer Name=\"Multiplexer\">                       \n"
    "    <pIndex>Selector</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">IntegerNI</pValueIndexed>     \n"
    "    <pValueIndexed Index=\"2\">IntegerNA</pValueIndexed>   \n"
    "    <pValueIndexed Index=\"3\">IntegerWO</pValueIndexed>   \n"
    "    <pValueIndexed Index=\"4\">IntegerRO</pValueIndexed>   \n"
    "    <pValueDefault>IntegerRW</pValueDefault>    \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"IntegerNI\">                           \n"
    "    <pIsImplemented>Zero</pIsImplemented>                             \n"
    "    <Value>100</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"IntegerNA\">                           \n"
    "    <pIsAvailable>Zero</pIsAvailable>                             \n"
    "    <Value>100</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"IntegerWO\">                           \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                             \n"
    "    <Value>100</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"IntegerRO\">                           \n"
    "    <ImposedAccessMode>RO</ImposedAccessMode>                             \n"
    "    <Value>100</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"IntegerRW\">                           \n"
    "    <Value>100</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"Selector\">                      \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"Zero\">                      \n"
    "    <Value>0</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"MultiplexerNI\">                       \n"
    "    <pIndex>SelectorNI</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">IntegerRW</pValueIndexed>     \n"
    "    <pValueDefault>IntegerRW</pValueDefault>    \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"SelectorNI\">                      \n"
    "    <pIsImplemented>Zero</pIsImplemented>                             \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"MultiplexerNA\">                       \n"
    "    <pIndex>SelectorNA</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">IntegerRW</pValueIndexed>     \n"
    "    <pValueDefault>IntegerRW</pValueDefault>    \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"SelectorNA\">                      \n"
    "    <pIsAvailable>Zero</pIsAvailable>                             \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"MultiplexerNAWO\">                       \n"
    "    <pIndex>SelectorWO</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">IntegerRW</pValueIndexed>     \n"
    "    <pValueDefault>IntegerRW</pValueDefault>    \n"
    "</Integer>                                           \n"
    "                                                   \n"
    "<Integer Name=\"SelectorWO\">                      \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>                             \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                           \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrMultiplexer = Camera._GetNode("Multiplexer");
    CIntegerPtr ptrSelector = Camera._GetNode("Selector");

    *ptrSelector = 1;
    CPPUNIT_ASSERT_EQUAL( NI, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 2;
    CPPUNIT_ASSERT_EQUAL( NA, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 3;
    CPPUNIT_ASSERT_EQUAL( WO, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 4;
    CPPUNIT_ASSERT_EQUAL( RO, ptrMultiplexer->GetAccessMode() );

    *ptrSelector = 222;
    CPPUNIT_ASSERT_EQUAL( RW, ptrMultiplexer->GetAccessMode() );

    CIntegerPtr ptrMultiplexerNI = Camera._GetNode("MultiplexerNI");
    CPPUNIT_ASSERT_EQUAL( NI, ptrMultiplexerNI->GetAccessMode() );

    CIntegerPtr ptrMultiplexerNA = Camera._GetNode("MultiplexerNA");
    CPPUNIT_ASSERT_EQUAL( NA, ptrMultiplexerNA->GetAccessMode() );

    CIntegerPtr ptrMultiplexerNAWO = Camera._GetNode("MultiplexerNAWO");
    CPPUNIT_ASSERT_EQUAL( NA, ptrMultiplexerNAWO->GetAccessMode() );
}


void IndexedIntegerTestSuite::Test_803()
{
    // added <pValueIndexed> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                   \n"
    "<Integer Name=\"Multiplexer\">                     \n"
    "    <pIndex>Selector</pIndex>                      \n"
    "    <pValueIndexed Index=\"1\">IntOn</pValueIndexed>     \n"
    "    <pValueIndexed Index=\"-1\">IntOff</pValueIndexed>   \n"
    "    <pValueDefault>IntDefault</pValueDefault>            \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IntOn\">                           \n"
    "    <Value>100</Value>                             \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IntOff\">                          \n"
    "    <Value>-100</Value>                            \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Integer Name=\"IntDefault\">                      \n"
    "    <Value>123</Value>                             \n"
    "</Integer>                                         \n"
    "                                                   \n"
    "<Enumeration Name=\"Selector\">                    \n"
    "    <EnumEntry Name=\"On\">                        \n"
    "        <Value>1</Value>                           \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Off\">                       \n"
    "        <Value>-1</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <EnumEntry Name=\"Unknown\">                   \n"
    "        <Value>99</Value>                          \n"
    "    </EnumEntry>                                   \n"
    "    <Value>-1</Value>                              \n"
    "</Enumeration>                                     \n"
    "                                                   \n"
    + CDFFooter;

    CNodeMapRef Camera;

    Camera._LoadXMLFromString(CameraDescriptionFile);

    NODE_POINTER( Node, Multiplexer );

    NodeList_t Terminals;
    ptrMultiplexer->GetChildren( Terminals, ctAllTerminalNodes );

    cout << "Terminals:\n";
    NodeList_t::iterator itTerminal;
    for( itTerminal=Terminals.begin(); itTerminal!=Terminals.end(); itTerminal++ )
    {
        cout << "- " << (*itTerminal)->GetName().c_str() << "\n";
    }

    CPPUNIT_ASSERT_EQUAL( (size_t)3, Terminals.size() );

}

// ---------------------------------------------------------------------------
// the test callback with a counter
// ---------------------------------------------------------------------------

void IndexedIntegerTestSuite::TestNumberOfCallbacks01()
{

    /*[ GenApiTest@IndexedIntegerTestSuite_TestNumberOfCallbacks01.xml|gxml

    <Integer Name="Multiplexer">
        <pIndex>Selector</pIndex>
        <pValueIndexed Index="1">IntOn</pValueIndexed>
        <pValueIndexed Index="-1">IntOff</pValueIndexed>
        <pValueDefault>IntDefault</pValueDefault>
    </Integer>

    <Integer Name="IntOn">
        <Value>100</Value>
    </Integer>
    
    <Integer Name="IntOff">
        <Value>-100</Value>
    </Integer>

    <Integer Name="IntDefault">
        <Value>123</Value>
    </Integer>
   
    <Enumeration Name="Selector">
        <EnumEntry Name="On">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Off">
            <Value>-1</Value>
        </EnumEntry>
        <EnumEntry Name="Default">
            <Value>99</Value>
        </EnumEntry>
        <Value>-1</Value>
    </Enumeration>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "IndexedIntegerTestSuite_TestNumberOfCallbacks01");

    // Make sure setting a value to a multiplexer node fires one callback only
    // even if the node has more than one terminal

    NODE_POINTER( Enumeration, Selector );
    NODE_POINTER( Integer, Multiplexer );

    ptrSelector->FromString("On");
    CallbackUtility::Reset();
    Register( ptrMultiplexer->GetNode(), &CallbackUtility::Callback );

    ptrMultiplexer->SetValue(4711);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackUtility::Count() );

}

void IndexedIntegerTestSuite::TestNumberOfCallbacks02()
{

    /*[ GenApiTest@IndexedIntegerTestSuite_TestNumberOfCallbacks02.xml|gxml

    <IntSwissKnife Name="Feature">
        <pVariable Name="A">RegisterA</pVariable>
        <pVariable Name="B">RegisterB</pVariable>
        <Formula>A+B</Formula>
    </IntSwissKnife>

    <IntReg Name="RegisterA">
        <Address>0x0000</Address>
        <Length>4</Length>
        <AccessMode>RW</AccessMode>
        <pPort>Port</pPort>
        <PollingTime>10</PollingTime>
        <Sign>Signed</Sign>
        <Endianess>LittleEndian</Endianess>
    </IntReg>

    <IntReg Name="RegisterB">
        <Address>0x0010</Address>
        <Length>4</Length>
        <AccessMode>RW</AccessMode>
        <pPort>Port</pPort>
        <PollingTime>10</PollingTime>
        <Sign>Signed</Sign>
        <Endianess>LittleEndian</Endianess>
    </IntReg>

    <Port Name="Port">       
    </Port>

    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "IndexedIntegerTestSuite_TestNumberOfCallbacks02");

    CTestPort Port;
    Port.CreateEntry(0x0000, (int32_t)42, RW);
    Port.CreateEntry(0x0010, (int32_t)43, RW);

    Camera._Connect(&Port, "Port");

    // Make sure polling two registers which have a common dependent feature does not make the feature's callback fire twice

    NODE_POINTER( Integer, Feature );

    CallbackUtility::Reset();
    Register( ptrFeature->GetNode(), &CallbackUtility::Callback );

    Camera._Poll(1000);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackUtility::Count() );

}

void IndexedIntegerTestSuite::TestNumberOfCallbacks03()
{

    /*[ GenApiTest@IndexedIntegerTestSuite_TestNumberOfCallbacks03.xml|gxml

    <Command Name="DoIt">
        <pValue>Multiplexer</pValue>
        <CommandValue>4711</CommandValue>
    </Command>    

    <Integer Name="Multiplexer">
        <pIndex>Selector</pIndex>
        <pValueIndexed Index="1">IntOn</pValueIndexed>
        <pValueIndexed Index="-1">IntOff</pValueIndexed>
        <pValueDefault>IntDefault</pValueDefault>
    </Integer>

    <IntReg Name="IntOn">
        <Address>0x0000</Address>
        <Length>4</Length>
        <AccessMode>RW</AccessMode>
        <pPort>Port</pPort>
        <Sign>Signed</Sign>
        <Endianess>LittleEndian</Endianess>
    </IntReg>
    
    <Integer Name="IntOff">
        <Value>0</Value>
    </Integer>

    <Integer Name="IntDefault">
        <Value>123</Value>
    </Integer>
   
    <Enumeration Name="Selector">
        <EnumEntry Name="On">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Off">
            <Value>-1</Value>
        </EnumEntry>
        <EnumEntry Name="Default">
            <Value>99</Value>
        </EnumEntry>
        <Value>-1</Value>
    </Enumeration>

    <Port Name="Port">       
    </Port>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "IndexedIntegerTestSuite_TestNumberOfCallbacks03");

    CTestPort Port;
    Port.CreateEntry(0x0000, (int32_t)42, RW);

    Camera._Connect(&Port, "Port");

    // Make sure setting a value to a multiplexer node fires one callback only
    // when read by an IsDone call

    NODE_POINTER( Command, DoIt );
    NODE_POINTER( Enumeration, Selector );
    NODE_POINTER( Integer, IntOn );

    Register( ptrDoIt->GetNode(), &CallbackUtility::Callback );

    ptrSelector->FromString("On");
    ptrDoIt->Execute();
    CPPUNIT_ASSERT_EQUAL( (int64_t)4711, ptrIntOn->GetValue() );
    
    CallbackUtility::Reset();
    ptrDoIt->IsDone();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0, CallbackUtility::Count() );

    int32_t Dummy=99;
    Port.Write( &Dummy, 0x0000, 4);
    CPPUNIT_ASSERT_EQUAL( (uint32_t)0, CallbackUtility::Count() );
    ptrDoIt->IsDone();
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CallbackUtility::Count() );  

}

void IndexedIntegerTestSuite::TestIgnoreCache()
{

    /*[ GenApiTest@IndexedIntegerTestSuite_TestIgnoreCache.xml|gxml

    <Integer Name="FeatureA">
        <pValue>Middle</pValue>
    </Integer>

    <Integer Name="FeatureB">
        <pValue>Middle</pValue>
    </Integer>

    <Integer Name="Middle">
        <pValue>Terminal</pValue>
    </Integer>

    <Integer Name="Terminal">
        <Value>42</Value>
    </Integer>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "IndexedIntegerTestSuite_TestIgnoreCache");

    NODE_POINTER( Integer, FeatureA );
    NODE_POINTER( Integer, FeatureB );
    NODE_POINTER( Integer, Middle );

    // Caches are empty
    CPPUNIT_ASSERT( ! ptrFeatureA->IsValueCacheValid() );
    CPPUNIT_ASSERT( ! ptrFeatureB->IsValueCacheValid() );
    CPPUNIT_ASSERT( ! ptrMiddle->IsValueCacheValid() );

    // Fill caches
    cout << ptrFeatureA->ToString() << endl;
    cout << ptrFeatureB->ToString() << endl;

    CPPUNIT_ASSERT( ptrFeatureA->IsValueCacheValid() );
    CPPUNIT_ASSERT( ptrFeatureB->IsValueCacheValid() );
    CPPUNIT_ASSERT( ptrMiddle->IsValueCacheValid() );

    // Read FeatureA with IgnoreCache = true
    cout << ptrFeatureA->ToString(false, true) << endl;
    CPPUNIT_ASSERT( ! ptrFeatureA->IsValueCacheValid() );
    CPPUNIT_ASSERT( ! ptrFeatureB->IsValueCacheValid() );
    CPPUNIT_ASSERT( ! ptrMiddle->IsValueCacheValid() );

}

void IndexedIntegerTestSuite::TestGetProperty()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescriptionFile =

        CDFHeader +
        "<Integer Name=\"intNode\">                                   \n"
        "   <pIndex>Index</pIndex>                                     \n"
        "   <pValueIndexed Index=\"0\">A</pValueIndexed>               \n"
        "   <pValueDefault>A</pValueDefault>                           \n"
        "</Integer>                                                    \n"
        "<Integer Name=\"Index\">                                      \n"
        "   <Value>0</Value>                                           \n"
        "</Integer>                                                    \n"
        "<Integer Name=\"A\">                                          \n"
        "   <Value>0</Value>                                           \n"
        "</Integer>                                                    \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePtr ptrNode = Camera._GetNode ("intNode");

    gcstring strValue;
    gcstring strAttr;

    ptrNode->GetProperty("Value", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pValue", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pValueCopy", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pIndex", strValue, strAttr) ;
    CPPUNIT_ASSERT_EQUAL( strValue, gcstring("Index"));
    ptrNode->GetProperty("ValueIndexed", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pValueIndexed", strValue, strAttr) ;
    CPPUNIT_ASSERT_EQUAL( strValue, gcstring("A"));
    ptrNode->GetProperty("pValueDefault", strValue, strAttr) ;
    CPPUNIT_ASSERT_EQUAL( strValue, gcstring("A"));
 
    ptrNode = Camera._GetNode ("Index");

    ptrNode->GetProperty("Value", strValue, strAttr) ;
    CPPUNIT_ASSERT_EQUAL( strValue, gcstring("0"));
    ptrNode->GetProperty("pValue", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pValueCopy", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pIndex", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("ValueIndexed", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pValueIndexed", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());
    ptrNode->GetProperty("pValueDefault", strValue, strAttr) ;
    CPPUNIT_ASSERT( strValue.empty());


    

}
