//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "IntRegTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include <cppunit/TestAssert.h>

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( IntRegTestSuite, "GenApiTest");

void IntRegTestSuite::TestValueAccess()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<IntReg Name=\"Int64\">                \n"
    "    <Address>0x0000</Address>            \n"
    "    <Length>8</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"Int32\">                \n"
    "    <Address>0x0010</Address>            \n"
    "    <Length>4</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"Int8\">                    \n"
    "    <Address>0x0020</Address>            \n"
    "    <Length>1</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"UInt64\">                \n"
    "    <Address>0x0030</Address>            \n"
    "    <Length>8</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Unsigned</Sign>                \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"UInt32\">                \n"
    "    <Address>0x0040</Address>            \n"
    "    <Length>4</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Unsigned</Sign>                \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"UInt8\">                \n"
    "    <Address>0x0050</Address>            \n"
    "    <Length>1</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Unsigned</Sign>                \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"Int8BitSigned\">        \n"
    "    <Address>0x0060</Address>            \n"
    "    <Length>1</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"Int8BitSignedNegAddr\">        \n"
    "    <Address>-4</Address>            \n"
    "    <Length>1</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<Port Name=\"Port\"/>                    \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x0000, (int64_t)117, RW);
    Port.CreateEntry(0x0010, (int32_t)117, RW);
    Port.CreateEntry(0x0020, (int8_t)117, RW);
    Port.CreateEntry(0x0030, (uint64_t)117, RW);
    Port.CreateEntry(0x0040, (uint32_t)117, RW);
    Port.CreateEntry(0x0050, (uint8_t)117, RW);
//    Port.CreateEntry(0x0060, (int8_t)0xFE, RW); // *JS* removed warning
    Port.CreateEntry(0x0060, (int8_t)(uint8_t)0xFE, RW);
    Port.CreateEntry(-4, (int8_t)(uint8_t)0xFE, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    const CIntegerPtr ptrInt64Reg(Camera._GetNode("Int64"));
    CPPUNIT_ASSERT( ptrInt64Reg.IsValid() );
    CPPUNIT_ASSERT_EQUAL (intfIInteger, ptrInt64Reg->GetNode()->GetPrincipalInterfaceType());

    const CIntegerPtr ptrInt32Reg(Camera._GetNode("Int32"));
    CPPUNIT_ASSERT( ptrInt32Reg.IsValid() );

    const CIntegerPtr ptrInt8Reg(Camera._GetNode("Int8"));
    CPPUNIT_ASSERT( ptrInt8Reg.IsValid() );

    const CIntegerPtr ptrUInt64Reg(Camera._GetNode("UInt64"));
    CPPUNIT_ASSERT( ptrUInt64Reg.IsValid() );

    const CIntegerPtr ptrUInt32Reg(Camera._GetNode("UInt32"));
    CPPUNIT_ASSERT( ptrUInt32Reg.IsValid() );

    const CIntegerPtr ptrUInt8Reg(Camera._GetNode("UInt8"));
    CPPUNIT_ASSERT( ptrUInt8Reg.IsValid() );

    CPPUNIT_ASSERT_EQUAL( int64_t(INT64_MIN), ptrInt64Reg->GetMin() );
    CPPUNIT_ASSERT_EQUAL( int64_t(INT64_MAX), ptrInt64Reg->GetMax() );

    CPPUNIT_ASSERT_EQUAL( int64_t(INT32_MIN), ptrInt32Reg->GetMin() );
    CPPUNIT_ASSERT_EQUAL( int64_t(INT32_MAX), ptrInt32Reg->GetMax() );

    CPPUNIT_ASSERT_EQUAL( int64_t(INT8_MIN), ptrInt8Reg->GetMin() );
    CPPUNIT_ASSERT_EQUAL( int64_t(INT8_MAX), ptrInt8Reg->GetMax() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)0, ptrUInt64Reg->GetMin() );
    CPPUNIT_ASSERT_EQUAL( uint64_t(INT64_MAX), uint64_t(ptrUInt64Reg->GetMax()) );

    CPPUNIT_ASSERT_EQUAL( (int64_t)0, ptrUInt32Reg->GetMin() );
    CPPUNIT_ASSERT_EQUAL( uint64_t(UINT32_MAX), (uint64_t)ptrUInt32Reg->GetMax() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)0, ptrUInt8Reg->GetMin() );
    CPPUNIT_ASSERT_EQUAL( uint64_t(UINT8_MAX), (uint64_t)ptrUInt8Reg->GetMax() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)117, ptrInt64Reg->GetValue() );
    ptrInt64Reg->SetValue(42);
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrInt64Reg->GetValue() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)117, ptrInt32Reg->GetValue() );
    ptrInt32Reg->SetValue(42);
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrInt32Reg->GetValue() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)117, ptrInt8Reg->GetValue() );
    ptrInt8Reg->SetValue(42);
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrInt8Reg->GetValue() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)117, ptrUInt64Reg->GetValue() );
    ptrUInt64Reg->SetValue(42);
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrUInt64Reg->GetValue() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)117, ptrUInt32Reg->GetValue() );
    ptrUInt32Reg->SetValue(42);
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrUInt32Reg->GetValue() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)117, ptrUInt8Reg->GetValue() );
    ptrUInt8Reg->SetValue(42);
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrUInt8Reg->GetValue() );

    ptrInt32Reg->FromString("181920");
    CPPUNIT_ASSERT_EQUAL( gcstring("181920"), ptrInt32Reg->ToString() );
    CPPUNIT_ASSERT_THROW_EX(
        ptrInt32Reg->FromString("abc"),
        GenICam::InvalidArgumentException
    );
    CPPUNIT_ASSERT_EQUAL (PureNumber, ptrInt32Reg->GetRepresentation());

    CIntegerPtr ptrInt8BitSigned = Camera._GetNode("Int8BitSigned");
    CPPUNIT_ASSERT( ptrInt8BitSigned.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)-2, ptrInt8BitSigned->GetValue() );
    ptrInt8BitSigned->SetValue(-1);
    CPPUNIT_ASSERT_EQUAL( (int64_t)-1, ptrInt8BitSigned->GetValue() );

    // also one reg with negative address to follow happy path
    CIntegerPtr ptrInt8BitSignedNegAddr = Camera._GetNode("Int8BitSignedNegAddr");
    CPPUNIT_ASSERT( ptrInt8BitSignedNegAddr.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)-2, ptrInt8BitSignedNegAddr->GetValue() );
    ptrInt8BitSignedNegAddr->SetValue(-1);
    CPPUNIT_ASSERT_EQUAL( (int64_t)-1, ptrInt8BitSignedNegAddr->GetValue() );

}

void IntRegTestSuite::TestpLength()
{

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<IntReg Name=\"Integer\">               \n"
        "    <Address>0x0000</Address>           \n"
        "    <pLength>Length</pLength>           \n"
        "    <AccessMode>RW</AccessMode>         \n"
        "    <pPort>Port</pPort>                 \n"
        "    <Sign>Unsigned</Sign>               \n"
        "    <Endianess>LittleEndian</Endianess> \n"
        "</IntReg>                               \n"
        "                                        \n"
        "<IntReg Name=\"Signed\">               \n"
        "    <Address>0x1000</Address>           \n"
        "    <pLength>Length</pLength>           \n"
        "    <AccessMode>RW</AccessMode>         \n"
        "    <pPort>Port</pPort>                 \n"
        "    <Sign>Signed</Sign>               \n"
        "    <Endianess>LittleEndian</Endianess> \n"
        "</IntReg>                               \n"
        "                                        \n"
        "<Integer Name=\"Length\">               \n"
        "    <Value>1</Value>                    \n"
        "</Integer>                              \n"
        "                                        \n"
        "<Port Name=\"Port\"/>                   \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x0000, 0xffffffffffffffffLL, RW);
    Port.CreateEntry(0x1000, 0x00000000000000ffLL, RW);

    // connect the node map to the port
    Camera._Connect(&Port, "Port");

    CIntegerPtr ptrInteger = Camera._GetNode("Integer");
    CIntegerPtr ptrSigned = Camera._GetNode("Signed");
    CIntegerPtr ptrLength = Camera._GetNode("Length");

    *ptrLength = 0;
    CPPUNIT_ASSERT_THROW_EX( *ptrInteger(), OutOfRangeException );
    CPPUNIT_ASSERT_THROW_EX( *ptrSigned(), OutOfRangeException );
    *ptrLength = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffLL, *ptrInteger() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffLL, ptrInteger->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) -1LL, *ptrSigned() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 127LL, ptrSigned->GetMax() );
    *ptrLength = 2;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffLL, *ptrInteger() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffLL, ptrInteger->GetMax() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 255LL, *ptrSigned() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 32767LL, ptrSigned->GetMax() );
    *ptrLength = 3;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffffLL, *ptrInteger() );
    *ptrLength = 4;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffffffLL, *ptrInteger() );
    *ptrLength = 5;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffffffffLL, *ptrInteger() );
    *ptrLength = 6;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffffffffffLL, *ptrInteger() );
    *ptrLength = 7;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffffffffffffLL, *ptrInteger() );
    *ptrLength = 8;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0xffffffffffffffffLL, *ptrInteger() );
    *ptrLength = 9;
    CPPUNIT_ASSERT_THROW_EX( *ptrInteger(), OutOfRangeException );

}

void IntRegTestSuite::TestpLengthThroughReg()
{

    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<IntReg Name=\"Integer\">               \n"
        "    <Address>0x0000</Address>           \n"
        "    <pLength>Length</pLength>           \n"
        "    <AccessMode>RW</AccessMode>         \n"
        "    <pPort>Port</pPort>                 \n"
        "    <Sign>Unsigned</Sign>               \n"
        "    <Endianess>LittleEndian</Endianess> \n"
        "</IntReg>                               \n"
        "                                        \n"
        "<IntReg Name=\"Length\">               \n"
        "    <Address>0x1000</Address>           \n"
        "    <Length>1</Length>           \n"
        "    <AccessMode>RW</AccessMode>         \n"
        "    <pPort>Port</pPort>                 \n"
        "    <Sign>Unsigned</Sign>               \n"
        "    <Endianess>LittleEndian</Endianess> \n"
        "</IntReg>                               \n"
        "                                        \n"
        "<Port Name=\"Port\"/>                   \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    // Loading the nodemap should not fail due to port access (pLength) before connecting
    CPPUNIT_ASSERT_NO_THROW (Camera._LoadXMLFromString(CameraDescriptionFile));

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x0000, 0xffffffffffffffffLL, RW);
    Port.CreateEntry(0x1000, 0LL, RW);

    // connect the node map to the port
    Camera._Connect(&Port, "Port");

    CIntegerPtr ptrInteger = Camera._GetNode("Integer");
    CIntegerPtr ptrLength = Camera._GetNode("Length");

    *ptrLength = 0;
    CPPUNIT_ASSERT_THROW_EX( *ptrInteger(), OutOfRangeException );
    *ptrLength = 1;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffLL, *ptrInteger() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffLL, ptrInteger->GetMax() );
    *ptrLength = 2;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffLL, *ptrInteger() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffLL, ptrInteger->GetMax() );
    *ptrLength = 3;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffffLL, *ptrInteger() );
    *ptrLength = 4;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffffffLL, *ptrInteger() );
    *ptrLength = 5;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffffffffLL, *ptrInteger() );
    *ptrLength = 6;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffffffffffLL, *ptrInteger() );
    *ptrLength = 7;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffffffffffffLL, *ptrInteger() );
    *ptrLength = 8;
    CPPUNIT_ASSERT_EQUAL( (int64_t)0xffffffffffffffffLL, *ptrInteger() );
    *ptrLength = 9;
    CPPUNIT_ASSERT_THROW_EX( *ptrInteger(), OutOfRangeException );

}


void IntRegTestSuite::TestRegOverlap()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                        \n"
    "<IntReg Name=\"Base\">                \n"
    "    <Address>0x0000</Address>            \n"
    "    <Length>4</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <pInvalidator>SecondBase</pInvalidator> \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"NoInvalidator\">                \n"
    "    <Address>0x0000</Address>            \n"
    "    <Length>4</Length>                    \n"
    "    <AccessMode>RO</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"WithInvalidator\">                \n"
    "    <Address>0x0000</Address>            \n"
    "    <Length>4</Length>                    \n"
    "    <AccessMode>RO</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <pInvalidator>Base</pInvalidator> \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                        \n"
    "<IntReg Name=\"SecondBase\">                \n"
    "    <Address>0x0000</Address>            \n"
    "    <Length>4</Length>                    \n"
    "    <AccessMode>RW</AccessMode>            \n"
    "    <pPort>Port</pPort>                    \n"
    "    <pInvalidator>Base</pInvalidator> \n"
    "    <Sign>Signed</Sign>                    \n"
    "    <Endianess>LittleEndian</Endianess>    \n"
    "</IntReg>                                \n"
    "                                                \n"
    "<StructReg Comment=\"MyStruct\" >               \n"
    "   <Address>0x0100</Address>                         \n"
    "   <Length>4</Length>                           \n"
    "   <AccessMode>RW</AccessMode>                  \n"
    "   <pPort>Port</pPort>                        \n"
    "   <Endianess>LittleEndian</Endianess>          \n"
    "   <StructEntry Name=\"Bit0NoInvalidator\">                     \n"
    "     <Bit>0</Bit>                               \n"
    "   </StructEntry>                               \n"
    "   <StructEntry Name=\"AllBits\">                     \n"
    "     <LSB>0</LSB>                               \n"
    "     <MSB>31</MSB>                               \n"
    "   </StructEntry>                               \n"
    "   <StructEntry Name=\"Bit0WithInvalidator\">                     \n"
    "    <pInvalidator>AllBits</pInvalidator> \n"
    "     <Bit>0</Bit>                               \n"
    "   </StructEntry>                               \n"
    "</StructReg>                                    \n"
    "                                        \n"
    "<Port Name=\"Port\"/>                    \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // create and initialize a test port
    CTestPort Port;
    Port.CreateEntry(0x0000, (int32_t)0, RW);
    Port.CreateEntry(0x0100, (int32_t)0, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");

    const CIntegerPtr ptrBase(Camera._GetNode("Base"));
    CPPUNIT_ASSERT( ptrBase.IsValid() );
    CPPUNIT_ASSERT_EQUAL ( int64_t(0), ptrBase->GetValue());
    const CIntegerPtr ptrNoInvalidator(Camera._GetNode("NoInvalidator"));
    CPPUNIT_ASSERT( ptrNoInvalidator.IsValid() );
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrNoInvalidator->GetValue());
    const CIntegerPtr ptrWithInvalidator(Camera._GetNode("WithInvalidator"));
    CPPUNIT_ASSERT( ptrWithInvalidator.IsValid() );
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrWithInvalidator->GetValue());
    const CIntegerPtr ptrSecondBase(Camera._GetNode("SecondBase"));
    CPPUNIT_ASSERT( ptrSecondBase.IsValid() );
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrSecondBase->GetValue());

    ptrBase->SetValue (1);
    // the register without invalidator will not see the new value (pity) because of own cache
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrNoInvalidator->GetValue());
    // the register with invalidator will be updated
    CPPUNIT_ASSERT_EQUAL (int64_t(1), ptrWithInvalidator->GetValue());
    // cyclic pInvalidator dependency is OK and works as expected
    CPPUNIT_ASSERT_EQUAL (int64_t(1), ptrSecondBase->GetValue());
    ptrSecondBase->SetValue (2);
    CPPUNIT_ASSERT_EQUAL (int64_t(2), ptrBase->GetValue());
    // also the pInvalidator chain will do its job (SecondBase->Base->WithInvalidator)
    CPPUNIT_ASSERT_EQUAL (int64_t(2), ptrWithInvalidator->GetValue());

    // test overlapping entries in a struct-reg
    const CIntegerPtr ptrAllBits(Camera._GetNode("AllBits"));
    CPPUNIT_ASSERT( ptrAllBits.IsValid() );
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrAllBits->GetValue());
    const CIntegerPtr ptrBit0NoInvalidator(Camera._GetNode("Bit0NoInvalidator"));
    CPPUNIT_ASSERT( ptrBit0NoInvalidator.IsValid() );
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrBit0NoInvalidator->GetValue());
    const CIntegerPtr ptrBit0WithInvalidator(Camera._GetNode("Bit0WithInvalidator"));
    CPPUNIT_ASSERT( ptrBit0WithInvalidator.IsValid() );
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrBit0WithInvalidator->GetValue());

    ptrAllBits->SetValue (0xFFFFFFFF);
    // the entry without invalidator will not see the new value (pity) because of own cache
    CPPUNIT_ASSERT_EQUAL (int64_t(0), ptrBit0NoInvalidator->GetValue());
    // the entry with invalidator will be updated
    CPPUNIT_ASSERT_EQUAL (int64_t(1), ptrBit0WithInvalidator->GetValue());
}
