//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/


#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "IntSwissKnifeTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "StopWatch.h"

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( IntSwissKnifeTestSuite, "GenApiTest");

void IntSwissKnifeTestSuite::TestValueAccess()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntSwissKnife Name=\"Result\">                \n"
    "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
    "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
    "    <Formula> X * Y + 12 </Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueX\">                        \n"
    "    <Value>3</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueY\">                        \n"
    "    <Value>10</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrResult(Camera._GetNode("Result"));
    CPPUNIT_ASSERT( ptrResult.IsValid() );
    CNodePtr ptrResultNode(ptrResult );
    CPPUNIT_ASSERT( ptrResultNode.IsValid() );
    CPPUNIT_ASSERT_EQUAL (intfIInteger, ptrResult->GetNode()->GetPrincipalInterfaceType());
    CPPUNIT_ASSERT_EQUAL (PureNumber, ptrResult->GetRepresentation());

    CPPUNIT_ASSERT_THROW_EX( ptrResult->SetValue(77), GenICam::GenericException );
    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrResult->GetValue() );

    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : IntSwissKnife <=> Integer (X = const)");
    LOG_PERFORMANCE("                      <=> Integer (Y = const)");

    ptrResultNode->InvalidateNode();
    TEST_BEGIN(1000)
        const int64_t Result(ptrResult->GetValue());
        StopWatch.PauseBegin();
        ptrResultNode->InvalidateNode();
        StopWatch.PauseEnd();
    TEST_END( IntSwissKnife::GetValue (dynamic, without caching) )

    {
        // tests the static case
        Test_TestCamera_v1_0::CTestCamera_v1_0 Camera;
        Camera._LoadDLL();

        CPPUNIT_ASSERT_EQUAL( (int64_t)42, Camera.MyIntSwissKnife.GetValue() );

        Camera.MyIntSwissKnife.GetNode()->InvalidateNode();
        TEST_BEGIN(1000)
            const int64_t Result(Camera.MyIntSwissKnife.GetValue());
            StopWatch.PauseBegin();
            Camera.MyIntSwissKnife.GetNode()->InvalidateNode();
            StopWatch.PauseEnd();
        TEST_END( IntSwissKnife::GetValue (static, without caching) )

    }
    {
        // tests the static case
        Test_TestCamera_v1_1::CTestCamera_v1_1 Camera;
        Camera._LoadDLL();

        CPPUNIT_ASSERT_EQUAL( (int64_t)42, Camera.MyIntSwissKnife.GetValue() );

        Camera.MyIntSwissKnife.GetNode()->InvalidateNode();
        TEST_BEGIN(1000)
            const int64_t Result(Camera.MyIntSwissKnife.GetValue());
            StopWatch.PauseBegin();
            Camera.MyIntSwissKnife.GetNode()->InvalidateNode();
            StopWatch.PauseEnd();
        TEST_END( IntSwissKnife::GetValue (static, without caching) )

    }
}


void IntSwissKnifeTestSuite::TestIntSwissKnifeTrig()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>2</Value>                                                \n"
    "    </Integer>                                                            \n"
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>-2</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                    \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Int0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>-1*((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsTrigInt\">                                    \n"
    "        <pVariable Name=\"SK0\">SwsMultDivInt</pVariable>                \n"
    "        <Formula>atan(tan(asin(sin(acos(cos(PI*(SK0)))/PI))))</Formula> \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr
        ptrSwsMultDivInt(Camera._GetNode("SwsMultDivInt")),
        ptrSwsTrigInt(Camera._GetNode("SwsTrigInt"));

    // Check trig parsing for IntSwsKnife
    const int64_t multDivInt(-1);
    CPPUNIT_ASSERT_EQUAL(multDivInt, ptrSwsMultDivInt->GetValue());

    // Beware: an IntSwissKnife does not support float functions like sin, cos etc.
    CPPUNIT_ASSERT_THROW_EX(
            ptrSwsTrigInt->GetValue(),
            GenICam::LogicalErrorException
        );
}


void IntSwissKnifeTestSuite::TestIntSwissKnifeReturnDbl()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>2</Value>                                                \n"
    "    </Integer>                                                            \n"
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>-2</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Int0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>-1*((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsTruncFracInt\">                            \n"
    "        <pVariable Name=\"SK0\">Int1</pVariable>                        \n"
    "        <Formula>(trunc(SK0*3) + frac(SK0*3))</Formula>                    \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsCeilFloorInt\">                            \n"
    "        <pVariable Name=\"SK0\">Int1</pVariable>                        \n"
    "        <Formula>(ceil(SK0) - floor(SK0))</Formula>                        \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsRoundInt\">                                \n"
    "        <pVariable Name=\"SK0\">Int0</pVariable>                        \n"
    "        <Formula>round(SK0)</Formula>                                    \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsSqrtInt\">                                    \n"
    "        <pVariable Name=\"SK0\">Int0</pVariable>                        \n"
    "        <Formula>sqrt(SK0*2)</Formula>                                    \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsPowersInt\">                                \n"
    "        <pVariable Name=\"SK0\">Int0</pVariable>                        \n"
    "        <Formula>ln(lg(SK0*5)*e*exp(SK0))</Formula>                        \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsNegSgnInt\">                                \n"
    "        <pVariable Name=\"SK0\">Int1</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">SwsMultDivInt</pVariable>                \n"
    "        <Formula>(neg(SK0)*sgn(SK0))*neg(SK1)</Formula>                    \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrSwsMultDivInt(Camera._GetNode("SwsMultDivInt"));
    CIntegerPtr ptrSwsTruncFracInt(Camera._GetNode("SwsTruncFracInt"));
    CIntegerPtr ptrSwsCeilFloorInt(Camera._GetNode("SwsCeilFloorInt"));
    CIntegerPtr ptrSwsRoundInt(Camera._GetNode("SwsRoundInt"));
    CIntegerPtr ptrSwsSqrtInt(Camera._GetNode("SwsSqrtInt"));
    CIntegerPtr ptrSwsPowersInt(Camera._GetNode("SwsPowersInt"));
    CIntegerPtr ptrSwsNegSgnInt(Camera._GetNode("SwsNegSgnInt"));

    // Check math functions that return a dbl within the IntSwissKnife
    const int64_t multDivInt(-1), negSgnInt(-2);

    CPPUNIT_ASSERT_EQUAL(multDivInt, ptrSwsMultDivInt->GetValue());
    CPPUNIT_ASSERT_THROW_EX( ptrSwsTruncFracInt->GetValue(), GenICam::LogicalErrorException);
    CPPUNIT_ASSERT_THROW_EX( ptrSwsCeilFloorInt->GetValue(), GenICam::LogicalErrorException);
    CPPUNIT_ASSERT_THROW_EX( ptrSwsRoundInt->GetValue(), GenICam::LogicalErrorException);
    CPPUNIT_ASSERT_THROW_EX( ptrSwsSqrtInt->GetValue(), GenICam::LogicalErrorException);
    CPPUNIT_ASSERT_THROW_EX( ptrSwsPowersInt->GetValue(), GenICam::LogicalErrorException);
    CPPUNIT_ASSERT_EQUAL( negSgnInt, ptrSwsNegSgnInt->GetValue());

}


void IntSwissKnifeTestSuite::TestIntSwissKnifeVariableMinMax()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>-1</Value>                                                \n"
    "        <Min>-1</Min>                                                    \n"
    "        <Max>2</Max>                                                    \n"
       "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Int0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>neg((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CIntegerPtr ptrSwsMultDivInt(Camera._GetNode("SwsMultDivInt"));
    // Check that variable ranges are respected
    const int64_t multDivInt(-1);
    CPPUNIT_ASSERT_EQUAL(multDivInt, ptrSwsMultDivInt->GetValue());

}

void IntSwissKnifeTestSuite::TestIntSwissKnifeVariableValue()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Int0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>-((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CIntegerPtr ptrSwsMultDivInt(Camera._GetNode("SwsMultDivInt"));
    // Check that variable values are defined
    const int64_t multDivInt(-1);
    CPPUNIT_ASSERT_EQUAL(multDivInt, ptrSwsMultDivInt->GetValue());
}

 void IntSwissKnifeTestSuite::TestIntSwissKnifeValueType()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Int0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>-((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CIntegerPtr ptrSwsMultDivInt(Camera._GetNode("SwsMultDivInt"));
    // Check that variable values are of the expected type
    const int64_t multDivInt(-1);
    CPPUNIT_ASSERT_EQUAL(multDivInt, ptrSwsMultDivInt->GetValue());
}

  void IntSwissKnifeTestSuite::TestIntSwissKnifeArgumentType()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <Boolean Name=\"Bln0\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Boolean>                                                            \n"
        "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Bln0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>-((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;

    if( GenApiSchemaVersion == v1_0 )
    {
        // pVariable must not point to a Boolean
        CPPUNIT_ASSERT_THROW_EX(Camera._LoadXMLFromString(CameraDescriptionFile), RuntimeException);
    }
    else
    {
        // pVariable is now polymorph
        Camera._LoadXMLFromString(CameraDescriptionFile);
    }
}


  // Pleora FG
// ---------------------------------------------------------------------------
// This method tests is used to trig cases where not all variables
// are present in order to evaluate an expression. This will trigger
// some yet-to-be-covered cases in VariableDelegate.cpp
//
void IntSwissKnifeTestSuite::TestVariableDelegates()
{
    gcstring CameraDescriptionFile =

    CDFHeader +

    "<Boolean Name =\"FalseNode\">                                \n"
    "  <Value>false</Value>                                        \n"
    "</Boolean>                                                    \n"

    "<Integer Name=\"Int0\">                                    \n"
    "  <Value>4</Value>                                            \n"
    "</Integer>                                                    \n"

    "<Integer Name=\"Int1\">                                    \n"
    "  <Value>2</Value>                                            \n"
    "</Integer>                                                    \n"

    "<IntSwissKnife Name=\"SwsAddInt\">                            \n"
    "  <pVariable Name=\"V0\">Int0</pVariable>                    \n"
    "  <pVariable Name=\"V1\">Int1</pVariable>                    \n"
    "  <Formula>V0+V1</Formula>                                    \n"
    "  <Representation>Linear</Representation>                    \n"
    "</IntSwissKnife>                                            \n"

    "<Integer Name=\"IntNotAvailable\">                            \n"
    "  <pIsAvailable>FalseNode</pIsAvailable>                    \n"
    "  <Value>3</Value>                                            \n"
    "</Integer>                                                    \n"

    "<IntSwissKnife Name=\"SwsAddIntNA\">                        \n"
    "  <pVariable Name=\"V0\">Int0</pVariable>                    \n"
    "  <pVariable Name=\"V1\">IntNotAvailable</pVariable>        \n"
    "  <Formula>V0+V1</Formula>                                    \n"
    "</IntSwissKnife>                                            \n"

    "<Integer Name=\"IntNotImplemented\">                        \n"
    "  <pIsImplemented>FalseNode</pIsImplemented>                \n"
    "  <Value>3</Value>                                            \n"
    "</Integer>                                                    \n"

    "<IntSwissKnife Name=\"SwsAddIntNI\">                        \n"
    "  <pVariable Name=\"V0\">Int0</pVariable>                    \n"
    "  <pVariable Name=\"V1\">IntNotImplemented</pVariable>        \n"
    "  <Formula>V0+V1</Formula>                                    \n"
    "</IntSwissKnife>                                            \n"

    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Regular add int
    CIntegerPtr ptrSwsAddInt(Camera._GetNode("SwsAddInt"));
    CPPUNIT_ASSERT( ptrSwsAddInt->GetAccessMode() == RO );
    CPPUNIT_ASSERT( ptrSwsAddInt->GetRepresentation() == Linear );
    CPPUNIT_ASSERT( ptrSwsAddInt->GetInc() == 1 ); // Is (and should be) 1 by definition
    const int64_t
        lIntMin(ptrSwsAddInt->GetMin()),
        lIntMax(ptrSwsAddInt->GetMax());

    // NI add int
    CIntegerPtr ptrSwsAddIntNI(Camera._GetNode("SwsAddIntNI"));
    CPPUNIT_ASSERT( ptrSwsAddIntNI->GetAccessMode() == NI );

    // NA add int
    CIntegerPtr ptrSwsAddIntNA(Camera._GetNode("SwsAddIntNA"));
    CPPUNIT_ASSERT( ptrSwsAddIntNA->GetAccessMode() == NA );
}


void IntSwissKnifeTestSuite::TestInvalidFormulas()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntSwissKnife Name=\"Result\">                \n"
    "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
    "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
    "    <Formula> Z + 12 </Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueX\">                        \n"
    "    <Value>3</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueY\">                        \n"
    "    <Value>10</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrResult(Camera._GetNode("Result"));
    CPPUNIT_ASSERT( ptrResult.IsValid() );
    CPPUNIT_ASSERT_THROW_EX (ptrResult->GetValue(), GenICam::LogicalErrorException);
}

// Mantis #288
void IntSwissKnifeTestSuite::TestVariableNames()
{

    // the implementation belongig to schema v1.0 can handle uppercase names onle
    if( GenApiSchemaVersion == v1_0 )
        return;

    gcstring CameraDescriptionFile =
    CDFHeader +
         "    <IntSwissKnife Name=\"Knife1\">                               \n"
         "        <pVariable Name=\"BLADE\">Blade</pVariable>        \n"
         "        <Formula>100*BLADE</Formula>                             \n"
         "    </IntSwissKnife>                                                \n"
         "    <IntSwissKnife Name=\"Knife2\">                               \n"
         "        <pVariable Name=\"bLaDe\">Blade</pVariable>        \n"
         "        <Formula>100*bLaDe</Formula>                             \n"
         "    </IntSwissKnife>                                                \n"
         "    <IntSwissKnife Name=\"Knife3\">                               \n"
         "        <pVariable Name=\"blade\">Blade</pVariable>        \n"
         "        <Formula>100*blade</Formula>                             \n"
         "    </IntSwissKnife>                                                \n"
         "    <IntSwissKnife Name=\"Knife4\">                               \n"
         "        <pVariable Name=\"BLADE\">Blade</pVariable>        \n"
         "        <Formula>100*Blade</Formula>                             \n"
         "    </IntSwissKnife>                                                \n"
         "                                                                    \n"
         "    <Integer Name=\"Blade\">                                 \n"
         "        <Value>1</Value>                                            \n"
         "    </Integer>                                                      \n"
    + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

    CIntegerPtr ptrBlade = Camera._GetNode("Knife1");
    CPPUNIT_ASSERT( (bool)ptrBlade );
    CPPUNIT_ASSERT_EQUAL(int64_t(100), ptrBlade->GetValue());

    ptrBlade = Camera._GetNode("Knife2");
    CPPUNIT_ASSERT( (bool)ptrBlade );
    CPPUNIT_ASSERT_EQUAL(int64_t(100), ptrBlade->GetValue());

    ptrBlade = Camera._GetNode("Knife3");
    CPPUNIT_ASSERT( (bool)ptrBlade );
    CPPUNIT_ASSERT_EQUAL(int64_t(100), ptrBlade->GetValue());

    ptrBlade = Camera._GetNode("Knife4");
    CPPUNIT_ASSERT_THROW_EX( ptrBlade->GetValue(), GenICam::LogicalErrorException );

}

// Added with std::vector change in Int64MathParser
void IntSwissKnifeTestSuite::TestBig()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "\n"
    "<Integer Name=\"MySelector\">\n"
    "    <Value>0</Value>\n"
    "</Integer>\n"
    "\n"
    "<IntSwissKnife Name=\"MyBigSwissKnife\">\n"
    "    <pVariable Name=\"SEL\">MySelector</pVariable>\n"
    "    <Formula>(SEL = 0) ? 0x00000000 : ((SEL = 1) ? 0x00000000 : ((SEL = 2) ? 0x00000000 : "
    "((SEL = 3) ? 0x00000000 : ((SEL = 4) ? 0x00000000 : ((SEL = 5) ? 0x00000000 : "
    "((SEL = 6) ? 0x00000000 : ((SEL = 7) ? 0x00000000 : ((SEL = 8) ? 0x00000000 : "
    "((SEL = 9) ? 0x00000000 : ((SEL = 10) ? 0x00000000 : ((SEL = 11) ? 0x00000000 : "
    "((SEL = 12) ? 0x00000000 : ((SEL = 13) ? 0x00000000 : ((SEL = 14) ? 0x00000000 : "
    "((SEL = 15) ? 0x00000000 : ((SEL = 16) ? 0x00000000 : ((SEL = 17) ? 0x00000000 : "
    "((SEL = 18) ? 0x00000000 : ((SEL = 19) ? 0x00000000 : ((SEL = 20) ? 0x00000000 : "
    "((SEL = 21) ? 0x00000000 : ((SEL = 22) ? 0x00000000 : ((SEL = 23) ? 0x00000000 : "
    "((SEL = 24) ? 0x00000000 : ((SEL = 25) ? 0x00000000 : ((SEL = 26) ? 0x00000000 : "
    "((SEL = 27) ? 0x00000000 : ((SEL = 28) ? 0x00000000 : ((SEL = 29) ? 0x00000000 : "
    "((SEL = 30) ? 0x00000000 : ((SEL = 31) ? 0x00000000 : ((SEL = 32) ? 0x00000000 : "
    "((SEL = 33) ? 0x00000000 : ((SEL = 34) ? 0x00000000 : ((SEL = 35) ? 0x00000000 : "
    "((SEL = 36) ? 0x00000000 : ((SEL = 37) ? 0x00000000 : ((SEL = 38) ? 0x00000000 : "
    "((SEL = 39) ? 0x00000000 : ((SEL = 40) ? 0x00000000 : ((SEL = 41) ? 0x00000000 : "
    "((SEL = 42) ? 0x12345678 : ((SEL = 43) ? 0x00000000 : ((SEL = 44) ? 0x00000000 : "
    "((SEL = 45) ? 0x00000000 : ((SEL = 46) ? 0x00000000 : ((SEL = 47) ? 0x00000000 : "
    "((SEL = 48) ? 0x00000000 : ((SEL = 49) ? 0x00000000 : ((SEL = 50) ? 0x00000000 : "
    "((SEL = 51) ? 0x00000000 : ((SEL = 52) ? 0x00000000 : ((SEL = 53) ? 0x00000000 : "
    "((SEL = 54) ? 0x00000000 : ((SEL = 55) ? 0x00000000 : ((SEL = 56) ? 0x00000000 : "
    "((SEL = 57) ? 0x00000000 : ((SEL = 58) ? 0x00000000 : ((SEL = 59) ? 0x00000000 : "
    "((SEL = 60) ? 0x00000000 : ((SEL = 61) ? 0x00000000 : ((SEL = 62) ? 0x00000000 : "
    "((SEL = 63) ? 0x00000000 : ((SEL = 64) ? 0x00000000 : ((SEL = 65) ? 0x00000000 : "
    "((SEL = 66) ? 0x00000000 : ((SEL = 67) ? 0x00000000 : ((SEL = 68) ? 0x00000000 : "
    "((SEL = 69) ? 0x00000000 : ((SEL = 70) ? 0x00000000 : ((SEL = 71) ? 0x00000000 : "
    "((SEL = 72) ? 0x00000000 : ((SEL = 73) ? 0x00000000 : ((SEL = 74) ? 0x00000000 : "
    "((SEL = 75) ? 0x00000000 : ((SEL = 76) ? 0x00000000 : ((SEL = 77) ? 0x00000000 : "
    "((SEL = 78) ? 0x00000000 : ((SEL = 79) ? 0x00000000 : ((SEL = 80) ? 0x00000000 : "
    "((SEL = 81) ? 0x00000000 : ((SEL = 82) ? 0x00000000 : ((SEL = 83) ? 0x00000000 : "
    "((SEL = 84) ? 0x00000000 : ((SEL = 85) ? 0x00000000 : ((SEL = 86) ? 0x00000000 : "
    "((SEL = 87) ? 0x00000000 : ((SEL = 88) ? 0x00000000 : ((SEL = 89) ? 0x00000000 : "
    "((SEL = 90) ? 0x00000000 : ((SEL = 91) ? 0x00000000 : ((SEL = 92) ? 0x00000000 : "
    "((SEL = 93) ? 0x00000000 : ((SEL = 94) ? 0x00000000 : ((SEL = 95) ? 0x00000000 : "
    "((SEL = 96) ? 0x00000000 : ((SEL = 97) ? 0x00000000 : ((SEL = 98) ? 0x00000000 : "
    "((SEL = 99) ? 0x00000000 : ((SEL = 100) ? 0x00000000 : ((SEL = 101) ? 0x00000000 : "
    "((SEL = 102) ? 0x00000000 : ((SEL = 103) ? 0x00000000 : ((SEL = 104) ? 0x00000000 : "
    "((SEL = 105) ? 0x00000000 : ((SEL = 106) ? 0x00000000 : ((SEL = 107) ? 0x00000000 : "
    "((SEL = 108) ? 0x00000000 : ((SEL = 109) ? 0x00000000 : ((SEL = 110) ? 0x00000000 : "
    "((SEL = 111) ? 0x00000000 : ((SEL = 112) ? 0x00000000 : ((SEL = 113) ? 0x00000000 : "
    "((SEL = 114) ? 0x00000000 : ((SEL = 115) ? 0x00000000 : ((SEL = 116) ? 0x00000000 : "
    "((SEL = 117) ? 0x00000000 : ((SEL = 118) ? 0x00000000 : ((SEL = 119) ? 0x00000000 : "
    "((SEL = 120) ? 0x00000000 : ((SEL = 121) ? 0x00000000 : ((SEL = 122) ? 0x00000000 : "
    "((SEL = 123) ? 0x00000000 : ((SEL = 124) ? 0x00000000 : ((SEL = 125) ? 0x00000000 : "
    "((SEL = 126) ? 0x00000000 : ((SEL = 127) ? 0x00000000 : (0xFFFFFFFF)))))))))))))))))"
    "))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))"
    ")))))))))))))))))))))))))))</Formula>\n"
    "</IntSwissKnife>\n"
    "\n"
    + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

    IInteger *lSelector = dynamic_cast<IInteger *>( Camera._GetNode("MySelector") );
    CPPUNIT_ASSERT( lSelector != NULL );

    IInteger *lSwissKnife = dynamic_cast<IInteger *>( Camera._GetNode("MyBigSwissKnife") );
    CPPUNIT_ASSERT( lSwissKnife != NULL );

    lSelector->SetValue( 0 );
    int64_t lValue = lSwissKnife->GetValue();
    CPPUNIT_ASSERT( lValue == 0x00000000 );

    lSelector->SetValue( 42 );
    lValue = lSwissKnife->GetValue();
    CPPUNIT_ASSERT( lValue == 0x12345678 );

    lSelector->SetValue( 128 );
    lValue = lSwissKnife->GetValue();
    CPPUNIT_ASSERT( lValue == 0xFFFFFFFF );
}

void IntSwissKnifeTestSuite::TestConstantAndExpression()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<IntSwissKnife Name=\"Result\">                \n"
    "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
    "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
    "    <Constant Name=\"Two\">2</Constant>    \n"
    "    <Expression Name=\"TwoX\">2*X</Expression>    \n"
    "    <Formula> TwoX * Y + Two </Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueX\">                        \n"
    "    <Value>5</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueY\">                        \n"
    "    <Value>4</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Result2\">                \n"
    "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
    "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
    "    <Constant Name=\"Two\">2</Constant>    \n"
    "    <Expression Name=\"TwoX\">2*X</Expression>    \n"
    "    <Expression Name=\"FourX\">2*TwoX</Expression>    \n"
    "    <Formula> 2*FourX </Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrResult(Camera._GetNode("Result"));
    CPPUNIT_ASSERT( ptrResult.IsValid() );
    CNodePtr ptrResultNode(ptrResult );
    CPPUNIT_ASSERT( ptrResultNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrResult->GetValue() );

    CIntegerPtr ptrResult2(Camera._GetNode("Result2"));
    CPPUNIT_ASSERT( ptrResult2.IsValid() );

    CPPUNIT_ASSERT_EQUAL( (int64_t)40, ptrResult2->GetValue() );
}

void IntSwissKnifeTestSuite::TestVariableExtensions()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescriptionFile =
        CDFHeader +

        "    <IntSwissKnife Name=\"Knife\">                             \n"
        "        <pVariable Name=\"BLADE.Min\">Blade</pVariable>     \n"
        "        <Formula>10*BLADE.Min</Formula>                         \n"
        "    </IntSwissKnife>                                           \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"BladeMin\">                             \n"
        "        <pVariable Name=\"BLADE.Min\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Min</Formula>                           \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"BladeMax\">                             \n"
        "        <pVariable Name=\"BLADE.Max\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Max</Formula>                           \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"BladeInc\">                             \n"
        "        <pVariable Name=\"BLADE.Inc\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Inc</Formula>                           \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"BladeVal\">                             \n"
        "        <pVariable Name=\"BLADE.Value\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Value</Formula>                           \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"BladeAccessMode\">                      \n"
        "        <pVariable Name=\"BLADE.AccessMode\">Blade</pVariable> \n"
        "        <Formula>BLADE.AccessMode</Formula>                    \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"BladeVisibility\">                      \n"
        "        <pVariable Name=\"BLADE.Visibility\">Blade</pVariable> \n"
        "        <Formula>BLADE.Visibility</Formula>                    \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"BladeCachingMode\">                      \n"
        "        <pVariable Name=\"BLADE.CachingMode\">Blade</pVariable> \n"
        "        <Formula>BLADE.CachingMode</Formula>                    \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <Integer Name=\"Blade\">                                   \n"
        "        <Value>2</Value>                                       \n"
        "        <Min>-10</Min>                                         \n"
        "        <Max>10</Max>                                          \n"
        "        <Inc>2</Inc>                                          \n"
        "    </Integer>                                                 \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"Dagger\">                                \n"
        "        <pVariable Name=\"OPTSEL.Entry.Option1\">OptionSelector</pVariable>            \n"
        "        <pVariable Name=\"OPTSEL.Entry.Option2\">OptionSelector</pVariable>            \n"
        "        <Formula>OPTSEL.Entry.Option1 + OPTSEL.Entry.Option2</Formula>                            \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "<Enumeration Name=\"OptionSelector\">              \n"
        "    <EnumEntry Name=\"Option1\">      \n"
        "        <Value>10</Value>                \n"
        "        <NumericValue>1.5</NumericValue> \n"
        "   </EnumEntry>                          \n"
        "    <EnumEntry Name=\"Option2\">      \n"
        "        <Value>20</Value>                \n"
        "   </EnumEntry>                          \n"
        "    <Value>10</Value>               \n"
        "</Enumeration>                           \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"DaggerNoEnum\">                      \n"
        "        <pVariable Name=\"Blade.Entry.Foo\">Blade</pVariable> \n"
        "        <Formula>Blade.Entry.Foo</Formula>                    \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"
        "    <IntSwissKnife Name=\"DaggerNoEnumEntry\">                      \n"
        "        <pVariable Name=\"OptionSelector.Entry.Foo\">OptionSelector</pVariable> \n"
        "        <Formula>OptionSelector.Entry.Foo</Formula>                    \n"
        "    </IntSwissKnife>                                              \n"
        "                                                               \n"

        + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

    CIntegerPtr ptrKnife = Camera._GetNode("Knife");
    CIntegerPtr ptrBlade = Camera._GetNode("Blade");
    CIntegerPtr ptrDagger = Camera._GetNode("Dagger");

    *ptrBlade = -4;
    CPPUNIT_ASSERT_EQUAL( (int64_t) -100LL, *ptrKnife() );

    CPPUNIT_ASSERT_EQUAL( (int64_t) 30LL, *ptrDagger() );

    CIntegerPtr ptrBladeMin = Camera._GetNode("BladeMin");
    CPPUNIT_ASSERT_EQUAL( (int64_t) -10LL, *ptrBladeMin() );

    CIntegerPtr ptrBladeMax = Camera._GetNode("BladeMax");
    CPPUNIT_ASSERT_EQUAL( (int64_t) 10LL, *ptrBladeMax() );

    CIntegerPtr ptrBladeInc = Camera._GetNode("BladeInc");
    CPPUNIT_ASSERT_EQUAL( (int64_t) 2LL, *ptrBladeInc() );

    CIntegerPtr ptrBladeVal = Camera._GetNode("BladeVal");
    CPPUNIT_ASSERT_EQUAL( (int64_t) -4LL, *ptrBladeVal() );

    CIntegerPtr ptrBladeVisibility = Camera._GetNode("BladeVisibility");
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0LL, *ptrBladeVisibility() );

    CIntegerPtr ptrBladeAccessMode = Camera._GetNode("BladeAccessMode");
    CPPUNIT_ASSERT_EQUAL( (int64_t) 4LL, *ptrBladeAccessMode() );

    CIntegerPtr ptrBladeCachingMode = Camera._GetNode("BladeCachingMode");
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, *ptrBladeCachingMode() );

    CIntegerPtr ptrDaggerNoEnum = Camera._GetNode("DaggerNoEnum");
    CPPUNIT_ASSERT_THROW_EX( *ptrDaggerNoEnum(), RuntimeException );

    CIntegerPtr ptrDaggerNoEnumEntry = Camera._GetNode("DaggerNoEnumEntry");
    CPPUNIT_ASSERT_THROW_EX( *ptrDaggerNoEnumEntry(), RuntimeException );

}


void IntSwissKnifeTestSuite::TestMathParser()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Integer Name=\"A\">                            \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                      \n"
    "                                                \n"
    "<Integer Name=\"B\">                            \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                      \n"
    "                                                \n"
    "<Integer Name=\"C\">                            \n"
    "    <Value>0</Value>                            \n"
    "</Integer>                                      \n"
    "                                                \n"
    "<IntSwissKnife Name=\"BitNot\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[  ~A ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"BitAnd\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A&B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"BitOr\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A|B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"BitXor\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A^B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"BitLeft\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<<B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"BitRight\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A>>B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Power\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A**B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Neq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<>B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Eq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A=B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Geq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A>=B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Leq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<=B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Greater\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A>B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Less\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"And\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A&&B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Or\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A||B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Plus\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A+B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Minus\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A-B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Times\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A*B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Div\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A/B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Mod\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A%B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Ternary\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <pVariable Name=\"C\">C</pVariable>    \n"
    "    <Formula><![CDATA[ A?B:C ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Precedence1\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <pVariable Name=\"C\">C</pVariable>    \n"
    "    <Formula><![CDATA[ A+B*C ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Brackets1\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <pVariable Name=\"C\">C</pVariable>    \n"
    "    <Formula><![CDATA[ (A+B)*C ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Sgn\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ SGN(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Neg\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ NEG(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Digits1\">                    \n"
    "    <Formula><![CDATA[ 10+0x10+010 ]]></Formula>                \n" // 010 treated as decimal !?!
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Digits2\">                    \n"
    "    <Formula><![CDATA[ +2 ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Digits3\">                    \n"
    "    <Formula><![CDATA[ -2 ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "                                                \n"
    "                                                \n"
    "                                                \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn1\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ TRUNC(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn2\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ FLOOR(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn3\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ CEIL(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn4\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ROUND(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn5\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ ROUND(A,B) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn6\">                    \n"
    "    <Formula><![CDATA[ PI ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn7\">                    \n"
    "    <Formula><![CDATA[ E ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn8\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ABS(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn9\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ EXP(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn10\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ LN(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn11\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ LG(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn12\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ SQRT(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn13\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ SIN(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn14\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ COS(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn15\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ TAN(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn16\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ASIN(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn17\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ACOS(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn18\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ATAN(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"FloatOnlyFn19\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ FRAC(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "                                                \n"
    "                                                \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid1\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ /A ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid2\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ A/ ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid3\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ / ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid4\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A==B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid5\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A$B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid6\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A#B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid7\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ !A ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid8\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ FOOBAR(A) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid9\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ (A+B ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid10\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A+B) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid11\">                    \n"
    "    <Formula></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid12\">                    \n"
    "    <Formula>    </Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid13\">                    \n"
    "    <Formula>1 2 3</Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid14\">                    \n"
    "    <Formula>0x</Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid15\">                    \n"
    "    <Formula>\"abc\"</Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid16\">                    \n"
    "    <Formula>\'abc\'</Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid17\">                    \n"
    "    <Formula>/* aha */ 1</Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid18\">                    \n"
    "    <Formula>\'abc</Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid19\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ () ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid20\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ SGN() ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "<IntSwissKnife Name=\"Invalid21\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ SGN(A,B) ]]></Formula>                \n"
    "</IntSwissKnife>                                        \n"
    "                                                \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrA(Camera._GetNode("A"));
    CPPUNIT_ASSERT( ptrA.IsValid() );
    CIntegerPtr ptrB(Camera._GetNode("B"));
    CPPUNIT_ASSERT( ptrB.IsValid() );
    CIntegerPtr ptrC(Camera._GetNode("C"));
    CPPUNIT_ASSERT( ptrC.IsValid() );

    CIntegerPtr ptrBitNot(Camera._GetNode("BitNot"));
    CPPUNIT_ASSERT( ptrBitNot.IsValid() );
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0xFFFFFFFFFFFFFFFFLL, ptrBitNot->GetValue());
    ptrA->SetValue (0x1111111111111111LL);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0xEEEEEEEEEEEEEEEELL, ptrBitNot->GetValue());

    CIntegerPtr ptrBitAnd(Camera._GetNode("BitAnd"));
    CPPUNIT_ASSERT( ptrBitAnd.IsValid() );
    ptrA->SetValue (0xCCCCCCCCCCCCCCCCLL);
    ptrB->SetValue (0xAAAAAAAAAAAAAAAALL);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0x8888888888888888LL, ptrBitAnd->GetValue());

    CIntegerPtr ptrBitOr(Camera._GetNode("BitOr"));
    CPPUNIT_ASSERT( ptrBitOr.IsValid() );
    ptrA->SetValue (0xCCCCCCCCCCCCCCCCLL);
    ptrB->SetValue (0xAAAAAAAAAAAAAAAALL);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0xEEEEEEEEEEEEEEEELL, ptrBitOr->GetValue());

    CIntegerPtr ptrBitXor(Camera._GetNode("BitXor"));
    CPPUNIT_ASSERT( ptrBitXor.IsValid() );
    ptrA->SetValue (0xCCCCCCCCCCCCCCCCLL);
    ptrB->SetValue (0xAAAAAAAAAAAAAAAALL);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0x6666666666666666LL, ptrBitXor->GetValue());

    CIntegerPtr ptrBitLeft(Camera._GetNode("BitLeft"));
    CPPUNIT_ASSERT( ptrBitLeft.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)4, ptrBitLeft->GetValue());

    CIntegerPtr ptrBitRight(Camera._GetNode("BitRight"));
    CPPUNIT_ASSERT( ptrBitRight.IsValid() );
    ptrA->SetValue (8);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)2, ptrBitRight->GetValue());

    CIntegerPtr ptrPower(Camera._GetNode("Power"));
    CPPUNIT_ASSERT( ptrPower.IsValid() );
    ptrA->SetValue (3);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)9, ptrPower->GetValue());

    CIntegerPtr ptrNeq(Camera._GetNode("Neq"));
    CPPUNIT_ASSERT( ptrNeq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrNeq->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrNeq->GetValue());

    CIntegerPtr ptrEq(Camera._GetNode("Eq"));
    CPPUNIT_ASSERT( ptrEq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrEq->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrEq->GetValue());

    CIntegerPtr ptrGeq(Camera._GetNode("Geq"));
    CPPUNIT_ASSERT( ptrGeq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrGeq->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrGeq->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrGeq->GetValue());

    CIntegerPtr ptrLeq(Camera._GetNode("Leq"));
    CPPUNIT_ASSERT( ptrLeq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrLeq->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrLeq->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrLeq->GetValue());

    CIntegerPtr ptrGreater(Camera._GetNode("Greater"));
    CPPUNIT_ASSERT( ptrGreater.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrGreater->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrGreater->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrGreater->GetValue());

    CIntegerPtr ptrLess(Camera._GetNode("Less"));
    CPPUNIT_ASSERT( ptrLess.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrLess->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrLess->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrLess->GetValue());

    CIntegerPtr ptrAnd(Camera._GetNode("And"));
    CPPUNIT_ASSERT( ptrAnd.IsValid() );
    ptrA->SetValue (-5);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrAnd->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrAnd->GetValue());
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrAnd->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrAnd->GetValue());

    CIntegerPtr ptrOr(Camera._GetNode("Or"));
    CPPUNIT_ASSERT( ptrOr.IsValid() );
    ptrA->SetValue (-5);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrOr->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrOr->GetValue());
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrOr->GetValue());
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrOr->GetValue());

    CIntegerPtr ptrPlus(Camera._GetNode("Plus"));
    CPPUNIT_ASSERT( ptrPlus.IsValid() );
    ptrA->SetValue (25);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)32, ptrPlus->GetValue());

    CIntegerPtr ptrMinus(Camera._GetNode("Minus"));
    CPPUNIT_ASSERT( ptrMinus.IsValid() );
    ptrA->SetValue (25);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)18, ptrMinus->GetValue());

    CIntegerPtr ptrTimes(Camera._GetNode("Times"));
    CPPUNIT_ASSERT( ptrTimes.IsValid() );
    ptrA->SetValue (25);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)175, ptrTimes->GetValue());

    CIntegerPtr ptrDiv(Camera._GetNode("Div"));
    CPPUNIT_ASSERT( ptrDiv.IsValid() );
    ptrA->SetValue (25);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)3, ptrDiv->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_THROW( ptrDiv->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrMod(Camera._GetNode("Mod"));
    CPPUNIT_ASSERT( ptrMod.IsValid() );
    ptrA->SetValue (25);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)4, ptrMod->GetValue());
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_THROW( ptrMod->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrTernary(Camera._GetNode("Ternary"));
    CPPUNIT_ASSERT( ptrTernary.IsValid() );
    ptrA->SetValue (5);
    ptrB->SetValue (7);
    ptrC->SetValue (8);
    CPPUNIT_ASSERT_EQUAL ((int64_t)7, ptrTernary->GetValue());
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)8, ptrTernary->GetValue());

    CIntegerPtr ptrPrecedence1(Camera._GetNode("Precedence1"));
    CPPUNIT_ASSERT( ptrPrecedence1.IsValid() );
    ptrA->SetValue (2);
    ptrB->SetValue (3);
    ptrC->SetValue (4);
    CPPUNIT_ASSERT_EQUAL ((int64_t)14, ptrPrecedence1->GetValue());

    CIntegerPtr ptrBrackets1(Camera._GetNode("Brackets1"));
    CPPUNIT_ASSERT( ptrBrackets1.IsValid() );
    ptrA->SetValue (2);
    ptrB->SetValue (3);
    ptrC->SetValue (4);
    CPPUNIT_ASSERT_EQUAL ((int64_t)20, ptrBrackets1->GetValue());

    CIntegerPtr ptrSgn(Camera._GetNode("Sgn"));
    CPPUNIT_ASSERT( ptrSgn.IsValid() );
    ptrA->SetValue (5);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1, ptrSgn->GetValue());
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrSgn->GetValue());
    ptrA->SetValue (-7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)-1, ptrSgn->GetValue());

    CIntegerPtr ptrNeg(Camera._GetNode("Neg"));
    CPPUNIT_ASSERT( ptrNeg.IsValid() );
    ptrA->SetValue (5);
    CPPUNIT_ASSERT_EQUAL ((int64_t)-5, ptrNeg->GetValue());
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0, ptrNeg->GetValue());
    ptrA->SetValue (-7);
    CPPUNIT_ASSERT_EQUAL ((int64_t)7, ptrNeg->GetValue());

    CIntegerPtr ptrDigits1(Camera._GetNode("Digits1"));
    CPPUNIT_ASSERT( ptrDigits1.IsValid() );
    CPPUNIT_ASSERT_EQUAL ((int64_t)36, ptrDigits1->GetValue());

    CIntegerPtr ptrDigits2(Camera._GetNode("Digits2"));
    CPPUNIT_ASSERT( ptrDigits2.IsValid() );
    CPPUNIT_ASSERT_EQUAL ((int64_t)2, ptrDigits2->GetValue());

    CIntegerPtr ptrDigits3(Camera._GetNode("Digits3"));
    CPPUNIT_ASSERT( ptrDigits3.IsValid() );
    CPPUNIT_ASSERT_EQUAL ((int64_t)-2, ptrDigits3->GetValue());


    // float only functions
    CIntegerPtr ptrFloatOnlyFn1(Camera._GetNode("FloatOnlyFn1"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn1.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn1->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn2(Camera._GetNode("FloatOnlyFn2"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn2.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn2->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn3(Camera._GetNode("FloatOnlyFn3"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn3.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn3->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn4(Camera._GetNode("FloatOnlyFn4"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn4.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn4->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn5(Camera._GetNode("FloatOnlyFn5"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn5.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn5->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn6(Camera._GetNode("FloatOnlyFn6"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn6.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn6->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn7(Camera._GetNode("FloatOnlyFn7"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn7.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn7->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn8(Camera._GetNode("FloatOnlyFn8"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn8.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn8->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn9(Camera._GetNode("FloatOnlyFn9"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn9.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn9->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn10(Camera._GetNode("FloatOnlyFn10"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn10.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn10->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn11(Camera._GetNode("FloatOnlyFn11"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn11.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn11->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn12(Camera._GetNode("FloatOnlyFn12"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn12.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn12->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn13(Camera._GetNode("FloatOnlyFn13"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn13.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn13->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn14(Camera._GetNode("FloatOnlyFn14"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn14.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn14->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn15(Camera._GetNode("FloatOnlyFn15"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn15.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn15->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn16(Camera._GetNode("FloatOnlyFn16"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn16.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn16->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn17(Camera._GetNode("FloatOnlyFn17"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn17.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn17->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn18(Camera._GetNode("FloatOnlyFn18"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn18.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn18->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrFloatOnlyFn19(Camera._GetNode("FloatOnlyFn19"));
    CPPUNIT_ASSERT( ptrFloatOnlyFn19.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrFloatOnlyFn19->GetValue(), GenICam::GenericException );


    // some invalid stuff
    CIntegerPtr ptrInvalid1(Camera._GetNode("Invalid1"));
    CPPUNIT_ASSERT( ptrInvalid1.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid1->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid2(Camera._GetNode("Invalid2"));
    CPPUNIT_ASSERT( ptrInvalid2.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid2->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid3(Camera._GetNode("Invalid3"));
    CPPUNIT_ASSERT( ptrInvalid3.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid3->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid4(Camera._GetNode("Invalid4"));
    CPPUNIT_ASSERT( ptrInvalid4.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid4->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid5(Camera._GetNode("Invalid5"));
    CPPUNIT_ASSERT( ptrInvalid5.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid5->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid6(Camera._GetNode("Invalid6"));
    CPPUNIT_ASSERT( ptrInvalid6.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid6->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid7(Camera._GetNode("Invalid7"));
    CPPUNIT_ASSERT( ptrInvalid7.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid7->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid8(Camera._GetNode("Invalid8"));
    CPPUNIT_ASSERT( ptrInvalid8.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid8->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid9(Camera._GetNode("Invalid9"));
    CPPUNIT_ASSERT( ptrInvalid9.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid9->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid10(Camera._GetNode("Invalid10"));
    CPPUNIT_ASSERT( ptrInvalid10.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid10->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid11(Camera._GetNode("Invalid11"));
    CPPUNIT_ASSERT( ptrInvalid11.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid11->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid12(Camera._GetNode("Invalid12"));
    CPPUNIT_ASSERT( ptrInvalid12.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid12->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid13(Camera._GetNode("Invalid13"));
    CPPUNIT_ASSERT( ptrInvalid13.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid13->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid14(Camera._GetNode("Invalid14"));
    CPPUNIT_ASSERT( ptrInvalid14.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid14->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid15(Camera._GetNode("Invalid15"));
    CPPUNIT_ASSERT( ptrInvalid15.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid15->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid16(Camera._GetNode("Invalid16"));
    CPPUNIT_ASSERT( ptrInvalid16.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid16->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid17(Camera._GetNode("Invalid17"));
    CPPUNIT_ASSERT( ptrInvalid17.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid17->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid18(Camera._GetNode("Invalid18"));
    CPPUNIT_ASSERT( ptrInvalid18.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid18->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid19(Camera._GetNode("Invalid19"));
    CPPUNIT_ASSERT( ptrInvalid19.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid19->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid20(Camera._GetNode("Invalid20"));
    CPPUNIT_ASSERT( ptrInvalid20.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid20->GetValue(), GenICam::GenericException );

    CIntegerPtr ptrInvalid21(Camera._GetNode("Invalid21"));
    CPPUNIT_ASSERT( ptrInvalid21.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid21->GetValue(), GenICam::GenericException );
}
