//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "IntegerReplicatorTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "GenApi/NodeCallbackImpl.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( IntegerReplicatorTestSuite, "GenApiTest");

// ---------------------------------------------------------------------------
// the test callback with a counter
// ---------------------------------------------------------------------------
struct CCallbackUtility
{
    static void Reset() { m_Count=0; }
    static void Callback( INode* ) { ++m_Count; }
    static uint32_t Count() { return m_Count; }
private:
    static unsigned m_Count;
};
unsigned CCallbackUtility::m_Count=0;



void IntegerReplicatorTestSuite::TestValueAccess()
{
    // added <pValueCopy> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                          \n"
    "<Integer Name=\"ValueA\">                 \n"
    "    <Value>42</Value>                     \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueB\">                 \n"
    "    <Value>4711</Value>                   \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueC\">                 \n"
    "    <Value>0815</Value>                   \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueP\">                 \n"
    "    <pValueCopy>ValueA</pValueCopy>       \n"
    "    <pValue>ValueB</pValue>               \n"
    "    <pValueCopy>ValueC</pValueCopy>       \n"
    "</Integer>                                \n"
    "                                          \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrValueA = Camera._GetNode("ValueA");
    CIntegerPtr ptrValueB = Camera._GetNode("ValueB");
    CIntegerPtr ptrValueC = Camera._GetNode("ValueC");
    CIntegerPtr ptrValueP = Camera._GetNode("ValueP");

    CPPUNIT_ASSERT_EQUAL( (int64_t) 4711LL, ptrValueP->GetValue() );

    *ptrValueP = 100;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 100LL, ptrValueA->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 100LL, ptrValueB->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 100LL, ptrValueC->GetValue() );

}

void IntegerReplicatorTestSuite::TestMinMaxHandling()
{
    // added <pValueCopy> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                          \n"
    "<Integer Name=\"ValueA\">                 \n"
    "    <Value>0</Value>                      \n"
    "    <Min>-10</Min>                        \n"
    "    <Max>100</Max>                        \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueB\">                 \n"
    "    <Value>0</Value>                      \n"
    "    <Min>-100</Min>                       \n"
    "    <Max>10</Max>                         \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueC\">                 \n"
    "    <Value>1500</Value>                   \n"
    "    <Min>1000</Min>                       \n"
    "    <Max>2000</Max>                       \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueP\">                 \n"
    "    <pValueCopy>ValueA</pValueCopy>       \n"
    "    <pValue>ValueB</pValue>               \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueQ\">                 \n"
    "    <pValueCopy>ValueA</pValueCopy>       \n"
    "    <pValue>ValueB</pValue>               \n"
    "    <pValueCopy>ValueC</pValueCopy>       \n"
    "</Integer>                                \n"
    "                                          \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrValueA = Camera._GetNode("ValueA");
    CIntegerPtr ptrValueB = Camera._GetNode("ValueB");
    CIntegerPtr ptrValueC = Camera._GetNode("ValueC");
    CIntegerPtr ptrValueP = Camera._GetNode("ValueP");
    CIntegerPtr ptrValueQ = Camera._GetNode("ValueQ");

    CPPUNIT_ASSERT_EQUAL( (int64_t) -10LL, ptrValueP->GetMin() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 10LL, ptrValueP->GetMax() );

    // ValueQ is not consistent: you can read the Value and get 0, but you cannot write it
    // back because the combined Min/Max results in max < min
    CPPUNIT_ASSERT_EQUAL( (int64_t)0LL, ptrValueQ->GetValue());
    CPPUNIT_ASSERT_THROW_EX(ptrValueQ->GetValue(true), GenICam::OutOfRangeException);
    CPPUNIT_ASSERT_THROW_EX(ptrValueQ->SetValue(0), GenICam::OutOfRangeException);

}

void IntegerReplicatorTestSuite::TestIncHandling()
{
    // added <pValueCopy> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                          \n"
    "<Integer Name=\"ValueA\">                 \n"
    "    <Value>0</Value>                      \n"
    "    <Inc>1</Inc>                          \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueB\">                 \n"
    "    <Value>0</Value>                      \n"
    "    <Inc>1</Inc>                          \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueC\">                 \n"
    "    <Value>0</Value>                      \n"
    "    <Inc>2</Inc>                          \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueP\">                 \n"
    "    <pValueCopy>ValueA</pValueCopy>       \n"
    "    <pValue>ValueB</pValue>               \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueQ\">                 \n"
    "    <pValueCopy>ValueA</pValueCopy>       \n"
    "    <pValue>ValueB</pValue>               \n"
    "    <pValueCopy>ValueC</pValueCopy>       \n"
    "</Integer>                                \n"
    "                                          \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrValueP = Camera._GetNode("ValueP");
    CIntegerPtr ptrValueQ = Camera._GetNode("ValueQ");

    CPPUNIT_ASSERT_EQUAL( RW, ptrValueP->GetAccessMode() );
    CPPUNIT_ASSERT_EQUAL( RO, ptrValueQ->GetAccessMode() );

}

void IntegerReplicatorTestSuite::TestCallbacks()
{

    // added <pValueCopy> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                          \n"
    "<Integer Name=\"ValueA\">                 \n"
    "    <Value>42</Value>                     \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueB\">                 \n"
    "    <Value>4711</Value>                   \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueC\">                 \n"
    "    <Value>0815</Value>                   \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueP\">                 \n"
    "    <pValueCopy>ValueA</pValueCopy>       \n"
    "    <pValue>ValueB</pValue>               \n"
    "    <pValueCopy>ValueC</pValueCopy>       \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueX\">                 \n"
    "    <pValue>ValueB</pValue>               \n"
    "</Integer>                                \n"
    "                                          \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrValueA = Camera._GetNode("ValueA");
    CIntegerPtr ptrValueB = Camera._GetNode("ValueB");
    CIntegerPtr ptrValueC = Camera._GetNode("ValueC");
    CIntegerPtr ptrValueP = Camera._GetNode("ValueP");
    CIntegerPtr ptrValueX = Camera._GetNode("ValueX");

    CCallbackUtility::Reset();
    intptr_t Coockie = Register( ptrValueP->GetNode(), &CCallbackUtility::Callback );
    *ptrValueP = 100;
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CCallbackUtility::Count() );
    ptrValueP->GetNode()->DeregisterCallback(Coockie);

    CCallbackUtility::Reset();
    Register( ptrValueX->GetNode(), &CCallbackUtility::Callback );
    *ptrValueP = 100;
    CPPUNIT_ASSERT_EQUAL( (uint32_t)1, CCallbackUtility::Count() );

}
/*! Use Case given by Vincent Rowley, Pleora

    Feature to implement:
    - GainRaw

    Camera existing underlying features:
    - MyGainRawHigh
    - MyGainRawLow

    Desired Behavior:
    MyGainRawHigh = (0xFF00 & GainRaw) >> 8; MyGainRawLow = 0x00FF & GainRaw;
*/
void IntegerReplicatorTestSuite::TestUseCase1()
{
    // added <pValueCopy> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                               \n"
    "<Integer Name=\"GainRaw\">                                     \n"
    "    <pValue>WriteHighConverter</pValue>                        \n"
    "    <pValueCopy>WriteLowConverter</pValueCopy>                 \n"
    "    <Min>0</Min>                                               \n"
    "    <Max>65535</Max>                                           \n"
    "</Integer>                                                     \n"
    "                                                               \n"
    "<IntConverter Name=\"WriteHighConverter\">                     \n"
    "    <pVariable Name=\"LOW\">GainRawLow</pVariable>             \n"
    "    <FormulaTo><![CDATA[  (0xFF00 & FROM) >> 8 ]]></FormulaTo> \n"
    "    <FormulaFrom><![CDATA[ LOW + (TO << 8) ]]></FormulaFrom>   \n"
    "    <pValue>GainRawHigh</pValue>                               \n"
    "    <Slope>Varying</Slope>                                     \n"
    "</IntConverter>                                                \n"
    "                                                               \n"
    "<IntConverter Name=\"WriteLowConverter\">                      \n"
    "    <FormulaTo><![CDATA[ 0x00FF & FROM ]]></FormulaTo>         \n"
    "    <FormulaFrom>TO</FormulaFrom>                              \n"
    "    <pValue>GainRawLow</pValue>                                \n"
    "    <Slope>Increasing</Slope>                                  \n"
    "</IntConverter>                                                \n"
    "                                                               \n"
    "<Integer Name=\"GainRawHigh\">                                 \n"
    "    <Value>0x12</Value>                                        \n"
    "</Integer>                                                     \n"
    "                                                               \n"
    "<Integer Name=\"GainRawLow\">                                  \n"
    "    <Value>0x34</Value>                                        \n"
    "</Integer>                                                     \n"
    "                                                               \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrGainRaw = Camera._GetNode("GainRaw");
    CIntegerPtr ptrGainRawHigh = Camera._GetNode("GainRawHigh");
    CIntegerPtr ptrGainRawLow = Camera._GetNode("GainRawLow");

    CPPUNIT_ASSERT_EQUAL( (int64_t) 0x1234LL, ptrGainRaw->GetValue() );
    ptrGainRaw->SetValue(0x5678LL);
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0x56LL, ptrGainRawHigh->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 0x78LL, ptrGainRawLow->GetValue() );

}

/*! Use Case given by Vincent Rowley, Pleora

    Feature to implement:
    - TriggerMode (Off=0, Software=1, Hardware=2)
    - TriggerSource (Line1=1, Line2=2)

    Camera existing underlying features:
    - MyTriggerMode (MyOff=0, MyOn=1)
    - MyTriggerSource (MySoftware=0, MyLine1=1, MyLine2=2)

    Desired Behavior:
    = on write TriggerMode :
      (1) MyTriggerMode := (TriggerMode == Off) ? MyOff : MyOn;
      (2) MyTriggerSource := (TriggerMode == Software) ? MySoftware : MyTriggerSource;
    = on write TriggerSource
      (3) MyTriggerSource := TriggerSource;
      (4) MyTriggerMode is not affected
    = on read TriggerMode :
      (5) TriggerMode := (MyTriggerMode == MyOff) ? Off : ( (MyTriggerSource == MySoftware) ? Software : Hardware );
    = on read TriggerSource :
      (6) TriggerSource := MyTriggerSource;

    Some comments:
    - The formulas assume that the coding of Line1==MyLine1 and Line2==MyLine2.
    - Formula (2) uses the the old value of the variable to set in oder to compute the new value
    - Due to logical reasons setting TriggerMode automatically sets TriggerMode to Hardware
*/
void IntegerReplicatorTestSuite::TestUseCase2()
{
    // added <pValueCopy> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                               \n"
    "<Enumeration Name=\"TriggerMode\">                             \n"
    "    <EnumEntry Name=\"Off\">                                   \n"
    "        <Value>0</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <EnumEntry Name=\"Software\">                              \n"
    "        <Value>1</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <EnumEntry Name=\"Hardware\">                              \n"
    "        <Value>2</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <pValue>TriggerModeReplicator</pValue>                     \n"
    "</Enumeration>                                                 \n"
    "                                                               \n"
    "<Integer Name=\"TriggerModeReplicator\">                       \n"
    "    <pValue>Mode2MyModeConverter</pValue>                      \n"
    "    <pValueCopy>Mode2MySourceConverter</pValueCopy>            \n"
    "</Integer>                                                     \n"
    "                                                               \n"
    "<IntConverter Name=\"Mode2MyModeConverter\">                   \n"
    "    <pVariable Name=\"MYSOURCE\">MyTriggerSourceReg</pVariable>\n"
    // (1) MyTriggerMode = (TriggerMode == Off) ? MyOff : MyOn;
    //                   = (TriggerMode == 0  ) ? 0     : 1;
    "    <FormulaTo><![CDATA[ (FROM=0) ? 0 : 1 ]]></FormulaTo>      \n"
    // (5) TriggerMode = (MyTriggerMode == MyOff) ? Off : ( (MyTriggerSource == MySoftware) ? Software : Hardware );
    //     TriggerMode = (MyTriggerMode == 0    ) ? 0   : ( (MyTriggerSource == 0         ) ? 1        : 2        );
    "    <FormulaFrom><![CDATA[ (TO=0) ? 0 : ( (MYSOURCE=0) ? 1 : 2 )  ]]></FormulaFrom>   \n"
    "    <pValue>MyTriggerModeReg</pValue>                          \n"
    "    <Slope>Varying</Slope>                                     \n"
    "</IntConverter>                                                \n"
    "                                                               \n"
    "<Enumeration Name=\"TriggerSource\">                           \n"
    "    <EnumEntry Name=\"Line1\">                                 \n"
    "        <Value>1</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <EnumEntry Name=\"Line2\">                                 \n"
    "        <Value>2</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <pValue>Source2MySourceConverter</pValue>                  \n"
    "</Enumeration>                                                 \n"
    "                                                               \n"
    "<IntConverter Name=\"Source2MySourceConverter\">               \n"
    // (3) MyTriggerSource = TriggerSource;
    "    <FormulaTo><![CDATA[ FROM ]]></FormulaTo>\n"
    // (6) TriggerSource = MyTriggerSource;
    "    <FormulaFrom><![CDATA[ TO ]]></FormulaFrom>                \n"
    "    <pValue>MyTriggerSourceReg</pValue>                        \n"
    "    <Slope>Varying</Slope>                                     \n"
    "</IntConverter>                                                \n"
    "                                                               \n"
    "<IntConverter Name=\"Mode2MySourceConverter\">                 \n"
    "    <pVariable Name=\"MYSOURCE\">MyTriggerSourceReg</pVariable>\n"
    // (2) MyTriggerSource = (TriggerMode == Software) ? MySoftware : MyTriggerSource;
    //                     = (TriggerMode == 1       ) ? 0          : previous value of MyTriggerSource;
    "    <FormulaTo><![CDATA[ (FROM=1) ? 0 : MYSOURCE ]]></FormulaTo>\n"
    // (4) Dummy, should never be called
    "    <FormulaFrom><![CDATA[ 42 ]]></FormulaFrom>                \n"
    "    <pValue>MyTriggerSourceReg</pValue>                        \n"
    "    <Slope>Varying</Slope>                                     \n"
    "</IntConverter>                                                \n"
    "                                                               \n"
    "<Enumeration Name=\"MyTriggerMode\">                           \n"
    "    <EnumEntry Name=\"Off\">                                   \n"
    "        <Value>0</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <EnumEntry Name=\"On\">                                    \n"
    "        <Value>1</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <pValue>MyTriggerModeReg</pValue>                          \n"
    "</Enumeration>                                                 \n"
    "                                                               \n"
    "<Integer Name=\"MyTriggerModeReg\">                            \n"
    "    <Value>0</Value>                                           \n"
    "</Integer>                                                     \n"
    "                                                               \n"
    "<Enumeration Name=\"MyTriggerSource\">                         \n"
    "    <EnumEntry Name=\"Software\">                              \n"
    "        <Value>0</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <EnumEntry Name=\"Line1\">                                 \n"
    "        <Value>1</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <EnumEntry Name=\"Line2\">                                 \n"
    "        <Value>2</Value>                                       \n"
    "   </EnumEntry>                                                \n"
    "    <pValue>MyTriggerSourceReg</pValue>                        \n"
    "</Enumeration>                                                 \n"
    "                                                               \n"
    "<Integer Name=\"MyTriggerSourceReg\">                          \n"
    "    <Value>0</Value>                                           \n"
    "</Integer>                                                     \n"
    "                                                               \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CEnumerationPtr ptrTriggerMode = Camera._GetNode("TriggerMode");
    CEnumerationPtr ptrTriggerSource = Camera._GetNode("TriggerSource");
    CEnumerationPtr ptrMyTriggerMode = Camera._GetNode("MyTriggerMode");
    CEnumerationPtr ptrMyTriggerSource = Camera._GetNode("MyTriggerSource");

    *ptrTriggerMode = "Off";
    *ptrTriggerSource = "Line1";
    CPPUNIT_ASSERT_EQUAL( gcstring("Off"), ptrMyTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Line1"), ptrMyTriggerSource->ToString() );
    *ptrTriggerSource = "Line2";
    CPPUNIT_ASSERT_EQUAL( gcstring("Off"), ptrMyTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Line2"), ptrMyTriggerSource->ToString() );

    *ptrTriggerMode = "Hardware";
    *ptrTriggerSource = "Line1";
    CPPUNIT_ASSERT_EQUAL( gcstring("On"), ptrMyTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Line1"), ptrMyTriggerSource->ToString() );
    *ptrTriggerSource = "Line2";
    CPPUNIT_ASSERT_EQUAL( gcstring("On"), ptrMyTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Line2"), ptrMyTriggerSource->ToString() );

    *ptrTriggerMode = "Software";
    CPPUNIT_ASSERT_EQUAL( gcstring("On"), ptrMyTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Software"), ptrMyTriggerSource->ToString() );

    *ptrTriggerMode = "Software";
    *ptrTriggerSource = "Line1";
    // Beware: due to logcal reasons setting TriggerSource automatically sets TriggerMode to Hardware
    CPPUNIT_ASSERT_EQUAL( gcstring("Hardware"), ptrTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("On"), ptrMyTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Line1"), ptrMyTriggerSource->ToString() );
    *ptrTriggerSource = "Line2";
    CPPUNIT_ASSERT_EQUAL( gcstring("On"), ptrMyTriggerMode->ToString() );
    CPPUNIT_ASSERT_EQUAL( gcstring("Line2"), ptrMyTriggerSource->ToString() );


}

void IntegerReplicatorTestSuite::TestAccessMode()
{
    // added <pValueCopy> element
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                          \n"
    "<Integer Name=\"Value\">                 \n"
    "    <pValue>MainValue</pValue>               \n"
    "    <pValueCopy>ValueCopy</pValueCopy>       \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"MainValue\">                 \n"
    "    <pIsLocked>LockMain</pIsLocked>                     \n"
    "    <Value>42</Value>                     \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"ValueCopy\">                 \n"
    "    <pIsLocked>LockCopy</pIsLocked>                     \n"
    "    <Value>42</Value>                     \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"LockMain\">                 \n"
    "    <Value>0</Value>                   \n"
    "</Integer>                                \n"
    "                                          \n"
    "<Integer Name=\"LockCopy\">                 \n"
    "    <Value>0</Value>                   \n"
    "</Integer>                                \n"
    "                                          \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrValue = Camera._GetNode("Value");

    CIntegerPtr ptrLockMain = Camera._GetNode("LockMain");
    CIntegerPtr ptrLockCopy = Camera._GetNode("LockCopy");

    CPPUNIT_ASSERT (IsWritable (ptrValue->GetAccessMode()));

    ptrLockMain->SetValue (1);
    CPPUNIT_ASSERT (!IsWritable (ptrValue->GetAccessMode()));

    ptrLockMain->SetValue (0);
    ptrLockCopy->SetValue (1);
    CPPUNIT_ASSERT (!IsWritable (ptrValue->GetAccessMode()));

}
