//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif
#include <Base/GCUtilities.h>
#include "GenApiTest.h"
#include "NodeMapTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "GenApi/GenApiVersion.h"
#include "GenApi/impl/DllManager.h"
#include "GenApi/DLLLoad.h"
#include <fstream>
#include <ctime>

// hn ++ 10012007
//#include "Base/GCTypes.h"
//#pragma message( "include GCTypes.h" )
// hn --

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( NodeMapTestSuite, "GenApiTest");



void NodeMapTestSuite::TestNodeMap()
{
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<Integer Name=\"TestInt\">                \n"
        "    <Value>0</Value>                    \n"
        "</Integer>                                \n"
        "                                        \n"
        "<Register Name=\"TestReg\">            \n"
        "    <Address>0x0000</Address>            \n"
        "    <Length>1</Length>                    \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "</Register>                            \n"
        "                                        \n"
        "<Port Name=\"Port\"/>                    \n"
        + CDFFooter;

    CNodeMapRef Camera;
    CTestPort Port1, Port2;
    NodeList_t NodeList;
    size_t Size;

    // test uninitialzied map
    CPPUNIT_ASSERT_THROW_EX( Camera._Connect(&Port1, "Port"), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( Camera._Connect(&Port2), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( Camera._GetNode("TestInt"), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( Camera._GetNodes(NodeList), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( Camera._GetDeviceName(), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( Camera._InvalidateNodes(), GenICam::AccessException );
    CPPUNIT_ASSERT_THROW_EX( Camera._Poll(500), GenICam::AccessException );

    // now test initialized map
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CNodeMapDynPtr pNodeMapDyn(Camera._Ptr);
    CPPUNIT_ASSERT_NO_THROW( Camera._Connect(&Port1, "Port") );
    CPPUNIT_ASSERT_NO_THROW( Camera._Connect(&Port2) );
    CPPUNIT_ASSERT_NO_THROW( Camera._GetNode("TestInt") );
    CPPUNIT_ASSERT_NO_THROW( Camera._GetNodes(NodeList) );
    Size = NodeList.size();
    CPPUNIT_ASSERT_NO_THROW( Camera._GetNodes(NodeList) );
    CPPUNIT_ASSERT_EQUAL ( Size, NodeList.size() );
    CPPUNIT_ASSERT_NO_THROW( Camera._GetDeviceName() );
    CPPUNIT_ASSERT_NO_THROW( Camera._InvalidateNodes() );
    CPPUNIT_ASSERT_EQUAL ( gcstring("Device"), Camera._GetDeviceName() );
    CPPUNIT_ASSERT_EQUAL ( gcstring("Device"), Camera._GetNode("TestInt")->GetDeviceName() );
    
    {   // test the interface
        CNodeMapRef Camera;
        CTestPort Port1, Port2;
        NodeList_t NodeList;
        size_t Size = 0;
        Camera._LoadXMLFromString(CameraDescriptionFile);
        INodeMap *pINodemap( Camera._Ptr );
        CPPUNIT_ASSERT( NULL != pINodemap );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->Connect(&Port1, "Port") );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->Connect(&Port2) );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->GetNode("TestInt") );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->GetNodes(NodeList) );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->GetLock() );
        Size = NodeList.size();
        CPPUNIT_ASSERT_EQUAL( (uint64_t) Size, pINodemap->GetNumNodes() );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->GetDeviceName() );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->InvalidateNodes() );
        CPPUNIT_ASSERT_EQUAL (  gcstring("Device"), pINodemap->GetDeviceName() );
        CPPUNIT_ASSERT_EQUAL (  gcstring("Device"), pINodemap->GetNode("TestInt")->GetDeviceName() );
        CPPUNIT_ASSERT_NO_THROW( pINodemap->Poll( 500 ) ); 
    }

    {
        // test loading from file
        CNodeMapRef Camera2, Camera3, Camera4, Camera4b;
        CPPUNIT_ASSERT_NO_THROW( Camera2._LoadXMLFromFile("Test", "TestCamera_v1_0") );
        CPPUNIT_ASSERT ( Camera2._GetNode("MyNode") != NULL );
        gcstring FileName(XMLPATH "Test/TestCamera_v1_0.xml");
        GenICam::ReplaceEnvironmentVariables(FileName);
        CPPUNIT_ASSERT_NO_THROW( Camera3._LoadXMLFromFile(FileName) );
        CPPUNIT_ASSERT ( Camera3._GetNode("MyNode") != NULL );
        gcstring DllFileName(GenApi::_MakeDLLFileName("grzzz", "barghhh"));
        CPPUNIT_ASSERT_THROW_EX( Camera4._LoadDLL(DllFileName), GenICam::RuntimeException );
        DllFileName = _MakeDLLFileName("Test", "TestCamera_v1_0");
        CPPUNIT_ASSERT_NO_THROW( Camera4._LoadDLL(DllFileName) );
        CPPUNIT_ASSERT ( Camera4._GetNode("MyNode") != NULL );
        CPPUNIT_ASSERT_THROW_EX( Camera4b._LoadXMLFromFile("vrrrmm", "psssst"), GenICam::RuntimeException );
        CPPUNIT_ASSERT_THROW_EX( Camera4b._LoadDLL(DllFileName), GenICam::RuntimeException  );
        CPPUNIT_ASSERT_THROW_EX( Camera4b._LoadDLL("vrrrmm", "psssst"), GenICam::RuntimeException  );
        CPPUNIT_ASSERT_THROW_EX( Camera4b._LoadXMLFromFile(DllFileName), GenICam::RuntimeException );
        CPPUNIT_ASSERT_THROW_EX( Camera4b._LoadXMLFromStringInject("vrrrmm", "psssst"), GenICam::RuntimeException );
        CPPUNIT_ASSERT_THROW_EX( Camera4b._LoadXMLFromFileInject("vrrrmm", "psssst"), GenICam::RuntimeException );
    }
    {
        // test loading from file
        CNodeMapRef Camera2, Camera3, Camera4, Camera4b;
        CPPUNIT_ASSERT_NO_THROW( Camera2._LoadXMLFromFile("Test", "TestCamera_v1_1") );
        CPPUNIT_ASSERT ( Camera2._GetNode("MyNode") != NULL );
        gcstring FileName(XMLPATH "Test/TestCamera_v1_1.xml");
        GenICam::ReplaceEnvironmentVariables(FileName);
        CPPUNIT_ASSERT_NO_THROW( Camera3._LoadXMLFromFile(FileName) );
        CPPUNIT_ASSERT ( Camera3._GetNode("MyNode") != NULL );
        gcstring DllFileName(_MakeDLLFileName("grzzz", "barghhh"));
        CPPUNIT_ASSERT_THROW_EX( Camera4._LoadDLL(DllFileName), GenICam::RuntimeException );
        DllFileName = _MakeDLLFileName("Test", "TestCamera_v1_1");
        CPPUNIT_ASSERT_NO_THROW( Camera4._LoadDLL(DllFileName) );
        CPPUNIT_ASSERT ( Camera4._GetNode("MyNode") != NULL );
        CPPUNIT_ASSERT_THROW_EX( Camera4b._LoadXMLFromFile("vrrrmm", "psssst"), GenICam::RuntimeException );
    }
    // test empty map
    pNodeMapDyn->ClearAllNodes ();
    CPPUNIT_ASSERT_EQUAL ( (GenApi::INode*)NULL, Camera._GetNode("FooBar") );

    // try loading again without first cleaning up
    CPPUNIT_ASSERT_THROW_EX(
        Camera._LoadXMLFromString(CameraDescriptionFile),
        GenICam::RuntimeException
    );


    // cover all paths in node factory (namely converter and invalid nodes)
    CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<Integer Name=\"TestInt\">                \n"
        "    <Value>0</Value>                    \n"
        "</Integer>                                \n"
        "                                        \n"
        "<Converter Name=\"TestConv\">            \n"
        "    <FormulaTo>(FROM &#47; 20)</FormulaTo>    \n"
        "    <FormulaFrom>(TO * 20)</FormulaFrom>    \n"
        "    <pValue>TestInt</pValue>            \n"
        "</Converter>                            \n"
        "                                        \n"
        "<FooBar Name=\"TestInvalid\">                \n"
        "    <ToolTip>Invalid node</ToolTip>        \n"
        "</FooBar>                                \n"
        "                                        \n"
        "<Port Name=\"Port\"/>                    \n"
        + CDFFooter;

    CNodeMapRef Camera5;
    CPPUNIT_ASSERT_THROW_EX( Camera5._LoadXMLFromString(CameraDescriptionFile), GenICam::RuntimeException );

    // pass invalid XML so taht the Xalan transformer fails
    CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "</Integer>                                \n"
        "                                        \n"
        + CDFFooter;

    CNodeMapRef Camera6;
    CPPUNIT_ASSERT_THROW_EX( Camera6._LoadXMLFromString(CameraDescriptionFile), GenICam::RuntimeException );
}


void NodeMapTestSuite::TestTheRest()
{
    using namespace Test_TestCamera_v1_0;
    using namespace Test_TestCamera_v1_1;

    gcstring DllFileName(_MakeDLLFileName("Test", "TestCamera_v1_0"));
    DllManager::Handle _hDLL(DllManager::LoadLib(DllFileName));
    CPPUNIT_ASSERT ( DllManager::_INVALID_HANDLE() != _hDLL );
    typedef void (__cdecl* SetGenApiVersion_f)(Version_t &Version);
    GetGenApiVersion_f fGetGenApiVersion((GetGenApiVersion_f) DllManager::FindSymbol(_hDLL, "GetGenApiVersion"));
    CPPUNIT_ASSERT ( DllManager::_INVALID_SYMBOL() != fGetGenApiVersion );
    SetGenApiVersion_f fSetGenApiVersion = (SetGenApiVersion_f) DllManager::FindSymbol(_hDLL, "SetGenApiVersion");
    CPPUNIT_ASSERT ( DllManager::_INVALID_SYMBOL() != fSetGenApiVersion );
    SetGenApiVersion_f fUnknownSymbol = (SetGenApiVersion_f) DllManager::FindSymbol(_hDLL, "ThisFunctionDoesNotExist");
    CPPUNIT_ASSERT ( DllManager::_INVALID_SYMBOL() == fUnknownSymbol );

    {
        // test incorrect GenApi versions
        CTestCamera_v1_0 Camera1, Camera2, Camera3, Camera4, Camera5;

        // with default values should be OK
        CPPUNIT_ASSERT_NO_THROW ( Camera1._LoadDLL() );

        // now the incorrect ones
        Version_t GenApiVersion2 = {GENAPI_VERSION_MAJOR+1, GENAPI_VERSION_MINOR, GENAPI_VERSION_SUBMINOR};
        (fSetGenApiVersion)(GenApiVersion2);
        CPPUNIT_ASSERT_THROW_EX( Camera2._LoadDLL(), GenICam::RuntimeException );

        Version_t GenApiVersion3 = {GENAPI_VERSION_MAJOR, GENAPI_VERSION_MINOR+1, GENAPI_VERSION_SUBMINOR};
        (fSetGenApiVersion)(GenApiVersion3);
        CPPUNIT_ASSERT_THROW_EX( Camera3._LoadDLL(), GenICam::RuntimeException );

        // and again these ones should pass
        Version_t GenApiVersion4 = {GENAPI_VERSION_MAJOR, GENAPI_VERSION_MINOR, GENAPI_VERSION_SUBMINOR+1};
        (fSetGenApiVersion)(GenApiVersion4);
        CPPUNIT_ASSERT_NO_THROW( Camera4._LoadDLL() );

        Version_t GenApiVersion5 = {GENAPI_VERSION_MAJOR, GENAPI_VERSION_MINOR, GENAPI_VERSION_SUBMINOR};
        (fSetGenApiVersion)(GenApiVersion5);
        CPPUNIT_ASSERT_NO_THROW( Camera5._LoadDLL() );
    }

    // Free the library
    DllManager::FreeLib( _hDLL );
    // And try to free one not previously loaded
    CPPUNIT_ASSERT_EQUAL ( false, DllManager::FreeLib( DllManager::_INVALID_HANDLE() ) );
}

void NodeMapTestSuite::TestSchemaLocation()
{

    gcstring SubTest( XMLPATH "Test/SubTest/SubTest.xml" );
    ReplaceEnvironmentVariables(SubTest);

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile(SubTest);

    CIntegerPtr ptrValue = Camera._GetNode("Value");
    CPPUNIT_ASSERT_EQUAL( (int64_t)3, ptrValue->GetValue());
}
// Mantis #127: XMLLoader needs the optional ToolTip to be
// present in RegisterDescription element
void NodeMapTestSuite::TestWithoutTooltip()
{
    const GenICam::gcstring CDFHeader =
    "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
    "<RegisterDescription\n"
    "      ModelName=\"GenApiTest\"\n"
    "      VendorName=\"Generic\"\n"
//    "      ToolTip=\"nodes for testing the GenApi reference implementation\"\n"
    "      StandardNameSpace=\"GEV\"\n"
    "      SchemaMajorVersion=\"1\"\n"
    "      SchemaMinorVersion=\"0\"\n"
    "      SchemaSubMinorVersion=\"1\"\n"
    "      MajorVersion=\"2\"\n"
    "      MinorVersion=\"3\"\n"
    "      SubMinorVersion=\"4\"\n"
    "      ProductGuid=\"2D932CC6-EB68-40bd-B6CC-F03B55B7D653\"\n"
    "      VersionGuid=\"02A8C268-BEE8-463b-A6C0-53ED8256E3D8\"\n"
    "      xmlns=\"http://www.genicam.org/GenApi/Version_1_0\"\n"
    "      xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
    "      xsi:schemaLocation=\"http://www.genicam.org/GenApi/Version_1_0\n"
    "            http://www.genicam.org/GenApi/GenApiSchema_Version_1_0.xsd\">\n";


    gcstring CameraDescriptionFile =
        CDFHeader +
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString( CameraDescriptionFile );

}

void NodeMapTestSuite::TestSchemaVersions()
{
    CNodeMapRef Camera;

    GenICam::gcstring_vector SchemaVersions;
    Camera._GetSupportedSchemaVersions( SchemaVersions );

    CPPUNIT_ASSERT_EQUAL( (size_t)2, SchemaVersions.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("1.1"), SchemaVersions[0] );
    CPPUNIT_ASSERT_EQUAL( gcstring("1.2"), SchemaVersions[1] );
}

void NodeMapTestSuite::TestExposeStatic()
{
    using namespace Test_TestCamera_v1_1;

    CTestCamera_v1_1 Camera;
    Camera._LoadDLL();

    cout << Camera.Root.GetNode()->GetName().c_str() << "\n";
    cout << Camera.NoFeatureButExposed.GetNode()->GetName().c_str() << "\n";
    //cout << Camera.FeatureButNotExposed.GetNode()->GetName().c_str() << "\n";
    cout << Camera.CategoryButExposed.GetNode()->GetName().c_str() << "\n";
    //cout << Camera.CategoryNotExposed.GetNode()->GetName().c_str() << "\n";
}

void NodeMapTestSuite::TestExtractSubtree()
{
    // Load an XML file which contains "all" nodes with "all" features
    gcstring XmlFileName = XMLPATH "Test/AllGenapiNodesTest.xml";
    ReplaceEnvironmentVariables(XmlFileName);

    CNodeMapRef::_ClearXMLCache();

    gcstring xmlData;
    //read the text file and put it into a string
    {
        //open the file to read
        std::ifstream file(XmlFileName.c_str());
        CPPUNIT_ASSERT( file.is_open());
        //read the data
        for( char c = 0; file.get( c); xmlData += c);
        CPPUNIT_ASSERT( file.eof());
    }

    //try no injection with and without cache
    gcstring extractedDataNoCache;
    gcstring extractedDataCache;
    {
        CNodeMapRef Camera;
        Camera._ExtractIndependentSubtree(xmlData, "", "ChunkData", extractedDataNoCache);
        Camera._ExtractIndependentSubtree(xmlData, "", "ChunkData", extractedDataCache);
    }

    //results should be equal
    CPPUNIT_ASSERT( extractedDataNoCache == extractedDataCache);

    //small check if node map is OK, node ChunkPayloadCRC16 should exist
    {
        CNodeMapRef Camera;
        Camera._LoadXMLFromString( extractedDataNoCache);
        CPPUNIT_ASSERT( Camera._GetNode("ChunkPayloadCRC16"));
        CPPUNIT_ASSERT( Camera._GetNode("GainRaw") == NULL);
    }

    gcstring Inject =
        "<RegisterDescription xmlns=\"http://www.genicam.org/GenApi/Version_1_0\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" MajorVersion=\"2\" MinorVersion=\"9\" ModelName=\"GigECameraParams\" ProductGuid=\"1F3C6A72-7842-4edd-9130-E2E90A2058BA\" SchemaMajorVersion=\"1\" SchemaMinorVersion=\"0\" SchemaSubMinorVersion=\"1\" StandardNameSpace=\"GEV\" SubMinorVersion=\"0\" ToolTip=\"Basler GigEVision camera interface\" VendorName=\"Basler\" VersionGuid=\"D1865140-07C0-11DE-82C9-00105A5BAE55\" xsi:schemaLocation=\"http://www.genicam.org/GenApi/Version_1_0 C:\\Projects\\GenICam\\xml\\GenApi\\GenApiSchema_Version_1_0.xsd\">"
        "                                         \n"
        "<Integer Name=\"ValueA\" MergePriority=\"+1\">  \n"
        "   <Value>2</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "                                         \n"
        "<Category Name=\"ChunkData\">            \n"
        "   <pFeature>ValueA</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        + CDFFooter;

    //try with injection with and without cache
    extractedDataNoCache = "";
    extractedDataCache = "";
    {
        CNodeMapRef Camera;
        Camera._ExtractIndependentSubtree(xmlData, Inject, "ChunkData", extractedDataNoCache);
        Camera._ExtractIndependentSubtree(xmlData, Inject, "ChunkData", extractedDataCache);
    }

    //results should be equal
    CPPUNIT_ASSERT( extractedDataNoCache == extractedDataCache);

    //small check if node map is OK, node ChunkPayloadCRC16 and ValueA should exist
    {
        CNodeMapRef Camera;
        Camera._LoadXMLFromString( extractedDataNoCache);
        CPPUNIT_ASSERT( Camera._GetNode("ChunkPayloadCRC16"));
        CPPUNIT_ASSERT( Camera._GetNode("ValueA"));
        CPPUNIT_ASSERT( Camera._GetNode("GainRaw") == NULL);        
    }
}

void NodeMapTestSuite::TestRealLifeNodeMap()
{
    gcstring strXMLFileName("$(" GENICAM_ROOT_VERSION ")/xml/GenApi/Test/AllGenapiNodesTest.xml");

    {
        // cache miss, create nodemap from scratch
        CNodeMapRef::_ClearXMLCache();
        CNodeMapRef Camera;
        CPPUNIT_ASSERT_NO_THROW(Camera._LoadXMLFromFile(strXMLFileName));
    }
    
    // this time we should have a cache hit 
    {
        CNodeMapRef Camera;
        CPPUNIT_ASSERT_NO_THROW(Camera._LoadXMLFromFile(strXMLFileName));
    }
}

#define MEASURE_TIME_START(x) \
    { \
        debug("-->start(%s)", (const char*) x); \
        clock_t start, finish; \
        start = clock();

#define MEASURE_TIME_STOP(x) \
        finish = clock(); \
        debug("-->stop (%s), time : %d ms", (const char*) x, (finish - start)); \
    }

void NodeMapTestSuite::TestRealLifeNodeMapPerformance()
{
    gcstring Prefix("$(" GENICAM_ROOT_VERSION ")/xml/GenApi/Test/");
    gcstring FileTemplate(Prefix + "AllGenapiNodesTest*.xml");
    gcstring_vector FileNames;

    GetFiles(FileTemplate, FileNames);

    for (gcstring_vector::const_iterator it = FileNames.begin(); it != FileNames.end(); ++it)
    {
        gcstring Filename(Prefix + *it);
        MEASURE_TIME_START(Filename)
        {
            // cache miss, create nodemap from scratch
            CNodeMapRef::_ClearXMLCache();
            CNodeMapRef Camera;
            CPPUNIT_ASSERT_NO_THROW(Camera._LoadXMLFromFile(Filename));
        }
        MEASURE_TIME_STOP(Filename)
    
        MEASURE_TIME_START(Filename)
        // this time we should have a cache hit 
        {
            CNodeMapRef Camera;
            CPPUNIT_ASSERT_NO_THROW(Camera._LoadXMLFromFile(Filename));
        }
        MEASURE_TIME_STOP(Filename)
    }
}

#ifdef _WIN32

#include <windows.h>
#include <process.h>

class ProbeThreadParams
{
public:
    gcstring    strXMLFilename;
    NodeMapTestSuite* pThis;
};

unsigned __stdcall NodeMapTestSuite::CreateNodeMapThread(void* p)
{
    ProbeThreadParams* pParams = reinterpret_cast<ProbeThreadParams*>(p);

    try
    {
        CNodeMapRef Camera;

        FILETIME times[4] = {0};
        ::GetThreadTimes(GetCurrentThread(), &times[0], &times[1], &times[2], &times[3]);
        ULARGE_INTEGER userTime1;
        ULARGE_INTEGER kernelTime1;
        kernelTime1.LowPart =  times[2].dwLowDateTime; 
        kernelTime1.HighPart = times[2].dwHighDateTime; 
        userTime1.LowPart =    times[3].dwLowDateTime; 
        userTime1.HighPart =   times[3].dwHighDateTime; 

        CPPUNIT_ASSERT_NO_THROW(Camera._LoadXMLFromFile(pParams->strXMLFilename));

        ::GetThreadTimes(GetCurrentThread(), &times[0], &times[1], &times[2], &times[3]);
        ULARGE_INTEGER userTime2;
        ULARGE_INTEGER kernelTime2;
        kernelTime2.LowPart =  times[2].dwLowDateTime; 
        kernelTime2.HighPart = times[2].dwHighDateTime; 
        userTime2.LowPart =    times[3].dwLowDateTime; 
        userTime2.HighPart =   times[3].dwHighDateTime; 
        
        pParams->pThis->debug("Thread %4u: Kernel %4u ms; User %6u ms ", GetCurrentThreadId(), unsigned((kernelTime2.QuadPart - kernelTime1.QuadPart) / 10000), unsigned((userTime2.QuadPart - userTime1.QuadPart) / 10000));

        return 0;
    }
    catch (const std::exception& e)
    {
        pParams->pThis->debug("Exception in CreateNodeMapThread: '%hs'", e.what());
        return unsigned(-1);
    }
}

void NodeMapTestSuite::CreateNodeMapsThreaded()
{
    ProbeThreadParams params;
    params.pThis = this;
    params.strXMLFilename = "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/Test/AllGenapiNodesTest.xml";

    CNodeMapRef::_ClearXMLCache();
    // cache the file
//    CreateNodeMapThread(&params);

    HANDLE handles[10] = {0};

    DWORD cntCreated = 0;
    for (size_t i = 0; i < GC_COUNTOF(handles); ++i)
    {
        HANDLE h = reinterpret_cast<HANDLE>(_beginthreadex(NULL, 0, CreateNodeMapThread, &params, CREATE_SUSPENDED, NULL));
        if (h != 0)
        {
            handles[cntCreated] = h;
            ++cntCreated;
        }
    }

    debug("Created %u LoadXMLFromFile threads.", cntCreated);
    CPPUNIT_ASSERT_MESSAGE("Could not start probe threads", cntCreated > 0);

    // let them run
    DWORD dwStart1 = GetTickCount();
    for (size_t i = 0; i < cntCreated; ++i)
    {
        ResumeThread(handles[i]);
    }

    bool waitSucc = false;
    if (cntCreated > 0)
    {
        debug("Running LoadFromXML threads.");

        DWORD dwTimeout = 5 * 60 * 1000;
        if (IsDebuggerPresent())
        {
            dwTimeout = 10 * 60 *1000;
        }
        // wait for threads to exit
        DWORD res = WaitForMultipleObjects(cntCreated, handles, 1 /*wait_all*/, dwTimeout);
        waitSucc = (res >= WAIT_OBJECT_0 && res < (WAIT_OBJECT_0 + cntCreated));
    }

    DWORD dwDuration1 = GetTickCount() - dwStart1;

    debug("Multithreaded nodemap creation finished. Duration: %u ms", dwDuration1);

    size_t cntFailed = 0;
    for (size_t i = 0; i < cntCreated; ++i)
    {
        DWORD ec = DWORD(-1);
        ::GetExitCodeThread(handles[i], &ec);
        if (ec == DWORD(-1))
        {
            ++cntFailed;
        }

        ::CloseHandle(handles[i]);
    }

    CPPUNIT_ASSERT_MESSAGE("Could not wait for threads to finish", waitSucc);
    CPPUNIT_ASSERT_MESSAGE("ProbeThread returned an error. Check your PortIDDeviceIDCache.ini file!", cntFailed == 0);


    ///////////////////////////////
    CNodeMapRef::_ClearXMLCache();

    DWORD dwStart2 = GetTickCount();

    unsigned ret = 0;
    for (size_t i = 0; i < cntCreated && ret == 0; ++i)
    {
        ret = CreateNodeMapThread(&params);
    }

    DWORD dwDuration2 = GetTickCount() - dwStart2;

    CPPUNIT_ASSERT_MESSAGE("Error creating nodemaps single threaded", ret == 0);

    debug("Singlethreaded nodemap creation finished. Duration: %u ms", dwDuration2);
}
#endif // _WIN32
