//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/


#ifndef __NODETEST_H
#define __NODETEST_H

#include "GenApiTest.h"

/**
\brief Tests CNodeImpl
*/

class NodeTestSuite : public CPPUNIT_NS::TestCaseEx
{
    CPPUNIT_TEST_SUITE( NodeTestSuite );
        CPPUNITEX_TEST( TestStringConversions );
        CPPUNITEX_TEST( TestAccessMode );
        CPPUNITEX_TEST( TestPropertyAccess );
        CPPUNITEX_TEST( TestNameSpace );
        CPPUNITEX_TEST( TestStaticNameSpace );
        CPPUNITEX_TEST( TestDeviceInformation );
        CPPUNITEX_TEST( TestCachingMode );
        CPPUNITEX_TEST( TestValueNode );
        CPPUNITEX_TEST( TestTheRest );
        CPPUNITEX_TEST( TestBool );
        CPPUNITEX_TEST( TestConst );
        CPPUNITEX_TEST( TestImposeAccessMode );
        CPPUNITEX_TEST( TestReadOnlyPort );
        CPPUNITEX_TEST( TestInfiniteRecursion );
        CPPUNITEX_TEST( TestURL );
        CPPUNITEX_TEST( TestDeprecated );
        CPPUNITEX_TEST( TestStructReg );
        CPPUNITEX_TEST( TestExtension );
        CPPUNITEX_TEST( TestAccessModeCache );
        CPPUNITEX_TEST( TestIsUncached );
        CPPUNITEX_TEST( TestWriteCache );
        CPPUNITEX_TEST( TestLinkTypes );
    CPPUNIT_TEST_SUITE_END();

public:
    //! Tests the propagation of the access mode
    void TestAccessMode();

    //! Tests access of basic properties like, e.g. ToolTip
    void TestPropertyAccess();

    //! Tests name space handling
    void TestNameSpace();

    //! Tests name space handling using the code generator
    void TestStaticNameSpace();

    //! Tests the caching mode
    void TestCachingMode();

    //! Tests the IDeviceInfo interface
    void TestDeviceInformation();

    //! Tests the ValueNode class
    void TestValueNode();

    //! Any other tests not covered by functions above
    void TestTheRest();

    //! check the fromstring tostring methods
    void TestStringConversions();

    //! check pXXX to IBoolean nodes
    void TestBool();

    //! using constant node references
    void TestConst();

    //! Mantis 0000005 : ImposeAccessMode is not taken into account when GetAccessMode returns the cached value
    void TestImposeAccessMode();

    //! Mantis 000015 : The Nodes accessing registers via a port do not check if the port is RO
    void TestReadOnlyPort();

    void TestInfiniteRecursion();

    void TestURL();

    void TestDeprecated();

    void TestStructReg();

    void TestExtension();

    //! Mantis 0000378: CacheMode influences the value cache but not the AccesMode cache
    void TestAccessModeCache();

    void TestIsUncached();

    //! Test for #548
    void TestWriteCache();

    void TestLinkTypes ();
};


#endif // __NODETEST_H

