//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApi
//  Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
\brief    Implemention of CRegisterMap
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include <fstream>
#include <map>
#include "GenApi/Types.h"
#include "Base/GCException.h"
#include "RegisterMap.h"

using namespace GenApi;
using namespace GenICam;
using namespace std;


CRegisterMap::CRegisterMap()
{
}

CRegisterMap::~CRegisterMap()
{
}

void CRegisterMap::LookupEntry( int64_t Address, int64_t Size, void *pValue, GenApi::EAccessMode &AccessMode  )
{
    // IDEA : handle addresses which do not hit an entry exactly but the interval [address, address+length]
    Map_t::iterator it = m_Map.find( Address );
    if (it == m_Map.end())
        throw RUNTIME_EXCEPTION("No entry at address 0x%llx ", Address);

    CRegisterEntry &ve = it->second;
    if(Size > ve.m_DataSize)
        throw RUNTIME_EXCEPTION("Lenght of data item at address 0x%llx not sufficient (actual = %lld, required = %lld)", Address, ve.m_DataSize, Size);

    memcpy( pValue, ve.m_pData, (size_t)Size );
    AccessMode = ve.m_AccessMode;

}

void CRegisterMap::CreateEntry( int64_t Address, int64_t Size, void *pValue, EAccessMode AccessMode )
{
    // IDEA : handle addresses which do not hit an entry exaclty but the interval [address, address+length]
    Map_t::iterator it = m_Map.find( Address );
    if (it != m_Map.end())
        throw RUNTIME_EXCEPTION("Entry already present at address 0x%llx ", Address);

    Map_t::value_type ve( Address, CRegisterEntry( Address, Size, AccessMode, pValue ) );
    m_Map.insert( ve );
}

void CRegisterMap::UpdateEntry( int64_t Address, int64_t Size, const void *pValue, GenApi::EAccessMode &AccessMode)
{
    Map_t::iterator it = m_Map.find( Address );
    if (it == m_Map.end())
        throw RUNTIME_EXCEPTION("No entry at address 0x%llx ", Address);

    CRegisterEntry &ve = it->second;
    if(Size > ve.m_DataSize)
        throw RUNTIME_EXCEPTION("Lenght of data item at address 0x%llx not sufficient (actual = %lld, required = %lld)", Address, ve.m_DataSize, Size);

    memcpy( ve.m_pData, pValue, (size_t)Size );
    AccessMode = ve.m_AccessMode;
}

void CRegisterMap::ClearAllEntries()
{
    m_Map.clear();
}

