//-----------------------------------------------------------------------------
//  (c) 2006 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "SchemaTest.h"
#include "GenApi/GenApi.h"
#include <cppunitex/LogEx.h>
#include "StopWatch.h"
#include "TestPort.h"

using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SchemaTestSuite, "GenApiTest");


void SchemaTestSuite::TestAllNodes()
{
    // added ChunkBaseAddress and ChunkLenght
    if(GenApiSchemaVersion == v1_0)
        return;

    // Load an XML file which congains "all" nodes with "all" features
    gcstring XmlFileName = XMLPATH "Test/AllGenapiNodesTest.xml";

    CNodeMapRef::_ClearXMLCache();

    CStopWatch StopWatch;
    StopWatch.Start();
    {
        CNodeMapRef Camera;
        Camera._LoadXMLFromFile(XmlFileName);
    }
    double ElapsedTime1 = StopWatch.Stop();
    CLog::GetLogger("CppUnit.Performance").info( "SchemaTestSuite::TestAllNodes (fist load) : t = %.1f s", ElapsedTime1 ); 

    StopWatch.Start();
    {
        CNodeMapRef Camera;
        Camera._LoadXMLFromFile(XmlFileName);
    }
    double ElapsedTime2 = StopWatch.Stop();
    CLog::GetLogger("CppUnit.Performance").info( "SchemaTestSuite::TestAllNodes (load using cache) : t = %.2f s", ElapsedTime2 );
    CLog::GetLogger("CppUnit.Performance").info( "SchemaTestSuite::TestAllNodes With caching the XML loades %.2f times faster", ElapsedTime1/ElapsedTime2 );

}

void SchemaTestSuite::TestInjection()
{
    gcstring Target =
        CDFHeader +
        "                                         \n"
        "<Integer Name=\"ValueA\">                \n"
        "   <Value>1</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Integer Name=\"ValueB\">                \n"
        "   <Value>1</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Group Comment=\"Test1\">                \n"
        "                                         \n"
        "   <Integer Name=\"ValueD\">             \n"
        "       <Value>1</Value>                  \n"
        "   </Integer>                            \n"
        "                                         \n"
        "   <Integer Name=\"ValueE\">             \n"
        "       <Value>1</Value>                  \n"
        "   </Integer>                            \n"
        "                                         \n"
        "</Group>                                 \n"
        "                                         \n"
        "<Category Name=\"CategoryX\">            \n"
        "   <Description>some stuff</Description> \n"
        "   <pFeature>ValueA</pFeature>           \n"
        "   <pFeature>ValueB</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        "<Category Name=\"CategoryY\">            \n"
        "   <Description>some stuff</Description> \n"
        "   <pFeature>ValueD</pFeature>           \n"
        "   <pFeature>ValueA</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        + CDFFooter;

    gcstring Inject =
        CDFHeader +
        "                                         \n"
        "<Integer Name=\"ValueA\" MergePriority=\"+1\">  \n"
        "   <Value>2</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Group Comment=\"Test2\">                \n"
        "                                         \n"
        "   <Integer Name=\"ValueC\">             \n"
        "       <Value>2</Value>                  \n"
        "   </Integer>                            \n"
        "                                         \n"
        "</Group>                                 \n"
        "                                         \n"
        "<Integer Name=\"ValueE\"  MergePriority=\"-1\">\n"
        "   <Value>2</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Category Name=\"CategoryX\">            \n"
        "   <Description>some stuff</Description> \n"
        "   <pFeature>ValueA</pFeature>           \n"
        "   <pFeature>ValueC</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        "<Category Name=\"CategoryZ\">            \n"
        "   <Description>some other stuff</Description> \n"
        "   <pFeature>ValueE</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromStringInject(Target, Inject);

    CIntegerPtr ptrValueA = Camera._GetNode("ValueA");
    CPPUNIT_ASSERT( ptrValueA.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 2LL, *ptrValueA() );

    CIntegerPtr ptrValueB = Camera._GetNode("ValueB");
    CPPUNIT_ASSERT( ptrValueB.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, *ptrValueB() );

    CIntegerPtr ptrValueC = Camera._GetNode("ValueC");
    CPPUNIT_ASSERT( ptrValueC.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 2LL, *ptrValueC() );

    CIntegerPtr ptrValueD = Camera._GetNode("ValueD");
    CPPUNIT_ASSERT( ptrValueD.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, *ptrValueD() );

    CIntegerPtr ptrValueE = Camera._GetNode("ValueE");
    CPPUNIT_ASSERT( ptrValueE.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 1LL, *ptrValueE() );



    FeatureList_t FeatureList;
    CCategoryPtr ptrCategoryX = Camera._GetNode("CategoryX");
    CPPUNIT_ASSERT( ptrCategoryX.IsValid() );
    ptrCategoryX->GetFeatures(FeatureList);
    cout << FeatureList[0]->GetNode()->GetName() << endl;
    cout << FeatureList[1]->GetNode()->GetName() << endl;
    cout << FeatureList[2]->GetNode()->GetName() << endl;
    CPPUNIT_ASSERT_EQUAL( gcstring("ValueA"), FeatureList[0]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("ValueB"), FeatureList[1]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("ValueC"), FeatureList[2]->GetNode()->GetName() );

    CCategoryPtr ptrCategoryY = Camera._GetNode("CategoryY");
    CPPUNIT_ASSERT( ptrCategoryY.IsValid() );
    ptrCategoryY->GetFeatures(FeatureList);
    CPPUNIT_ASSERT_EQUAL( gcstring("ValueD"), FeatureList[0]->GetNode()->GetName() );
    CPPUNIT_ASSERT_EQUAL( gcstring("ValueA"), FeatureList[1]->GetNode()->GetName() );

    CCategoryPtr ptrCategoryZ = Camera._GetNode("CategoryZ");
    CPPUNIT_ASSERT( ptrCategoryZ.IsValid() );
    ptrCategoryZ->GetFeatures(FeatureList);
    CPPUNIT_ASSERT_EQUAL( gcstring("ValueE"), FeatureList[0]->GetNode()->GetName() );


}

void SchemaTestSuite::TestHiddenRegister()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring Target =
        CDFHeader +
        "                                           \n"
        "<IntReg Name=\"Public\">                   \n"
        "    <Address>0x0000</Address>              \n"
        "    <Length>4</Length>                     \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Signed</Sign>                    \n"
        "    <Endianess>LittleEndian</Endianess>    \n"
        "</IntReg>                                  \n"
        "                                           \n"
        "<Port Name=\"Port\" />                     \n"
        "                                           \n"
        + CDFFooter;

    gcstring Inject =
        CDFHeader +
        "                                           \n"
        "<IntReg Name=\"Hidden\">                   \n"
        "    <Address>0xff00</Address>              \n"
        "    <Length>4</Length>                     \n"
        "    <AccessMode>RW</AccessMode>            \n"
        "    <pPort>Port</pPort>                    \n"
        "    <Sign>Signed</Sign>                    \n"
        "    <Endianess>LittleEndian</Endianess>    \n"
        "</IntReg>                                  \n"
        "                                           \n"
        "<Port Name=\"Port\" MergePriority=\"-1\"/> \n"
        "                                           \n"
        + CDFFooter;

    // Load target and Incet File
    CNodeMapRef Camera;
    Camera._LoadXMLFromStringInject(Target, Inject);

    // create, initialize, and connect a test port
    CTestPort Port;
    Port.CreateEntry(0x0000, (int32_t)42, RW);
    Port.CreateEntry(0xff00, (int32_t)13, RW);
    Camera._Connect(&Port, "Port");

    CIntegerPtr ptrPublic = Camera._GetNode("Public");
    CPPUNIT_ASSERT( ptrPublic.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 42LL, *ptrPublic() );
    *ptrPublic = 99;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 99LL, *ptrPublic() );

    CIntegerPtr ptrHidden = Camera._GetNode("Hidden");
    CPPUNIT_ASSERT( ptrHidden.IsValid() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 13LL, *ptrHidden() );
    *ptrHidden = 66;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 66LL, *ptrHidden() );

}



void SchemaTestSuite::TestCLGlue()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraXML =
        CDFHeader +
        "                                         \n"
        "<Integer Name=\"Width\">                 \n"
        "    <pValue>WidthCamera</pValue>         \n"
        "    <pValueCopy>WidthGrabber</pValueCopy>\n"
        "</Integer>                               \n"
        "                                         \n"
        "<Integer Name=\"WidthCamera\">           \n"
        "   <Value>42</Value>                     \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Integer Name=\"WidthGrabber\">          \n"
        "   <Value>1</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        + CDFFooter;

    gcstring GrabberXML =
        CDFHeader +
        "                                             \n"
        "<Integer Name=\"WidthGrabber\" MergePriority=\"+1\">\n"
        "   <Value>2</Value>                          \n"
        "</Integer>                                   \n"
        "                                             \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromStringInject(CameraXML, GrabberXML);

    CIntegerPtr ptrWidth = Camera._GetNode("Width");
    CPPUNIT_ASSERT( ptrWidth.IsValid() );

    CIntegerPtr ptrWidthCamera = Camera._GetNode("WidthCamera");
    CPPUNIT_ASSERT( ptrWidthCamera.IsValid() );

    CIntegerPtr ptrWidthGrabber = Camera._GetNode("WidthGrabber");
    CPPUNIT_ASSERT( ptrWidthGrabber.IsValid() );

    *ptrWidth = 99;
    CPPUNIT_ASSERT_EQUAL( (int64_t) 99LL, *ptrWidth() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 99LL, *ptrWidthCamera() );
    CPPUNIT_ASSERT_EQUAL( (int64_t) 99LL, *ptrWidthGrabber() );

}

void SchemaTestSuite::TestInjectionBug()
{
    gcstring Target =
        CDFHeader +
        "                                         \n"
        "<Integer Name=\"ValueA\">                \n"
        "   <Value>1</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Integer Name=\"ValueB\">                \n"
        "   <Value>1</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Group Comment=\"Test1\">                \n"
        "                                         \n"
        // this is legal by the schema but only allowed for inject files => so we get an exceptio
        "   <Integer Name=\"ValueD\" MergePriority=\"+1\">             \n"
        "       <Value>1</Value>                  \n"
        "   </Integer>                            \n"
        "                                         \n"
        "   <Integer Name=\"ValueE\">             \n"
        "       <Value>1</Value>                  \n"
        "   </Integer>                            \n"
        "                                         \n"
        "</Group>                                 \n"
        "                                         \n"
        "<Category Name=\"CategoryX\">            \n"
        "   <Description>some stuff</Description> \n"
        "   <pFeature>ValueA</pFeature>           \n"
        "   <pFeature>ValueB</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        "<Category Name=\"CategoryY\">            \n"
        "   <Description>some stuff</Description> \n"
        "   <pFeature>ValueD</pFeature>           \n"
        "   <pFeature>ValueA</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        + CDFFooter;

    gcstring Inject =
        CDFHeader +
        "                                         \n"
        "<Integer Name=\"ValueA\" MergePriority=\"+1\">  \n"
        "   <Value>2</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Group Comment=\"Test2\">                \n"
        "                                         \n"
        "   <Integer Name=\"ValueC\">             \n"
        "       <Value>2</Value>                  \n"
        "   </Integer>                            \n"
        "                                         \n"
        "</Group>                                 \n"
        "                                         \n"
        "<Integer Name=\"ValueE\"  MergePriority=\"-1\">\n"
        "   <Value>2</Value>                      \n"
        "</Integer>                               \n"
        "                                         \n"
        "<Category Name=\"CategoryX\">            \n"
        "   <Description>some stuff</Description> \n"
        "   <pFeature>ValueA</pFeature>           \n"
        "   <pFeature>ValueC</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        "<Category Name=\"CategoryZ\">            \n"
        "   <Description>some other stuff</Description> \n"
        "   <pFeature>ValueE</pFeature>           \n"
        "</Category>                              \n"
        "                                         \n"
        + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_THROW_EX( Camera._LoadXMLFromStringInject(Target, Inject), GenICam::RuntimeException);
}

void SchemaTestSuite::TestTicket855()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring Target =
        CDFHeader +
        "                                           \n"
        "<Category Name=\"CategoryX\">              \n"
        "   <pFeature>ValueA</pFeature>             \n"
        "</Category>                                \n"
        "                                           \n"
        "<Integer Name=\"ValueA\">                  \n"
        "   <Value>2</Value>                        \n"
        "</Integer>                                 \n"
        "                                           \n"
        + CDFFooter;

    gcstring Inject =
        CDFHeader +
        "                                                    \n"
        "<Category Name=\"CategoryX\" MergePriority=\"+1\">  \n" // this is ok because in the target file another category with the same name exists
        "   <pFeature>ValueB</pFeature>                      \n"
        "</Category>                                         \n"
        "                                                    \n"
        "<Integer Name=\"ValueB\" MergePriority=\"+1\">      \n"
        "   <Value>3</Value>                                 \n"
        "</Integer>                                          \n"
        "                                                    \n"
        "<Category Name=\"CategoryY\" MergePriority=\"+1\">  \n" // this fails because the node is copied as is including the MergePriority attribute which 
        "   <pFeature>ValueC</pFeature>                      \n" //    is not accepted by the NodeMap creation process
        "</Category>                                         \n"
        "                                                    \n"
        "<Integer Name=\"ValueC\" MergePriority=\"+1\">      \n"
        "   <Value>3</Value>                                 \n"
        "</Integer>                                          \n"
        "                                                    \n"
        + CDFFooter;

    // Load target and Incet File
    CNodeMapRef Camera;
    Camera._LoadXMLFromStringInject(Target, Inject);

    NODE_POINTER( Integer, ValueA );
    NODE_POINTER( Integer, ValueB );

    cout << "ValueA = " << ptrValueA->GetValue() << endl;
    cout << "ValueB = " << ptrValueB->GetValue() << endl;

}

void SchemaTestSuite::TestTicket856()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    /*[ GenApiTest@SchemaTestSuite_TestTicket856_Target.xml|gxml

      <IntSwissKnife Name="SwissA">
        <pVariable Name="A">ValueA</pVariable>
        <Formula>1</Formula>
      </IntSwissKnife>
      
      <Integer Name="ValueA">
        <Value>1</Value>
      </Integer>                           
        
    */


    /*[ GenApiTest@SchemaTestSuite_TestTicket856_Inject.xml|gxml

      <IntSwissKnife Name="SwissA" MergePriority="+1">
        <pVariable Name="A">ValueA</pVariable>
        <Formula>2</Formula>
      </IntSwissKnife>

      <Integer Name="ValueA" MergePriority="-1">
        <Value>2</Value>
      </Integer>                                         

    */

    // Load target and Inject File
    {
        CNodeMapRef Camera;
        Camera._MergeXMLFiles( "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket856_Target.xml", 
            "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket856_Inject.xml", 
            "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket856_Merged.xml" );
    }

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile( "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket856_Merged.xml" );

    NODE_POINTER( Integer, SwissA );
    NODE_POINTER( Integer, ValueA );

    CPPUNIT_ASSERT_EQUAL( (int64_t)2, ptrSwissA->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrValueA->GetValue() );


}

void SchemaTestSuite::TestTicket873()
{

    if(GenApiSchemaVersion == v1_0)
        return;

    /*[ GenApiTest@SchemaTestSuite_TestTicket873_Target.xml|gxml

    <Enumeration Name="MyEnumeration">
        <EnumEntry Name="Enum1">
            <Value>11</Value>
        </EnumEntry>
        <EnumEntry Name="Enum2">
            <Value>12</Value>
        </EnumEntry>
        <EnumEntry Name="Enum3">
            <Value>13</Value>
        </EnumEntry>
        <Value>11</Value>
    </Enumeration>

    <Enumeration Name="MyEnumeration1">
        <EnumEntry Name="Enum1">
            <Value>1</Value>
        </EnumEntry>
        <Value>1</Value>
    </Enumeration>

    <Enumeration Name="MyEnumeration2">
        <EnumEntry Name="Enum1">
            <Value>2</Value>
        </EnumEntry>
        <Value>2</Value>
    </Enumeration>

    */


    /*[ GenApiTest@SchemaTestSuite_TestTicket873_Inject.xml|gxml

    <Enumeration Name="MyEnumeration">
        <EnumEntry Name="Enum1" MergePriority="-1">
            <Value>21</Value>
        </EnumEntry>
        <EnumEntry Name="Enum2" MergePriority="+1">
            <Value>22</Value>
        </EnumEntry>
        <EnumEntry Name="Enum4">
            <Value>24</Value>
        </EnumEntry>
        <Value>21</Value>
    </Enumeration>

    <Enumeration Name="MyEnumeration1" MergePriority="-1">
        <EnumEntry Name="Enum1">
            <Value>2</Value>
        </EnumEntry>
        <Value>2</Value>
    </Enumeration>

    <Enumeration Name="MyEnumeration2" MergePriority="+1">
        <EnumEntry Name="Enum1">
            <Value>2</Value>
        </EnumEntry>
        <Value>2</Value>
    </Enumeration>

    */

    // Load target and Inject File
    {
        CNodeMapRef Camera;
        Camera._MergeXMLFiles( "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket873_Target.xml", 
            "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket873_Inject.xml", 
            "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket873_Merged.xml" );
    }

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile( "$(" GENICAM_ROOT_VERSION ")/xml/GenApi/GenApiTest/SchemaTestSuite_TestTicket873_Merged.xml" );

    NODE_POINTER( EnumEntry, EnumEntry_MyEnumeration_Enum1 );
    NODE_POINTER( EnumEntry, EnumEntry_MyEnumeration_Enum2 );
    NODE_POINTER( EnumEntry, EnumEntry_MyEnumeration_Enum3 );
    NODE_POINTER( EnumEntry, EnumEntry_MyEnumeration_Enum4 );
    NODE_POINTER( EnumEntry, EnumEntry_MyEnumeration1_Enum1 );
    NODE_POINTER( EnumEntry, EnumEntry_MyEnumeration2_Enum1 );

    CPPUNIT_ASSERT_EQUAL( (int64_t)11, ptrEnumEntry_MyEnumeration_Enum1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)22, ptrEnumEntry_MyEnumeration_Enum2->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)13, ptrEnumEntry_MyEnumeration_Enum3->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)24, ptrEnumEntry_MyEnumeration_Enum4->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrEnumEntry_MyEnumeration1_Enum1->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)2, ptrEnumEntry_MyEnumeration2_Enum1->GetValue() );

}
