//-----------------------------------------------------------------------------
//  (c) 2010 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of ISelector
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "SelectorSetTest.h"
#include "GenApi/GenApi.h"
#include "GenApi/EnumClasses.h"
#include "GenApi/SelectorSet.h"


using namespace std;
using namespace GenICam;
using namespace GenApi;


CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SelectorSetTestSuite, "GenApiTest");

void SelectorSetTestSuite::TestParallelSelectors()
{

    /*[ GenApiTest@SelectorSetTestSuite_TestParallelSelectors.xml|gxml

    <Integer Name="LUTValue">
        <Value>42</Value>
    </Integer>

    <Integer Name="LUTIndex">
        <Value>4</Value>
        <Min>0</Min>
        <Max>7</Max>
        <pSelected>LUTValue</pSelected>
    </Integer>

    <Enumeration Name="LUTSelector">
        <EnumEntry Name="Red">
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Green">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Blue">
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>LUTValue</pSelected>
    </Enumeration>

    */


    {   // expected way to behave

        CNodeMapRef Camera;
        Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestParallelSelectors");

        NODE_POINTER( Integer, LUTValue )

        CSelectorSet ssLUTValue( ptrLUTValue );

        ostringstream TestBuffer;
        CPPUNIT_ASSERT( ssLUTValue.SetFirst() );
        do 
        {
            TestBuffer << ssLUTValue.ToString() << endl;
        } 
        while( ssLUTValue.SetNext() );
        cout << TestBuffer.str();

        ostringstream VerifyBuffer;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=0]" << endl;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=1]" << endl;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=2]" << endl;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=3]" << endl;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=4]" << endl;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=5]" << endl;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=6]" << endl;
        VerifyBuffer << "[LUTSelector=Red, LUTIndex=7]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=0]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=1]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=2]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=3]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=4]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=5]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=6]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=7]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=0]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=1]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=2]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=3]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=4]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=5]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=6]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=7]" << endl;
        
        CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );
    }

    {   // same again but without calling SetFirst

        CNodeMapRef Camera;
        Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestParallelSelectors");

        NODE_POINTER( Integer, LUTValue )

        CSelectorSet ssLUTValue( ptrLUTValue );

        ostringstream TestBuffer;
        do 
        {
            TestBuffer << ssLUTValue.ToString() << endl;
        } 
        while( ssLUTValue.SetNext() );
        cout << TestBuffer.str();

        ostringstream VerifyBuffer;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=4]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=5]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=6]" << endl;
        VerifyBuffer << "[LUTSelector=Green, LUTIndex=7]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=0]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=1]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=2]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=3]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=4]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=5]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=6]" << endl;
        VerifyBuffer << "[LUTSelector=Blue, LUTIndex=7]" << endl;

        CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );
    }
}

void SelectorSetTestSuite::TestSequentialSelectors()
{

    /*[ GenApiTest@SelectorSetTestSuite_TestSequentialSelectors.xml|gxml

    <Integer Name="LUTValue">
        <Value>42</Value>
    </Integer>

    <Integer Name="LUTIndex">
        <Value>0</Value>
        <Min>0</Min>
        <Max>7</Max>
        <pSelected>LUTValue</pSelected>
    </Integer>

    <Enumeration Name="LUTSelector">
        <EnumEntry Name="Red">
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Green">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Blue">
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>LUTIndex</pSelected>
    </Enumeration>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestSequentialSelectors");

    NODE_POINTER( Integer, LUTValue )

    CSelectorSet ssLUTValue( ptrLUTValue );

    ostringstream TestBuffer;
    CPPUNIT_ASSERT( ssLUTValue.SetFirst() );
    do 
    {
        TestBuffer << ssLUTValue.ToString() << endl;
    } 
    while( ssLUTValue.SetNext() );
    cout << TestBuffer.str();

    ostringstream VerifyBuffer;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=2]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=3]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=4]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=5]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=6]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=7]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=2]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=3]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=4]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=5]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=6]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=7]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=2]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=3]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=4]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=5]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=6]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=7]" << endl;

    CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );

}


void SelectorSetTestSuite::TestRealWordLUT()
{

    /*[ GenApiTest@SelectorSetTestSuite_TestRealWordLUT.xml|gxml

    <Integer Name="LUTValue">
        <Value>42</Value>
    </Integer>

    <Integer Name="LUTIndex">
        <Value>0</Value>
        <Min>0</Min>
        <Max>2</Max>
        <pSelected>LUTValue</pSelected>
    </Integer>

    <Boolean Name="LUTEnable">
        <Value>0</Value>
    </Boolean>

    <Register Name="LUTValueAll">
        <Address>0x123</Address>
        <Length>4</Length>
        <AccessMode>RW</AccessMode>
        <pPort>Device</pPort>
    </Register>
    
    <Enumeration Name="LUTSelector">
        <EnumEntry Name="Red">
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Green">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Blue">
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>LUTEnable</pSelected>
        <pSelected>LUTIndex</pSelected>
        <pSelected>LUTValue</pSelected>
        <pSelected>LUTValueAll</pSelected>
    </Enumeration>

    <Port Name="Device">        
    </Port>
    

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestRealWordLUT");

    NODE_POINTER( Integer, LUTValue )

    CSelectorSet ssLUTValue( ptrLUTValue );

    ostringstream TestBuffer;
    CPPUNIT_ASSERT( ssLUTValue.SetFirst() );
    do 
    {
        TestBuffer << ssLUTValue.ToString() << endl;
    } 
    while( ssLUTValue.SetNext() );
    cout << TestBuffer.str();

    ostringstream VerifyBuffer;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=2]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Green, LUTIndex=2]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=2]" << endl;

    CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );

}


void SelectorSetTestSuite::TestSelectorTree()
{

    /*[ GenApiTest@SelectorSetTestSuite_TestSelectorTree.xml|gxml

    <Integer Name="Value">
        <Value>42</Value>
    </Integer>

    <Integer Name="A">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Value</pSelected>
    </Integer>

    <Integer Name="B">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Value</pSelected>
    </Integer>

    <Integer Name="C">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Value</pSelected>
    </Integer>

    <Integer Name="Ba">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>B</pSelected>
    </Integer>

    <Integer Name="Bb">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>B</pSelected>
    </Integer>

    <Integer Name="Bc">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>B</pSelected>
    </Integer>

    <Integer Name="BbX">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Bb</pSelected>
    </Integer>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestSelectorTree");

    NODE_POINTER( Integer, Value )

    CSelectorSet ssValue( ptrValue );

    ostringstream TestBuffer;
    CPPUNIT_ASSERT( ssValue.SetFirst() );
    do 
    {
        TestBuffer << ssValue.ToString() << endl;
    } 
    while( ssValue.SetNext() );
    cout << TestBuffer.str();

    ostringstream VerifyBuffer;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=0, Bb=1, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=0, BbX=1, Bb=1, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=0, Bb=1, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=0, Bc=1, BbX=1, Bb=1, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=0, Bb=1, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=0, BbX=1, Bb=1, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=0, Bb=1, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=0, Ba=1, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=0, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=0, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=0, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=0, B=1, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=1, B=0, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=1, B=0, A=1]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=1, B=1, A=0]" << endl;
    VerifyBuffer << "[C=1, Bc=1, BbX=1, Bb=1, Ba=1, B=1, A=1]" << endl;
    
    CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );
}

void SelectorSetTestSuite::TestSelectorRing()
{
    /*[ GenApiTest@SelectorSetTestSuite_TestSelectorRing.xml|gxml

    <Integer Name="Value">
        <Value>42</Value>
    </Integer>

    <Integer Name="A">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Value</pSelected>
    </Integer>

    <Integer Name="B">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Value</pSelected>
    </Integer>

    <Integer Name="C">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Value</pSelected>
    </Integer>

    <Integer Name="Ba">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>B</pSelected>
    </Integer>

    <Integer Name="Bb">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>B</pSelected>
    </Integer>

    <Integer Name="Bc">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>B</pSelected>
    </Integer>

    <Integer Name="BbX">
        <Value>0</Value>
        <Min>0</Min>
        <Max>1</Max>
        <pSelected>Bb</pSelected>
        <pSelected>C</pSelected>
    </Integer>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestSelectorRing");

    NODE_POINTER( Integer, Value )

    CSelectorSet ssValue( ptrValue );

    FeatureList_t SelectorList;
    ssValue.GetSelectorList( SelectorList );

    {
        bool First = true;
        ostringstream TestBuffer;
        for( FeatureList_t::iterator itSelector=SelectorList.begin(); itSelector!=SelectorList.end(); itSelector++ )
        {
            if( !First )
                TestBuffer << ", ";
            else
                First = false;
            TestBuffer << (*itSelector)->GetNode()->GetName();
        }
        
        ostringstream VerifyBuffer;
        VerifyBuffer << "BbX, C, Bc, Bb, Ba, B, A";

        CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );
    }

    ostringstream TestBuffer;
    CPPUNIT_ASSERT( ssValue.SetFirst() );
    do 
    {
        TestBuffer << ssValue.ToString() << " : ";

        bool First = true;
        FeatureList_t SelectorList;
        ssValue.GetSelectorList( SelectorList, true );
        for( FeatureList_t::iterator itSelector=SelectorList.begin(); itSelector!=SelectorList.end(); itSelector++ )
        {
            if( !First )
                TestBuffer << ", ";
            else
                First = false;
            TestBuffer << (*itSelector)->GetNode()->GetName();
        }
        TestBuffer << endl;
    } 
    while( ssValue.SetNext() );
    cout << TestBuffer.str();

    ostringstream VerifyBuffer;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=0, B=0, A=0] : BbX, C, Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=0, Bb=1, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=0, B=0, A=0] : Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=0, Bc=1, Bb=1, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=0, B=0, A=0] : C, Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=0, Bb=1, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=0, B=0, A=0] : Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=0, C=1, Bc=1, Bb=1, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=0, B=0, A=0] : BbX, C, Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=0, Bb=1, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=0, B=0, A=0] : Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=0, Bc=1, Bb=1, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=0, B=0, A=0] : C, Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=0, Bb=1, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=0, B=0, A=0] : Bc, Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=0, Ba=1, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=0, B=0, A=0] : Bb, Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=0, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=0, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=0, B=1, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=1, B=0, A=0] : Ba, B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=1, B=0, A=1] : A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=1, B=1, A=0] : B, A" << endl;
    VerifyBuffer << "[BbX=1, C=1, Bc=1, Bb=1, Ba=1, B=1, A=1] : A" << endl;

    CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );

}

void SelectorSetTestSuite::TestNotAvailableEnum()
{

    /*[ GenApiTest@SelectorSetTestSuite_TestNotAvailableEnum.xml|gxml

    <Integer Name="LUTValue">
        <Value>42</Value>
    </Integer>

    <Integer Name="LUTIndex">
        <Value>0</Value>
        <Min>0</Min>
        <Max>7</Max>
        <pSelected>LUTValue</pSelected>
    </Integer>

    <Enumeration Name="LUTSelector">
        <EnumEntry Name="Red">
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Green">
            <pIsAvailable>Zero</pIsAvailable>
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Blue">
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>LUTValue</pSelected>
    </Enumeration>

    <Integer Name="Zero">
        <Value>0</Value>
    </Integer>
    

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestNotAvailableEnum");

    NODE_POINTER( Integer, LUTValue )

    CSelectorSet ssLUTValue( ptrLUTValue );

    ostringstream TestBuffer;
    CPPUNIT_ASSERT( ssLUTValue.SetFirst() );
    do 
    {
        TestBuffer << ssLUTValue.ToString() << endl;
    } 
    while( ssLUTValue.SetNext() );
    cout << TestBuffer.str();

    ostringstream VerifyBuffer;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=2]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=3]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=4]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=5]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=6]" << endl;
    VerifyBuffer << "[LUTSelector=Red, LUTIndex=7]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=0]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=1]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=2]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=3]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=4]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=5]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=6]" << endl;
    VerifyBuffer << "[LUTSelector=Blue, LUTIndex=7]" << endl;
    
    CPPUNIT_ASSERT_EQUAL( VerifyBuffer.str(), TestBuffer.str() );

}

void SelectorSetTestSuite::InvalidSelector()
{

    /*[ GenApiTest@SelectorSetTestSuite_InvalidSelector.xml|gxml

    <Integer Name="LUTValue">
        <Value>42</Value>
    </Integer>

    <Integer Name="LUTIndex">
        <Value>0</Value>
        <Min>0</Min>
        <Max>7</Max>
        <pSelected>LUTValue</pSelected>
    </Integer>

    <Enumeration Name="LUTSelector">
        <EnumEntry Name="Red">
            <pIsAvailable>Zero</pIsAvailable>
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Green">
            <pIsAvailable>Zero</pIsAvailable>
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Blue">
            <pIsAvailable>Zero</pIsAvailable>
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>LUTValue</pSelected>
    </Enumeration>

    <Integer Name="Zero">
        <Value>0</Value>
    </Integer>
    
    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_InvalidSelector");

    NODE_POINTER( Integer, LUTValue )
    CSelectorSet ssLUTValue( ptrLUTValue );

    // since the LUTSelector is not available the LUTIndex is the only selector
    FeatureList_t Selectors;
    ssLUTValue.GetSelectorList(Selectors);
    CPPUNIT_ASSERT_EQUAL( (size_t)1, Selectors.size() );
    CPPUNIT_ASSERT_EQUAL( gcstring("LUTIndex"), Selectors[0]->GetNode()->GetName() );

    // however the set is not empty
    CPPUNIT_ASSERT( !ssLUTValue.IsEmpty() );
}

void SelectorSetTestSuite::SelectorWithoutValues()
{

    /*[ GenApiTest@SelectorSetTestSuite_SelectorWithoutValues.xml|gxml

    <Integer Name="LUTValue">
        <Value>42</Value>
    </Integer>

    <Integer Name="LUTIndex">
        <pIsAvailable>Zero</pIsAvailable>
        <Value>0</Value>
        <Min>0</Min>
        <Max>7</Max>
        <pSelected>LUTValue</pSelected>
    </Integer>

    <Enumeration Name="LUTSelector">
        <EnumEntry Name="Red">
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Green">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Blue">
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>LUTIndex</pSelected>
    </Enumeration>

    <Integer Name="Zero">
        <Value>0</Value>
    </Integer>
    
    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_SelectorWithoutValues");

    NODE_POINTER( Integer, LUTValue )
    CSelectorSet ssLUTValue( ptrLUTValue );

    // since the LUTIndex is not available LUTValue has no selector (note that LUTSelector selects LUTIndex)
    FeatureList_t Selectors;
    ssLUTValue.GetSelectorList(Selectors);
    CPPUNIT_ASSERT(  Selectors.empty() );

}

void SelectorSetTestSuite::RestoreSelector()
{

    /*[ GenApiTest@SelectorSetTestSuite_RestoreSelector.xml|gxml

    <Integer Name="LUTValue">
        <Value>42</Value>
    </Integer>

    <Integer Name="LUTIndex">
        <Value>4</Value>
        <Min>0</Min>
        <Max>7</Max>
        <pSelected>LUTValue</pSelected>
    </Integer>

    <Enumeration Name="LUTSelector">
        <EnumEntry Name="Red">
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Green">
            <Value>1</Value>
        </EnumEntry>
        <EnumEntry Name="Blue">
            <Value>2</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>LUTValue</pSelected>
    </Enumeration>

    */


    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_RestoreSelector");

    NODE_POINTER( Integer, LUTValue )
    NODE_POINTER( Integer, LUTIndex )
    NODE_POINTER( Enumeration, LUTSelector )

    CSelectorSet ssLUTValue( ptrLUTValue );

    CPPUNIT_ASSERT( ssLUTValue.SetFirst() );
    ostringstream TestBuffer;
    do 
    {
        cout << ssLUTValue.ToString() << endl;
    } 
    while( ssLUTValue.SetNext() );

    ssLUTValue.Restore();

    CPPUNIT_ASSERT_EQUAL( (int64_t)42, ptrLUTValue->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)4, ptrLUTIndex->GetValue() );
    CPPUNIT_ASSERT_EQUAL( (int64_t)1, ptrLUTSelector->GetIntValue() );

}

void SelectorSetTestSuite::TestRecursionBreaker()
{

    /*[ GenApiTest@SelectorSetTestSuite_TestRecursionBreaker.xml|gxml
    
    <Integer Name="Feature">
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
        <!-- Recursion! -->
        <pSelected>OuterSelector</pSelected>
    </Integer>

    <Enumeration Name="InnerSelector">
        <EnumEntry Name="InnerEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>Feature</pSelected>
    </Enumeration>

    <Enumeration Name="OuterSelector">
        <EnumEntry Name="OuterEntry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="OuterEntry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>1</Value>
        <pSelected>InnerSelector</pSelected>
    </Enumeration>
       
    */


    CNodeMapRef Camera;
    CPPUNIT_ASSERT_THROW_EX( Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_TestRecursionBreaker"), GenICam::RuntimeException );

}



void SelectorSetTestSuite::SelectorNotImplemented()
{

    /*[ GenApiTest@SelectorSetTestSuite_SelectorNotImplemented.xml|gxml
    
    <Integer Name="Selectee">
        <Streamable>Yes</Streamable>
        <Value>192</Value>
        <Min>192</Min>
        <Max>1023</Max>
    </Integer>

    <Enumeration Name="Selector">
        <pIsImplemented>False</pIsImplemented>
        <EnumEntry Name="Entry0" >
            <Value>0</Value>
        </EnumEntry>
        <EnumEntry Name="Entry1" >
            <Value>1</Value>
        </EnumEntry>
        <Value>0</Value>
        <pSelected>Selectee</pSelected>
    </Enumeration>

    <Integer Name="False">
        <Value>0</Value>
    </Integer>
    
    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "SelectorSetTestSuite_SelectorNotImplemented");

    NODE_POINTER( Integer, Selectee )

    CSelectorSet ssSelectee( ptrSelectee );

    // since the selector is not implemented the list of selectors is empty
    FeatureList_t Selectors;
    ssSelectee.GetSelectorList(Selectors);
    CPPUNIT_ASSERT( Selectors.empty() );
    CPPUNIT_ASSERT( ssSelectee.IsEmpty() );
}


void SelectorSetTestSuite::TestRestoreSelectorNotAvailableEnum()
{

    /*[ GenApiTest@TestRestoreSelectorNotAvailableEnum.xml|gxml

    <Integer Name="LUTValue">
    <Value>42</Value>
    </Integer>

    <Enumeration Name="SelectorB">
    <EnumEntry Name="B1">
    <Value>0</Value>
    </EnumEntry>
    <EnumEntry Name="B2">
    <pIsImplemented>B2Implemented</pIsImplemented>
    <Value>1</Value>
    </EnumEntry>
    <Value>1</Value>
    <pSelected>LUTValue</pSelected>
    </Enumeration>

    <Enumeration Name="SelectorA">
    <EnumEntry Name="A1">
    <Value>0</Value>
    </EnumEntry>
    <EnumEntry Name="A2">
    <Value>1</Value>
    </EnumEntry>
    <pValue>SelectorAValue</pValue>
    <pSelected>SelectorB</pSelected>
    </Enumeration>

    <Integer Name="SelectorAValue">
    <Visibility>Invisible</Visibility>
    <Value>1</Value>
    </Integer>

    <IntSwissKnife Name="B2Implemented">
    <Visibility>Invisible</Visibility>
    <pVariable Name="SELEACTORA_VALUE">SelectorAValue</pVariable>
    <Formula>SELEACTORA_VALUE = 1</Formula>
    </IntSwissKnife>
    */

    CNodeMapRef Camera;
    Camera._LoadXMLFromFile("GenApiTest", "TestRestoreSelectorNotAvailableEnum");

    NODE_POINTER( Integer, LUTValue )

    CSelectorSet ssLUTValue( ptrLUTValue );

    ostringstream TestBuffer;
    CPPUNIT_ASSERT( ssLUTValue.SetFirst() );
    do 
    {
        TestBuffer << ssLUTValue.ToString() << endl;
    } 
    while( ssLUTValue.SetNext() );
    cout << TestBuffer.str();
    cout << "SelectorA=" << CValuePtr(Camera._GetNode("SelectorA"))->ToString() << endl;
    cout << "SelectorB=" <<  CValuePtr(Camera._GetNode("SelectorB"))->ToString() << endl;

    //Ticket #1065
    //SelectorA(A1,A2)->SelectorB(B1,B2)->AValue
    //B2 is only implemented if A2 is selected

    //The CSelectorSet remembers the setting of all selectors before iterating.
    //After iterating over the selector set the values A1 and B1 are selected.
    //Calling CSelectorSet::Restore fails because the original values are restored in the wrong order.
    //The implementation tries to select B2 for SelectorB while A1 is selected.
    //The values should be restored in order SelectorA then SelectorB.

    ssLUTValue.Restore();

    CPPUNIT_ASSERT( CValuePtr(Camera._GetNode("SelectorA"))->ToString() == "A2" );
    CPPUNIT_ASSERT( CValuePtr(Camera._GetNode("SelectorB"))->ToString() == "B2" );
}

