//-----------------------------------------------------------------------------
//  (c) 2008 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Test the implementation of CFloat
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "StringTest.h"
#include "GenApi/GenApi.h"
#include "GenApi/impl/PolyReference.h"
#include <string>

using namespace std;
using namespace GenICam;
using namespace GenApi;

#undef min

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( StringTestSuite, "GenApiTest");


// ---------------------------------------------------------------------------
// void StringTestSuite::TestValueAccess()
// ---------------------------------------------------------------------------
/*!
 * Perform basic tests on CString
 */
void StringTestSuite::TestValueAccess()
{
    if(GenApiSchemaVersion < v1_1)
        return;

    gcstring CameraDescription =
    CDFHeader +
    "                                         \n"
    "<String Name=\"A\">                      \n"
    "    <Value>Alle meine Entchen...</Value>   \n"
    "</String>                                \n"
    "                                         \n"
    "<String Name=\"RO\">                      \n"
    "    <ImposedAccessMode>RO</ImposedAccessMode>   \n"
    "    <Value>Alle meine Ganschen...</Value>   \n"
    "</String>                                \n"
    "                                         \n"
    "<String Name=\"WO\">                      \n"
    "    <ImposedAccessMode>WO</ImposedAccessMode>   \n"
    "    <Value>Alle meine Ganschen...</Value>   \n"
    "</String>                                \n"
    "                                         \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CStringPtr ptrA = Camera._GetNode( "A" );
    CPPUNIT_ASSERT( (bool) ptrA );
    CPPUNIT_ASSERT_EQUAL (intfIString, ptrA->GetNode()->GetPrincipalInterfaceType());
    CPPUNIT_ASSERT_EQUAL( gcstring( "Alle meine Entchen..." ), ptrA->GetValue() );

    ptrA->SetValue("...schwimmen auf dem See!");
    CPPUNIT_ASSERT_EQUAL( gcstring( "...schwimmen auf dem See!" ), ptrA->GetValue() );

    ptrA->FromString("Ehh? Don't understand Hungarian...");
    CPPUNIT_ASSERT_EQUAL( gcstring("Ehh? Don't understand Hungarian..."), ptrA->ToString() );

    // excercise also strings with non-RW access mode
    CStringPtr ptrRO = Camera._GetNode( "RO" );
    CPPUNIT_ASSERT( (bool) ptrRO );
    CPPUNIT_ASSERT_EQUAL( gcstring( "Alle meine Ganschen..." ), ptrRO->GetValue() );
    CPPUNIT_ASSERT_THROW_EX( ptrRO->FromString("Ehh? Don't understand Hungarian..."), GenICam::AccessException );
    CPPUNIT_ASSERT_EQUAL( gcstring("Alle meine Ganschen..."), ptrRO->ToString() );

    CStringPtr ptrWO = Camera._GetNode( "WO" );
    CPPUNIT_ASSERT( (bool) ptrWO );
    // note that the access mode is checked only because Verify==true
    CPPUNIT_ASSERT_THROW_EX( ptrWO->GetValue(true), GenICam::AccessException );
    ptrWO->FromString("Hmmm, how will I verify this value?");
    // note that the access mode is checked only because Verify==true
    CPPUNIT_ASSERT_THROW_EX( ptrWO->ToString(true), GenICam::AccessException );

    //cout << "MaxLength = " << ptrA->GetMaxLength() << "\n";
    gcstring Dummy;
    CPPUNIT_ASSERT_EQUAL( (int64_t)Dummy.max_size(), ptrA->GetMaxLength());

}

void StringTestSuite::TestPValueAccess()
{
    if(GenApiSchemaVersion < v1_1)
        return;

    gcstring CameraDescription =
    CDFHeader +
    "                                         \n"
    "<String Name=\"A\">                      \n"
    "    <Value>Hopsa hejsa do Brandejsa...</Value>   \n"
    "</String>                                \n"
    "                                         \n"
    "<String Name=\"B\">                      \n"
    "    <pValue>A</pValue>   \n"
    "</String>                                \n"
    "                                         \n"
    "<String Name=\"C\">                      \n"
    "    <pValue>B</pValue>   \n"
    "</String>                                \n"
    "                                         \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    CStringPtr ptrA = Camera._GetNode( "A" );
    CPPUNIT_ASSERT( (bool) ptrA );
    CPPUNIT_ASSERT_EQUAL( gcstring( "Hopsa hejsa do Brandejsa..." ), ptrA->GetValue() );

    CStringPtr ptrC = Camera._GetNode( "C" );
    CPPUNIT_ASSERT( (bool) ptrC );
    CPPUNIT_ASSERT_EQUAL( gcstring( "Hopsa hejsa do Brandejsa..." ), ptrC->GetValue() );

    ptrC->SetValue("...Brandejs nad Labem, to se nadlabem!");
    // <String> nodes with <pValue> entry to not have a cache. So they return the cache state of the node referred to
    CPPUNIT_ASSERT_EQUAL (true, ptrC->IsValueCacheValid());
    CPPUNIT_ASSERT_EQUAL( gcstring( "...Brandejs nad Labem, to se nadlabem!" ), ptrC->GetValue() );
    CPPUNIT_ASSERT_EQUAL (true, ptrC->IsValueCacheValid());
    CPPUNIT_ASSERT_EQUAL( gcstring( "...Brandejs nad Labem, to se nadlabem!" ), ptrA->GetValue() );

    // and test also the properties
    CNodePtr ptrAnode = (CNodePtr) ptrA;
    gcstring ValueStr;
    gcstring AttributeStr;
    CPPUNIT_ASSERT( ptrAnode->GetProperty("Value", ValueStr, AttributeStr) );
    CPPUNIT_ASSERT_EQUAL( gcstring("...Brandejs nad Labem, to se nadlabem!"), ValueStr );

    gcstring Dummy;
    CPPUNIT_ASSERT_EQUAL( (int64_t)Dummy.max_size(), ptrC->GetMaxLength());

}


void StringTestSuite::TestPolyReference()
{
    // test artificially the hard-to-test code paths
    CStringPolyRef poly;
    CPPUNIT_ASSERT_EQUAL (false, poly.IsInitialized());
    CPPUNIT_ASSERT_EQUAL (false, poly.IsPointer());
    CPPUNIT_ASSERT_THROW_EX (poly.SetValue ("gipf"), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (poly.GetValue (), GenICam::RuntimeException);
    CPPUNIT_ASSERT_THROW_EX (poly.GetMaxLength (), GenICam::RuntimeException);
    //CPPUNIT_ASSERT_THROW_EX (poly.IsValueCacheValid(), GenICam::RuntimeException);
    poly = "yinsh";
    CPPUNIT_ASSERT_EQUAL (true, poly.IsInitialized());
    CPPUNIT_ASSERT_EQUAL (false, poly.IsPointer());
    CPPUNIT_ASSERT_EQUAL ((INode*)NULL, poly.GetPointer());
    //CPPUNIT_ASSERT_THROW_EX (poly.IsValueCacheValid(), GenICam::RuntimeException);

}

#       ifdef _MSC_VER
// http://rishida.net/tools/conversion/
void StringTestSuite::TestUTF16Handling()
{
    wchar_t pBufferUTF16[] = L"\x00E8\x00E9\x00F8\x00DE\x01FD\x043B\x0459\x03A3\x00E6\x010Da\x00DF";
    const char pReferenceBufferUTF8[] = {0xC3u, 0xA8u, 0xC3u, 0xA9u, 0xC3u, 0xB8u, 0xC3u, 0x9Eu, 0xC7u, 0xBDu, 0xD0u, 0xBBu, 
        0xD1u, 0x99u, 0xCEu, 0xA3u, 0xC3u, 0xA6u, 0xE1u, 0x83u, 0x9Au, 0xC3u, 0x9Fu, 0x00u };
    // c3 a8 c3 a9 c3 b8 c3 9e c7 bd d0 bb d1 99 ce a3 c3 a6 c4 8d 61 c3 9f 
    // c3 a8 c3 a9 c3 b8 c3 9e c7 bd d0 bb d1 99 ce a3 c3 a6 e1 83 9a c3 9f
    {
        gcstring BufferUTF8;
        BufferUTF8.assign(pBufferUTF16);

        const char *pBufferUTF8 = BufferUTF8.c_str();
        CPPUNIT_ASSERT_EQUAL( 0, strcmp( pReferenceBufferUTF8, pBufferUTF8 ) );
    }

    {
        gcstring BufferUTF8;
        BufferUTF8 = pBufferUTF16;

        const char *pBufferUTF8 = BufferUTF8.c_str();
        CPPUNIT_ASSERT_EQUAL( 0, strcmp( pReferenceBufferUTF8, pBufferUTF8 ) );
    }

    {
        gcstring BufferUTF8(pBufferUTF16);

        const char *pBufferUTF8 = BufferUTF8.c_str();
        CPPUNIT_ASSERT_EQUAL( 0, strcmp( pReferenceBufferUTF8, pBufferUTF8 ) );
    }

    {
        gcstring BufferUTF8;
        BufferUTF8 = pBufferUTF16;

        wchar_t pBufferUTF16Test[256];
        wcscpy( pBufferUTF16Test, BufferUTF8.w_str() );

        CPPUNIT_ASSERT_EQUAL( 0,  wcscmp( pBufferUTF16Test, pBufferUTF16 ) );
    }

    {
        gcstring BufferUTF8;
        BufferUTF8 = pBufferUTF16;

        wostringstream TestStreamUTF16;
        TestStreamUTF16 << (const wchar_t*)BufferUTF8.w_str();

        wstring &TestStringUTF16 = TestStreamUTF16.str();
        wchar_t pBufferUTF16Test[256];
        wcscpy( pBufferUTF16Test, TestStringUTF16.c_str() );

        CPPUNIT_ASSERT_EQUAL( 0, wcscmp( pBufferUTF16, pBufferUTF16Test) );
        CPPUNIT_ASSERT_EQUAL( 0, wcscmp( pBufferUTF16, (const wchar_t*)BufferUTF8.w_str()) );
    }

#ifdef USE_CONSOLE_OUTPUT
    // print the register content
    {
        gcstring BufferUTF8;
        BufferUTF8.assign(pBufferUTF16);

        UINT oldcp = GetConsoleOutputCP();
        SetConsoleOutputCP(CP_UTF8);
        wprintf(L"%S\n", BufferUTF8.c_str());
        SetConsoleOutputCP(oldcp);
    }
#endif
}
#endif
