////-----------------------------------------------------------------------------
////  (c) 2005 by Basler Vision Technologies
////  Section: Vision Components
////  Project: GenApiTest
////    Author:  Fritz Dierks
////  $Header$
////-----------------------------------------------------------------------------
///*!
//\file
//*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "SwissKnifeTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "StopWatch.h"
#include <math.h>

#include "GenApi/Test/TestCamera_v1_0.h"
#include "GenApi/Test/TestCamera_v1_1.h"

#include <limits>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SwissKnifeTestSuite, "GenApiTest");

static void testMinimum()
    {
    gcstring CameraDescriptionFile =
    CDFHeader +
    "    <SwissKnife Name=\"SwsConstFormula\">                                    \n"
    "        <Formula>((3.0-4.0)+1.0) = 0.0</Formula>            \n"
    "    </SwissKnife>                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CFloatPtr ptrSwsConstFormula = Camera._GetNode("SwsConstFormula");
    const double minimum = ptrSwsConstFormula->GetMin();
    const double maximum = ptrSwsConstFormula->GetMax();
    const double value = ptrSwsConstFormula->GetValue();
    CPPUNIT_ASSERT_DOUBLES_EQUAL( -minimum, maximum, 1e-6 );
    CPPUNIT_ASSERT_EQUAL( int(value), 1 );
    CPPUNIT_ASSERT_EQUAL (intfIFloat, ptrSwsConstFormula->GetNode()->GetPrincipalInterfaceType());
    }

void SwissKnifeTestSuite::TestSwissKnife()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>2</Value>                                                \n"
    "    </Integer>                                                            \n"
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>-2</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <Float Name=\"Dbl0\">                                                \n"
    "        <Value>0.5</Value>                                                \n"
    "    </Float>                                                            \n"
    "                                                                        \n"
    "    <Float Name=\"Dbl1\">                                                \n"
    "        <Value>-0.5</Value>                                                \n"
    "    </Float>                                                            \n"
        "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsNoVarInt\">                                \n"
    "        <Formula>((3-4)*(2+(-1)))/(6-5)</Formula>                        \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Int0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>-1*((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsAddSubInt\">                                \n"
    "        <pVariable Name=\"SK0\">SwsMultDivInt</pVariable>                \n"
    "        <Formula>(SK0 + SK0) - (SK0 + SK0)</Formula>                    \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsBitsInt\">                                    \n"
    "        <pVariable Name=\"SK0\">Int0</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">Int1</pVariable>                        \n"
    "        <Formula>SK0/SK1*(0x01||(((((~(SK0^0x03)=SK1))+SK0-SK1)>>SK0)>0))%SK1</Formula>    \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                            \n"
    "    <SwissKnife Name=\"SwsNoVarDbl\">                                    \n"
    "        <Formula>((3.0-4.0)*(2+(-1.0)))/(6.0-0x05)</Formula>            \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsMultDivDbl\">                                    \n"
    "        <pVariable Name=\"V0\">Dbl0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>neg((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsAddSubDbl\">                                    \n"
    "        <pVariable Name=\"SK0\">SwsMultDivInt</pVariable>                \n"
    "        <pVariable Name=\"SK1\">Int0</pVariable>                        \n"
    "        <Formula>(SK0 + SK1) - (SK1 + SK0)</Formula>                    \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsTrigDbl\">                                    \n"
    "        <pVariable Name=\"SK0\">SwsMultDivDbl</pVariable>                \n"
    "        <pVariable Name=\"SK1\">Int1</pVariable>                        \n"
    "        <Formula>SK1*atan(tan(asin(sin(acos(cos(PI*(SK0)))/PI))))/SK1</Formula> \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsTruncFracDbl\">                                \n"
    "        <pVariable Name=\"SK0\">Dbl1</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">SwsAddSubInt</pVariable>                \n"
    "        <Formula>(trunc(SK0*3) + frac(SK0*3))+SK1</Formula>                \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsCeilFloorDbl\">                                \n"
    "        <pVariable Name=\"SK0\">Dbl1</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">Int0</pVariable>                        \n"
    "        <Formula>(ceil(SK0)*SK1 - floor(SK0)*SK1)/SK1</Formula>            \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsRoundDbl\">                                    \n"
    "        <pVariable Name=\"SK0\">Dbl0</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">Int0</pVariable>                        \n"
    "        <Formula>round(SK0+SK1)-SK1</Formula>                            \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsSqrtDbl\">                                    \n"
    "        <pVariable Name=\"SK0\">Dbl0</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">Int0</pVariable>                        \n"
    "        <Formula>sqrt(SK0*SK1)</Formula>                                \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsPowersDbl\">                                    \n"
    "        <pVariable Name=\"SK0\">Dbl0</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">Int0</pVariable>                        \n"
    "        <Formula>ln(lg(SK1*5)*e*exp(SK0*2))</Formula>                    \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    "    <SwissKnife Name=\"SwsNegSgnDbl\">                                    \n"
    "        <pVariable Name=\"SK0\">Dbl1</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">SwsMultDivInt</pVariable>                \n"
    "        <Formula>(neg(SK0)*sgn(SK0))*neg(SK1)</Formula>                    \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrInt0 = Camera._GetNode("Int0");
    CIntegerPtr ptrInt1 = Camera._GetNode("Int1");
    CFloatPtr ptrDbl0 = Camera._GetNode("Dbl0");
    CFloatPtr ptrDbl1 = Camera._GetNode("Dbl1");

    CIntegerPtr ptrSwsNoVarInt = Camera._GetNode("SwsNoVarInt");
    CIntegerPtr ptrSwsMultDivInt = Camera._GetNode("SwsMultDivInt");
    CIntegerPtr ptrSwsAddSubInt = Camera._GetNode("SwsAddSubInt");
    CIntegerPtr ptrSwsBitsInt = Camera._GetNode("SwsBitsInt");

    CFloatPtr ptrSwsNoVarDbl = Camera._GetNode("SwsNoVarDbl");
    CFloatPtr ptrSwsMultDivDbl = Camera._GetNode("SwsMultDivDbl");
    CFloatPtr ptrSwsAddSubDbl = Camera._GetNode("SwsAddSubDbl");
    CFloatPtr ptrSwsTrigDbl = Camera._GetNode("SwsTrigDbl");
    CFloatPtr ptrSwsTruncFracDbl = Camera._GetNode("SwsTruncFracDbl");
    CFloatPtr ptrSwsCeilFloorDbl = Camera._GetNode("SwsCeilFloorDbl");
    CFloatPtr ptrSwsRoundDbl = Camera._GetNode("SwsRoundDbl");
    CFloatPtr ptrSwsSqrtDbl = Camera._GetNode("SwsSqrtDbl");
    CFloatPtr ptrSwsPowersDbl = Camera._GetNode("SwsPowersDbl");
    CFloatPtr ptrSwsNegSgnDbl = Camera._GetNode("SwsNegSgnDbl");

    // Check functional correctness and SwissKnife nesting
    const int64_t noVarInt = -1;
    const int64_t multDivInt = -1;
    const int64_t addSubInt = 0;
    const int64_t bitsInt = -1;

    const double noVarDbl = -1.0;
    const double multDivDbl = -1.0;
    const double addSubDbl = 0.0;
    const double trigDbl = 1.0;
    const double truncFracDbl = -1.5;
    const double ceilFloorDbl = 1.0;
    const double roundDbl = 1.0;
    const double sqrtDbl = 1.0;
    const double powersDbl = 2.0;
    const double negSgnDbl = -0.5;

    CPPUNIT_ASSERT_THROW_EX( ptrSwsNoVarInt->SetValue(77), GenICam::GenericException );
    CPPUNIT_ASSERT_EQUAL(noVarInt, ptrSwsNoVarInt->GetValue());
    CPPUNIT_ASSERT_EQUAL(multDivInt, ptrSwsMultDivInt->GetValue());
    CPPUNIT_ASSERT_EQUAL(addSubInt, ptrSwsAddSubInt->GetValue());
    CPPUNIT_ASSERT_EQUAL(bitsInt, ptrSwsBitsInt->GetValue());

    CPPUNIT_ASSERT_THROW_EX( ptrSwsNoVarDbl->SetValue(7.7), GenICam::GenericException );
    CPPUNIT_ASSERT_EQUAL(noVarDbl, ptrSwsNoVarDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(multDivDbl, ptrSwsMultDivDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(addSubDbl, ptrSwsAddSubDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(trigDbl, ptrSwsTrigDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(truncFracDbl, ptrSwsTruncFracDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(ceilFloorDbl, ptrSwsCeilFloorDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(roundDbl, ptrSwsRoundDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(sqrtDbl, ptrSwsSqrtDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(powersDbl, ptrSwsPowersDbl->GetValue());
    CPPUNIT_ASSERT_EQUAL(negSgnDbl, ptrSwsNegSgnDbl->GetValue());

    testMinimum();
}


void SwissKnifeTestSuite::TestSwissKnifeBrokenBitOps()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>2</Value>                                                \n"
    "    </Integer>                                                            \n"
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>-2</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsBrokenBitOpsInt\">                            \n"
    "        <pVariable Name=\"SK0\">Int0</pVariable>                        \n"
    "        <pVariable Name=\"SK1\">Int1</pVariable>                        \n"
    "        <Formula><![CDATA[ ((((SK0<<1) & 0x01) <= (SK1<0)) >=0 ) && 1 ]]></Formula>            \n"
    // here an alternative formulation
    //"        <Formula>((((SK0&lt;&lt;1)&amp;0x01)&lt;=(SK1&lt;0))&gt;=0)&amp;&amp;1</Formula>            \n"
    "    </IntSwissKnife>                                                    \n"
    //operators not tested: "$" "?" ":" "," ";" "/*" ":=" "<>" "**"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CIntegerPtr ptrSwsBrokenBitOpsInt = Camera._GetNode("SwsBrokenBitOpsInt");
    // Check additional bit operations
    // Integers have a easy to calculate expected values for bit operations
    const int64_t brokenBitOpsInt = 1;
    CPPUNIT_ASSERT_EQUAL(brokenBitOpsInt, ptrSwsBrokenBitOpsInt->GetValue());

}


void SwissKnifeTestSuite::TestSwissKnifeVariableMinMax()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>-2</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <Float Name=\"Dbl0\">                                                \n"
    "        <Value>0.5</Value>                                                \n"
    "        <Min>1</Min>                                                    \n"
    "        <Max>2</Max>                                                    \n"
    "    </Float>                                                            \n"
        "                                                                        \n"
    "    <SwissKnife Name=\"SwsMultDivDbl\">                                    \n"
    "        <pVariable Name=\"V0\">Dbl0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>neg((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CFloatPtr ptrSwsMultDivDbl = Camera._GetNode("SwsMultDivDbl");
    // Check that variable ranges are respected when loading variables into formulas
    const double multDivDbl = -1.0;
    CPPUNIT_ASSERT_EQUAL(multDivDbl, ptrSwsMultDivDbl->GetValue());
}

void SwissKnifeTestSuite::TestSwissKnifeVariableValue()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <Float Name=\"Dbl0\">                                                \n"
    "        <Value>1.0</Value>                                                \n"
    "    </Float>                                                            \n"
        "                                                                        \n"
    "    <SwissKnife Name=\"SwsMultDivDbl\">                                    \n"
    "        <pVariable Name=\"V0\">Dbl0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>neg((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CFloatPtr ptrSwsMultDivDbl = Camera._GetNode("SwsMultDivDbl");
    // Check that variable values are defined
    const double multDivDbl = -1.0;
    CPPUNIT_ASSERT_EQUAL(multDivDbl, ptrSwsMultDivDbl->GetValue());
}


 void SwissKnifeTestSuite::TestSwissKnifeArgumentType()
{
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                                        \n"
    "    <Float Name=\"Dbl1\">                                                \n"
    "        <Value>1.0</Value>                                                \n"
    "    </Float>                                                            \n"
        "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>1</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                        \n"
    "    <SwissKnife Name=\"SwsMultDivDbl\">                                \n"
    "        <pVariable Name=\"V0\">Dbl1</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int0</pVariable>                            \n"
    "        <Formula>neg((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </SwissKnife>                                                        \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);
    CFloatPtr ptrSwsMultDivDbl = Camera._GetNode("SwsMultDivDbl");
    // Check that the arguments in the formula are all supported types
    const double multDivDbl = -1.0;
    CPPUNIT_ASSERT_EQUAL(multDivDbl , ptrSwsMultDivDbl->GetValue());
}



void SwissKnifeTestSuite::TestValueAccess()
{

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<SwissKnife Name=\"Result\">                    \n"
    "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
    "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
    "    <Formula> X * Y + 12 </Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<Float Name=\"ValueX\">                        \n"
    "    <Value>3</Value>                            \n"
    "</Float>                                        \n"
    "                                                \n"
    "<Integer Name=\"ValueY\">                        \n"
    "    <Value>10</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrResult = Camera._GetNode("Result");
    CPPUNIT_ASSERT( ptrResult.IsValid() );
    CNodePtr ptrResultNode(ptrResult );
    CPPUNIT_ASSERT( ptrResultNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( 42.0, ptrResult->GetValue() );

    LOG_PERFORMANCE("-------------------------------------------------");
    LOG_PERFORMANCE("Setup : SwissKnife <=> Integer (X = const)");
    LOG_PERFORMANCE("                   <=> Integer (Y = const)");

    ptrResultNode->InvalidateNode();
    TEST_BEGIN(1000)
        double Result = ptrResult->GetValue();
        StopWatch.PauseBegin();
        ptrResultNode->InvalidateNode();
        StopWatch.PauseEnd();
    TEST_END( SwissKnife::GetValue (dynamic, without caching) )

    {
        // tests the static case
        Test_TestCamera_v1_0::CTestCamera_v1_0 Camera;
        Camera._LoadDLL();

        CPPUNIT_ASSERT_EQUAL( 42.0, Camera.MySwissKnife.GetValue() );

        Camera.MySwissKnife.GetNode()->InvalidateNode();
        TEST_BEGIN(1000)
            double Result = Camera.MySwissKnife.GetValue();
            StopWatch.PauseBegin();
            Camera.MySwissKnife.GetNode()->InvalidateNode();
            StopWatch.PauseEnd();
        TEST_END( IntSwissKnife::GetValue (static, without caching) )

    }
    {
        // tests the static case
        Test_TestCamera_v1_1::CTestCamera_v1_1 Camera;
        Camera._LoadDLL();

        CPPUNIT_ASSERT_EQUAL( 42.0, Camera.MySwissKnife.GetValue() );

        Camera.MySwissKnife.GetNode()->InvalidateNode();
        TEST_BEGIN(1000)
            double Result = Camera.MySwissKnife.GetValue();
            StopWatch.PauseBegin();
            Camera.MySwissKnife.GetNode()->InvalidateNode();
            StopWatch.PauseEnd();
        TEST_END( IntSwissKnife::GetValue (static, without caching) )

    }
}


 void SwissKnifeTestSuite::TestIntSwissKnifeTrig()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                                        \n"
    "    <Integer Name=\"Int0\">                                                \n"
    "        <Value>2</Value>                                                \n"
    "    </Integer>                                                            \n"
    "                                                                        \n"
    "    <Integer Name=\"Int1\">                                                \n"
    "        <Value>-2</Value>                                                \n"
    "    </Integer>                                                            \n"
        "                                                                    \n"
    "    <IntSwissKnife Name=\"SwsMultDivInt\">                                \n"
    "        <pVariable Name=\"V0\">Int0</pVariable>                            \n"
    "        <pVariable Name=\"V1\">Int1</pVariable>                            \n"
    "        <Formula>-1*((V0*V1)/(V1*V0))</Formula>                            \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    "    <IntSwissKnife Name=\"SwsTrigInt\">                                    \n"
    "        <pVariable Name=\"SK0\">SwsMultDivInt</pVariable>                \n"
    "        <Formula>atan(tan(asin(sin(acos(cos(PI*(SK0)))/PI))))</Formula> \n"
    "    </IntSwissKnife>                                                    \n"
    "                                                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrSwsMultDivInt = Camera._GetNode("SwsMultDivInt");
    CIntegerPtr ptrSwsTrigInt = Camera._GetNode("SwsTrigInt");

    // Check trig parsing for IntSwsKnife
    const int64_t multDivInt = -1;
    CPPUNIT_ASSERT_EQUAL(multDivInt, ptrSwsMultDivInt->GetValue());

    // Beware: an IntSwissKnife does not support float functions like sin, cos etc.
    CPPUNIT_ASSERT_THROW_EX(
            ptrSwsTrigInt->GetValue(),
            GenICam::LogicalErrorException
        );
}


// Pleora FG
// ---------------------------------------------------------------------------
// This method tests is used to trig cases where not all variables
// are present in order to evaluate an expression. This will trigger
// some yet-to-be-covered cases in VariableDelegate.cpp
//
void SwissKnifeTestSuite::TestVariableDelegates()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "<Boolean Name =\"FalseNode\">                                \n"
    "  <Value>false</Value>                                        \n"
    "</Boolean>                                                    \n"
    "                                                            \n"
    "<Float Name=\"Float0\">                                    \n"
    "  <Value>4</Value>                                            \n"
    "</Float>                                                    \n"
    "<Float Name=\"Float1\">                                    \n"
    "  <Value>2</Value>                                            \n"
    "</Float>                                                    \n"
    "<SwissKnife Name=\"SwsAdd\">                                \n"
    "  <pVariable Name=\"V0\">Float0</pVariable>                \n"
    "  <pVariable Name=\"V1\">Float1</pVariable>                \n"
    "  <Formula>V0+V1</Formula>                                    \n"
    "  <Representation>Linear</Representation>                    \n"
    "</SwissKnife>                                                \n"
    "                                                            \n"
    "<Float Name=\"FloatNotAvailable\">                            \n"
    "  <pIsAvailable>FalseNode</pIsAvailable>                    \n"
    "  <Value>3</Value>                                            \n"
    "</Float>                                                    \n"
    "<SwissKnife Name=\"SwsAddNA\">                                \n"
    "  <pVariable Name=\"V0\">Float0</pVariable>                \n"
    "  <pVariable Name=\"V1\">FloatNotAvailable</pVariable>        \n"
    "  <Formula>V0+V1</Formula>                                    \n"
    "</SwissKnife>                                                \n"
    "                                                            \n"
    "<Float Name=\"FloatNotImplemented\">                        \n"
    "  <pIsImplemented>FalseNode</pIsImplemented>                \n"
    "  <Value>3</Value>                                            \n"
    "</Float>                                                    \n"
    "<SwissKnife Name=\"SwsAddNI\">                                \n"
    "  <pVariable Name=\"V0\">Float0</pVariable>                \n"
    "  <pVariable Name=\"V1\">FloatNotImplemented</pVariable>    \n"
    "  <Formula>V0+V1</Formula>                                    \n"
    "</SwissKnife>                                                \n"
    "                                                            \n"
    "<Float Name=\"FloatReadOnly\">                            \n"
    "  <ImposedAccessMode>RO</ImposedAccessMode>                    \n"
    "  <Value>3</Value>                                            \n"
    "</Float>                                                    \n"
    "<SwissKnife Name=\"SwsAddRO\">                                \n"
    "  <pVariable Name=\"V0\">Float0</pVariable>                \n"
    "  <pVariable Name=\"V1\">FloatReadOnly</pVariable>        \n"
    "  <Formula>V0+V1</Formula>                                    \n"
    "</SwissKnife>                                                \n"
    "                                                            \n"
    "<SwissKnife Name=\"SwsBadFormula\">                        \n"
    "  <pVariable Name=\"V0\">Float0</pVariable>                \n"
    "  <pVariable Name=\"V1\">Float1</pVariable>                \n"
    "  <Formula>ITALY2GERMANY0</Formula>                        \n"
    "</SwissKnife>                                                \n"
    "                                                            \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    // Regular float
    CFloatPtr ptrSwsAdd = Camera._GetNode("SwsAdd");
    CPPUNIT_ASSERT( ptrSwsAdd->GetAccessMode() == RO );
    CPPUNIT_ASSERT( ptrSwsAdd->GetRepresentation() == Linear );
    double lMin = ptrSwsAdd->GetMin();
    double lMax = ptrSwsAdd->GetMax();
    gcstring lUnit = ptrSwsAdd->GetUnit();
    CPPUNIT_ASSERT_THROW_EX( ptrSwsAdd->SetValue( 1.0 ), GenICam::AccessException);
    CPPUNIT_ASSERT_THROW_EX( ptrSwsAdd->FromString( "1.0" ), GenICam::AccessException);
    gcstring lStrVal = ptrSwsAdd->ToString();

    // NI float
    CFloatPtr ptrSwsAddNI = Camera._GetNode("SwsAddNI");
    CPPUNIT_ASSERT( ptrSwsAddNI->GetAccessMode() == NI );

    // NA float
    CFloatPtr ptrSwsAddNA = Camera._GetNode("SwsAddNA");
    CPPUNIT_ASSERT( ptrSwsAddNA->GetAccessMode() == NA );

    // RO float
    CFloatPtr ptrSwsAddRO = Camera._GetNode("SwsAddRO");
    CPPUNIT_ASSERT( ptrSwsAddRO->GetAccessMode() == RO );

    // Bad formula
    CFloatPtr ptrSwsBadFormula = Camera._GetNode("SwsBadFormula" );
    CPPUNIT_ASSERT_THROW_EX( ptrSwsBadFormula->GetValue(), GenICam::LogicalErrorException );
}


// Mantis #89
void SwissKnifeTestSuite::TestRecursion()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "                                                                    \n"
    "    <IntSwissKnife Name=\"WidthMax\">                               \n"
    "        <pVariable Name=\"OFFSETX\">OffsetXValue</pVariable>        \n"
    "        <Formula>1000-OFFSETX</Formula>                             \n"
    "    </IntSwissKnife>                                                \n"
    "                                                                    \n"
    "    <Integer Name=\"WidthValue\">                                   \n"
    "        <Value>1000</Value>                                         \n"
    "        <pMax>WidthMax</pMax>                                       \n"
    "    </Integer>                                                      \n"
    "                                                                    \n"
    "    <IntSwissKnife Name=\"OffsetXMax\">                             \n"
    "        <pVariable Name=\"WIDTH\">WidthValue</pVariable>            \n"
    "        <Formula>1000-WIDTH</Formula>                               \n"
    "    </IntSwissKnife>                                                \n"
    "                                                                    \n"
    "    <Integer Name=\"OffsetXValue\">                                 \n"
    "        <Value>0</Value>                                            \n"
    "        <pMax>OffsetXMax</pMax>                                     \n"
    "    </Integer>                                                      \n"
    "                                                                    \n"
    + CDFFooter;

    CNodeMapRef Camera;
    if(GenApiSchemaVersion == v1_0)
    {
        // for v1.0 schema the loader does not check for write cycles
        CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

        // due ot backward compatibility the nodemap is created and the cycle is broken at runtime
        // using the caches
        CIntegerPtr ptrWidth = Camera._GetNode("WidthValue");
        CPPUNIT_ASSERT( ptrWidth.IsValid() );

        CIntegerPtr ptrWMax = Camera._GetNode("WidthMax");
        CIntegerPtr ptrOffsetX = Camera._GetNode("OffsetXValue");
        CIntegerPtr ptrOXMax = Camera._GetNode("OffsetXMax");

        CPPUNIT_ASSERT_NO_THROW (ptrWidth->SetValue (500));
        CPPUNIT_ASSERT_NO_THROW (ptrOXMax->GetValue());
    }
    else
    {
        // for all other schema versions it does
        CPPUNIT_ASSERT_THROW_EX(Camera._LoadXMLFromString(CameraDescriptionFile), GenICam::RuntimeException);

        // no nodes are created because of the cycle
        CIntegerPtr ptrWidth = Camera._GetNode("WidthValue");
        CPPUNIT_ASSERT( !ptrWidth.IsValid() );

    }
}

void SwissKnifeTestSuite::VariableNames()
{
    // good examples
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                                    \n"
        "    <IntSwissKnife Name=\"Knife1\">                               \n"
        "        <pVariable Name=\"BLADE\">Blade</pVariable>        \n"
        "        <Formula>100*BLADE</Formula>                             \n"
        "    </IntSwissKnife>                                                \n"
        "                                                                    \n"
        "    <IntSwissKnife Name=\"Knife2\">                               \n"
        "        <pVariable Name=\"BLADE2\">Blade</pVariable>        \n"
        "        <Formula>100*BLADE2</Formula>                             \n"
        "    </IntSwissKnife>                                                \n"
        "                                                                    \n"
        "    <IntSwissKnife Name=\"Knife3\">                               \n"
        "        <pVariable Name=\"BLADE_\">Blade</pVariable>        \n"
        "        <Formula>100*BLADE_</Formula>                             \n"
        "    </IntSwissKnife>                                                \n"
        "                                                                    \n"
        "    <Integer Name=\"Blade\">                                 \n"
        "        <Value>1</Value>                                            \n"
        "    </Integer>                                                      \n"
        "                                                                    \n"
        + CDFFooter;

        CNodeMapRef Camera;
        CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

        CIntegerPtr ptrBlade = Camera._GetNode("Knife1");
        CPPUNIT_ASSERT( (bool)ptrBlade );
        CPPUNIT_ASSERT_EQUAL( (int64_t)100LL, ptrBlade->GetValue());

        ptrBlade = Camera._GetNode("Knife2");
        CPPUNIT_ASSERT( (bool)ptrBlade );
        CPPUNIT_ASSERT_EQUAL( (int64_t)100LL, ptrBlade->GetValue());

        ptrBlade = Camera._GetNode("Knife3");
        CPPUNIT_ASSERT( (bool)ptrBlade );
        CPPUNIT_ASSERT_EQUAL( (int64_t)100LL, ptrBlade->GetValue());
    }

    // bad examples
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                                    \n"
        "    <IntSwissKnife Name=\"Knife\">                               \n"
        "        <pVariable Name=\"blade\">Blade</pVariable>        \n"
        "        <Formula>100*blade</Formula>                             \n"
        "    </IntSwissKnife>                                                \n"
        "                                                                    \n"
        + CDFFooter;

        CNodeMapRef Camera;
        CPPUNIT_ASSERT_THROW_EX( Camera._LoadXMLFromString(CameraDescriptionFile), GenICam::GenericException );

    }
}

// Mantis #288
void SwissKnifeTestSuite::TestVariableNames()
{

    // the implementation belonging to schema v1.0 can handle uppercase names only
    if( GenApiSchemaVersion == v1_0 )
        return;

    gcstring CameraDescriptionFile =
        CDFHeader +
        "    <SwissKnife Name=\"Knife1\">                               \n"
        "        <pVariable Name=\"BLADE\">Blade</pVariable>        \n"
        "        <Formula>100*BLADE</Formula>                             \n"
        "    </SwissKnife>                                                \n"
        "    <SwissKnife Name=\"Knife2\">                               \n"
        "        <pVariable Name=\"bLaDe\">Blade</pVariable>        \n"
        "        <Formula>100*bLaDe</Formula>                             \n"
        "    </SwissKnife>                                                \n"
        "    <SwissKnife Name=\"Knife3\">                               \n"
        "        <pVariable Name=\"blade\">Blade</pVariable>        \n"
        "        <Formula>100*blade</Formula>                             \n"
        "    </SwissKnife>                                                \n"
        "    <SwissKnife Name=\"Knife4\">                               \n"
        "        <pVariable Name=\"BLADE\">Blade</pVariable>        \n"
        "        <Formula>100*Blade</Formula>                             \n"
        "    </SwissKnife>                                                \n"
        "                                                                    \n"
        "    <Integer Name=\"Blade\">                                 \n"
        "        <Value>1</Value>                                            \n"
        "    </Integer>                                                      \n"
        + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

    CFloatPtr
        ptrBlade = Camera._GetNode("Knife1");
    CPPUNIT_ASSERT( (bool)ptrBlade );
    CPPUNIT_ASSERT_EQUAL(100.0, ptrBlade->GetValue());

    ptrBlade = Camera._GetNode("Knife2");
    CPPUNIT_ASSERT( (bool)ptrBlade );
    CPPUNIT_ASSERT_EQUAL(100.0, ptrBlade->GetValue());

    ptrBlade = Camera._GetNode("Knife3");
    CPPUNIT_ASSERT( (bool)ptrBlade );
    CPPUNIT_ASSERT_EQUAL(100.0, ptrBlade->GetValue());

    ptrBlade = Camera._GetNode("Knife4");
    CPPUNIT_ASSERT( (bool)ptrBlade );
    CPPUNIT_ASSERT_THROW_EX( ptrBlade->GetValue(), GenICam::LogicalErrorException );

}

void SwissKnifeTestSuite::TestConstantAndExpression()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<SwissKnife Name=\"Result\">                \n"
    "    <pVariable Name=\"X\">ValueX</pVariable>    \n"
    "    <pVariable Name=\"Y\">ValueY</pVariable>    \n"
    "    <Constant Name=\"Two\">2.0</Constant>    \n"
    "    <Expression Name=\"TwoX\">2.0*X</Expression>    \n"
    "    <Formula> TwoX * Y + Two </Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<Float Name=\"ValueX\">                        \n"
    "    <Value>5.0</Value>                            \n"
    "</Float>                                        \n"
    "                                                \n"
    "<Float Name=\"ValueY\">                        \n"
    "    <Value>4.0</Value>                            \n"
    "</Float>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrResult(Camera._GetNode("Result"));
    CPPUNIT_ASSERT( ptrResult.IsValid() );
    CNodePtr ptrResultNode(ptrResult );
    CPPUNIT_ASSERT( ptrResultNode.IsValid() );

    CPPUNIT_ASSERT_EQUAL( 42.0, ptrResult->GetValue() );
}



// Added with std::vector change in Int64MathParser
void SwissKnifeTestSuite::TestBig()
{
    gcstring CameraDescriptionFile =

    CDFHeader +
    "\n"
    "<Integer Name=\"MySelector\">\n"
    "    <Value>0</Value>\n"
    "</Integer>\n"
    "\n"
    "<SwissKnife Name=\"MyBigSwissKnife\">\n"
    "    <pVariable Name=\"SEL\">MySelector</pVariable>\n"
    "    <Formula>(SEL = 0) ? 0.0 : ((SEL = 1) ? 0.0 : ((SEL = 2) ? 0.0 : ((SEL = 3) ? 0.0 : "
    "((SEL = 4) ? 0.0 : ((SEL = 5) ? 0.0 : ((SEL = 6) ? 0.0 : ((SEL = 7) ? 0.0 : ((SEL = 8) ? "
    "0.0 : ((SEL = 9) ? 0.0 : ((SEL = 10) ? 0.0 : ((SEL = 11) ? 0.0 : ((SEL = 12) ? 0.0 : "
    "((SEL = 13) ? 0.0 : ((SEL = 14) ? 0.0 : ((SEL = 15) ? 0.0 : ((SEL = 16) ? 0.0 : "
    "((SEL = 17) ? 0.0 : ((SEL = 18) ? 0.0 : ((SEL = 19) ? 0.0 : ((SEL = 20) ? 0.0 : "
    "((SEL = 21) ? 0.0 : ((SEL = 22) ? 0.0 : ((SEL = 23) ? 0.0 : ((SEL = 24) ? 0.0 : "
    "((SEL = 25) ? 0.0 : ((SEL = 26) ? 0.0 : ((SEL = 27) ? 0.0 : ((SEL = 28) ? 0.0 : "
    "((SEL = 29) ? 0.0 : ((SEL = 30) ? 0.0 : ((SEL = 31) ? 0.0 : ((SEL = 32) ? 0.0 : "
    "((SEL = 33) ? 0.0 : ((SEL = 34) ? 0.0 : ((SEL = 35) ? 0.0 : ((SEL = 36) ? 0.0 : "
    "((SEL = 37) ? 0.0 : ((SEL = 38) ? 0.0 : ((SEL = 39) ? 0.0 : ((SEL = 40) ? 0.0 : "
    "((SEL = 41) ? 0.0 : ((SEL = 42) ? 3.1416 : ((SEL = 43) ? 0.0 : ((SEL = 44) ? 0.0 : "
    "((SEL = 45) ? 0.0 : ((SEL = 46) ? 0.0 : ((SEL = 47) ? 0.0 : ((SEL = 48) ? 0.0 : "
    "((SEL = 49) ? 0.0 : ((SEL = 50) ? 0.0 : ((SEL = 51) ? 0.0 : ((SEL = 52) ? 0.0 : "
    "((SEL = 53) ? 0.0 : ((SEL = 54) ? 0.0 : ((SEL = 55) ? 0.0 : ((SEL = 56) ? 0.0 : "
    "((SEL = 57) ? 0.0 : ((SEL = 58) ? 0.0 : ((SEL = 59) ? 0.0 : ((SEL = 60) ? 0.0 : "
    "((SEL = 61) ? 0.0 : ((SEL = 62) ? 0.0 : ((SEL = 63) ? 0.0 : ((SEL = 64) ? 0.0 : "
    "((SEL = 65) ? 0.0 : ((SEL = 66) ? 0.0 : ((SEL = 67) ? 0.0 : ((SEL = 68) ? 0.0 : "
    "((SEL = 69) ? 0.0 : ((SEL = 70) ? 0.0 : ((SEL = 71) ? 0.0 : ((SEL = 72) ? 0.0 : "
    "((SEL = 73) ? 0.0 : ((SEL = 74) ? 0.0 : ((SEL = 75) ? 0.0 : ((SEL = 76) ? 0.0 : "
    "((SEL = 77) ? 0.0 : ((SEL = 78) ? 0.0 : ((SEL = 79) ? 0.0 : ((SEL = 80) ? 0.0 : "
    "((SEL = 81) ? 0.0 : ((SEL = 82) ? 0.0 : ((SEL = 83) ? 0.0 : ((SEL = 84) ? 0.0 : "
    "((SEL = 85) ? 0.0 : ((SEL = 86) ? 0.0 : ((SEL = 87) ? 0.0 : ((SEL = 88) ? 0.0 : "
    "((SEL = 89) ? 0.0 : ((SEL = 90) ? 0.0 : ((SEL = 91) ? 0.0 : ((SEL = 92) ? 0.0 : "
    "((SEL = 93) ? 0.0 : ((SEL = 94) ? 0.0 : ((SEL = 95) ? 0.0 : ((SEL = 96) ? 0.0 : "
    "((SEL = 97) ? 0.0 : ((SEL = 98) ? 0.0 : ((SEL = 99) ? 0.0 : ((SEL = 100) ? 0.0 : "
    "((SEL = 101) ? 0.0 : ((SEL = 102) ? 0.0 : ((SEL = 103) ? 0.0 : ((SEL = 104) ? 0.0 : "
    "((SEL = 105) ? 0.0 : ((SEL = 106) ? 0.0 : ((SEL = 107) ? 0.0 : ((SEL = 108) ? 0.0 : "
    "((SEL = 109) ? 0.0 : ((SEL = 110) ? 0.0 : ((SEL = 111) ? 0.0 : ((SEL = 112) ? 0.0 : "
    "((SEL = 113) ? 0.0 : ((SEL = 114) ? 0.0 : ((SEL = 115) ? 0.0 : ((SEL = 116) ? 0.0 : "
    "((SEL = 117) ? 0.0 : ((SEL = 118) ? 0.0 : ((SEL = 119) ? 0.0 : ((SEL = 120) ? 0.0 : "
    "((SEL = 121) ? 0.0 : ((SEL = 122) ? 0.0 : ((SEL = 123) ? 0.0 : ((SEL = 124) ? 0.0 : "
    "((SEL = 125) ? 0.0 : ((SEL = 126) ? 0.0 : ((SEL = 127) ? 0.0 : (-1.0))))))))))))))))))"
    "))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))"
    "))))))))))))))))))))))))</Formula>\n"
    "</SwissKnife>\n"
    "\n"
    + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

    IInteger *lSelector = dynamic_cast<IInteger *>( Camera._GetNode("MySelector") );
    CPPUNIT_ASSERT( lSelector != NULL );

    IFloat *lSwissKnife = dynamic_cast<IFloat *>( Camera._GetNode("MyBigSwissKnife") );
    CPPUNIT_ASSERT( lSwissKnife != NULL );

    lSelector->SetValue( 0 );
    double lValue = lSwissKnife->GetValue();
    CPPUNIT_ASSERT( fabs( lValue ) < 0.000001 );

    lSelector->SetValue( 42 );
    lValue = lSwissKnife->GetValue();
    CPPUNIT_ASSERT( fabs( lValue - 3.1416 ) < 0.000001 );

    lSelector->SetValue( 128 );
    lValue = lSwissKnife->GetValue();
    CPPUNIT_ASSERT( fabs( lValue + 1.0 ) < 0.000001 );
}

void SwissKnifeTestSuite::TestVariableExtensions()
{
    if(GenApiSchemaVersion == v1_0)
        return;

    gcstring CameraDescriptionFile =
        CDFHeader +

        "    <SwissKnife Name=\"Knife\">                                \n"
        "        <pVariable Name=\"BLADE.Min\">Blade</pVariable>            \n"
        "        <Formula>10*BLADE.Min</Formula>                            \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"BladeMin\">                             \n"
        "        <pVariable Name=\"BLADE.Min\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Min</Formula>                           \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"BladeMax\">                             \n"
        "        <pVariable Name=\"BLADE.Max\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Max</Formula>                           \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"BladeInc\">                             \n"
        "        <pVariable Name=\"BLADE.Inc\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Inc</Formula>                           \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"BladeVal\">                             \n"
        "        <pVariable Name=\"BLADE.Value\">Blade</pVariable>        \n"
        "        <Formula>BLADE.Value</Formula>                           \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"BladeAccessMode\">                      \n"
        "        <pVariable Name=\"BLADE.AccessMode\">Blade</pVariable> \n"
        "        <Formula>BLADE.AccessMode</Formula>                    \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"BladeVisibility\">                      \n"
        "        <pVariable Name=\"BLADE.Visibility\">Blade</pVariable> \n"
        "        <Formula>BLADE.Visibility</Formula>                    \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"BladeCachingMode\">                      \n"
        "        <pVariable Name=\"BLADE.CachingMode\">Blade</pVariable> \n"
        "        <Formula>BLADE.CachingMode</Formula>                    \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <Float Name=\"Blade\">                                     \n"
        "        <Value>0.2</Value>                                     \n"
        "        <Min>-1.0</Min>                                        \n"
        "        <Max>1.0</Max>                                         \n"
        "        <Inc>0.1</Inc>                                         \n"
        "    </Float>                                                   \n"
        "                                                               \n"
        "    <SwissKnife Name=\"Dagger\">                                \n"
        "        <pVariable Name=\"OPTSEL.Entry.Option1\">OptionSelector</pVariable>            \n"
        "        <pVariable Name=\"OPTSEL.Entry.Option2\">OptionSelector</pVariable>            \n"
        "        <Formula>OPTSEL.Entry.Option1 + OPTSEL.Entry.Option2</Formula>                            \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <Enumeration Name=\"OptionSelector\">              \n"
        "       <EnumEntry Name=\"Option1\">      \n"
        "          <Value>10</Value>                \n"
        "          <NumericValue>1.5</NumericValue> \n"
        "       </EnumEntry>                          \n"
        "       <EnumEntry Name=\"Option2\">      \n"
        "          <Value>20</Value>                \n"
        "       </EnumEntry>                          \n"
        "       <Value>10</Value>                                  \n"
        "    </Enumeration>                                            \n"
        "                                                               \n"
        "    <SwissKnife Name=\"DaggerNoEnum\">                      \n"
        "        <pVariable Name=\"Blade.Entry.Foo\">Blade</pVariable> \n"
        "        <Formula>Blade.Entry.Foo</Formula>                    \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"
        "    <SwissKnife Name=\"DaggerNoEnumEntry\">                      \n"
        "        <pVariable Name=\"OptionSelector.Entry.Foo\">OptionSelector</pVariable> \n"
        "        <Formula>OptionSelector.Entry.Foo</Formula>                    \n"
        "    </SwissKnife>                                              \n"
        "                                                               \n"

        + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_NO_THROW( Camera._LoadXMLFromString(CameraDescriptionFile) );

    CFloatPtr ptrKnife = Camera._GetNode("Knife");
    CFloatPtr ptrBlade = Camera._GetNode("Blade");
    CFloatPtr ptrDagger = Camera._GetNode("Dagger");

    *ptrBlade = -0.3;
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)-10.0, *ptrKnife(), std::numeric_limits<double>::epsilon() );

    // note that the ".Entry.XX" variable uses Value rather than NumericValue by design
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)30.0, *ptrDagger(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBladeMin = Camera._GetNode("BladeMin");
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)-1.0, *ptrBladeMin(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBladeMax = Camera._GetNode("BladeMax");
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)1.0, *ptrBladeMax(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBladeInc = Camera._GetNode("BladeInc");
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)0.1, *ptrBladeInc(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBladeVal = Camera._GetNode("BladeVal");
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)-0.3, *ptrBladeVal(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBladeVisibility = Camera._GetNode("BladeVisibility");
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)0.0, *ptrBladeVisibility(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBladeAccessMode = Camera._GetNode("BladeAccessMode");
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)4.0, *ptrBladeAccessMode(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBladeCachingMode = Camera._GetNode("BladeCachingMode");
    CPPUNIT_ASSERT_DOUBLES_EQUAL( (double)WriteAround, *ptrBladeCachingMode(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrDaggerNoEnum = Camera._GetNode("DaggerNoEnum");
    CPPUNIT_ASSERT_THROW_EX( *ptrDaggerNoEnum(), RuntimeException );

    CFloatPtr ptrDaggerNoEnumEntry = Camera._GetNode("DaggerNoEnumEntry");
    CPPUNIT_ASSERT_THROW_EX( *ptrDaggerNoEnumEntry(), RuntimeException );
}

void SwissKnifeTestSuite::TestMathParser()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Float Name=\"A\">                            \n"
    "    <Value>0.0</Value>                            \n"
    "</Float>                                      \n"
    "                                                \n"
    "<Float Name=\"B\">                            \n"
    "    <Value>0.0</Value>                            \n"
    "</Float>                                      \n"
    "                                                \n"
    "<Float Name=\"C\">                            \n"
    "    <Value>0.0</Value>                            \n"
    "</Float>                                      \n"
    "                                                \n"
    "<SwissKnife Name=\"BitNot\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[  ~A ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"BitAnd\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A&B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"BitOr\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A|B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"BitXor\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A^B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"BitLeft\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<<B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"BitRight\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A>>B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Power\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A**B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Neq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<>B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Eq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A=B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Geq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A>=B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Leq\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<=B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Greater\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A>B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Less\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A<B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"And\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A&&B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Or\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A||B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Plus\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A+B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Minus\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A-B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Times\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A*B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Div\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A/B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Mod\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A%B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Ternary\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <pVariable Name=\"C\">C</pVariable>    \n"
    "    <Formula><![CDATA[ A?B:C ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Precedence1\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <pVariable Name=\"C\">C</pVariable>    \n"
    "    <Formula><![CDATA[ A+B*C ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Brackets1\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <pVariable Name=\"C\">C</pVariable>    \n"
    "    <Formula><![CDATA[ (A+B)*C ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Sgn\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ SGN(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Neg\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ NEG(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Frac\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ FRAC(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Trunc\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ TRUNC(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Floor\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ FLOOR(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Ceil\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ CEIL(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Round\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ROUND(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"RoundPrec\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ ROUND(A,B) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Pi\">                    \n"
    "    <Formula><![CDATA[ PI ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"E\">                    \n"
    "    <Formula><![CDATA[ E ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Abs\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ABS(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Exp\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ EXP(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Ln\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ LN(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Lg\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ LG(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Sqrt\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ SQRT(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Sin\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ SIN(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Cos\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ COS(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Tan\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ TAN(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Asin\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ASIN(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Acos\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ACOS(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Atan\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ ATAN(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Digits1\">                    \n"
    "    <Formula><![CDATA[ 10.2+10+0x10+010 ]]></Formula>                \n" // 010 treated as decimal !?!
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Digits2\">                    \n"
    "    <Formula><![CDATA[ +2.2 ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Digits3\">                    \n"
    "    <Formula><![CDATA[ -2.2 ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "                                                \n"
    "                                                \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid1\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ /A ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid2\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ A/ ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid3\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ / ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid4\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A==B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid5\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A$B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid6\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A#B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid7\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ !A ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid8\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <Formula><![CDATA[ FOOBAR(A) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid9\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ (A+B ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid10\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ A+B) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid11\">                    \n"
    "    <Formula></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid12\">                    \n"
    "    <Formula>    </Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid13\">                    \n"
    "    <Formula>1 2 3</Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid14\">                    \n"
    "    <Formula>0x</Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid15\">                    \n"
    "    <Formula>\"abc\"</Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid16\">                    \n"
    "    <Formula>\'abc\'</Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid17\">                    \n"
    "    <Formula>/* aha */ 1</Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid18\">                    \n"
    "    <Formula>\'abc</Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid19\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ () ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid20\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ SGN() ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid21\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <Formula><![CDATA[ SGN(A,B) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "<SwissKnife Name=\"Invalid22\">                    \n"
    "    <pVariable Name=\"A\">A</pVariable>    \n"
    "    <pVariable Name=\"B\">B</pVariable>    \n"
    "    <pVariable Name=\"C\">C</pVariable>    \n"
    "    <Formula><![CDATA[ ROUND(A,B,C) ]]></Formula>                \n"
    "</SwissKnife>                                        \n"
    "                                                \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CFloatPtr ptrA(Camera._GetNode("A"));
    CPPUNIT_ASSERT( ptrA.IsValid() );
    CFloatPtr ptrB(Camera._GetNode("B"));
    CPPUNIT_ASSERT( ptrB.IsValid() );
    CFloatPtr ptrC(Camera._GetNode("C"));
    CPPUNIT_ASSERT( ptrC.IsValid() );

    /*CFloatPtr ptrBitNot(Camera._GetNode("BitNot"));
    CPPUNIT_ASSERT( ptrBitNot.IsValid() );
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL ((double)(int64_t)0xFFFFFFFFFFFFFFFF, ptrBitNot->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue ((double)(int64_t)0x1111111111111111);
    CPPUNIT_ASSERT_DOUBLES_EQUAL ((double)(int64_t)0xEEEEEEEEEEEEEEEE, ptrBitNot->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBitAnd(Camera._GetNode("BitAnd"));
    CPPUNIT_ASSERT( ptrBitAnd.IsValid() );
    ptrA->SetValue ((double)(int64_t)0xCCCCCCCCCCCCCCCC);
    ptrB->SetValue ((double)(int64_t)0xAAAAAAAAAAAAAAAA);
    CPPUNIT_ASSERT_EQUAL ((double)(int64_t)0x8888888888888888, ptrBitAnd->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBitOr(Camera._GetNode("BitOr"));
    CPPUNIT_ASSERT( ptrBitOr.IsValid() );
    ptrA->SetValue ((double)(int64_t)0xCCCCCCCCCCCCCCCC);
    ptrB->SetValue ((double)(int64_t)0xAAAAAAAAAAAAAAAA);
    CPPUNIT_ASSERT_DOUBLES_EQUAL ((double)(int64_t)0xEEEEEEEEEEEEEEEE, ptrBitOr->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBitXor(Camera._GetNode("BitXor"));
    CPPUNIT_ASSERT( ptrBitXor.IsValid() );
    ptrA->SetValue ((double)(int64_t)0xCCCCCCCCCCCCCCCC);
    ptrB->SetValue ((double)(int64_t)0xAAAAAAAAAAAAAAAA);
    CPPUNIT_ASSERT_DOUBLES_EQUAL ((double)(int64_t)0x6666666666666666, ptrBitXor->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBitLeft(Camera._GetNode("BitLeft"));
    CPPUNIT_ASSERT( ptrBitLeft.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (4, ptrBitLeft->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBitRight(Camera._GetNode("BitRight"));
    CPPUNIT_ASSERT( ptrBitRight.IsValid() );
    ptrA->SetValue (8);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (2, ptrBitRight->GetValue(), std::numeric_limits<double>::epsilon() );*/

    CFloatPtr ptrPower(Camera._GetNode("Power"));
    CPPUNIT_ASSERT( ptrPower.IsValid() );
    ptrA->SetValue (3);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (9, ptrPower->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrNeq(Camera._GetNode("Neq"));
    CPPUNIT_ASSERT( ptrNeq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrNeq->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrNeq->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrEq(Camera._GetNode("Eq"));
    CPPUNIT_ASSERT( ptrEq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrEq->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrEq->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrGeq(Camera._GetNode("Geq"));
    CPPUNIT_ASSERT( ptrGeq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrGeq->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrGeq->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrGeq->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrLeq(Camera._GetNode("Leq"));
    CPPUNIT_ASSERT( ptrLeq.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrLeq->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrLeq->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrLeq->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrGreater(Camera._GetNode("Greater"));
    CPPUNIT_ASSERT( ptrGreater.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrGreater->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrGreater->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrGreater->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrLess(Camera._GetNode("Less"));
    CPPUNIT_ASSERT( ptrLess.IsValid() );
    ptrA->SetValue (1);
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrLess->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrLess->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrLess->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrAnd(Camera._GetNode("And"));
    CPPUNIT_ASSERT( ptrAnd.IsValid() );
    ptrA->SetValue (-5);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrAnd->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrAnd->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrAnd->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrAnd->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrOr(Camera._GetNode("Or"));
    CPPUNIT_ASSERT( ptrOr.IsValid() );
    ptrA->SetValue (-5);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrOr->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrOr->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrOr->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrOr->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrPlus(Camera._GetNode("Plus"));
    CPPUNIT_ASSERT( ptrPlus.IsValid() );
    ptrA->SetValue (25.2);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (32.2, ptrPlus->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrMinus(Camera._GetNode("Minus"));
    CPPUNIT_ASSERT( ptrMinus.IsValid() );
    ptrA->SetValue (25.2);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (18.2, ptrMinus->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrTimes(Camera._GetNode("Times"));
    CPPUNIT_ASSERT( ptrTimes.IsValid() );
    ptrA->SetValue (25.2);
    ptrB->SetValue (7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (176.4, ptrTimes->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrDiv(Camera._GetNode("Div"));
    CPPUNIT_ASSERT( ptrDiv.IsValid() );
    ptrA->SetValue (15);
    ptrB->SetValue (6);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (2.5, ptrDiv->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_THROW( ptrDiv->GetValue(), GenICam::GenericException );

    CFloatPtr ptrMod(Camera._GetNode("Mod"));
    CPPUNIT_ASSERT( ptrMod.IsValid() );
    ptrA->SetValue (25.2);
    ptrB->SetValue (7);
    //CPPUNIT_ASSERT_DOUBLES_EQUAL (4.2, ptrMod->GetValue(), std::numeric_limits<double>::epsilon() ); // lacks precision
    CPPUNIT_ASSERT_DOUBLES_EQUAL (4.2, ptrMod->GetValue(), 10e-10 );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_THROW( ptrMod->GetValue(), GenICam::GenericException );

    CFloatPtr ptrTernary(Camera._GetNode("Ternary"));
    CPPUNIT_ASSERT( ptrTernary.IsValid() );
    ptrA->SetValue (5);
    ptrB->SetValue (7);
    ptrC->SetValue (8);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (7, ptrTernary->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (8, ptrTernary->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrPrecedence1(Camera._GetNode("Precedence1"));
    CPPUNIT_ASSERT( ptrPrecedence1.IsValid() );
    ptrA->SetValue (2);
    ptrB->SetValue (3);
    ptrC->SetValue (4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (14, ptrPrecedence1->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrBrackets1(Camera._GetNode("Brackets1"));
    CPPUNIT_ASSERT( ptrBrackets1.IsValid() );
    ptrA->SetValue (2);
    ptrB->SetValue (3);
    ptrC->SetValue (4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (20, ptrBrackets1->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrSgn(Camera._GetNode("Sgn"));
    CPPUNIT_ASSERT( ptrSgn.IsValid() );
    ptrA->SetValue (5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrSgn->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrSgn->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-1, ptrSgn->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrNeg(Camera._GetNode("Neg"));
    CPPUNIT_ASSERT( ptrNeg.IsValid() );
    ptrA->SetValue (5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-5, ptrNeg->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0, ptrNeg->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (7, ptrNeg->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrFrac(Camera._GetNode("Frac"));
    CPPUNIT_ASSERT( ptrFrac.IsValid() );
    ptrA->SetValue (5.4);
    //CPPUNIT_ASSERT_DOUBLES_EQUAL (0.4, ptrFrac->GetValue(), std::numeric_limits<double>::epsilon() ); // lacks precision
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.4, ptrFrac->GetValue(), 10e-10 ); 
    ptrA->SetValue (-5.4);
    //CPPUNIT_ASSERT_DOUBLES_EQUAL (-0.4, ptrFrac->GetValue(), std::numeric_limits<double>::epsilon() ); // lacks precision
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-0.4, ptrFrac->GetValue(), 10e-10 ); 

    CFloatPtr ptrTrunc(Camera._GetNode("Trunc"));
    CPPUNIT_ASSERT( ptrTrunc.IsValid() );
    ptrA->SetValue (5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (5.0, ptrTrunc->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (5.0, ptrTrunc->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-5.0, ptrTrunc->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-5.0, ptrTrunc->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrFloor(Camera._GetNode("Floor"));
    CPPUNIT_ASSERT( ptrFloor.IsValid() );
    ptrA->SetValue (5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (5.0, ptrFloor->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (5.0, ptrFloor->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-6.0, ptrFloor->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-6.0, ptrFloor->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrCeil(Camera._GetNode("Ceil"));
    CPPUNIT_ASSERT( ptrCeil.IsValid() );
    ptrA->SetValue (5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (6.0, ptrCeil->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (6.0, ptrCeil->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-5.0, ptrCeil->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-5.0, ptrCeil->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrRound(Camera._GetNode("Round"));
    CPPUNIT_ASSERT( ptrRound.IsValid() );
    ptrA->SetValue (5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (5.0, ptrRound->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (6.0, ptrRound->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-5.0, ptrRound->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-6.0, ptrRound->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrRoundPrec(Camera._GetNode("RoundPrec"));
    CPPUNIT_ASSERT( ptrRoundPrec.IsValid() );
    ptrA->SetValue (45.4545);
    ptrB->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (45.5, ptrRoundPrec->GetValue(), 10e-2/*std::numeric_limits<double>::epsilon()*/ );
    ptrB->SetValue (2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (45.45, ptrRoundPrec->GetValue(), 10e-3/*std::numeric_limits<double>::epsilon()*/ );
    ptrB->SetValue (3);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (45.455, ptrRoundPrec->GetValue(), 10e-4 /*std::numeric_limits<double>::epsilon()*/ );
    ptrB->SetValue (0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (45.0, ptrRoundPrec->GetValue(), 10e-1/*std::numeric_limits<double>::epsilon()*/ );
    ptrB->SetValue (-1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (50.0, ptrRoundPrec->GetValue(), 1/*std::numeric_limits<double>::epsilon()*/ );

    CFloatPtr ptrPi(Camera._GetNode("Pi"));
    CPPUNIT_ASSERT( ptrPi.IsValid() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL (3.1415926536, ptrPi->GetValue(), 10e-10 );

    CFloatPtr ptrE(Camera._GetNode("E"));
    CPPUNIT_ASSERT( ptrE.IsValid() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL (2.7182818285, ptrE->GetValue(), 10e-10 );

    CFloatPtr ptrAbs(Camera._GetNode("Abs"));
    CPPUNIT_ASSERT( ptrAbs.IsValid() );
    ptrA->SetValue (5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (5.4, ptrAbs->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (-5.4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (5.4, ptrAbs->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrExp(Camera._GetNode("Exp"));
    CPPUNIT_ASSERT( ptrExp.IsValid() );
    ptrA->SetValue (1);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (ptrE->GetValue(), ptrExp->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrLn(Camera._GetNode("Ln"));
    CPPUNIT_ASSERT( ptrLn.IsValid() );
    ptrA->SetValue (ptrE->GetValue()*ptrE->GetValue());
    CPPUNIT_ASSERT_DOUBLES_EQUAL (2.0, ptrLn->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrLg(Camera._GetNode("Lg"));
    CPPUNIT_ASSERT( ptrLg.IsValid() );
    ptrA->SetValue (1000);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (3.0, ptrLg->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrSqrt(Camera._GetNode("Sqrt"));
    CPPUNIT_ASSERT( ptrSqrt.IsValid() );
    ptrA->SetValue (2.25);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1.5, ptrSqrt->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrSin(Camera._GetNode("Sin"));
    CPPUNIT_ASSERT( ptrSin.IsValid() );
    ptrA->SetValue (0.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.0, ptrSin->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (ptrPi->GetValue()/2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1.0, ptrSin->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrCos(Camera._GetNode("Cos"));
    CPPUNIT_ASSERT( ptrCos.IsValid() );
    ptrA->SetValue (0.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1.0, ptrCos->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (ptrPi->GetValue()/2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.0, ptrCos->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrTan(Camera._GetNode("Tan"));
    CPPUNIT_ASSERT( ptrTan.IsValid() );
    ptrA->SetValue (0.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.0, ptrTan->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (ptrPi->GetValue()/4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (1, ptrTan->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrAsin(Camera._GetNode("Asin"));
    CPPUNIT_ASSERT( ptrAsin.IsValid() );
    ptrA->SetValue (0.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.0, ptrAsin->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (1.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (ptrPi->GetValue()/2, ptrAsin->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrAcos(Camera._GetNode("Acos"));
    CPPUNIT_ASSERT( ptrAcos.IsValid() );
    ptrA->SetValue (0.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (ptrPi->GetValue()/2, ptrAcos->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (1.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.0, ptrAcos->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrAtan(Camera._GetNode("Atan"));
    CPPUNIT_ASSERT( ptrAtan.IsValid() );
    ptrA->SetValue (0.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (0.0, ptrAtan->GetValue(), std::numeric_limits<double>::epsilon() );
    ptrA->SetValue (1.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL (ptrPi->GetValue()/4, ptrAtan->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrDigits1(Camera._GetNode("Digits1"));
    CPPUNIT_ASSERT( ptrDigits1.IsValid() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL (46.2, ptrDigits1->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrDigits2(Camera._GetNode("Digits2"));
    CPPUNIT_ASSERT( ptrDigits2.IsValid() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL (2.2, ptrDigits2->GetValue(), std::numeric_limits<double>::epsilon() );

    CFloatPtr ptrDigits3(Camera._GetNode("Digits3"));
    CPPUNIT_ASSERT( ptrDigits3.IsValid() );
    CPPUNIT_ASSERT_DOUBLES_EQUAL (-2.2, ptrDigits3->GetValue(), std::numeric_limits<double>::epsilon() );


    // some invalid stuff
    CFloatPtr ptrInvalid1(Camera._GetNode("Invalid1"));
    CPPUNIT_ASSERT( ptrInvalid1.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid1->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid2(Camera._GetNode("Invalid2"));
    CPPUNIT_ASSERT( ptrInvalid2.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid2->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid3(Camera._GetNode("Invalid3"));
    CPPUNIT_ASSERT( ptrInvalid3.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid3->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid4(Camera._GetNode("Invalid4"));
    CPPUNIT_ASSERT( ptrInvalid4.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid4->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid5(Camera._GetNode("Invalid5"));
    CPPUNIT_ASSERT( ptrInvalid5.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid5->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid6(Camera._GetNode("Invalid6"));
    CPPUNIT_ASSERT( ptrInvalid6.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid6->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid7(Camera._GetNode("Invalid7"));
    CPPUNIT_ASSERT( ptrInvalid7.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid7->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid8(Camera._GetNode("Invalid8"));
    CPPUNIT_ASSERT( ptrInvalid8.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid8->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid9(Camera._GetNode("Invalid9"));
    CPPUNIT_ASSERT( ptrInvalid9.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid9->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid10(Camera._GetNode("Invalid10"));
    CPPUNIT_ASSERT( ptrInvalid10.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid10->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid11(Camera._GetNode("Invalid11"));
    CPPUNIT_ASSERT( ptrInvalid11.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid11->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid12(Camera._GetNode("Invalid12"));
    CPPUNIT_ASSERT( ptrInvalid12.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid12->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid13(Camera._GetNode("Invalid13"));
    CPPUNIT_ASSERT( ptrInvalid13.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid13->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid14(Camera._GetNode("Invalid14"));
    CPPUNIT_ASSERT( ptrInvalid14.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid14->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid15(Camera._GetNode("Invalid15"));
    CPPUNIT_ASSERT( ptrInvalid15.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid15->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid16(Camera._GetNode("Invalid16"));
    CPPUNIT_ASSERT( ptrInvalid16.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid16->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid17(Camera._GetNode("Invalid17"));
    CPPUNIT_ASSERT( ptrInvalid17.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid17->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid18(Camera._GetNode("Invalid18"));
    CPPUNIT_ASSERT( ptrInvalid18.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid18->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid19(Camera._GetNode("Invalid19"));
    CPPUNIT_ASSERT( ptrInvalid19.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid19->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid20(Camera._GetNode("Invalid20"));
    CPPUNIT_ASSERT( ptrInvalid20.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid20->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid21(Camera._GetNode("Invalid21"));
    CPPUNIT_ASSERT( ptrInvalid21.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid21->GetValue(), GenICam::GenericException );

    CFloatPtr ptrInvalid22(Camera._GetNode("Invalid22"));
    CPPUNIT_ASSERT( ptrInvalid22.IsValid() );
    CPPUNIT_ASSERT_THROW( ptrInvalid22->GetValue(), GenICam::GenericException );
}

void SwissKnifeTestSuite::TestTheFrenchWay()
{
    // create and initialize node map
    gcstring CameraDescriptionFile =
        CDFHeader +
        "                                        \n"
        "<SwissKnife Name=\"Value\">             \n"
        "    <Formula>1.234</Formula>                \n"
        "</SwissKnife>                                \n"
        "                                        \n"
        + CDFFooter;

    log4cpp::Category& testlogger(GenICam::CLog::GetLogger("CppUnit.Tests"));
    gcstring OldLocale = setlocale(LC_ALL, NULL);
    
    //cs << "Original locale = " << OldLocale << "\n";
    stringstream ss; ss << "Original locale = " << OldLocale << "\n";
    testlogger.info( ss.str() );
    ss.clear();

    if (!setlocale(LC_ALL, "French"))
        if (!setlocale(LC_ALL, "fr_FR.UTF-8"))
            setlocale(LC_ALL, "fr_FR");

    ss << "French locale = " << setlocale(LC_ALL, NULL) << "\n";
    testlogger.info( ss.str() );
    ss.clear();

    // Ticket #704 seems to imply that this goes wrong because the French locale
    // denotes float numbers as 1,234 instead of 1.234.
    // However this does not fail. I tried to set the locale in DLLMain directly
    // but it made no difference
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    setlocale(LC_ALL, OldLocale.c_str() );
    ss << "Restored locale = " << setlocale(LC_ALL, NULL) << "\n";
    testlogger.info( ss.str() );

}

void SwissKnifeTestSuite::TestTickets_788_789_790()
{

    gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                                        \n"
        "    <IntSwissKnife Name=\"Ticket788\">                                  \n"
        "        <pVariable Name=\"A\">A</pVariable>                             \n"
        "        <Formula>A % 0</Formula>                                        \n"
        "    </IntSwissKnife>                                                    \n"
        "                                                                        \n"
        "    <IntSwissKnife Name=\"Ticket789\">                                  \n"
        "        <Formula></Formula>                                             \n"
        "    </IntSwissKnife>                                                    \n"
        "                                                                        \n"
        "    <IntSwissKnife Name=\"Ticket790\">                                  \n"
        "        <pVariable Name=\"A\">A</pVariable>                             \n"
        "        <pVariable Name=\"B\">B</pVariable>                             \n"
        "        <Formula>A}B</Formula>                                          \n"
        "    </IntSwissKnife>                                                    \n"
        "                                                                        \n"
        "    <IntSwissKnife Name=\"Ticket788_Float\">                            \n"
        "        <pVariable Name=\"A\">A</pVariable>                             \n"
        "        <Formula>A % 0</Formula>                                        \n"
        "    </IntSwissKnife>                                                    \n"
        "                                                                        \n"
        "    <IntSwissKnife Name=\"Ticket789_Float\">                            \n"
        "        <Formula></Formula>                                             \n"
        "    </IntSwissKnife>                                                    \n"
        "                                                                        \n"
        "    <IntSwissKnife Name=\"Ticket790_Float\">                            \n"
        "        <pVariable Name=\"A\">A</pVariable>                             \n"
        "        <pVariable Name=\"B\">B</pVariable>                             \n"
        "        <Formula>A}B</Formula>                                          \n"
        "    </IntSwissKnife>                                                    \n"
        "                                                                        \n"
        "    <Integer Name=\"A\">                                                \n"
        "        <Value>1</Value>                                                \n"
        "    </Integer>                                                          \n"
        "                                                                        \n"
        "    <Integer Name=\"B\">                                                \n"
        "        <Value>1</Value>                                                \n"
        "    </Integer>                                                          \n"
        "                                                                        \n"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    NODE_POINTER( Integer, Ticket788 );
    NODE_POINTER( Integer, Ticket789 );
    NODE_POINTER( Integer, Ticket790 );

    NODE_POINTER( Integer, Ticket788_Float );
    NODE_POINTER( Integer, Ticket789_Float );
    NODE_POINTER( Integer, Ticket790_Float );

    CPPUNIT_ASSERT_THROW_EX( ptrTicket788->GetValue() ,LogicalErrorException );
    CPPUNIT_ASSERT_THROW_EX( ptrTicket789->GetValue() ,LogicalErrorException );
    CPPUNIT_ASSERT_THROW_EX( ptrTicket790->GetValue() ,LogicalErrorException );

    CPPUNIT_ASSERT_THROW_EX( ptrTicket788_Float->GetValue() ,LogicalErrorException );
    CPPUNIT_ASSERT_THROW_EX( ptrTicket789_Float->GetValue() ,LogicalErrorException );
    CPPUNIT_ASSERT_THROW_EX( ptrTicket790_Float->GetValue() ,LogicalErrorException );
}
