//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Hartmut Nebelung
//  $Header$
//-----------------------------------------------------------------------------
/*!
  \file
  Implementation of the TerminalNodeTestSuite test suite
*/


#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "TerminalNodeTestSuite.h"
#include "GenApi/GenApi.h"
#include "GenApi/Pointer.h"
#include "GenApi/impl/PointerImpl.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( TerminalNodeTestSuite, "GenApiTest");

using namespace GenApi;
using namespace GenICam;

extern bool operator == (const NodeList_t &lhs, const NodeList_t &rhs); // defined in DependencyTest.cpp

static bool TestTerminals(CNodeMapRef& map, INodePrivate* pNode, const char *names[])
{
    NodeList_t expected;
    const char **ppCur = names;
    while (*ppCur != NULL)
    {
        expected.push_back( map._GetNode( *ppCur ) );
        ++ppCur;
    }

    NodeList_t actual;
    pNode->GetTerminalNodes(actual);

    return actual == expected;
}

void TerminalNodeTestSuite::TestInteger01()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +
        "<Integer Name=\"N1\">"
        "  <pValue>N2</pValue>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N2\">"
        "  <Value>2</Value>"
        "  <Min>1</Min>"
        "  <pMax>N3</pMax>"
        "</Integer>"
        "<Integer Name=\"N3\">"
        "  <Value>3</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        + CDFFooter;
    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N1"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N2", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N2"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N2", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }
    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N3"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N3", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }
}



void TerminalNodeTestSuite::TestInteger02()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +

        "<Integer Name=\"N1\">"
        "  <pValue>N2</pValue>"
        "  <pMin>N3</pMin>"
        "  <pMax>N8</pMax>"
        "</Integer>"
        "<Integer Name=\"N2\">"
        "  <pValue>N5</pValue>"
        "  <pMin>N4</pMin>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N3\">"
        "  <pValue>N6</pValue>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N4\">"
        "  <Value>50</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N5\">"
        "  <pValue>N7</pValue>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N6\">"
        "  <Value>6</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N7\">"
        "  <Value>7</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N8\">"
        "  <Value>8</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N1"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N7", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N2"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N7", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N3"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N6", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N4"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N4", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N5"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N7", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N6"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N6", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N7"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N7", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N8"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N8", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }
}

void TerminalNodeTestSuite::TestInteger03()
{
    // create and initilize node map
    gcstring CameraDescription =
        CDFHeader +
        "<Integer Name=\"N1\">"
        "  <pValue>N5</pValue>"
        "  <pMin>N2</pMin>"
        "  <pMax>N3</pMax>"
        "</Integer>"
        "<Integer Name=\"N2\">"
        "  <Value>2</Value>"
        "  <Min>1</Min>"
        "  <pMax>N9</pMax>"
        "</Integer>"
        "<Integer Name=\"N3\">"
        "  <pValue>N4</pValue>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N4\">"
        "  <Value>4</Value>"
        "  <Min>1</Min>"
        "  <pMax>N7</pMax>"
        "</Integer>"
        "<Integer Name=\"N5\">"
        "  <pValue>N6</pValue>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N6\">"
        "  <Value>6</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N7\">"
        "  <pValue>N8</pValue>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N8\">"
        "  <Value>8</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N9\">"
        "  <pValue>N12</pValue>"
        "  <Min>1</Min>"
        "  <pMax>N10</pMax>"
        "</Integer>"
        "<Integer Name=\"N10\">"
        "  <pValue>N11</pValue>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N11\">"
        "  <Value>11</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        "<Integer Name=\"N12\">"
        "  <Value>12</Value>"
        "  <Min>1</Min>"
        "  <Max>100</Max>"
        "</Integer>"
        + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescription);

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N1"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N6", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N2"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N2", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N3"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N4", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N4"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N4", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N5"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N6", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N6"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N6", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N7"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N8", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N8"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N8", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N9"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N12", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N10"));
        CPPUNIT_ASSERT( ! ptrNode->IsTerminalNode() );
        const char* names[] = { "N11", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N11"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N11", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

    {
        CNodePrivatePtr ptrNode(Camera._GetNode("N12"));
        CPPUNIT_ASSERT( ptrNode->IsTerminalNode() );
        const char* names[] = { "N12", NULL };
        CPPUNIT_ASSERT( TestTerminals( Camera, ptrNode, names ) );
    }

}
/*
    Carefully constructed register description triggering
    the Depency Bug in the V1.1.0 RC1
*/
void TerminalNodeTestSuite::TestSelectorTerminals()
{
    gcstring CameraDescriptionFile =

    CDFHeader +

"    <Category Name=\"Root\">                               \n"
"       <pFeature>Enable</pFeature>                         \n"
"       <pFeature>Selector</pFeature>                       \n"
"       <pFeature>SelectorValue</pFeature>                 \n"
"    </Category>                                            \n"

"    <Integer Name=\"SelectorValue\">                       \n"
"    <Value>1</Value>                                       \n"
"    </Integer>                                             \n"

"    <Boolean Name=\"Enable\">                              \n"
"       <Value>1</Value>                                    \n"
"    </Boolean>                                             \n"

"    <Enumeration Name=\"Selector\">                        \n"
"    <pIsImplemented>Selector_Presence</pIsImplemented>     \n"
"    <pIsAvailable>Selector_Available</pIsAvailable>        \n"
"    <EnumEntry Name=\"EA\">            \n"
"       <pIsImplemented>SelectorValue_Presence</pIsImplemented> \n"
"       <Value>1</Value>                \n"
"    </EnumEntry>                       \n"
"    <EnumEntry Name=\"EB\">            \n"
"       <pIsImplemented>SelectorValue_Presence</pIsImplemented> \n"
"       <Value>2</Value>                \n"
"    </EnumEntry>                       \n"
"    <pValue>SelectorValue</pValue>     \n"
"    <pSelected>Enable</pSelected>      \n"
"    </Enumeration>                                         \n"

"    <Integer Name=\"SelectorValue_Presence\">              \n"
"       <pValue>Selector_Presence</pValue>                                    \n"
"    </Integer>                                             \n"

"    <Integer Name=\"Selector_Presence\">                   \n"
"       <Value>1</Value>                                    \n"
"    </Integer>                                             \n"

"    <Integer Name=\"Selector_Available\">                   \n"
"       <pValue>Selector_Presence</pValue>                                    \n"
"    </Integer>                                             \n"
+ CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CNodePrivatePtr pSelector = dynamic_cast<INodePrivate*>( Camera._GetNode( "Selector" ) );
    const char* names[] = { "SelectorValue", NULL };
    CPPUNIT_ASSERT( TestTerminals( Camera, pSelector, names ) );

}

