//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "TestPort.h"
#include "GenApi/GenApi.h"
#include "GenApi/Compatibility.h"

#ifndef _WIN32
#  define _snprintf snprintf
#endif

using namespace std;
using namespace GenICam;
using namespace GenApi;

GenApi::EAccessMode CTestPort::GetAccessMode() const
{
    return RW;
}

//! Get the type of the main interface of a node
GenApi::EInterfaceType CTestPort::GetPrincipalInterfaceType() const
{
    return intfIPort;
}


void CTestPort::Read(void *pBuffer, int64_t Address, int64_t Length)
{
    EAccessMode AccessMode = NI;
    LookupEntry(Address, Length, pBuffer, AccessMode);

    if(m_Verbose)
    {
        static const char fmt[] =
            "CTestPort::Read( 0x%016"FMT_I64"X, %"FMT_I64"d ) = 0x";
        int BufferLen(256);
        char _pBuffer[256];
        int BufferLeft(_snprintf(_pBuffer, BufferLen, fmt, Address, Length));

        for(int i=0; i<Length; i++)
        {
            int temp = ((unsigned char*)pBuffer)[i];
            const int n = _snprintf(_pBuffer+BufferLeft, BufferLen-BufferLeft, "%02X", temp);

#           ifdef _MSC_VER
                if (n < 0)
                    break;
#           else
                if (BufferLeft + n >= BufferLen)
                    break;
#           endif

            BufferLeft += n;
        }
        cout << _pBuffer << "\n";
    }
    if( !IsReadable(AccessMode) )
        throw RUNTIME_EXCEPTION("Register not readable at address 0x%llx with length %lld", Address, Length);
}

void CTestPort::Write(const void *pBuffer, int64_t Address, int64_t Length)
{

    if(m_Verbose)
    {
        static const char fmt[] =
            "CTestPort::Write( 0x%016"FMT_I64"X, %"FMT_I64"d ) = 0x";

        int BufferLen(256);
        char _pBuffer[256];
        int BufferLeft(_snprintf(_pBuffer, BufferLen, fmt, Address, Length));

        for(int i=0; i<Length; i++)
        {
            int temp = ((unsigned char*)pBuffer)[i];
            const int n = _snprintf(_pBuffer+BufferLeft, BufferLen-BufferLeft, "%02X", temp);

#           ifdef _MSC_VER
                if (n < 0)
                    break;
#           else
                if (BufferLeft + n >= BufferLen)
                    break;
#           endif

            BufferLeft += n;
        }
        cout << _pBuffer << "\n";
    }

    EAccessMode AccessMode = NI;
    UpdateEntry(Address, Length, pBuffer, AccessMode);

    if( !IsWritable(AccessMode) )
        throw RUNTIME_EXCEPTION("Register not writable at address 0x%llx with length %lld", Address, Length);
}


void CTestPortReplay::Replay( IPortWriteList *pPortRecorder )
{
    cout << "CTestPortReplay::Replay\n";

    CPackerPort PackerPort;
    // collect the write commands in the packer port
    pPortRecorder->Replay(&PackerPort);
    // TODO : transfer them as one block to the camera

}

void CTestPortReplay::CPackerPort::Write(const void *pBuffer, int64_t Address, int64_t Length)
{

    static const char fmt[] =
        "CTestPortReplay::CPackerPort::Write( 0x%016"FMT_I64"X, %"FMT_I64"d ) = 0x";
    int BufferLen(256);
    char _pBuffer[256];
    int BufferLeft(_snprintf(_pBuffer, BufferLen, fmt, Address, Length));

    for(int i=0; i<Length; i++)
    {
        int temp = ((unsigned char*)pBuffer)[i];
        const int n = _snprintf(_pBuffer+BufferLeft, BufferLen-BufferLeft, "%02X", temp);
#ifdef _MSC_VER
        if (n < 0)
#else
        if (BufferLeft + n >= BufferLen)
#endif
            break;

        BufferLeft += n;
    }
    cout << _pBuffer << "\n";

}




