//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies / (c) 2006 by STEMMER IMAGING
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks, Rupert Stelz, Sascha Dorenbeck
//  $Header$
//-----------------------------------------------------------------------------
/*!
\file UtilitiesTest.cpp
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#undef GENAPI_EXPORTS
#include "GenApi/impl/Value2String.h"

#include "GenApiTest.h"
#include "UtilitiesTest.h"
#include "GenApi/Synch.h"
#include "GenApi/Container.h"
#include "GenApi/impl/GenApiUtilities.h"
#include "GenApi/EnumClasses.h"
#include "TestPort.h"
#include <Base/GCSynch.h>

#include <stdlib.h>
#include <algorithm>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( UtilitiesTestSuite, "GenApiTest");

void UtilitiesTestSuite::TestString2Value()
{
    uint8_t Buffer[256];
    CPPUNIT_ASSERT( String2Value("ff1234", Buffer, 256 ) );
    CPPUNIT_ASSERT( Buffer[0]==0xff && Buffer[1]==0x12 && Buffer[2]==0x34 );

    CPPUNIT_ASSERT( String2Value("0xff1234", Buffer, 256 ) );
    CPPUNIT_ASSERT( Buffer[0]==0xff && Buffer[1]==0x12 && Buffer[2]==0x34 );

    CPPUNIT_ASSERT( String2Value("0Xff1234", Buffer, 256 ) );
    CPPUNIT_ASSERT( Buffer[0]==0xff && Buffer[1]==0x12 && Buffer[2]==0x34 );

    int uc = 0;
    CPPUNIT_ASSERT( String2Value("25", &uc ) );
    CPPUNIT_ASSERT_EQUAL ( (int)25, uc);
    CPPUNIT_ASSERT( String2Value("0X17", &uc ) );
    CPPUNIT_ASSERT_EQUAL ( (int)0x17, uc);

    CPPUNIT_ASSERT( !String2Value("blaff1234", Buffer, 256 ) );
    CPPUNIT_ASSERT( !String2Value("f1234", Buffer, 256 ) );
    CPPUNIT_ASSERT( !String2Value("ff1234bla", Buffer, 256 ) );

  _DEFINE_GUID(guid, 0x82397b9d, 0xeb3d, 0x4552, 0xb5, 0x1e, 0x89, 0x5d, 0xad, 0xed, 0x5, 0x6 );
  gcstring strGUID;
  Value2String(guid, strGUID);
  CPPUNIT_ASSERT_EQUAL( gcstring("82397B9D-EB3D-4552-B51E-895DADED0506"), strGUID);
  GUID toCompare;
  String2Value(strGUID, &toCompare);
  CPPUNIT_ASSERT( guid == toCompare );

  NodeList_t NoNodes;
  gcstring strNodes;
  Values2String (NoNodes, strNodes);
  CPPUNIT_ASSERT (gcstring("") == strNodes);

    gcstring str;
    Value2String (0, str, Boolean);
    CPPUNIT_ASSERT_EQUAL (gcstring("false"), str);

    int64_t val = 55;

    String2Value ("true", &val, Boolean);
    CPPUNIT_ASSERT_EQUAL ((int64_t)1LL, val);
    String2Value ("false", &val, Boolean);
    CPPUNIT_ASSERT_EQUAL ((int64_t)0LL, val);
    CPPUNIT_ASSERT (!String2Value ("arrgh", &val, Boolean));

    CPPUNIT_ASSERT (!String2Value ("arrgh", &val, IPV4Address));
    CPPUNIT_ASSERT (String2Value ("0.0.0.0", &val, IPV4Address));
    CPPUNIT_ASSERT (!String2Value ("0.0.0.0.0", &val, IPV4Address));
    CPPUNIT_ASSERT (!String2Value ("abc.0.0.0", &val, IPV4Address));
    CPPUNIT_ASSERT (!String2Value ("444.0.0.0", &val, IPV4Address));
    CPPUNIT_ASSERT (!String2Value ("-4.0.0.0", &val, IPV4Address));

    CPPUNIT_ASSERT (!String2Value ("arrgh", &val, MACAddress));
    CPPUNIT_ASSERT (String2Value ("0:0:0:0:0:0", &val, MACAddress));
    CPPUNIT_ASSERT (!String2Value ("0:0:0:0:0:0:0", &val, MACAddress));
    CPPUNIT_ASSERT (!String2Value ("ijk:0:0:0:0:0", &val, MACAddress));
    CPPUNIT_ASSERT (!String2Value ("444:0:0:0:0:0", &val, MACAddress));
    CPPUNIT_ASSERT (!String2Value ("-4:0:0:0:0:0", &val, MACAddress));
}

void UtilitiesTestSuite::TestReplaceEnvironmentVariable()
{
#if defined (_WIN32)
    bool res;
    res = (SetEnvironmentVariable("__TEST1", "12345") == TRUE);
    assert(res);
    res = (SetEnvironmentVariable("__TEST2", "67890") == TRUE);
    assert(res);
    res = (SetEnvironmentVariable("__TEST3", "c:\\Program Files (x86)\\GenICam_v1_2") == TRUE);
    assert(res);
#elif defined (HAVE_CONFIG_H)
    putenv((char*)"__TEST1=12345");
    putenv((char*)"__TEST2=67890");
    putenv((char*)"__TEST3=c:\\Program Files (x86)\\GenICam_v1_2");
#else
#   error Unknown platform
#endif

    gcstring Buffer;

    // one variable to replace
    Buffer = "bla$(__TEST1)blubb";
    ReplaceEnvironmentVariables( Buffer );
    CPPUNIT_ASSERT_EQUAL( gcstring("bla12345blubb"), Buffer );

    // two variables to replace
    Buffer = "gurgel$(__TEST1)schlabber$(__TEST2)schlonz";
    ReplaceEnvironmentVariables( Buffer );
    CPPUNIT_ASSERT_EQUAL( gcstring("gurgel12345schlabber67890schlonz"), Buffer );

    // A problem with Win64
#   define GENICAM_ROOT_VERSION_TEST "__TEST3"
#   define PREFIX "$(" GENICAM_ROOT_VERSION_TEST ")/xml/GenApi/"
    Buffer = PREFIX "PreProcess_1.xsl";
    
    cout << "Before replacement : '" << Buffer << "'\n";
    ReplaceEnvironmentVariables(Buffer, true);
    cout << "After replacement : '"<< Buffer << "'\n";
    CPPUNIT_ASSERT_EQUAL( gcstring("c:\\Program%20Files%20(x86)\\GenICam_v1_2/xml/GenApi/PreProcess_1.xsl"), Buffer );

    gcstring_vector Names;
    Tokenize(Buffer, Names, "\n");
    for( gcstring_vector::iterator it=Names.begin(); it!=Names.end(); it++ )
    {
        cout << "After Tokenize : '" << (*it).c_str() << "'\n";
    }


    // unknown variable
    Buffer = "ping$(__TEST4)pong";
    CPPUNIT_ASSERT_THROW_EX(
        ReplaceEnvironmentVariables( Buffer ),
        GenICam::RuntimeException
        );

    // tricky syntax 1
    Buffer = "bla$(__TEST1))blubb$(";
    ReplaceEnvironmentVariables( Buffer );
    CPPUNIT_ASSERT_EQUAL( gcstring("bla12345)blubb$("), Buffer );

    // tricky syntax 2
    // note that there is nothing to replace
    Buffer = "ping$(__TEST4pong";
    ReplaceEnvironmentVariables( Buffer );
    CPPUNIT_ASSERT_EQUAL( gcstring("ping$(__TEST4pong"), Buffer );

    // tricky syntax 3
    // note that there is nothing to replace
    Buffer = "ping$__TEST4)pong";
    ReplaceEnvironmentVariables( Buffer );
    CPPUNIT_ASSERT_EQUAL( gcstring("ping$__TEST4)pong"), Buffer );

    // tricky syntax 4
    // note that only one item is to replace is nothing to replace
    Buffer = "ping$(__TEST1)gung$__TEST3)pong";
    ReplaceEnvironmentVariables( Buffer );
    CPPUNIT_ASSERT_EQUAL( gcstring("ping12345gung$__TEST3)pong"), Buffer );

    // trashy syntax 2
    // note that the environment variable's name should read "$(__TEST1"
    Buffer = "bla$($(__TEST1)blubb";
    CPPUNIT_ASSERT_THROW_EX(
        ReplaceEnvironmentVariables( Buffer ),
        GenICam::RuntimeException
        );

#if defined (HAVE_CONFIG_H)
    // Clean up
    putenv((char *)"__TEST1");
    putenv((char *)"__TEST2");
    putenv((char *)"__TEST3");
#endif

    // do other env related tests
        const char *EnvVarName = "okolohradcevmalezahradce";
        const char *EnvVarValue = "rostoutamtriruze";
    
    CPPUNIT_ASSERT_EQUAL (false, DoesEnvironmentVariableExist (EnvVarName));
    #if defined (_WIN32)
        res = (SetEnvironmentVariable(EnvVarName, EnvVarValue) == TRUE);
        assert(res);
    #elif defined (HAVE_CONFIG_H)
        char EnvString[128];
        sprintf (EnvString, "%s=%s", EnvVarName, EnvVarValue);
        putenv(EnvString);
    #else
    #   error Unknown platform
    #endif
        
    CPPUNIT_ASSERT_EQUAL (true, DoesEnvironmentVariableExist (EnvVarName));

}

/// Test EAccessModeClass
void UtilitiesTestSuite::TestEAccessModeClass( void )
{
    // function FromString
    CPPUNIT_ASSERT( !EAccessModeClass::FromString("RW", NULL));

    EAccessMode Value;
    CPPUNIT_ASSERT( EAccessModeClass::FromString("RW", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, RW);
    CPPUNIT_ASSERT( EAccessModeClass::FromString("RO", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, RO);
    CPPUNIT_ASSERT( EAccessModeClass::FromString("WO", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, WO);
    CPPUNIT_ASSERT( EAccessModeClass::FromString("NA", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, NA);
    CPPUNIT_ASSERT( EAccessModeClass::FromString("NI", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, NI);

    CPPUNIT_ASSERT_EQUAL( EAccessModeClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( EAccessModeClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = RW;
    EAccessModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("RW"));
    Value = RO;
    EAccessModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("RO"));
    Value = WO;
    EAccessModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("WO"));
    Value = NA;
    EAccessModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("NA"));
    Value = NI;
    EAccessModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("NI"));

    Value = (EAccessMode)1000;
    EAccessModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedAccessMode"));

    // function ToString ( returning string )
    Value = RW;
    CPPUNIT_ASSERT_EQUAL(EAccessModeClass::ToString(Value), gcstring("RW"));

    // Test Global Functions
    CPPUNIT_ASSERT_EQUAL(NI, Combine(NI, NA));
    CPPUNIT_ASSERT_EQUAL(NA, Combine(NA, RW));
    CPPUNIT_ASSERT_EQUAL(NA, Combine(RO, WO));
    CPPUNIT_ASSERT_EQUAL(WO, Combine(WO, RW));
    CPPUNIT_ASSERT_EQUAL(RO, Combine(RO, RW));
    CPPUNIT_ASSERT_EQUAL(RW, Combine(RW, RW));

    CPPUNIT_ASSERT_EQUAL(true, IsImplemented(RO));
    CPPUNIT_ASSERT_EQUAL(false, IsImplemented(NI));

    CPPUNIT_ASSERT_EQUAL(true, IsAvailable(RO));
    CPPUNIT_ASSERT_EQUAL(false, IsAvailable(NI));

    CPPUNIT_ASSERT_EQUAL(true, IsReadable(RO));
    CPPUNIT_ASSERT_EQUAL(false, IsReadable(WO));

    CPPUNIT_ASSERT_EQUAL(true, IsWritable(RW));
    CPPUNIT_ASSERT_EQUAL(false, IsWritable(RO));
}

/// Test ECachingModeClass
void UtilitiesTestSuite::TestECachingModeClass()
{
    // function FromString
    CPPUNIT_ASSERT( !ECachingModeClass::FromString("NoCache", NULL));

    ECachingMode Value;
    CPPUNIT_ASSERT( ECachingModeClass::FromString("NoCache", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, NoCache);
    CPPUNIT_ASSERT( ECachingModeClass::FromString("WriteThrough", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, WriteThrough);
    CPPUNIT_ASSERT( ECachingModeClass::FromString("WriteAround", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, WriteAround);

    CPPUNIT_ASSERT_EQUAL( ECachingModeClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( ECachingModeClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = NoCache;
    ECachingModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("NoCache"));
    Value = WriteThrough;
    ECachingModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("WriteThrough"));
    Value = WriteAround;
    ECachingModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("WriteAround"));

    Value = (ECachingMode)1000;
    ECachingModeClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedCachingMode"));

    // function ToString ( returning string )
    Value = NoCache;
    CPPUNIT_ASSERT_EQUAL(ECachingModeClass::ToString(Value), gcstring("NoCache"));

    // test global functions
    // NOTE: IsCacheable should be changed to IsCachable in both implementation and tests, there's a typo!
    CPPUNIT_ASSERT_EQUAL (false, IsCacheable(NoCache));
    CPPUNIT_ASSERT_EQUAL (true, IsCacheable(WriteThrough));
    CPPUNIT_ASSERT_EQUAL (true, IsCacheable(WriteAround));
}

/// Test ESignClass
void UtilitiesTestSuite::TestESignClass()
{
    // function FromString
    CPPUNIT_ASSERT( !ESignClass::FromString("Signed", NULL));

    ESign Value;
    CPPUNIT_ASSERT( ESignClass::FromString("Signed", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Signed);
    CPPUNIT_ASSERT( ESignClass::FromString("Unsigned", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Unsigned);

    CPPUNIT_ASSERT_EQUAL( ESignClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( ESignClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = Signed;
    ESignClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Signed"));
    Value = Unsigned;
    ESignClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Unsigned"));

    Value = (ESign)1000;
    ESignClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedSign"));

    // function ToString ( returning string )
    Value = Signed;
    CPPUNIT_ASSERT_EQUAL(ESignClass::ToString(Value), gcstring("Signed"));
}

/// Test EEndianessClass
void UtilitiesTestSuite::TestEEndianessClass()
{
    // function FromString
    CPPUNIT_ASSERT( !EEndianessClass::FromString("BigEndian", NULL));

    EEndianess Value;
    CPPUNIT_ASSERT( EEndianessClass::FromString("BigEndian", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, BigEndian);
    CPPUNIT_ASSERT( EEndianessClass::FromString("LittleEndian", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, LittleEndian);

    CPPUNIT_ASSERT_EQUAL( EEndianessClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( EEndianessClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = BigEndian;
    EEndianessClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("BigEndian"));
    Value = LittleEndian;
    EEndianessClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("LittleEndian"));

    Value = (EEndianess)1000;
    EEndianessClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedEndian"));

    // function ToString ( returning string )
    Value = BigEndian;
    CPPUNIT_ASSERT_EQUAL(EEndianessClass::ToString(Value), gcstring("BigEndian"));
}


/// Test ENameSpace Class
void UtilitiesTestSuite::TestENameSpaceClass()
{
    // function FromString
    CPPUNIT_ASSERT( !ENameSpaceClass::FromString("Custom", NULL));

    ENameSpace Value;
    CPPUNIT_ASSERT( ENameSpaceClass::FromString("Custom", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Custom);
    CPPUNIT_ASSERT( ENameSpaceClass::FromString("Standard", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Standard);

    CPPUNIT_ASSERT_EQUAL( ENameSpaceClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( ENameSpaceClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = Custom;
    ENameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Custom"));
    Value = Standard;
    ENameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Standard"));

    Value = (ENameSpace)1000;
    ENameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedNameSpace"));

    // function ToString ( returning string )
    Value = Custom;
    CPPUNIT_ASSERT_EQUAL(ENameSpaceClass::ToString(Value), gcstring("Custom"));
}

/// Test ERepresentation Class
void UtilitiesTestSuite::TestERepresentationClass()
{
    // function FromString
    CPPUNIT_ASSERT( !ERepresentationClass::FromString("Linear", NULL));

    ERepresentation Value;
    CPPUNIT_ASSERT( ERepresentationClass::FromString("Linear", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Linear);
    CPPUNIT_ASSERT( ERepresentationClass::FromString("Logarithmic", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Logarithmic);
    CPPUNIT_ASSERT( ERepresentationClass::FromString("Boolean", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Boolean);
    CPPUNIT_ASSERT( ERepresentationClass::FromString("PureNumber", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, PureNumber);
    CPPUNIT_ASSERT( ERepresentationClass::FromString("HexNumber", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, HexNumber);

    CPPUNIT_ASSERT_EQUAL( ERepresentationClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( ERepresentationClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = Linear;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Linear"));
    Value = Logarithmic;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Logarithmic"));
    Value = Boolean;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Boolean"));
    Value = PureNumber;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("PureNumber"));
    Value = HexNumber;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("HexNumber"));
    Value = IPV4Address;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("IPV4Address"));
    Value = MACAddress;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("MACAddress"));

    Value = (ERepresentation)1000;
    ERepresentationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedRepresentation"));

    // function ToString ( returning string )
    Value = Linear;
    CPPUNIT_ASSERT_EQUAL(ERepresentationClass::ToString(Value), gcstring("Linear"));
}

/// Test EVisibility Class
void UtilitiesTestSuite::TestEVisibilityClass()
{
    // function FromString
    CPPUNIT_ASSERT( !EVisibilityClass::FromString("Beginner", NULL));

    EVisibility Value;
    CPPUNIT_ASSERT( EVisibilityClass::FromString("Beginner", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Beginner);
    CPPUNIT_ASSERT( EVisibilityClass::FromString("Expert", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Expert);
    CPPUNIT_ASSERT( EVisibilityClass::FromString("Guru", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Guru);
    CPPUNIT_ASSERT( EVisibilityClass::FromString("Invisible", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Invisible);

    CPPUNIT_ASSERT_EQUAL( EVisibilityClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( EVisibilityClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = Beginner;
    EVisibilityClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Beginner"));
    Value = Expert;
    EVisibilityClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Expert"));
    Value = Guru;
    EVisibilityClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Guru"));
    Value = Invisible;
    EVisibilityClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Invisible"));

    Value = (EVisibility)1000;
    EVisibilityClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedVisibility"));

    // function ToString ( returning string )
    Value = Beginner;
    CPPUNIT_ASSERT_EQUAL(EVisibilityClass::ToString(Value), gcstring("Beginner"));

  // global function test
  CPPUNIT_ASSERT_EQUAL( IsVisible( Expert, Beginner ), false );
  CPPUNIT_ASSERT_EQUAL( IsVisible( Beginner, Expert ), true );
}


/// Test EYesNo Class
void UtilitiesTestSuite::TestEYesNoClass()
{
    // function FromString
    CPPUNIT_ASSERT( !EYesNoClass::FromString("Yes", NULL));

    EYesNo Value;
    CPPUNIT_ASSERT( EYesNoClass::FromString("Yes", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, Yes);
    CPPUNIT_ASSERT( EYesNoClass::FromString("No", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, No);

    CPPUNIT_ASSERT_EQUAL( EYesNoClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( EYesNoClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = Yes;
    EYesNoClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Yes"));
    Value = No;
    EYesNoClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("No"));

    Value = (EYesNo)1000;
    EYesNoClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedYesNo"));

    // function ToString ( returning string )
    Value = Yes;
    CPPUNIT_ASSERT_EQUAL(EYesNoClass::ToString(Value), gcstring("Yes"));
}


/// Test EStandardNameSpace Class
void UtilitiesTestSuite::TestEStandardNameSpaceClass()
{
    // function FromString
    CPPUNIT_ASSERT( !EStandardNameSpaceClass::FromString("None", NULL));

    EStandardNameSpace Value;
    CPPUNIT_ASSERT( EStandardNameSpaceClass::FromString("None", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, None);
    CPPUNIT_ASSERT( EStandardNameSpaceClass::FromString("GEV", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, GEV);
    CPPUNIT_ASSERT( EStandardNameSpaceClass::FromString("IIDC", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, IIDC);
    CPPUNIT_ASSERT( EStandardNameSpaceClass::FromString("CL", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, CL);
    CPPUNIT_ASSERT( EStandardNameSpaceClass::FromString("USB", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, USB);

    CPPUNIT_ASSERT_EQUAL( EStandardNameSpaceClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( EStandardNameSpaceClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = None;
    EStandardNameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("None"));
    Value = GEV;
    EStandardNameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("GEV"));
    Value = IIDC;
    EStandardNameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("IIDC"));
    Value = CL;
    EStandardNameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("CL"));
    Value = USB;
    EStandardNameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("USB"));

    Value = (EStandardNameSpace)1000;
    EStandardNameSpaceClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedStandardNameSpace"));

    // function ToString ( returning string )
    Value = None;
    CPPUNIT_ASSERT_EQUAL(EStandardNameSpaceClass::ToString(Value), gcstring("None"));
}

/// Test TestSyncClasses (for MD config only)
void UtilitiesTestSuite::TestSyncClasses()
{

  // test real lock (only single threaded test because of simple critical section calls)
  GenApi::CLock realLock;
  realLock.Lock();
  CPPUNIT_ASSERT( realLock.TryLock() ); // true because of same thread context
  realLock.Unlock();
  realLock.Unlock();

  // test autolock
  {
    GenApi::AutoLock lock(realLock);
    CPPUNIT_ASSERT( true );
  }

}


#ifdef _WIN32

#include <windows.h>
#include <process.h>

class TestLockThreadParams
{
public:
    bool        Run;
    gcstring    strLockName;
    unsigned    SleepTimeout_ms;
    UtilitiesTestSuite* pThis;
};

unsigned __stdcall UtilitiesTestSuite::TestLockThread(void* p)
{
    TestLockThreadParams* pParams = reinterpret_cast<TestLockThreadParams*>(p);

    CGlobalLock lock(pParams->strLockName);

    // create the name of the port in form of "Many02", "Many03"
    try
    {
        while (pParams->Run)
        {
            if (!lock.Lock(1000))
            {
                throw std::exception("timeout while trying to acquire global lock.");
            }
            CGlobalLockUnlocker unlocker(lock);

            if (!pParams->Run)
            {
                break;
            }

            ::Sleep(pParams->SleepTimeout_ms);

            // global unlocker will unlock the lock
        }

        return 0;
    }
    catch (const std::exception& e)
    {
        pParams->pThis->debug("Exception in TestLock: '%hs'", e.what());
        return unsigned(-1);
    }
}



/// Test GlobalLock Classes
void UtilitiesTestSuite::TestGlobalLock()
{
    {
        CGlobalLock testLockA("GenICam_testLockA");
        CPPUNIT_ASSERT_MESSAGE("Could not create ansi named global lock", testLockA.IsValid());

        CGlobalLock testLockW(L"GenICam_testLockW");
        CPPUNIT_ASSERT_MESSAGE("Could not create unicode named global lock", testLockW.IsValid());

        CGlobalLock testLockG(gcstring("GenICam_testLockG"));
        CPPUNIT_ASSERT_MESSAGE("Could not create gcstring named global lock", testLockG.IsValid());
    }

    {
        CGlobalLock unnamedTestLockA("");
        CPPUNIT_ASSERT_MESSAGE("Could not create ansi unnamed global lock", unnamedTestLockA.IsValid());

        CGlobalLock unnamedTestLockW(L"");
        CPPUNIT_ASSERT_MESSAGE("Could not create unicode unnamed global lock", unnamedTestLockW.IsValid());

        CGlobalLock unnamedTestLockG(gcstring(""));
        CPPUNIT_ASSERT_MESSAGE("Could not create gcstring unnamed global lock", unnamedTestLockG.IsValid());
    }

    {
        CGlobalLock testLock("");

        CPPUNIT_ASSERT_MESSAGE("Could not create unnamed global lock", testLock.IsValid());
    
        CPPUNIT_ASSERT_MESSAGE("Could not acquire lock", testLock.Lock(500));
        // recursive acquisition should work
        CPPUNIT_ASSERT_MESSAGE("Could not acquire lock", testLock.TryLock());
        CPPUNIT_ASSERT_NO_THROW(testLock.Unlock());
        CPPUNIT_ASSERT_NO_THROW(testLock.Unlock());
    }

    {
        char szProcId[33] = {0};
        _ltoa(GetCurrentProcessId(), szProcId, 16);

        TestLockThreadParams params;
        params.pThis = this;
        params.Run = true;
        params.SleepTimeout_ms = 2 * 1000;
        params.strLockName = gcstring("TestGlobalLock_0x") + szProcId;

        CGlobalLock lock(params.strLockName);
        CPPUNIT_ASSERT_MESSAGE("Could not create named global lock", lock.IsValid());
        
        HANDLE hThread = reinterpret_cast<HANDLE>(_beginthreadex(NULL, 0, TestLockThread, &params, 0, NULL));
        CPPUNIT_ASSERT_MESSAGE("Could not create thread", hThread != NULL);

        // this thread should be little bit higher the the main thread
        SetThreadPriority(hThread, THREAD_PRIORITY_ABOVE_NORMAL);

        // let the thread acquire the lock
        ::Sleep(100);

        try
        {
            // try to get the lock (should fail)
            CPPUNIT_ASSERT_MESSAGE("Lock shouldn't available at this time", !lock.TryLock());
            // try to get the lock (should fail)
            CPPUNIT_ASSERT_MESSAGE("Lock shouldn't available at this time 2", !lock.Lock(10));
            // wait until the lock is available (should be successful
            CPPUNIT_ASSERT_MESSAGE("Lock should be available at this time", lock.Lock(2500));
            
            // tell the thread to stop
            params.Run = false;
            params.SleepTimeout_ms = 1000;
            // let the thread exit
            lock.Unlock();

            // wait for it to exit
            WaitForSingleObject(hThread, 5000);
            CloseHandle(hThread);

        }
        catch (...)
        {
            // wait for thread to exit
            params.Run = false;
            WaitForSingleObject(hThread, 5000);
            CloseHandle(hThread);

            throw;
        }
    }
}

#else

#include <unistd.h>
#include <sstream>

class TestLockThreadParams
{
public:
    bool        Run;
    gcstring    strLockName;
    unsigned    SleepTimeout_ms;
    UtilitiesTestSuite* pThis;
};

void* UtilitiesTestSuite::TestLockThread(void* p)
{
    TestLockThreadParams* pParams = reinterpret_cast<TestLockThreadParams*>(p);

    CGlobalLock lock(pParams->strLockName);

    // create the name of the port in form of "Many02", "Many03"
    try
    {
        while (pParams->Run)
        {
            if (!lock.Lock(1000))
            {
                throw std::runtime_error("timeout while trying to acquire global lock.");
            }
            CGlobalLockUnlocker unlocker(lock);

            if (!pParams->Run)
            {
                break;
            }

            usleep(pParams->SleepTimeout_ms * 1000);

            // global unlocker will unlock the lock
        }

        pthread_exit(NULL);
    }
    catch (const std::exception& e)
    {
        pParams->pThis->debug("Exception in TestLock: '%hs'", e.what());
        pthread_exit(NULL);
    }
}

#include <pthread.h>

/// Test GlobalLock Classes
void UtilitiesTestSuite::TestGlobalLock()
{
    {
        CGlobalLock testLockA("GenICam_testLockA");
        CPPUNIT_ASSERT_MESSAGE("Could not create ansi named global lock", testLockA.IsValid());

#ifdef _WIN32
        CGlobalLock testLockW(L"GenICam_testLockW");
        CPPUNIT_ASSERT_MESSAGE("Could not create unicode named global lock", testLockW.IsValid());
#endif

        CGlobalLock testLockG(gcstring("GenICam_testLockG"));
        CPPUNIT_ASSERT_MESSAGE("Could not create gcstring named global lock", testLockG.IsValid());
    }

    {
        CGlobalLock unnamedTestLockA("");
        CPPUNIT_ASSERT_MESSAGE("Could not create ansi unnamed global lock", unnamedTestLockA.IsValid());

#ifdef _WIN32
        CGlobalLock unnamedTestLockW(L"");
        CPPUNIT_ASSERT_MESSAGE("Could not create unicode unnamed global lock", unnamedTestLockW.IsValid());
#endif

        CGlobalLock unnamedTestLockG(gcstring(""));
        CPPUNIT_ASSERT_MESSAGE("Could not create gcstring unnamed global lock", unnamedTestLockG.IsValid());
    }

    {
        CGlobalLock testLock("");

        CPPUNIT_ASSERT_MESSAGE("Could not create unnamed global lock", testLock.IsValid());

        CPPUNIT_ASSERT_MESSAGE("Could not acquire lock", testLock.Lock(500));
        // try to get the lock (should fail)
        CPPUNIT_ASSERT_MESSAGE("Lock shouldn't be available at this time", !testLock.TryLock());
        CPPUNIT_ASSERT_NO_THROW(testLock.Unlock());
        CPPUNIT_ASSERT_MESSAGE("Could not acquire lock", testLock.TryLock());
        CPPUNIT_ASSERT_NO_THROW(testLock.Unlock());
    }

    {
        std::ostringstream szProcId;
        szProcId << getpid();

        TestLockThreadParams params;
        params.pThis = this;
        params.Run = true;
        params.SleepTimeout_ms = 2 * 1000;
        params.strLockName = gcstring("TestGlobalLock_0x") + szProcId.str().c_str();

        CGlobalLock lock(params.strLockName);
        CPPUNIT_ASSERT_MESSAGE("Could not create named global lock", lock.IsValid());

        pthread_t hThread;
        int threadCreation = pthread_create(&hThread, NULL, &TestLockThread, &params);
        CPPUNIT_ASSERT_MESSAGE("Could not create thread", threadCreation == 0);

        // this thread should be little bit higher the the main thread
        //SetThreadPriority(hThread, THREAD_PRIORITY_ABOVE_NORMAL);

        // let the thread acquire the lock
        usleep(100*1000);

        try
        {
            // try to get the lock (should fail)
            CPPUNIT_ASSERT_MESSAGE("Lock shouldn't available at this time", !lock.TryLock());
            // try to get the lock (should fail)
            CPPUNIT_ASSERT_MESSAGE("Lock shouldn't available at this time 2", !lock.Lock(10));
            // tell the thread to stop else the thread will acquire the lock immediately again
            params.Run = false;
            params.SleepTimeout_ms = 1000;
            // wait until the lock is available (should be successful
            CPPUNIT_ASSERT_MESSAGE("Lock should be available at this time", lock.Lock(2500));
            
            // let the thread exit
            lock.Unlock();

            // wait for it to exit
            pthread_join(hThread, NULL);

        }
        catch (...)
        {
            // wait for thread to exit
            params.Run = false;
            pthread_join(hThread, NULL);

            throw;
        }
    }
}
#endif



void UtilitiesTestSuite::TestOpOnNodeVectors()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Category Name=\"ScalarFeatures\">                \n"
    "    <pFeature>Shutter</pFeature>                \n"
    "    <pFeature>Gain</pFeature>                    \n"
    "</Category>                                    \n"
    "                                                \n"
    "<Integer Name=\"Shutter\">                        \n"
    "    <Value>20</Value>                            \n"
    "    <Min>0</Min>                                \n"
    "    <Max>100</Max>                                \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Integer Name=\"Gain\">                        \n"
    "    <Value>20</Value>                            \n"
    "    <Min>5</Min>                                \n"
    "    <Max>200</Max>                                \n"
    "</Integer>                                        \n"
    "                                                \n"
    "<Enumeration Name=\"PixelFormat\">        \n"
    "<EnumEntry Name=\"Mono8\">             \n"
    "            <Value>0</Value>            \n"
    "</EnumEntry>                           \n"
    "<EnumEntry Name=\"Mono16\">            \n"
    "            <Value>1</Value>            \n"
    "</EnumEntry>                           \n"
    "<EnumEntry Name=\"RGB24\">                \n"
    "            <Value>2</Value>            \n"
    "</EnumEntry>                           \n"
    "    <pValue>Value</pValue>                \n"
    "</Enumeration>                            \n"
    "<Integer Name=\"Value\">                \n"
    "    <Value>10</Value>                    \n"
    "</Integer>                                \n"
    "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CTestPort Port;
    Port.CreateEntry(0x0104, (uint32_t)1024, RW);

    // conntect the node map to the port
    Camera._Connect(&Port, "Port");


    // 1) Test node_vector
    CEnumerationPtr ptrValue(Camera._GetNode("PixelFormat"));
    CPPUNIT_ASSERT( ptrValue.IsValid() );

    NodeList_t Entries;
    ptrValue->GetEntries(Entries);

    NodeList_t::iterator pEntry;
    NodeList_t::iterator pLastEntry;

    intptr_t nNbElement(0);

    for(pEntry = Entries.begin(); pEntry != Entries.end(); pEntry++ )
    {
        pLastEntry = pEntry;
        nNbElement ++;
    }

    CEnumEntryPtr ptrEnumEntry(*pLastEntry);
    gcstring strLastSymbolic(ptrEnumEntry->GetSymbolic());


    if (nNbElement)
        nNbElement --;

#ifdef _WIN32
    node_vector::const_iterator it(::operator +(nNbElement, Entries.begin()));
#else
    node_vector::const_iterator it = nNbElement + Entries.begin();
#endif
    CEnumEntryPtr ptrEnumEntry1(*it);
    gcstring strLastSymbolic1(ptrEnumEntry1->GetSymbolic());
    CPPUNIT_ASSERT_EQUAL(strLastSymbolic, strLastSymbolic1);

    node_vector::const_iterator it1 = Entries.begin();
#ifdef _WIN32
    it = ::operator +(nNbElement, it1);
#else
    it = nNbElement + it1;
#endif
    ptrEnumEntry1 = *it;
    strLastSymbolic1 = ptrEnumEntry1->GetSymbolic();
    CPPUNIT_ASSERT_EQUAL(strLastSymbolic, strLastSymbolic1);


    // 2) Test value_vector
    CCategoryPtr ptrCat = Camera._GetNode("ScalarFeatures");
    CPPUNIT_ASSERT( ptrCat.IsValid() );

    FeatureList_t pFeatureList;
    ptrCat->GetFeatures(pFeatureList);

    nNbElement = 0;
    FeatureList_t::const_iterator itFeat;
    FeatureList_t::const_iterator pLastFeature;
    gcstring strCat;

    for(itFeat = pFeatureList.begin(); itFeat != pFeatureList.end(); itFeat++ )
    {
        pLastFeature = itFeat;
        strCat = (*itFeat)->GetNode()->GetName();
        nNbElement ++;
    }

    if (nNbElement)
        nNbElement --;

#ifdef _WIN32
    value_vector::const_iterator ValIt = ::operator +(nNbElement, pFeatureList.begin());
#else
    value_vector::const_iterator ValIt = nNbElement + pFeatureList.begin();
#endif
    gcstring strLastCat = (*ValIt)->GetNode()->GetName();
    CPPUNIT_ASSERT_EQUAL(strCat, strLastCat);

    value_vector::const_iterator ValIt1 = pFeatureList.begin();
#ifdef _WIN32
    ValIt = ::operator +(nNbElement, ValIt1);
#else
    ValIt = nNbElement + ValIt1;
#endif
    strLastCat = (*ValIt)->GetNode()->GetName();
    CPPUNIT_ASSERT_EQUAL(strCat, strLastCat);
}



void UtilitiesTestSuite::TestMiscellaneous()
{

    // 1) String2Value
    gcstring ValueStr = "0x1234P1";
    uint8_t pTabVal[10];
    CPPUNIT_ASSERT_EQUAL(::String2Value(ValueStr, pTabVal, sizeof(pTabVal) / sizeof(uint8_t)), false);

    // 2) Combine
    EVisibility First = Invisible;
    EVisibility Second = Guru;
    CPPUNIT_ASSERT_EQUAL(::Combine(First, Second), Invisible);

    First = Expert;
    Second = Beginner;
    CPPUNIT_ASSERT_EQUAL(::Combine(First, Second), Expert);

#if _WIN32
    // Test some Utility functions
    gcstring DLLName (GetGenICamDLLName("Log"));
    gcstring DLLFolder (GetGenICamDLLFolder());
    gcstring DLLPath (GetGenICamDLLPath("Log"));
    gcstring DLLPathCombined (DLLFolder + "/" + DLLName);
    struct _finddata_t fileinfo;
    intptr_t lFile = ::_findfirst(DLLPath.c_str(), &fileinfo);
    CPPUNIT_ASSERT (lFile != -1);
    if (lFile != -1) ::_findclose(lFile);
    lFile = ::_findfirst(DLLPathCombined.c_str(), &fileinfo);
    CPPUNIT_ASSERT (lFile != -1);
    if (lFile != -1) ::_findclose(lFile);
#endif

    CPPUNIT_ASSERT_THROW_EX ((INTEGRAL_CAST2<unsigned char, unsigned>(500)), GenICam::RuntimeException);
}

void UtilitiesTestSuite::TestGetInterfaceName()
{
    // create and initilize node map
    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Integer Name=\"Shutter\">                        \n"
    "    <Value>20</Value>                            \n"
    "</Integer>                                        \n"
    "                                                \n"
    + CDFFooter;

    CNodeMapRef Camera;
    Camera._LoadXMLFromString(CameraDescriptionFile);

    CIntegerPtr ptrInteger = Camera._GetNode("Shutter");
    gcstring InterfaceName = GetInterfaceName( ptrInteger );
    CPPUNIT_ASSERT_EQUAL( InterfaceName, gcstring("IInteger") );
    CPPUNIT_ASSERT_EQUAL( intfIInteger, ptrInteger->GetNode()->GetPrincipalInterfaceType() );


}
#if _MSC_VER < 1400
typedef int errno_t;
inline errno_t _putenv_s( char a[], char b[] )
{
    gcstring s = gcstring(a)+ "=" + b;
    return putenv( const_cast<char *>(s.c_str() ) );
}
#endif
void UtilitiesTestSuite::TestMissingRoot()
{
    // First test the missing root variable
#if defined (_WIN32)
    char sOrigRoot[MAX_PATH+1];
    errno_t res;
    char *env = getenv (GENICAM_ROOT_VERSION);
    assert (env && strlen(env)<MAX_PATH);
    strcpy (sOrigRoot, env);
    res = _putenv_s (GENICAM_ROOT_VERSION, "");
    assert(res==0);


    gcstring CameraDescriptionFile =
    CDFHeader +
    "                                                \n"
    "<Integer Name=\"Dummy\">                        \n"
    "    <Value>20</Value>                            \n"
    "</Integer>                                        \n"
   "                                        \n"
    + CDFFooter;

    CNodeMapRef Camera;
    CPPUNIT_ASSERT_THROW_EX(
        Camera._LoadXMLFromString(CameraDescriptionFile),
        GenICam::RuntimeException
        );

    // Test with too long path
    char sModifRoot[MAX_PATH+2];
    memset (sModifRoot, 0, MAX_PATH+2);
    memset (sModifRoot, 'x', MAX_PATH+1);
    res = _putenv_s (GENICAM_ROOT_VERSION, sModifRoot);
    assert(res==0);

    CNodeMapRef Camera1;
    CPPUNIT_ASSERT_THROW_EX(
        Camera1._LoadXMLFromString(CameraDescriptionFile),
        GenICam::RuntimeException
        );

    // Now try the path with ending slash/backslash
    strcpy (sModifRoot, sOrigRoot);
    strcat (sModifRoot, "/");
    res = _putenv_s (GENICAM_ROOT_VERSION, sModifRoot);
    assert(res==0);

    CNodeMapRef Camera2;
    CPPUNIT_ASSERT_NO_THROW( Camera2._LoadXMLFromString(CameraDescriptionFile) );

    strcpy (sModifRoot, sOrigRoot);
    strcat (sModifRoot, "\\");
    res = _putenv_s (GENICAM_ROOT_VERSION, sModifRoot);
    assert(res==0);

    CNodeMapRef Camera3;
    CPPUNIT_ASSERT_NO_THROW( Camera3._LoadXMLFromString(CameraDescriptionFile) );

    // Restore the environment
    res = _putenv_s (GENICAM_ROOT_VERSION, sOrigRoot);
    assert(res==0);
#elif defined (HAVE_CONFIG_H)
    // TODO
#else
#   error Unknown platform
#endif
}

/// Test EDisplayNotation Class
void UtilitiesTestSuite::TestEDisplayNotationClass()
{
    // function FromString
    CPPUNIT_ASSERT( !EDisplayNotationClass::FromString("Automatic", NULL));

    EDisplayNotation Value;
    CPPUNIT_ASSERT( EDisplayNotationClass::FromString("Automatic", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, fnAutomatic);
    CPPUNIT_ASSERT( EDisplayNotationClass::FromString("Fixed", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, fnFixed);
    CPPUNIT_ASSERT( EDisplayNotationClass::FromString("Scientific", &Value));
    CPPUNIT_ASSERT_EQUAL(Value, fnScientific);

    CPPUNIT_ASSERT_EQUAL( EDisplayNotationClass::FromString("BLA", &Value), false);

    // function ToString
    gcstring strValue;
    CPPUNIT_ASSERT_THROW_EX( EDisplayNotationClass::ToString(strValue, NULL), GenICam::InvalidArgumentException);

    Value = fnAutomatic;
    EDisplayNotationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Automatic"));
    Value = fnFixed;
    EDisplayNotationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Fixed"));
    Value = fnScientific;
    EDisplayNotationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("Scientific"));

    Value = (EDisplayNotation)1000;
    EDisplayNotationClass::ToString(strValue, &Value);
    CPPUNIT_ASSERT_EQUAL(strValue, gcstring("_UndefinedEDisplayNotation"));

    // function ToString ( returning string )
    Value = fnAutomatic;
    CPPUNIT_ASSERT_EQUAL(EDisplayNotationClass::ToString(Value), gcstring("Automatic"));
}

void UtilitiesTestSuite::TestURLEscape( void )
{
    std::ostringstream ssBuffer;        
    for( char ASCII=0x20; ASCII<=126; ASCII++ )
        ssBuffer << ASCII;

    gcstring UnEscaped( ssBuffer.str().c_str() );
    gcstring Escaped = UrlEncode(UnEscaped);
    gcstring Reference("%20%21\"%23%24%%26%27%28%29%2a%2b%2c-./0123456789%3a%3b<%3d>%3f%40ABCDEFGHIJKLMNOPQRSTUVWXYZ%5b/%5d^_`abcdefghijklmnopqrstuvwxyz{|}~");

    cout << "UnEscaped       = " << UnEscaped << endl;
    cout << "Escaped         = " << Escaped << endl;

    CPPUNIT_ASSERT_EQUAL(Escaped, Reference);

    std::string _ReferenceBack( UnEscaped );
    // UrlEncode replaces \ by /
    std::replace(_ReferenceBack.begin(), _ReferenceBack.end(), '\\', '/' );
    gcstring ReferenceBack( _ReferenceBack.c_str() );

    gcstring ConvertedBack = UrlDecode(Escaped);
    cout << "convertred back = " << ConvertedBack << endl;

    CPPUNIT_ASSERT_EQUAL(ReferenceBack, ConvertedBack);

}

