//-----------------------------------------------------------------------------
//  (c) 2005 by Basler Vision Technologies
//  Section: Vision Components
//  Project: GenApiTest
//    Author:  Fritz Dierks
//  $Header$
//-----------------------------------------------------------------------------

/*!
\file
*/

#ifdef _MSC_VER
#   include "GenApi/impl/GlobalPragmas.h"  // this must be the first include!
#endif

#include "GenApiTest.h"
#include "XMLCacheTest.h"
#include "GenApi/GenApi.h"

#include <stdlib.h>

using namespace std;
using namespace GenICam;
using namespace GenApi;

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( XMLCacheTestSuite, "GenApiTest");

#if defined (_WIN32)
#   define setenv(key, val, repl) SetEnvironmentVariable(key, val)
#endif

void XMLCacheTestSuite::TestXMLCache()
{
    gcstring CacheName = GENICAM_CACHE_VERSION;
    CNodeMapRef::_ClearXMLCache();

    // remember the cache directory
    gcstring CacheDirectory = GetValueOfEnvironmentVariable( GENICAM_CACHE_VERSION );

    // clear the cache directory; this disables the caching
    // Don't use setenv()!
    // setenv() potentially has issues on Windows when dealing with another
    // DLL setting an env variable with a different C runtime as GenICam is using. Use
    // SetEnvironmentVariable() which will always work regardless of runtime mixing
#   if defined (_MSC_VER)
        ::SetEnvironmentVariableW(CacheName.w_str(), NULL);
#   elif defined (HAVE_CONFIG_H)
        setenv(CacheName.c_str(), "", 1);
#   else
#       error Unknown platform
#   endif

    // cache disabled
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                        \n"
        "    <Node Name=\"Node\">                                \n"
        "    </Node>                                             \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

        CNodeMapRef Camera;

        Camera._LoadXMLFromString(CameraDescriptionFile);

        const CNodePtr ptrNode(Camera._GetNode("Node"));
        CPPUNIT_ASSERT( ptrNode.IsValid() );
    }

    // set the cache directory to the old value; this enables the caching
#   if defined (_MSC_VER)
        ::SetEnvironmentVariableW(CacheName.w_str(), CacheDirectory.w_str());
#   elif defined (HAVE_CONFIG_H)
        setenv(CacheName.c_str(), CacheDirectory.c_str(), 1);
#   else
#       error Unknown platform
#   endif

    // cache enabled - cache empty
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                        \n"
        "    <Node Name=\"Node\">                                \n"
        "    </Node>                                             \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        const CNodePtr ptrNode(Camera._GetNode("Node"));
        CPPUNIT_ASSERT( ptrNode.IsValid() );
    }

    // cache enabled - cache full
    {
        gcstring CameraDescriptionFile =

        CDFHeader +
        "                                                        \n"
        "    <Node Name=\"Node\">                                \n"
        "    </Node>                                             \n"
        "                                                        \n"
        "                                                        \n"
        + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        const CNodePtr ptrNode(Camera._GetNode("Node"));
        CPPUNIT_ASSERT( ptrNode.IsValid() );
    }
}

void XMLCacheTestSuite::TestTicket777()
{
#ifdef _MSC_VER

    // remember the cache directory
    gcstring CacheDirectory( GetValueOfEnvironmentVariable(GENICAM_CACHE_VERSION) );
    gcstring LogConfigFile( GetValueOfEnvironmentVariable(GENICAM_LOG_CONFIG_VERSION) );

    gcstring GenICamCacheVersion( GENICAM_CACHE_VERSION );
    gcstring GenICamLogConfigVersion( GENICAM_LOG_CONFIG_VERSION );

	try
    {

        // Create a caching directory with Japanese name

        // NOTE : VisualStudio can deal with UniCode but GNU seems to have trouble
        // See http://rishida.net/tools/conversion/ for what this means
        // create a cache directory in $(GENICAM_ROOT)/xml/genapi/cache/<japanese>
        gcstring TestCacheDirectory = CacheDirectory + "\\" + gcstring( L"\x5C71\x7530\x592A\x90CE" );
        RemoveDirectoryW( TestCacheDirectory.w_str() );
        CreateDirectoryW( TestCacheDirectory.w_str(), NULL );

        // set GENICAM_CACHE to the new location
        // note that the environemtn variable name must also be wchar_t
        BOOL res = ::SetEnvironmentVariableW( GenICamCacheVersion.w_str(), TestCacheDirectory.w_str() );
        CPPUNIT_ASSERT( res );

        // copy the log file to the new location
        gcstring TestLogFile = TestCacheDirectory + "\\DebugLogging.properties";
        res = CopyFileW( LogConfigFile.w_str(), TestLogFile.w_str(), false );
        CPPUNIT_ASSERT(res);

        // set the new log file
        res = ::SetEnvironmentVariableW( GenICamLogConfigVersion.w_str(), TestLogFile.w_str() );
        CPPUNIT_ASSERT( res );

        CLog::ConfigureFromEnvironment();

        // just for test: read back the environment variable
        // print the register content
        {
            UINT oldcp = GetConsoleOutputCP();
            SetConsoleOutputCP(CP_UTF8);
            cout << "TestCacheDirectory = " << TestCacheDirectory.c_str() << endl;
            cout << "TestLogFile = " << TestLogFile.c_str() << endl;
            SetConsoleOutputCP(oldcp);
        }

        CNodeMapRef::_ClearXMLCache();

        // Do something with the cache
        gcstring CameraDescriptionFile =

            CDFHeader +
            "                                                        \n"
            "    <Node Name=\"Node\">                                \n"
            "    </Node>                                             \n"
            "                                                        \n"
            + CDFFooter;

        CNodeMapRef Camera;
        Camera._LoadXMLFromString(CameraDescriptionFile);

        NODE_POINTER(Node, Node);
        cout << ptrNode->GetName() << endl;

        gcstring_vector FileNames;
        GetFiles( TestCacheDirectory + "\\*.*", FileNames );
        for( gcstring_vector::iterator itFileName=FileNames.begin(); itFileName!=FileNames.end(); itFileName++ )
        {
            gcstring FileName(TestCacheDirectory + gcstring("\\") + *itFileName);    
            ::DeleteFileW(FileName.w_str());
        }
        RemoveDirectoryW( TestCacheDirectory.w_str() );
    }
    catch(...)
    {
        // Restore the cache directory
        ::SetEnvironmentVariableW( GenICamCacheVersion.w_str(), CacheDirectory.w_str() );
        ::SetEnvironmentVariableW( GenICamLogConfigVersion.w_str(), LogConfigFile.w_str() );
        CLog::ConfigureFromEnvironment();

        throw;
    }

    // Restore the cache directory
    ::SetEnvironmentVariableW( GenICamCacheVersion.w_str(), CacheDirectory.w_str() );
    ::SetEnvironmentVariableW( GenICamLogConfigVersion.w_str(), LogConfigFile.w_str() );
    CLog::ConfigureFromEnvironment();

#endif
}
